% GOAT GUI for Auto downloading and installing various observational datasets
% Copyright (C) Ori Adam Jan.2012.
% Updated Dec.2013 by Ori Adam
% Compatible with version 2013L or later
%
% see also GOAT, GOAT_MyDefaults, GOAT_Environment, GOAT_Defaults

function GOAT_AutoDownload
warning off all
D.Version = '2014A' ;
D = Consts(D)       ; % Global Consts
D = DataSets(D)     ; % Produce Data Set Tables
D = NewDataSet(D)   ; % Set Default Dataset
D = Download_GUI(D) ; % Produce User Interface

GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',D.Download_Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 1  % Data Set
                D.DataSets.DataSetIdx = get(D.h_DataSet,'value') ;
                D = NewDataSet(D) ;
                set(D.h_Fields,'string',D.DataSet.FieldsStr) ;
                D.Field = eval(['D.DataSet.Fields.',cell2mat(D.DataSet.Fields.Names(D.DataSet.FieldsIdx))]) ;
                set(D.h_Fields,'TooltipString',['[ ',D.Field.Units,' ]   ',D.Field.FieldDescription]) ;
                YearsStr = '' ;
                for i = D.Field.FirstYear : D.Field.LastYear
                    YearsStr = [YearsStr,int2str(i),' |'] ;
                end
                YearsStr = YearsStr(1:end-1) ;
                D.Field.BeginYear = D.Field.FirstYear ;
                D.Field.EndYear   = D.Field.LastYear ;
                set(D.h_year1 ,'string',YearsStr,'value',1) ;
                set(D.h_year2 ,'string',YearsStr,'value',D.Field.EndYear - D.Field.FirstYear + 1) ;
                set(D.h_Folder,'string',D.DataSet.Address)
                try
                    set(D.h_DataSet,'TooltipString',D.DataSet.DescriptionTip) ;
                end
                if D.DataSet.TimeSelect
                    set(D.h_year1 ,'enable','on') ;
                    set(D.h_year2 ,'enable','on') ;
                else
                    set(D.h_year1 ,'enable','off') ;
                    set(D.h_year2 ,'enable','off') ;
                end
            case 2  % Field
                D.DataSet.FieldsIdx = get(D.h_Fields,'value') ;
                if D.DataSet.FieldsIdx<= length(D.DataSet.Fields.Names)
                    D.Field = eval(['D.DataSet.Fields.',cell2mat(D.DataSet.Fields.Names(D.DataSet.FieldsIdx))]) ;
                    set(D.h_Fields,'TooltipString',['[ ',D.Field.Units,' ]   ',D.Field.FieldDescription]) ;
                    YearsStr = '' ;
                    for i = D.Field.FirstYear : D.Field.LastYear
                        YearsStr = [YearsStr,int2str(i),' |'] ;
                    end
                    YearsStr = YearsStr(1:end-1) ;
                    D.Field.BeginYear = D.Field.FirstYear ;
                    D.Field.EndYear   = D.Field.LastYear ;
                    set(D.h_year1 ,'string',YearsStr,'value',1) ;
                    set(D.h_year2 ,'string',YearsStr,'value',D.Field.EndYear - D.Field.FirstYear + 1) ;
                else
                    set(D.h_Fields,'TooltipString','Download all fields within the specified period') ;
                end
            case 11 % Year Begin
                D.Field.BeginYear = D.Field.FirstYear + get(D.h_year1,'value') - 1 ;
                if D.Field.EndYear < D.Field.BeginYear
                    D.Field.EndYear = D.Field.BeginYear ;
                    set(D.h_year2,'value',get(D.h_year1,'value')) ;
                end
            case 12 % Year End
                D.Field.EndYear = D.Field.FirstYear + get(D.h_year2,'value') - 1 ;
                if D.Field.BeginYear > D.Field.EndYear
                    D.Field.BeginYear = D.Field.EndYear ;
                    set(D.h_year1,'value',get(D.h_year2,'value')) ;
                end
            case 21 % Edit Download Directory
                D.DataSet.Address = get(D.h_Folder,'string') ;
            case 22 % Browse
                Address0 = uigetdir ; 
                try
                  if Address0 == 0
                  continue ; 
                  else
                    D.DataSet.Address = Address0 ;  
                    set(D.h_Folder,'string',D.DataSet.Address) ;
                  end
                catch
                  D.DataSet.Address = Address0 ;  
                  set(D.h_Folder,'string',D.DataSet.Address) ;  
                end
            case 100 % GO
                if D.DataSet.FieldsIdx > length(D.DataSet.Fields.Names) % All
                    BeginYear = D.Field.BeginYear ;
                    EndYear   = D.Field.EndYear   ;
                    for i = 1:length(D.DataSet.Fields.Names)
                        D.Field = eval(['D.DataSet.Fields.',cell2mat(D.DataSet.Fields.Names(i))]) ;
                        D.Field.BeginYear = BeginYear ;
                        D.Field.EndYear   = EndYear ;
                        try
                            eval(['D = ',D.DataSets.DataSetFuncNames{D.DataSets.DataSetIdx},'(''Download'',D) ; ']) ;
                            %D = DownloadField(D) ;
                        end
                    end
                else
                    eval(['D = ',D.DataSets.DataSetFuncNames{D.DataSets.DataSetIdx},'(''Download'',D) ; ']) ;
                    %D = DownloadField(D) ;
                end
            case 999 % EXIT
                GUIcon = 0 ;
        end
    catch Error
        beep ;
        set(D.h_message,'string',['Error: ',Error.message]) ; drawnow ;
        try
            set(0,'CurrentFigure',D.Download_Fig) ;
        catch % Figure was deleted
            GUIcon = 0 ;
            continue ;
        end
        disp(['Error: ',Error.message]) ;
        for e = 1: length(Error.stack)
            disp(['Error in ''',Error.stack(e).name,''', Line: ',int2str(Error.stack(e).line)]) ;
        end
    end
end
close(D.Download_Fig) ;
return ;
% General Consts and Defaults
function D = Consts(D)

D.DataSets.DataSetIdx = 1 ;
D.scrsz      = get(0,'ScreenSize')                       ;
D.pos        = [D.scrsz(3)/4 D.scrsz(4)/10 (D.scrsz(3)*2/5) (D.scrsz(4)*3/5)] ;
D.FigBackgroudColor = [1 1 1]          ; % White Figure Background color
D.ButtonColor       = [.9 .9 .91]      ; % Default Button Color
D.ActiveColor       = [.7  1  .7 ]     ; % During Runs, the "Run" Button" is set to this color
D.DropTextColor     = [.25 .25 .25]    ; % Text Color in Popup Menus
D.HeaderColor       = [.1 .1 .1]       ; % Popup and button Header Color
D.ButtonTextColor   = [.11 .1 .1]      ; % Button Text Color
D.GUI.colors.celeste = [.5 .7 .9]      ; % celeste background
D.TextSize_L        = 16               ; % Standardized Text Font Size 'Large'
D.TextSize_M        = 14               ; % Standardized Text Font Size 'Medium'
D.TextSize_S        = 12               ; % Standardized Text Font Size 'Small'
D.NumLines = 10                        ; % Number of lines in left action panel
D.NumEdit  = 10                        ; % Number of lines with a panel
D.Hgui = 1                             ; % nondimensional gui height
D.Hedge = 2/100                        ; % (%) distance from edge
D.Hparts = (D.Hgui-2*D.Hedge )/D.NumLines ; % ND Height of each rubric line
D.HbuttonHeight = D.Hparts*0.9         ; % ND Button height
D.HbuttonLength = 5/100                ; % ND Button length
D.TextLength    = 12/100               ; % ND text length
D.EditLength    = 6/100                ; % ND edit box length
D.TextSpacing   = .5/100               ; % ND text spacing
D.b_left        = .5/100               ; % ND margin from left
D.panelext      = .2/100               ; % ND panel width ~0 draws a line
D.linext        = .1*D.Hparts          ; % ND line extension
D.GUI.Fonts.gcaFont          = 'sans-serif' ; % Matlab Default  = Helvetica
D.GUI.Fonts.uiButtonFont     = 'Bangla MN'  ; % Matlab Default  = Helvetica
D.GUI.Fonts.uiHeaderFont     = 'Bangla MN'  ; % Matlab Default  = Helvetica
D.GUI.Fonts.MessageFont      = 'Cambria'  ; % Matlab Default  = Helvetica
% Message Area
D.message        = 'Welcome !  Select Data Fields and analysis type. Press ''GO'' to process and display data.' ; % Default Message
D.messageColor   =  [.35 .27 .24]      ; % text color of displayed messages
% Consts
D.Consts.LatentHeatOfVapor  = 2.5e6    ; % Latent heat of vaporization 2.26
% New Data Set selection
function D = NewDataSet(D)
D.DataSets.Default = cell2mat(D.DataSets.DataSetNames(D.DataSets.DataSetIdx)) ;
eval(['D.DataSet = D.DataSets.',D.DataSets.DataSetFolderNames{D.DataSets.DataSetIdx},'  ; ']) ;

D.DataSets.DataSetStr = '' ;
for i = 1:length(D.DataSets.DataSetNames)
    D.DataSets.DataSetStr = [D.DataSets.DataSetStr,cell2mat(D.DataSets.DataSetNames(i)), ' |'] ;
end
D.DataSets.DataSetStr = D.DataSets.DataSetStr(1:end-1);

D.DataSet.FieldsStr = '' ;
for i = 1:length(D.DataSet.Fields.Names)
    D.DataSet.FieldsStr = [D.DataSet.FieldsStr,cell2mat(D.DataSet.Fields.Names(i)), ' |'] ;
end
D.DataSet.FieldsStr = [D.DataSet.FieldsStr,'Download All'] ;
%D.DataSet.FieldsStr = D.DataSet.FieldsStr(1:end-1);
D.DataSet.FieldsIdx = 1 ;
% Produce User Inteface
function D = Download_GUI(D)
NumLines = D.NumLines ;
bl = 0.02              ; % Left Edge
bt = 0.05              ; % Top Edge
lh = (1-2*bt)/NumLines ; % Line Height
cl = (1-2*bl)/2        ; % column width (a total of 5 columns)

D.Download_Fig = figure('name',['GOAT Auto Data Downloadlysis Tool. Ver: ',D.Version,'   Written By Ori Adam.'],'position',D.pos) ;
set(D.Download_Fig,'Color',D.FigBackgroudColor,'MenuBar','none')

messageStr = 'GOAT Auto Download. Select Dataset and Field and press ''GO'' to begin Download.' ;
% Top Message
D.h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-lh,1,lh], 'string',messageStr);
uicontrol(D.h_message) ;  set(D.h_message,'ForegroundColor',D.messageColor,'FontSize',13,'FontName',D.GUI.Fonts.MessageFont) ;
% DataSet
Line = 2 ;
h_DataSetT = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Data Set');
uicontrol(h_DataSetT) ;  set(h_DataSetT,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',D.HeaderColor,'FontSize',D.TextSize_L,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiHeaderFont) ;
D.h_DataSet = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
    'string',D.DataSets.DataSetStr,  'callback','set(gcbf,''userdata'',1); uiresume;');
uicontrol(D.h_DataSet) ;  set(D.h_DataSet,'BackgroundColor',[1 1 1],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_M,'HorizontalAlignment','left','value',D.DataSets.DataSetIdx,'FontName',D.GUI.Fonts.uiButtonFont) ;
try
    set(D.h_DataSet,'TooltipString',D.DataSet.DescriptionTip) ;
end
% Fields
Line = 3 ;
h_FieldsT = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Fields');
uicontrol(h_FieldsT) ;  set(h_FieldsT,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',D.HeaderColor,'FontSize',D.TextSize_L,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiHeaderFont) ;
D.h_Fields = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
    'string',D.DataSet.FieldsStr,  'callback','set(gcbf,''userdata'',2); uiresume;');
uicontrol(D.h_Fields) ;  set(D.h_Fields,'BackgroundColor',[1 1 1],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_M,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiButtonFont) ;
% Years
D.Field = eval(['D.DataSet.Fields.',cell2mat(D.DataSet.Fields.Names(D.DataSet.FieldsIdx))]) ;
set(D.h_Fields,'TooltipString',['[ ',D.Field.Units,' ]   ',D.Field.FieldDescription]) ;
YearsStr = '' ;
for i = D.Field.FirstYear : D.Field.LastYear
    YearsStr = [YearsStr,int2str(i),' |'] ;
end
YearsStr = YearsStr(1:end-1) ;
D.Field.BeginYear = D.Field.FirstYear ;
D.Field.EndYear   = D.Field.LastYear ;
Line = 4 ;
h_year1T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Begin Year');
uicontrol(h_year1T) ;  set(h_year1T,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',D.HeaderColor,'FontSize',D.TextSize_L,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiHeaderFont) ;
D.h_year1 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
    'string',YearsStr,  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(D.h_year1) ;  set(D.h_year1,'BackgroundColor',[1 1 1],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_M,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiButtonFont) ;
Line = 5 ;
h_year2T = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','End Year');
uicontrol(h_year2T) ;  set(h_year2T,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',D.HeaderColor,'FontSize',D.TextSize_L,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiHeaderFont) ;
D.h_year2 = uicontrol('style','popup','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line),...
    'string',YearsStr,  'callback','set(gcbf,''userdata'',12); uiresume;');
uicontrol(D.h_year2) ;  set(D.h_year2,'BackgroundColor',[1 1 1],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_M,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiButtonFont) ;
set(D.h_year2 ,'string',YearsStr,'value',D.Field.EndYear - D.Field.FirstYear + 1) ;
% Download Folder
Line = 7 ;
h_FolderT = uicontrol('style','text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Download Folder');
uicontrol(h_FolderT) ;  set(h_FolderT,'BackgroundColor',D.FigBackgroudColor,'ForegroundColor',D.HeaderColor,'FontSize',D.TextSize_L,'HorizontalAlignment','left','FontName',D.GUI.Fonts.uiHeaderFont) ;
Line = 8 ;
D.h_Folder = uicontrol('style','edit','Units','normalized','pos',[.05,1-lh*Line,.7,lh], 'string',D.DataSet.Address);
uicontrol(D.h_Folder) ;  set(D.h_Folder,'BackgroundColor',[1 1 1],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_M,'HorizontalAlignment','left', ...
    'callback','set(gcbf,''userdata'',21); uiresume;','FontName',D.GUI.Fonts.uiButtonFont) ;
D.h_Browse = uicontrol('Units','normalized','pos',[.8,1-lh*Line,.15,lh], 'string','Browse');
uicontrol(D.h_Browse) ;  set(D.h_Browse,'BackgroundColor',[.85 .85 .85],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_L,'HorizontalAlignment','left', ...
    'callback','set(gcbf,''userdata'',22); uiresume;','FontName',D.GUI.Fonts.uiButtonFont) ;
% GO
Line = 10 ;
D.h_GO = uicontrol('Units','normalized','pos',[.05,1-lh*Line,.7,lh], 'string','GO');
uicontrol(D.h_GO) ;  set(D.h_GO,'BackgroundColor',[.5 .7 .9],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_L, ...
    'fontweight','bold','callback','set(gcbf,''userdata'',100); uiresume;','FontName',D.GUI.Fonts.uiButtonFont) ;
D.h_EXIT = uicontrol('Units','normalized','pos',[.8,1-lh*Line,.15,lh], 'string','EXIT');
uicontrol(D.h_EXIT) ;  set(D.h_EXIT,'BackgroundColor',[.85 .85 .85],'ForegroundColor',D.DropTextColor,'FontSize',D.TextSize_L, ...
    'callback','set(gcbf,''userdata'',999); uiresume;','FontName',D.GUI.Fonts.uiButtonFont) ;

if D.DataSet.TimeSelect
    set(D.h_year1 ,'enable','on') ;
    set(D.h_year2 ,'enable','on') ;
else
    set(D.h_year1 ,'enable','off') ;
    set(D.h_year2 ,'enable','off') ;
end

% DataSet Tables
function D = DataSets(D)
today = date ;
thisyear = round(str2double(today(end-3:end))) ;
D.DataSets.DataSetNames       = cell(1,1) ;
D.DataSets.DataSetFolderNames = cell(1,1) ;
D.DataSets.DataSetFuncNames = cell(1,1) ;
Nsets = 0 ;
datasetdir = dir(fullfile(cd,'Utils','DataManagementCode','AutoDownload','DataSets')) ;

for i = 1:length(datasetdir)
    if ~strncmp(datasetdir(i,1).name,'.',1)
        if strcmp(datasetdir(i,1).name(end-1:end),'.m')
            eval(['D = ',datasetdir(i,1).name(1:end-2),'(''Meta'',D) ; ']) ;
            if D.AutoDownloadDataset.ActiveFlag == 1
                Nsets = Nsets+1 ; 
                D.DataSets.DataSetNames(1,Nsets) = {D.AutoDownloadDataset.Name} ;
                D.DataSets.DataSetFolderNames(Nsets,1) = {D.AutoDownloadDataset.FolderName} ;
                D.DataSets.DataSetFuncNames(Nsets,1) = {datasetdir(i,1).name(1:end-2)} ;
            end
        end
    end
end

GOATpath = fileparts(cd) ;
for i = 1:length(D.DataSets.DataSetFuncNames)
    eval(['D = ',D.DataSets.DataSetFuncNames{i},'(''Meta'',D) ; ']) ;
    SetName = D.AutoDownloadDataset.FolderName ;
    CurrentYearFlag = D.AutoDownloadDataset.CurrentYear ;
    eval(['D.DataSets.',SetName,'.DescriptionTip = D.AutoDownloadDataset.DescriptionTip ; ']) ;
    eval(['D.DataSets.',SetName,'.Address = fullfile(GOATpath,''Data'',''',SetName,''') ; ']) ;
    eval(['D.DataSets.',SetName,'.TimeSelect = D.AutoDownloadDataset.TimeSelect ; ']) ;
    eval(['D.DataSets.',SetName,'.Fields.Names = D.AutoDownloadDataset.Fields.Names ; ']) ;
    eval(['D.DataSets.',SetName,'.Fields.Descriptions = cell(length(D.AutoDownloadDataset.Fields.Names),1) ; ']) ;
    for f = 1:length(D.AutoDownloadDataset.Fields.Names)
        eval(['D = ',D.DataSets.DataSetFuncNames{i},'(''Field'',D,''',D.AutoDownloadDataset.Fields.Names{f},''') ; ']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.Name  = ''',D.AutoDownloadDataset.Fields.Names{f},'''   ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.FieldDescription = D.AutoDownloadDataset.FieldDescription ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.Descriptions(f) = {D.AutoDownloadDataset.FieldDescription} ; ']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.Units = D.AutoDownloadDataset.Units ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.VerticalOrdinate = D.AutoDownloadDataset.VerticalOrdinate ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.LevelType = D.AutoDownloadDataset.LevelType ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.FirstYear = D.AutoDownloadDataset.FirstYear ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.LastYear = min([D.AutoDownloadDataset.LastYear,thisyear-1+CurrentYearFlag]) ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.TimeResolution = D.AutoDownloadDataset.TimeResolution ;']) ;
        eval(['D.DataSets.',SetName,'.Fields.',D.AutoDownloadDataset.Fields.Names{f},'.url = D.AutoDownloadDataset.url ;']) ;
    end
end

% GUI Column Position
function PosVec = PosFunc(bl,bt,lh,cl,C,Line)
PosVec = [bl*C + cl*(C-1),1-bt-Line*lh,cl-bl,lh] ;
return
