% GOAT_Produce_DefaultAddress  produce address vector for a given TemporalResolution and span
% copyright (C) ori adam, Mar.3.2014
% syntax:
% >> G = GOAT_Produce_DefaultAddress(G,year1,year2,Calendar,ext)
%


function G = GOAT_Produce_DefaultAddress(G,year1,year2,TemporalResolution,ext)
if nargin < 5
    ext = 'mat' ; 
end
% make sure TemporalResolution is acceptable
AcceptableGregorian = {'Monthly','Daily','DailyX1','DailyX4','DailyX6','DailyX8','DailyX2','DailyX12','DailyX24','Daily365', ... 
                      'Daily365X1','Daily365X4','Daily365X6','Daily365X8','Daily365X2','Daily365X12','Daily365X24', ... 
                      'Daily360','Daily360X1','Daily360X4','Daily360X6','Daily360X8','Daily360X2','Daily360X12','Daily360X24'} ; 

if ~(sum(strcmp(AcceptableGregorian,TemporalResolution)) || strncmp(TemporalResolution,'Julian',6))
    disp(['GOAT_Produce_DefaultAddress Error: Unrecognized TemporalREsolution - ',TemporalResolution])
    return ; 
end

if year1<0 || year2<0
    disp 'GOAT_Produce_DefaultAddress Error: year1 and year2 must be >= 0 '
    return ; 
end

y1 = min([year1,year2]) ;
y2 = max([year1,year2]) ;

if strncmp(TemporalResolution,'Julian',6)
    Type = 'Julian' ; 
%     if length(TemporalResolution) == 6
%         YYY = 365 ; 
%         X = 1 ; 
%     else
%         Xidx = regexpi(TemporalResolution,'X') ; 
%         if isempty(Xidx)
%         YYY = str2num(TemporalResolution(7:end)) ; 
%         X = 1 ; 
%         else
%         YYY = str2num(TemporalResolution(7:Xidx-1)) ;     
%         X =   str2num(TemporalResolution(Xidx+1:end)) ;       
%         end
%     end
else
    Type = TemporalResolution ; 
end

[YYY,X] = GOAT_ParseTimeResolution(TemporalResolution) ; 

switch Type
    case 'Monthly'
        y2length = length(int2str(y2)) ;
        ys = fullfile(num2str(y1,['%0',int2str(y2length),'.0f']),'.') ;
        %ys = fullfile(int2str(y1),'.') ;
        yearsDefault = repmat(ys(1:end-1),[12*(year2-year1+1) 1]) ;
        GregorianMonthRep = G.Addresses.GregorianMonthRep ; 
        if ~strcmp(ext,'mat')
           ext0 = cell(size(GregorianMonthRep,1),1) ; ext0(:) = {'.mat'} ; 
           ext1 = cell(size(GregorianMonthRep,1),1) ; ext1(:) = {['.',ext]} ; 
           GregorianMonthRep = cellfun(@regexprep,GregorianMonthRep,ext0,ext1,'UniformOutput',false) ; 
        end
        q = 12 ;
        if y2>y1
            for y = y1+1:y2
                %ys = fullfile(int2str(y),'.') ;
                ys = fullfile(num2str(y,['%0',int2str(y2length),'.0f']),'.') ;
                yearsDefault(q+1:q+12,:) = repmat(ys(1:end-1),[12 1]) ;
                q = q + 12 ;
            end
            G.Addresses.Default = cellstr([yearsDefault,repmat(char(GregorianMonthRep),[y2-y1+1 1])]) ;
        else
            G.Addresses.Default = cellstr([yearsDefault,char(GregorianMonthRep)]) ;
        end
        
    case {'Daily','DailyX1','DailyX4','DailyX6','DailyX8','DailyX2','DailyX12','DailyX24','Daily365','Daily365X1','Daily365X4','Daily365X6','Daily365X8','Daily365X2','Daily365X12','Daily365X24'}
        % produce an address vector of all leap years and remove non-leap years
%         Xidx = regexpi(Type,'X') ;
%         if isempty(Xidx)
%             X = 1 ; % TimesDaily
%         else
%             X =   str2num(Type(Xidx+1:end)) ;       % TimesDaily
%         end
        GregorianAddressesleap = G.Addresses.GregorianDailyX24Rep(1:24/X:end) ;
        if ~strcmp(ext,'mat')
           ext0 = cell(size(GregorianAddressesleap,1),1) ; ext0(:) = {'.mat'} ; 
           ext1 = cell(size(GregorianAddressesleap,1),1) ; ext1(:) = {['.',ext]} ; 
           GregorianAddressesleap = cellfun(@regexprep,GregorianAddressesleap,ext0,ext1,'UniformOutput',false) ; 
        end
        GregorianAddressesleap = char(GregorianAddressesleap) ;
        y2length = length(int2str(y2)) ;  
        ys = fullfile(num2str(y1,['%0',int2str(y2length),'.0f']),'.') ;
        lyIdx = [] ;
        q = 0 ;
        YYY = 366 ;
        yearsDefault = repmat(ys(1:end-1),[YYY*X*(y2-y1+1) 1]) ;
        if ~GOAT_leapyear(y1,Type)
                lyIdx = [lyIdx ; (q + 59*X  + [1:X])'] ;
        end
        q = q + YYY*X ;
        
        
        if y2>y1
            for y = y1+1:y2
                ys = fullfile(num2str(y,['%0',int2str(y2length),'.0f']),'.') ;
                yearsDefault(q+1:q+YYY*X,:) = repmat(ys(1:end-1),[YYY*X 1]) ;
                if ~GOAT_leapyear(y,Type)
                    lyIdx = [lyIdx ; (q + 59*X + [1:X])'] ;
                end
                q = q + YYY*X ;
            end
            G.Addresses.Default = cellstr([yearsDefault,repmat(GregorianAddressesleap,[y2-y1+1 1])]) ;
        else
            G.Addresses.Default = cellstr([yearsDefault,GregorianAddressesleap]) ;
        end
        G.Addresses.Default(lyIdx) = [] ;
        G.Addresses.Default = G.Addresses.Default(~cellfun(@isempty,G.Addresses.Default)) ;
    case {'Daily360','Daily360X1','Daily360X4','Daily360X6','Daily360X8','Daily360X2','Daily360X12','Daily360X24'}
        Gregorian360 = cell(360,1) ; 
        GregorianAddressesJan30 = G.Addresses.GregorianDailyX24Rep(1:24/X:30*24) ;
        if ~strcmp(ext,'mat')
           ext0 = cell(size(GregorianAddressesJan30,1),1) ; ext0(:) = {'.mat'} ; 
           ext1 = cell(size(GregorianAddressesJan30,1),1) ; ext1(:) = {['.',ext]} ; 
           GregorianAddressesJan30 = cellfun(@regexprep,GregorianAddressesJan30,ext0,ext1,'UniformOutput',false) ; 
        end
        GregorianAddressesJan30 = char(GregorianAddressesJan30) ;
        Gregorian360(1:30*X,1) = cellstr(GregorianAddressesJan30) ; 
        for i = 2:12
        Gregorian360((i-1)*30*X+1:i*30*X,1) = cellstr([repmat(int2str(i),[30*X 1]),GregorianAddressesJan30(:,2:end)]) ; 
        end
        Gregorian360 = char(Gregorian360) ;
        y2length = length(int2str(y2)) ;  
        ys = fullfile(num2str(y1,['%0',int2str(y2length),'.0f']),'.') ;
        q = 0 ;
        yearsDefault = repmat(ys(1:end-1),[YYY*X*(y2-y1+1) 1]) ;
        q = q + YYY*X ;
        if y2>y1
            for y = y1+1:y2
                ys = fullfile(num2str(y,['%0',int2str(y2length),'.0f']),'.') ;
                yearsDefault(q+1:q+YYY*X,:) = repmat(ys(1:end-1),[YYY*X 1]) ;
                q = q + YYY*X ;
            end
            G.Addresses.Default = cellstr([yearsDefault,repmat(Gregorian360,[y2-y1+1 1])]) ;
        else
            G.Addresses.Default = cellstr([yearsDefault,Gregorian360]) ;
        end
    case 'Julian'
        RepAddresses = G.Addresses.JulianDailyX24Rep(1:24/X:YYY*24) ;
        if ~strcmp(ext,'mat')
           ext0 = cell(size(RepAddresses,1),1) ; ext0(:) = {'.mat'} ; 
           ext1 = cell(size(RepAddresses,1),1) ; ext1(:) = {['.',ext]} ; 
           RepAddresses = cellfun(@regexprep,RepAddresses,ext0,ext1,'UniformOutput',false) ; 
        end
        RepAddresses = char(RepAddresses) ;
        y2length = length(int2str(y2)) ;
        ys = fullfile(num2str(y1,['%0',int2str(y2length),'.0f']),'.') ;
        yearsDefault = repmat(ys(1:end-1),[YYY*X*(y2-y1+1) 1]) ;
        q = YYY*X ;
        if y2>y1
            for y = y1+1:y2
                ys = fullfile(num2str(y,['%0',int2str(y2length),'.0f']),'.') ;
                yearsDefault(q+1:q+YYY*X,:) = repmat(ys(1:end-1),[YYY*X 1]) ;
                q = q + YYY*X ;
            end
            G.Addresses.Default = cellstr([yearsDefault,repmat(RepAddresses,[y2-y1+1 1])]) ;
        else
            G.Addresses.Default = cellstr([yearsDefault,RepAddresses]) ;
        end
        
    otherwise
        disp(['GOAT_Produce_DefaultAddress Error: Unrecognized Calendar - ',TemporalResolution])
end

G.Addresses.Default(end+1) = {TemporalResolution} ;
