% Auto Link Protocol (ALP) for
% MERRA: MODERN-ERA RETROSPECTIVE ANALYSIS FOR RESEARCH AND APPLICATIONS (NASA)
% source:
% OPeNDAP GES DISC GrADS Data Server

% ALP uses the csv table: 'link_standard_variables.csv' which contains (by columns)
% 1. Field Name
% 2. Standard Field Name
% 3. Original Field Description
% 4. Standard Field Description
% 5. Original Units
% 6. Standard Units
% 7. Vertical Ordinates
% 8. Factor (scalar) for converting from Original Units to Standard Units
% 9. Meta data 1
% 10.Meta data 2

% The table 'CM' contains the following attributes for each dataset (columns):
% 1. Dataset name
% 2. OPeNDAP extention
% 3. goldsmr number
% 4. info


% Each data set is linked to GOAT by first constructing the table 'links'
% and then creating the Dims, info, and link_field  files for each variable in the dataset

%'link' table columns:
% 1.  Temporal Resolution
% 2.  nc variable name
% 3.  file path
% 4. type: 1) opendap  2) nc file 3) hdf file (TBD)  4) GRIB1 file (TBD) 5) GRIB2 file (TBD)
% 5.  lon   (x) dimension number (1,2,3,4) or 0 if does not exist
% 6.  lat   (y) dimention number (1,2,3,4) or 0 if does not exist
% 7.  level (z) dimension number (1,2,3,4) or 0 if does not exist
% 8.  time  (t) dimensoon number (1,2,3,4) or 0 if does not exist
% example1: if variable has size [128,64,17,365]  with dimensions (longitude,latitude,pressure,time)
% then lon dimension number = 1, lat dimension number  = 2, level dimension number = 3, time dimension number = 4
% example2: if variable has size [365,128,64]  with dimensions (time,longitude,latitude)
% then lon dimension number = 2, lat dimension number  = 3, level dimension number = 0, time dimension number = 1
% 9.  first year
% 10. first month
% 11. first day
% 12. first hour
% 13. last year
% 14. last month
% 15. last day
% 16. last hour
% 17. stride. for example if temporal resolution of the field is 4XDaily but the nc file contains only 12:00 hours, stride = 4
% 18. (optional) linear folder name
% 19. (optional) 'assimilated' | 'analyzed' | 'forecast' | 'initialconditions'
% 20. (optional) 'accumilated fields'

G = GOAT_Environment ;
link_dir = fullfile(fileparts(which('GOAT.m')),'Utils','DataManagementCode','ALP','MERRA') ; % MERRA ALP code folder
Data_ALP = fullfile(fileparts(fileparts(which('GOAT.m'))),'Data_LINK') ; % MERRA_ALP Data folder
CV = csvimport_g(fullfile(link_dir,'link_standard_variables.csv')) ;
DODS = 'http://goldsmrX.sci.gsfc.nasa.gov:80/dods/' ;


CM={'inst3_3d_asm_Cp','MAI3CPASM','DailyX8' ,'3','MERRA IAU 3d assimilated state on pressure, \nInstantaneous, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, pressure levels: 42' ;
    'instM_3d_asm_Cp','MAIMCPASM','Monthly' ,'3','MERRA IAU 3d assimilated state on pressure, \nInstantaneous, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, pressure levels: 42' ;
    'const_2d_asm_Nx','MAC0NXASM','List'    ,'2','MERRA DAS 2d constants at native resolution, \n540 X 361 (lon X lat)' ;
    'const_2d_mld_Nx','MSC0NXMLD','List'    ,'2','MERRA Land 2d land surface constants at native resolution, \n540 X 361 (lon X lat)' ;
    'inst6_3d_ana_Nv','MAI6NVANA','DailyX4' ,'3','MERRA DAS 3d analyzed state \nInstantaneous, on model levels, at native resolution \nlongitude: 540, latitude: 361, levels: 72 \nTimes: 00, 06, 12, 18 GMT' ;
    'inst6_3d_ana_Np','MAI6NPANA','DailyX4' ,'3','MERRA DAS 3d analyzed state on pressure \nInstantaneous, on pressure levels, at native resolution \nlongitude: 540, latitude: 361, pressure levels: 42' ;
    'instM_3d_ana_Np','MAIMNPANA','Monthly' ,'3','MERRA DAS 3d analyzed state on pressure \nInstantaneous, on pressure levels, at native resolution \nlongitude: 540, latitude: 361, pressure levels: 42' ;
    'tavg3_3d_cld_Cp','MAT3CPCLD','DailyX8' ,'3','MERRA IAU 3d cloud diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30' ;
    'tavgM_3d_cld_Cp','MATMCPCLD','Monthly' ,'3','MERRA IAU 3d cloud diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nongitude: 288, latitude: 144, levels: 42' ;
    'tavg3_3d_mst_Cp','MAT3CPMST','DailyX8' ,'3','MERRA IAU 3d moist processes diagnostic \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_mst_Cp','MATMCPMST','Monthly' ,'3','MERRA IAU 3d moist processes diagnostic \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42'  ;
    'tavg3_3d_rad_Cp','MAT3CPRAD','DailyX8' ,'3','MERRA IAU 3d radiation diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_rad_Cp','MATMCPRAD','Monthly' ,'3','MERRA IAU 3d radiation diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
    'tavg3_3d_trb_Cp','MAT3CPTRB','DailyX8' ,'3','MERRA IAU 3d turbulence diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_trb_Cp','MATMCPTRB','Monthly' ,'3','MERRA IAU 3d turbulence diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42';
    'tavg3_3d_tdt_Cp','MAT3CPTDT','DailyX8' ,'3','MERRA IAU 3d temperature tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_tdt_Cp','MATMCPTDT','Monthly' ,'3','MERRA IAU 3d temperature tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
    'tavg3_3d_udt_Cp','MAT3CPUDT','DailyX8' ,'3','MERRA IAU 3d eastward wind tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_udt_Cp','MATMCPUDT','Monthly' ,'3','MERRA IAU 3d eastward wind tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
    'tavg3_3d_qdt_Cp','MAT3CPQDT','DailyX8' ,'3','MERRA IAU 3d moisture tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_qdt_Cp','MATMCPQDT','Monthly' ,'3','MERRA IAU 3d moisture tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
    'tavg3_3d_odt_Cp','MAT3CPODT','DailyX8' ,'3','MERRA IAU 3d ozone tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_3d_odt_Cp','MATMCPODT','Monthly' ,'3','MERRA IAU 3d ozone tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42'  ;
    'tavg1_2d_slv_Nx','MAT1NXSLV','DailyX24','2','MERRA IAU 2d atmospheric single-level diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgM_2d_slv_Nx','MATMNXSLV','Monthly' ,'2','MERRA IAU 2d atmospheric single-level diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
    'tavg1_2d_flx_Nx','MAT1NXFLX','DailyX24','2','MERRA IAU 2d surface turbulent flux diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgM_2d_flx_Nx','MATMNXFLX','Monthly' ,'2','MERRA IAU 2d surface turbulent flux diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
    'tavg1_2d_rad_Nx','MAT1NXRAD','DailyX24','2','MERRA IAU 2d surface and TOA radiation fluxes \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgM_2d_rad_Nx','MATMNXRAD','Monthly' ,'2','MERRA IAU 2d surface and TOA radiation fluxes \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
    'tavg1_2d_lnd_Nx','MAT1NXLND','DailyX24','2','MERRA IAU 2d land surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgM_2d_lnd_Nx','MATMNXLND','Monthly' ,'2','MERRA IAU 2d land surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
    'tavg1_2d_ocn_Nx','MAT1NXOCN','DailyX24','2','MERRA IAU 2d ocean surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgM_2d_ocn_Nx','MATMNXOCN','Monthly' ,'2','MERRA IAU 2d ocean surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
    'tavg1_2d_int_Nx','MAT1NXINT','DailyX24','2','MERRA IAU 2d Vertical integrals \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgM_2d_int_Nx','MATMNXINT','Monthly' ,'2','MERRA IAU 2d Vertical integrals \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
    'inst1_2d_int_Nx','MAI1NXINT','DailyX24','2','MERRA IAU 2d Vertical integrals \nInstantaneous, single level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0, 1, 2, 3, 4, ...,23 GMT' ;
    'instM_2d_int_Nx','MAIMNXINT','Monthly' ,'2','MERRA IAU 2d Vertical integrals \nInstantaneous, single level, at native resolution \nlongitude: 540, latitude: 361' ;
    'const_2d_chm_Fx','MAC0FXCHM','List'    ,'1','MERRA CHM 2d constants \nConstant at reduced FV resolution \nlongitude: 288, latitude: 181' ;
    'tavg3_3d_chm_Fv','MAT3FVCHM','DailyX8' ,'1','MERRA IAU 3d Chem On Layers \nTime-averaged, 3D model levels, at reduced FV resolution \nlongitude: 288, latitude: 181, levels: 72 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavg3_3d_chm_Fe','MAT3FECHM','DailyX8' ,'1','MERRA IAU 3d Chem On Layer Edges \nTime-averaged, 3D model levels, at reduced FV resolution \nlongitude: 288, latitude: 181, levels: 73 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavg3_2d_chm_Fx','MAT3FXCHM','DailyX8' ,'1','MERRA IAU 2d Chem \nTime-averaged, single-level, at reduced FV resolution \nlongitude: 288, latitude: 181 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgM_2d_chm_Fx','MATMFXCHM','Monthly' ,'1','MERRA IAU 2d Chem \nTime-averaged, single-level, at reduced FV resolution \nlongitude: 288, latitude: 181' ;
    'tavg3_3d_chm_Nv','MAT3NVCHM','DailyX8' ,'1','MERRA IAU 3d Chem On Layers \nTime-averaged, 3D model levels, at native resolution \nlongitude: 540, latitude: 361, model levels: 72 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavg3_3d_chm_Ne','MAT3NECHM','DailyX8' ,'1','MERRA IAU 3d Chem On Layer Edges \nTime-averaged, 3D model levels, at native resolution \nlongitude: 540, latitude: 361, model levels: 73 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'inst3_3d_chm_Ne','MAI3NECHM','DailyX8' ,'1','MERRA IAU 3d Chem On Layer Edges \nInstantaneous, 3D model levels, at native resolution \nlongitude: 540, latitude: 361, model levels: 73 \nTimes: 0, 3, 6, 9, 12, 15, 18, 21 -GMT' ;
    'instD_3d_asm_Cp','MAI3CPASM','Daily'   ,'3','MERRA IAU 3d assimilated state on pressure, \nDaily averages from Instantaneous, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, pressure levels: 42' ;
    'instD_3d_ana_Nv','MAI6NVANA','Daily'   ,'3','MERRA DAS 3d analyzed state \nDaily averages from Instantaneous, on model levels, at native resolution \nlongitude: 540, latitude: 361, levels: 72 \nTimes: 00, 06, 12, 18 GMT' ;
    'instD_3d_ana_Np','MAI6NPANA','Daily'   ,'3','MERRA DAS 3d analyzed state on pressure \nDaily averages from Instantaneous, on pressure levels, at native resolution \nlongitude: 540, latitude: 361, pressure levels: 42' ;
    'tavgD_3d_cld_Cp','MAT3CPCLD','Daily'   ,'3','MERRA IAU 3d cloud diagnostics \nDaily averages from Time-averaged, on pressure levels, at reduced resolution \nongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30' ;
    'tavgD_3d_mst_Cp','MAT3CPMST','Daily'   ,'3','MERRA IAU 3d moist processes diagnostic \nDaily averages from Time-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_rad_Cp','MAT3CPRAD','Daily'   ,'3','MERRA IAU 3d radiation diagnostics \nDaily averages from Time-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_trb_Cp','MAT3CPTRB','Daily'   ,'3','MERRA IAU 3d turbulence diagnostics \nDaily averages from Time-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_tdt_Cp','MAT3CPTDT','Daily'   ,'3','MERRA IAU 3d temperature tendencies \nDaily averages from Time-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_udt_Cp','MAT3CPUDT','Daily'   ,'3','MERRA IAU 3d eastward wind tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_qdt_Cp','MAT3CPQDT','Daily'   ,'3','MERRA IAU 3d moisture tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_odt_Cp','MAT3CPODT','Daily'   ,'3','MERRA IAU 3d ozone tendencies \nDaily averages from Time-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_2d_slv_Nx','MAT1NXSLV','Daily'   ,'2','MERRA IAU 2d atmospheric single-level diagnostics \nDaily Averages from Time-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgD_2d_flx_Nx','MAT1NXFLX','Daily'   ,'2','MERRA IAU 2d surface turbulent flux diagnostics \nDaily Averages from Time-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgD_2d_rad_Nx','MAT1NXRAD','Daily'   ,'2','MERRA IAU 2d surface and TOA radiation fluxes \nDaily Averages from Time-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgD_2d_lnd_Nx','MAT1NXLND','Daily'   ,'2','MERRA IAU 2d land surface diagnostics \nDaily Averages from Time-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgD_2d_ocn_Nx','MAT1NXOCN','Daily'   ,'2','MERRA IAU 2d ocean surface diagnostics \nDaily Averages from Time-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'tavgD_2d_int_Nx','MAT1NXINT','Daily'   ,'2','MERRA IAU 2d Vertical integrals \nDaily Averages from Time-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
    'instD_2d_int_Nx','MAI1NXINT','Daily'   ,'2','MERRA IAU 2d Vertical integrals \nDaily averages from Instantaneous, single level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0, 1, 2, 3, 4, ...,23 GMT' ;
    'tavgD_3d_chm_Fv','MAT3FVCHM','Daily'   ,'1','MERRA IAU 3d Chem On Layers \nDaily averages from Time-averaged, 3D model levels, at reduced FV resolution \nlongitude: 288, latitude: 181, levels: 72 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_chm_Fe','MAT3FECHM','Daily'   ,'1','MERRA IAU 3d Chem On Layer Edges \nDaily averages from Time-averaged, 3D model levels, at reduced FV resolution \nlongitude: 288, latitude: 181, levels: 73 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_2d_chm_Fx','MAT3FXCHM','Daily'   ,'1','MERRA IAU 2d Chem \nDaily averages from Time-averaged, single-level, at reduced FV resolution \nlongitude: 288, latitude: 181 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_chm_Nv','MAT3NVCHM','Daily'   ,'1','MERRA IAU 3d Chem On Layers \nDaily averages from Time-averaged, 3D model levels, at native resolution \nlongitude: 540, latitude: 361, model levels: 72 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'tavgD_3d_chm_Ne','MAT3NECHM','Daily'   ,'1','MERRA IAU 3d Chem On Layer Edges \nDaily averages from Time-averaged, 3D model levels, at native resolution \nlongitude: 540, latitude: 361, model levels: 73 \nTimes: 1:30, 4:30, 7:30, 10:30, 13:30, 16:30, 19:30, 22:30 GMT' ;
    'instD_3d_chm_Ne','MAI3NECHM','Daily'   ,'1','MERRA IAU 3d Chem On Layer Edges \nDaily averages from Instantaneous, 3D model levels, at native resolution \nlongitude: 540, latitude: 361, model levels: 73 \nTimes: 0, 3, 6, 9, 12, 15, 18, 21 -GMT' ;
    } ;
% 'tavg1_2d_mld_Nx','MAT1NXMLD','DailyX24','2','MERRA-Land 2d land surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361 \nTimes: 0:30, 1:30, 2:30, 3:30, 4:30, ... GMT' ;
%     'tavgM_2d_mld_Nx','MATMNXMLD','Monthly' ,'2','MERRA-Land 2d land surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_mld_Nx','MATUNXMLD','Daily'   ,'2','MERRA-Land 2d land surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'instU_3d_asm_Cp','MAIUCPASM','Daily'   ,'3','MERRA IAU 3d assimilated state on pressure, \nInstantaneous, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, pressure levels: 42' ;
%     'instU_3d_ana_Np','MAIUNPANA','Daily'   ,'3','MERRA DAS 3d analyzed state on pressure \nInstantaneous, on pressure levels, at native resolution \nlongitude: 540, latitude: 361, pressure levels: 42' ;
%     'tavgU_3d_cld_Cp','MATUCPCLD','Daily'   ,'3','MERRA IAU 3d cloud diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nongitude: 288, latitude: 144, levels: 42' ;
%     'tavgU_3d_mst_Cp','MATUCPMST','Daily'   ,'3','MERRA IAU 3d moist processes diagnostic \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42'  ;
%     'tavgU_3d_rad_Cp','MATUCPRAD','Daily'   ,'3','MERRA IAU 3d radiation diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
%     'tavgU_3d_trb_Cp','MATUCPTRB','Daily'   ,'3','MERRA IAU 3d turbulence diagnostics \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42';
%     'tavgU_3d_tdt_Cp','MATUCPTDT','Daily'   ,'3','MERRA IAU 3d temperature tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
%     'tavgU_3d_udt_Cp','MATUCPUDT','Daily'   ,'3','MERRA IAU 3d eastward wind tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
%     'tavgU_3d_qdt_Cp','MATUCPQDT','Daily'   ,'3','MERRA IAU 3d moisture tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42' ;
%     'tavgU_3d_odt_Cp','MATUCPODT','Daily'   ,'3','MERRA IAU 3d ozone tendencies \nTime-averaged, on pressure levels, at reduced resolution \nlongitude: 288, latitude: 144, levels: 42'  ;
%     'tavgU_2d_slv_Nx','MATUNXSLV','Daily'   ,'2','MERRA IAU 2d atmospheric single-level diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_rad_Nx','MATUNXRAD','Daily'   ,'2','MERRA IAU 2d surface and TOA radiation fluxes \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_flx_Nx','MATUNXFLX','Daily'   ,'2','MERRA IAU 2d surface turbulent flux diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_lnd_Nx','MATUNXLND','Daily'   ,'2','MERRA IAU 2d land surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_ocn_Nx','MATUNXOCN','Daily'   ,'2','MERRA IAU 2d ocean surface diagnostics \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_int_Nx','MATUNXINT','Daily'   ,'2','MERRA IAU 2d Vertical integrals \nTime-averaged, single-level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'instU_2d_int_Nx','MAIUNXINT','Daily'   ,'2','MERRA IAU 2d Vertical integrals \nInstantaneous, single level, at native resolution \nlongitude: 540, latitude: 361' ;
%     'tavgU_2d_chm_Fx','MATUFXCHM','Daily'   ,'1','MERRA IAU 2d Chem \nTime-averaged, single-level, at reduced FV resolution \nlongitude: 288, latitude: 181' ;
CompressFlag = 1 ;
totaltoc = 0 ;
Ext = 'dap' ;
longitude_dimname = 'lon' ;
latitude_dimname  = 'lat' ;
level_dimname     = 'lev' ;
time_dimname      = 'time';
today = date ;
thisyear = round(str2double(today(end-3:end))) ;
%
for ds = 1:size(CM,1)
    tic ;
    Stride = 1 ;
    dataset = CM{ds,1} ;
    opendap_address = regexprep([DODS,CM{ds,2}],'X.sci',[CM{ds,4},'.sci']) ;
    TemporalResolution = CM{ds,3} ;
    infostr = sprintf(CM{ds,5}) ;
    
    switch dataset
        case {'inst3_3d_asm_Cp','instM_3d_asm_Cp','instU_3d_asm_Cp','instD_3d_asm_Cp'}
            datasetsave = 'inst_3d_asm_Cp' ;
        case {'inst6_3d_ana_Nv','instM_3d_ana_Nv','instU_3d_ana_Nv','instD_3d_ana_Nv'}
            datasetsave = 'inst_3d_ana_Nv' ;
        case {'inst6_3d_ana_Np','instM_3d_ana_Np','instU_3d_ana_Np','instD_3d_ana_Np'}
            datasetsave = 'inst_3d_ana_Np' ;
        case {'tavg3_3d_cld_Cp','tavgM_3d_cld_Cp','tavgU_3d_cld_Cp','tavgD_3d_cld_Cp'}
            datasetsave = 'tavg_3d_cld_Cp' ;
        case {'tavg3_3d_mst_Cp','tavgM_3d_mst_Cp','tavgU_3d_mst_Cp','tavgD_3d_mst_Cp'}
            datasetsave = 'tavg_3d_mst_Cp' ;
        case {'tavg3_3d_rad_Cp','tavgM_3d_rad_Cp','tavgU_3d_rad_Cp','tavgD_3d_rad_Cp'}
            datasetsave = 'tavg_3d_rad_Cp' ;
        case {'tavg3_3d_trb_Cp','tavgM_3d_trb_Cp','tavgU_3d_trb_Cp','tavgD_3d_trb_Cp'}
            datasetsave = 'tavg_3d_trb_Cp' ;
        case {'tavg3_3d_tdt_Cp','tavgM_3d_tdt_Cp','tavgU_3d_tdt_Cp','tavgD_3d_tdt_Cp'}
            datasetsave = 'tavg_3d_tdt_Cp' ;
        case {'tavg3_3d_udt_Cp','tavgM_3d_udt_Cp','tavgU_3d_udt_Cp','tavgD_3d_udt_Cp'}
            datasetsave = 'tavg_3d_udt_Cp' ;
        case {'tavg3_3d_qdt_Cp','tavgM_3d_qdt_Cp','tavgU_3d_qdt_Cp','tavgD_3d_qdt_Cp'}
            datasetsave = 'tavg_3d_qdt_Cp' ;
        case {'tavg3_3d_odt_Cp','tavgM_3d_odt_Cp','tavgU_3d_odt_Cp','tavgD_3d_odt_Cp'}
            datasetsave = 'tavg_3d_odt_Cp' ;
        case {'tavg3_2d_chm_Fx','tavgM_2d_chm_Fx','tavgU_2d_chm_Fx','tavgD_2d_chm_Fx'}
            datasetsave = 'tavg_2d_chm_Fx' ;
        case {'tavg3_2d_chm_Fv','tavgM_2d_chm_Fv','tavgU_2d_chm_Fv','tavgD_2d_chm_Fv'}
            datasetsave = 'tavg_2d_chm_Fv' ;
        case {'tavg3_2d_chm_Fe','tavgM_2d_chm_Fe','tavgU_2d_chm_Fe','tavgD_2d_chm_Fe'}
            datasetsave = 'tavg_2d_chm_Fe' ;
        case {'tavg3_2d_chm_Nv','tavgM_2d_chm_Nv','tavgU_2d_chm_Nv','tavgD_2d_chm_Nv'}
            datasetsave = 'tavg_2d_chm_Nv' ;
        case {'tavg3_2d_chm_Ne','tavgM_2d_chm_Ne','tavgU_2d_chm_Ne','tavgD_2d_chm_Ne'}
            datasetsave = 'tavg_2d_chm_Ne' ;
        case {'inst3_2d_chm_Ne','instM_2d_chm_Ne','instU_2d_chm_Ne','instD_2d_chm_Ne'}
            datasetsave = 'inst_2d_chm_Ne' ;
        case {'tavg1_2d_slv_Nx','tavgM_2d_slv_Nx','tavgU_2d_slv_Nx','tavgD_2d_slv_Nx'}
            datasetsave = 'tavg_2d_slv_Nx' ;
        case {'tavg1_2d_flx_Nx','tavgM_2d_flx_Nx','tavgU_2d_flx_Nx','tavgD_2d_flx_Nx'}
            datasetsave = 'tavg_2d_flx_Nx' ;
        case {'tavg1_2d_rad_Nx','tavgM_2d_rad_Nx','tavgU_2d_rad_Nx','tavgD_2d_rad_Nx'}
            datasetsave = 'tavg_2d_rad_Nx' ;
        case {'tavg1_2d_lnd_Nx','tavgM_2d_lnd_Nx','tavgU_2d_lnd_Nx','tavgD_2d_lnd_Nx'}
            datasetsave = 'tavg_2d_lnd_Nx' ;
        case {'tavg1_2d_mld_Nx','tavgM_2d_mld_Nx','tavgU_2d_mld_Nx','tavgD_2d_mld_Nx'}
            datasetsave = 'tavg_2d_mld_Nx' ;
        case {'tavg1_2d_ocn_Nx','tavgM_2d_ocn_Nx','tavgU_2d_ocn_Nx','tavgD_2d_ocn_Nx'}
            datasetsave = 'tavg_2d_ocn_Nx' ;
        case {'tavg1_2d_int_Nx','tavgM_2d_int_Nx','tavgU_2d_int_Nx','tavgD_2d_int_Nx'}
            datasetsave = 'tavg_2d_int_Nx' ;
        case {'inst1_2d_int_Nx','instM_2d_int_Nx','instD_2d_int_Nx'}
            datasetsave = 'inst_2d_int_Nx' ;
        case {'inst3_3d_chm_Ne','instD_3d_chm_Ne'}
            datasetsave = 'inst_3d_chm_Ne' ;
        case {'tavg3_3d_chm_Fv','tavgD_3d_chm_Fv'}
            datasetsave = 'tavg_3d_chm_Fv' ;
        case {'tavg3_3d_chm_Fe','tavgD_3d_chm_Fe'}
            datasetsave = 'tavg_3d_chm_Fv' ;
        case {'tavg3_3d_chm_Nv','tavgD_3d_chm_Nv'}
            datasetsave = 'tavg_3d_chm_Nv' ;
        case {'tavg3_3d_chm_Ne','tavgD_3d_chm_Ne'}
            datasetsave = 'tavg_3d_chm_Ne' ;
        otherwise
            datasetsave = dataset ;
    end
    
    switch dataset
        case {'instD_3d_asm_Cp','tavgD_3d_cld_Cp','tavgD_3d_mst_Cp','tavgD_3d_rad_Cp','tavgD_3d_trb_Cp','tavgD_3d_tdt_Cp','tavgD_3d_udt_Cp','tavgD_3d_qdt_Cp','tavgD_3d_odt_Cp' ...
                'tavgD_2d_chm_Fx','tavgD_2d_chm_Fv','tavgD_2d_chm_Fe','tavgD_2d_chm_Nv','tavgD_2d_chm_Ne','instD_2d_chm_Ne'}
            SeqFileNum = 8 ;
        case {'instD_3d_ana_Nv','instD_3d_ana_Np'}
            SeqFileNum = 4 ;
        case {'instD_2d_int_Nx','tavgD_2d_slv_Nx','tavgD_2d_flx_Nx','tavgD_2d_rad_Nx','tavgD_2d_lnd_Nx','tavgD_2d_mld_Nx','tavgD_2d_ocn_Nx','tavgD_2d_int_Nx'}
            SeqFileNum = 24 ;
        otherwise
            SeqFileNum = 1 ;
    end
    a = ncinfo(opendap_address) ;
    varlist = cell(1,1) ;
    q = 0 ;
    for i = 1:length(a.Variables)
        if ~sum(strcmp(a.Variables(i).Name,{longitude_dimname,latitude_dimname,level_dimname,time_dimname}))
            q = q + 1 ;
            varlist(q) = {a.Variables(i).Name} ;
        end
    end
    disp(['Building link table ',dataset,' . . .'])
    year1 = 1979 ; month1 = 1 ; day1 = 1 ; hour1 = 0 ;
    year2 = thisyear ; month2 = 12 ; day2 = 31 ; hour2 = 21 ;
    links    = cell(1,20) ;
    [links(1,:)] = deal([{'Temporal Resolution'},{'link type'},{'file path'},{'varname'},{'x dim #'},{'y dim #'},{'z dim #'},{'t dim #'},{'year1'},{'month1'},{'day1'},{'hour1'},{'year2'},{'month2'},{'day2'},{'hour2'},{'stride'},{''},{''},{''}]) ;
    q = 1 ;
    for d = 1:length(varlist)
        a = ncinfo(opendap_address,varlist{d}) ;
        q = q + 1 ;
        links(q,1) = {TemporalResolution}  ; % Temporal Resolution
        links(q,2) = {1}                   ; % type
        links(q,3) = {opendap_address}     ; % opendap url
        links(q,4) = varlist(d)            ; % variable name (short)
        links(q,5:8) = {0}                 ; % default dimension number
        for j = 1:length(a.Dimensions)
            if strcmpi(a.Dimensions(j).Name,longitude_dimname) ; links(q,5) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,latitude_dimname)  ; links(q,6) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,level_dimname)     ; links(q,7) = {j} ; end
            if strcmpi(a.Dimensions(j).Name,time_dimname)      ; links(q,8) = {j} ; end
        end
        [links(q,9:18)] = deal([{year1},{month1},{day1},{hour1},{year2},{month2},{day2},{hour2},{Stride},{''}]) ;
        Idx = find(strcmpi(a.Name,CV(:,1))) ;
        
    end
    CB = links ;
    % link Fields
    Data_Link = fullfile(Data_ALP,'MERRA_LINK',datasetsave) ;
    if ~exist(Data_Link,'dir')
        mkdir(Data_Link) ;
    end
    % step 1. identify Valid Time Resolutions
    TimeResolutions = unique(CB(:,1)) ;
    ValidTimeResolutions = zeros(size(TimeResolutions)) ;
    for i = 1:length(ValidTimeResolutions)
        if strncmpi(TimeResolutions{i},'Daily',5)  || ...
                strncmpi(TimeResolutions{i},'Month',5)  || ...
                strncmpi(TimeResolutions{i},'Julian',6) || ...
                strncmpi(TimeResolutions{i},'List',4)
            ValidTimeResolutions(i) = 1 ;
        end
    end
    TimeResolutions = TimeResolutions(ValidTimeResolutions==1) ;
    
    for t = 1:length(TimeResolutions)
        CT = CB(strcmp(TimeResolutions(t),CB(:,1)),:) ;
        if ~exist(fullfile(Data_Link,TimeResolutions{t}),'dir')
            mkdir(fullfile(Data_Link,TimeResolutions{t})) ;
        end
        save(fullfile(Data_Link,TimeResolutions{t},'info.mat'),'infostr') ;
        % Identify Fields
        Fields = unique(CT(:,4)) ;
        for f = 1:length(Fields)
            CF = CT(strcmp(Fields(f),CT(:,4)),:) ;
            % Find standard field name
            Idx = find(strcmpi(Fields(f),CV(:,1))) ;
            if isempty(Idx)
                FieldName = lower(Fields{f}) ;
                if strcmpi(FieldName,'ens')
                    continue ;
                else
                    % check for variance fields
                    if strncmpi(FieldName,'var',3)
                        Idv = find(strcmpi(FieldName(5:end),CV(:,1))) ;
                        if ~isempty(Idv)
                            if isempty(CV{Idv,2})
                                FieldName = [lower(CV{Idv,1}),'_var'] ;
                            else
                                FieldName = [lower(CV{Idv,2}),'_var'] ;
                            end
                            VerticalOrdinate = CV{Idv,7} ;
                            if isempty(CV{Idv,6})
                                Units            = ['(',CV{Idv,5},')^2'] ;
                            else
                                Units            = ['(',CV{Idv,6},')^2'] ;
                            end
                            % Field Description (long-name)
                            if isempty(CV{Idv,4})
                                FieldDescription = [CV{Idv,3},' - variance'] ;
                            else
                                FieldDescription = [CV{Idv,4},' - variance'] ;
                            end
                        else
                            disp(['ERROR: Unidentified field: ',FieldName]);
                            continue ;
                        end
                    else
                        disp(['ERROR: Unidentified field: ',FieldName]);
                        continue ;
                    end
                end
            else
                if isempty(CV{Idx,2})
                    FieldName = lower(CV{Idx,1}) ;
                else
                    FieldName = lower(CV{Idx,2}) ;
                end
            end
            disp(['linking field ',dataset,'/',TimeResolutions{t},'  ',int2str(f),'/',int2str(length(Fields)),' :  ',FieldName,' . . . '])
            mkdir(fullfile(Data_Link,TimeResolutions{t},FieldName)) ;
            if ~isempty(Idx)
                VerticalOrdinate = CV{Idx,7} ;
                if isempty(CV{Idx,6})
                    Units            = CV{Idx,5} ;
                else
                    Units            = CV{Idx,6} ;
                end
                % Field Description (long-name)
                if isempty(CV{Idx,4})
                    FieldDescription = CV{Idx,3} ;
                else
                    FieldDescription = CV{Idx,4} ;
                end
            else
                Idx = Idv ;
            end
            % retrieve spatial vectors
            filename = CF{1,3} ;
            a = ncinfo(filename) ;
            lon = double(ncread(filename,longitude_dimname)) ;
            lat = double(ncread(filename,latitude_dimname)) ;
            av = ncinfo(filename,CF{1,4}) ;
            if cell2mat(CF(1,7)) == 0
                P = 1000 ;
            else
                P = double(ncread(filename,level_dimname)) ;
            end
            save(fullfile(Data_Link,TimeResolutions{t},FieldName,'Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
            % move on to creating Attributes
            dates1 = cell2mat(CF(:,9 ))*1.e10 + cell2mat(CF(:,10)).*1.e4 + cell2mat(CF(:,11)).*1.e2 + cell2mat(CF(:,12))  ;
            dates2 = cell2mat(CF(:,13))*1.e10 + cell2mat(CF(:,14)).*1.e4 + cell2mat(CF(:,15)).*1.e2 + cell2mat(CF(:,16))  ;
            [S, Si] = sort(dates1) ;
            year1 = min(cell2mat(CF(:,9 ))) ;
            year2 = max(cell2mat(CF(:,13))) ;
            if strcmp(TimeResolutions{t},'List')
                G.Addresses.Default      = cell(2,1)                   ;
                G.Addresses.Default(1,1) = {fullfile('Invariant','data.mat')} ;
                G.Addresses.Default(2,1) = {'List'}          ;
            else
                G = GOAT_Produce_DefaultAddress(G,year1,year2,TimeResolutions{t},Ext) ;
            end
            Addresses = G.Addresses.Default ;
            if ~CompressFlag && ~strcmp(TimeResolutions{t},'List')
                Attributes = cell(length(Addresses),6) ;
                % Attributes(i,1) = filename ;
                % Attributes(i,2) = varname  ;
                % Attributes(i,3) = start    ;
                % Attributes(i,4) = count    ;
                % Attributes(i,5) = factor   ;
                % Attributes(i,6) = permute   ;
                % Attributes(i,7) = reshape (add dimensions)   ;
                % Assign Attributes to each date
                for k = 1:length(Addresses) - 1 ;
                    if strcmp(TimeResolutions{t},'List')
                        Idf = 1 ;
                        S   = 1 ;
                        Sf  = 1 ;
                    else
                        [year,month,day,hour] = GOAT_serial2date(Addresses,k) ;
                        %                     date = year*1.e10 + month.*1.e4 + day.*1.e2 + hour  ;
                        %                     if date < min(dates1) ||date > max(dates2)
                        %                         if date < min(dates1)
                        %                         [Attributes(k,:)] = deal([{'Date preceeds earliest date in dataset'},CF(1,4),{[]},{[]},{[]},{[]},{[]}]) ;
                        %                         else
                        %                         [Attributes(k,:)] = deal([{'Date exceeds latest date in dataset'},CF(1,4),{[]},{[]},{[]},{[]},{[]}]) ;
                        %                         end
                        %                         continue ;
                        %                     end
                        %                     % find appropriate file
                        %                     Idf = find(dates1 <= date & dates2 >= date) ;
                        %                     if isempty(Idf)
                        %                         [Attributes(k,:)] = deal([{'No matching file'},CF(1,4),{[]},{[]},{[]},{[]},{[]}]) ;
                        %                         continue ;
                        %                     elseif length(Idf) > 1
                        %                         [Attributes(k,:)] = deal([{'More than one matching file'},CF(1,4),{[]},{[]},{[]},{[]},{[]}]) ;
                        %                         continue ;
                        %                     end
                        Idf = 1 ;
                        S  = (GOAT_getSerial(year,month,day,hour,TimeResolutions{t},year1)-1)*SeqFileNum + 1 ;
                        Sf = (GOAT_getSerial(CF{Idf,9},CF{Idf,10},CF{Idf,11},CF{Idf,12},TimeResolutions{t},year1)-1)*SeqFileNum + 1 ;
                    end
                    if SeqFileNum == 1
                        Attributes(k,1) = CF(Idf,3) ;
                        Attributes(k,2) = CF(Idf,4) ;
                        if isempty(CV{Idx,8})
                            Attributes(k,5) = {1}  ;
                        else
                            Attributes(k,5) = {cell2mat(CV(Idx,8))}  ;
                        end
                    else
                        Attributes_1 = cell(SeqFileNum,1) ;
                        Attributes_2 = cell(SeqFileNum,1) ;
                        Attributes_3 = cell(SeqFileNum,1) ;
                        Attributes_4 = cell(SeqFileNum,1) ;
                        Attributes_5 = cell(SeqFileNum,1) ;
                        Attributes_6 = cell(SeqFileNum,1) ;
                        %Attributes_7 = cell(SeqFileNum,1) ;
                        for sf = 1:SeqFileNum
                            Attributes_1(sf,1) = CF(Idf,3) ;
                            Attributes_2(sf,1) = CF(Idf,4) ;
                            if isempty(CV{Idx,8})
                                Attributes_5(sf,1) = {1/SeqFileNum}  ;
                            else
                                Attributes_5(sf,1) = {cell2mat(CV(Idx,8))/SeqFileNum}  ;
                            end
                        end
                    end
                    % locate dimensions
                    dims = cell2mat(CF(Idf,5:8)) ;
                    dims = dims(dims>0) ;
                    if SeqFileNum == 1
                        if ~(isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                            %                         Attributes(k,6) = {0}   ;
                            %                     else
                            Attributes(k,6) = {dims}   ;
                        end
                    else
                        for sf = 1:SeqFileNum
                            if ~(isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                                %                             Attributes_6(sf,1) = {0}   ;
                                %                         else
                                Attributes_6(sf,1) = {dims}   ;
                            end
                        end
                    end
                    switch length(dims)
                        case 2 % lat,lon
                            start = [1 1] ;
                            count = [inf inf] ;
                        case 3 % lat,lon,time
                            start = [1 1 1] ;
                            start(dims(3)) = (S-Sf)/CF{Idf,17}+1 ;
                            count = [1 1 1] ;
                            count(dims(1)) = inf ;
                            count(dims(2)) = inf ;
                        case 4 % lat lon level time
                            start = [1 1 1 1] ;
                            start(dims(4)) = (S-Sf)/CF{Idf,17}+1 ;
                            count = [1 1 1 1] ;
                            count(dims(1)) = inf ;
                            count(dims(2)) = inf ;
                            count(dims(3)) = inf ;
                    end
                    if SeqFileNum == 1
                        Attributes(k,3) = {start} ;
                        Attributes(k,4) = {count} ;
                    else
                        for sf = 1:SeqFileNum
                            AddStart = zeros(size(start)) ; AddStart(end) = 1 ;
                            Attributes_3(sf,1) = {start + AddStart*(sf-1)} ;
                            Attributes_4(sf,1) = {count} ;
                        end
                    end
                    if SeqFileNum >1
                        Attributes(k,1) = {Attributes_1} ;
                        Attributes(k,2) = {Attributes_2} ;
                        Attributes(k,3) = {Attributes_3} ;
                        Attributes(k,4) = {Attributes_4} ;
                        Attributes(k,5) = {Attributes_5} ;
                        Attributes(k,6) = {Attributes_6} ;
                        %Attributes(k,7) = {Attributes_7} ;
                    end
                end
                
            else
                qc = length(Addresses)-1 ;
                Idf = 1 ;
                dims = cell2mat(CF(Idf,5:8)) ;
                dims = dims(dims>0) ;
                if (isequal(dims,[1 2]) || isequal(dims,[1 2 3]) || isequal(dims,[1 2 3 4]))
                    Permute = {[]}   ;
                else
                    Permute = {dims}   ;
                end
                Attributes = cell(2,6) ;
                Attributes(end,5) = {qc} ;
                switch SeqFileNum
                    case 1
                        Col = cell(1,2) ;
                        % Column 1: OPeNDAP Address
                        Col(1,1) = CF(Idf,3) ;
                        Col(1,2) = {qc} ;
                        Attributes(1,1) = {Col} ;
                        % Column 2: source varname
                        Col(1,1) = CF(Idf,4) ;
                        Col(1,2) = {qc} ;
                        Attributes(1,2) = {Col} ;
                        switch length(dims)
                            case 3 % lat,lon,time
                                % Column 3: start
                                Col(1,1) = {['Start = ones(',int2str(qc),',3) ; ', ...
                                    'Start(:,',int2str(dims(3)),') = (1:',int2str(qc),')'' ; ', ...
                                    'Col = num2cell(Start,2) ; ']} ;
                                Col(1,2) = {'eval'} ;
                                Attributes(1,3) = {Col} ;
                                % Column 4: count
                                count = [1 1 1] ;
                                count(dims(1)) = inf ;
                                count(dims(2)) = inf ;
                                Col(1,1) = {count} ;
                                Col(1,2) = {qc} ;
                                Attributes(1,4) = {Col} ;
                            case 4 % lat lon level time
                                % Column 3: start
                                Col(1,1) = {['Start = ones(',int2str(qc),',4) ; ', ...
                                    'Start(:,',int2str(dims(4)),') = (1:',int2str(qc),')'' ; ', ...
                                    'Col = num2cell(Start,2) ; ']} ;
                                Col(1,2) = {'eval'} ;
                                Attributes(1,3) = {Col} ;
                                % Column 4: count
                                count = [1 1 1 1] ;
                                count(dims(1)) = inf ;
                                count(dims(2)) = inf ;
                                count(dims(3)) = inf ;
                                Col(1,1) = {count} ;
                                Col(1,2) = {qc} ;
                                Attributes(1,4) = {Col} ;
                        end
                        % Column 5: Factor
                        Col(1,2) = {qc} ;
                        if isempty(CV{Idx,8})
                            Col(1,1) = {1}  ;
                        else
                            Col(1,1) = {cell2mat(CV(Idx,8))}  ;
                        end
                        Attributes(1,5) = {Col} ;
                        % Column 6: Permute
                        if ~isempty(Permute{1})
                            Col(1,2) = {qc} ;
                            Col(1,1) = Permute  ;
                            Attributes(1,6) = {Col} ;
                        end
                    otherwise
                        Attributes_1 = cell(SeqFileNum,1) ;
                        Attributes_2 = cell(SeqFileNum,1) ;
                        Attributes_3 = cell(SeqFileNum,1) ;
                        Attributes_4 = cell(SeqFileNum,1) ;
                        Attributes_5 = cell(SeqFileNum,1) ;
                        Attributes_6 = cell(SeqFileNum,1) ;
                        %Attributes_7 = cell(SeqFileNum,1) ;
                        for sf = 1:SeqFileNum
                            Attributes_1(sf,1) = CF(Idf,3) ;
                            Attributes_2(sf,1) = CF(Idf,4) ;
                            if isempty(CV{Idx,8})
                                Attributes_5(sf,1) = {1/SeqFileNum}  ;
                            else
                                Attributes_5(sf,1) = {cell2mat(CV(Idx,8))/SeqFileNum}  ;
                            end
                            Attributes_6(sf,1) = Permute ;
                        end
                        Col = cell(1,2) ;
                        % Column 1: OPeNDAP Address
                        Col(1,1) = {Attributes_1} ;
                        Col(1,2) = {qc} ;
                        Attributes(1,1) = {Col} ;
                        % Column 2: source varname
                        Col(1,1) = {Attributes_2} ;
                        Col(1,2) = {qc} ;
                        Attributes(1,2) = {Col} ;
                        switch length(dims)
                            case 3 % lat,lon,time
                                % Column 3: start
                                Col(1,1) = {['Start = ones(',int2str(qc*SeqFileNum),',3) ; ', ...
                                    'Start(:,',int2str(dims(3)),') = (1:',int2str(qc*SeqFileNum),')'' ; ', ...
                                    'Col = cell(',int2str(qc),',1) ; ', ...
                                    'for ijk = 1:',int2str(qc),' ; ', ...
                                    'Col(ijk) = {num2cell(Start((ijk-1)*',int2str(SeqFileNum),'+1:ijk*',int2str(SeqFileNum),',:,:),2)} ; ', ...
                                    'end ;  ']} ;
                                Col(1,2) = {'eval'} ;
                                Attributes(1,3) = {Col} ;
                                % Column 4: count
                                count = [1 1 1] ;
                                count(dims(1)) = inf ;
                                count(dims(2)) = inf ;
                                for sf = 1:SeqFileNum
                                    Attributes_4(sf,1) = {count} ;
                                end
                                Col(1,1) = {Attributes_4} ;
                                Col(1,2) = {qc} ;
                                Attributes(1,4) = {Col} ;
                            case 4 % lat lon level time
                                % Column 3: start
                                Col(1,1) = {['Start = ones(',int2str(qc*SeqFileNum),',4) ; ', ...
                                    'Start(:,',int2str(dims(4)),') = (1:',int2str(qc*SeqFileNum),')'' ; ', ...
                                    'Col = cell(',int2str(qc),',1) ; ', ...
                                    'for ijk = 1:',int2str(qc),' ; ', ...
                                    'Col(ijk) = {num2cell(Start((ijk-1)*',int2str(SeqFileNum),'+1:ijk*',int2str(SeqFileNum),',:,:,:),2)} ; ', ...
                                    'end ;  ']} ;
                                Col(1,2) = {'eval'} ;
                                Attributes(1,3) = {Col} ;
                                % Column 4: count
                                count = [1 1 1 1] ;
                                count(dims(1)) = inf ;
                                count(dims(2)) = inf ;
                                count(dims(3)) = inf ;
                                for sf = 1:SeqFileNum
                                    Attributes_4(sf,1) = {count} ;
                                end
                                Col(1,1) = {Attributes_4} ;
                                Col(1,2) = {qc} ;
                                Attributes(1,4) = {Col} ;
                        end
                        % Column 5: Factor
                        Col(1,1) = {Attributes_5} ;
                        Col(1,2) = {qc} ;
                        Attributes(1,5) = {Col} ;
                        % Column 6: Permute
                        Col(1,1) = {Attributes_6} ;
                        Col(1,2) = {qc} ;
                        Attributes(1,6) = {Col} ;
                end
                
            end
            Attributes(end,1) = {Ext} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
            Attributes(end,2) = {[year1 year2]} ;
            Attributes(end,3) = {[]} ; % Defs Struct
            Attributes(end,4) = TimeResolutions(t) ;
            if strcmp(TimeResolutions{t},'List')
                mkdir(fullfile(Data_Link,TimeResolutions{t},FieldName,'Invariant')) ;
                save(fullfile(Data_Link,TimeResolutions{t},FieldName,'Invariant','link_field.mat'),'Attributes') ;
            else
                save(fullfile(Data_Link,TimeResolutions{t},FieldName,'link_field.mat'),'Attributes') ;
            end
            if strcmp(FieldName,'ps')
                if ~exist(fullfile(Data_Link,TimeResolutions{t},'BoerBeta'),'dir') ;
                    mkdir(fullfile(Data_Link,TimeResolutions{t},'BoerBeta')) ;
                end
                Attributes(end,1) = {'bns'} ;
                save(fullfile(Data_Link,TimeResolutions{t},'BoerBeta','link_field.mat'),'Attributes') ;
                %FieldDescription = 'Boer Beta Function' ;
                %Units = '1' ;
                %save(fullfile(Data_Link,TimeResolutions{t},'BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
            end
        end
    end
    
    
    TT = toc ;
    totaltoc = totaltoc + TT ;
    HH = floor(TT/3600) ;
    MM = floor((TT - HH*3600)/60) ;
    SS = TT - HH*3600 - MM*60 ;
    if HH > 0
        TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif MM > 0
        TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
    elseif SS > 0
        TimeString = [int2str(SS),' seconds.'] ;
    end
    disp(['Linking MERRA ',int2str(ds),'/',int2str(size(CM,1)),': ',dataset,' completed in ',TimeString]) ;
    
end

% 'Borrow' Boer Beta Functions where needed
disp 'assigning BoerBeta fields where needed . . .'
TargetBoer = {'tavg_3d_cld_Cp','tavg_3d_mst_Cp','tavg_3d_qdt_Cp','tavg_3d_rad_Cp','tavg_3d_udt_Cp','tavg_3d_odt_Cp','tavg_3d_trb_Cp','tavg_3d_tdt_Cp'} ;
SourceBoer = 'inst_3d_asm_Cp' ;
load(fullfile(Data_ALP,'MERRA_LINK','inst_3d_asm_Cp','Monthly','ta','Dims.mat')) ;
Units = '1' ;
FieldDescription = 'Boer Beta Function' ;
for i = 1:length(TargetBoer)
    mkdir(fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'Daily','BoerBeta')) ;
    save(fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'Daily','BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
    copyfile(fullfile(Data_ALP,'MERRA_LINK',SourceBoer,'Daily','BoerBeta','link_field.mat'),fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'Daily','BoerBeta','link_field.mat'))
    mkdir(fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'DailyX8','BoerBeta')) ;
    save(fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'DailyX8','BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
    copyfile(fullfile(Data_ALP,'MERRA_LINK',SourceBoer,'DailyX8','BoerBeta','link_field.mat'),fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'DailyX8','BoerBeta','link_field.mat'))
    mkdir(fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'Monthly','BoerBeta')) ;
    save(fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'Monthly','BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
    copyfile(fullfile(Data_ALP,'MERRA_LINK',SourceBoer,'Monthly','BoerBeta','link_field.mat'),fullfile(Data_ALP,'MERRA_LINK',TargetBoer{i},'Monthly','BoerBeta','link_field.mat'))
end
% Add Dims files where needed
TargetDimsBoer = {'inst_3d_ana_Np','inst_3d_ana_Nv','inst_3d_asm_Cp'} ;
Calendars = {'Daily','DailyX4','DailyX8','DailyX24','Monthly'} ;
for i = 1:length(TargetDimsBoer)
    for c = 1: length(Calendars)
        if exist(fullfile(Data_ALP,'MERRA_LINK',TargetDimsBoer{i},Calendars{c},'BoerBeta'),'dir')
            if exist(fullfile(Data_ALP,'MERRA_LINK',TargetDimsBoer{i},Calendars{c},'BoerBeta','Dims.mat'),'file')
                continue
            else
                Path =  fullfile(Data_ALP,'MERRA_LINK',TargetDimsBoer{i},Calendars{c}) ;
                fieldsdir = dir(Path) ;
                Fields = cell(1,1) ;
                NF = 0 ;
                for f = 1:length(fieldsdir)
                    if fieldsdir(f,1).isdir && ~strncmp(fieldsdir(f,1).name,'.',1) && ~strcmp(fieldsdir(f,1).name,'BoerBeta')
                        NF = NF + 1 ;
                        Fields(NF,1) = {fieldsdir(f,1).name} ;
                    end
                end
                % look for 3D fields
                for f = 1:length(Fields)
                    clear P lon lat Plabel lonlabel latlabel Units VerticalOrdinate FieldDescription
                    load(fullfile(Path,Fields{f},'Dims.mat')) ;
                    if length(P) > 1
                        FieldDescription = 'Boer Beta function' ;
                        Units = '1' ;
                        try
                            save(fullfile(Path,'BoerBeta','Dims.mat'),'P','lon','lat','Plabel','latlabel','lonlabel','Units','VerticalOrdinate','FieldDescription') ;
                        catch
                            save(fullfile(Path,'BoerBeta','Dims.mat'),'P','lon','lat','Units','VerticalOrdinate','FieldDescription') ;
                        end
                        break
                    end
                end
            end
        end
    end
end

% Summarize
TT = totaltoc ;
HH = floor(TT/3600) ;
MM = floor((TT - HH*3600)/60) ;
SS = TT - HH*3600 - MM*60 ;
if HH > 0
    TimeString = [int2str(HH),' hours, ',int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif MM > 0
    TimeString = [int2str(MM),' minutes and ',int2str(SS),' seconds.'] ;
elseif SS > 0
    TimeString = [int2str(SS),' seconds.'] ;
end
disp(['Linking entire MERRA archive completed in ',TimeString]) ;