% GOAT_AutoDownload_20CR
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_20CR(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = '20C Reanalysis V2' ;
        D.AutoDownloadDataset.FolderName = 'R20C' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['The Twentieth Century Reanalysis (20CR) provides a comprehensive global atmospheric circulation data set spanning 1870-2010.', ...
            '\nIts chief motivation is to provide an observational validation data set, with quantified uncertainties,', ...
            '\nfor assessing climate model simulations of the 20th century, with emphasis on the statistics of daily weather.', ...
            '\nThe analyses are generated by assimilating only surface pressures and using monthly SST and sea ice distributions as boundary conditions within a ''deterministic'' Ensemble Kalman Filter (EKF).', ...
            '\nA unique feature of the 20CR is that estimates of uncertainty are derived using a 56 member ensemble.',...
            '\nOverall, the quality is approximately that of current three-day NWP forecasts.',...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 0 ; 
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'zg','ta','hur','hus','wap','ua','va','pr',} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1871 ;
        D.AutoDownloadDataset.LastYear = 2011 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Daily' ;
        switch Field
            case 'zg'
                D.AutoDownloadDataset.FieldDescription = 'Geopotential divided by g' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/hgt.yyyy.nc' ;
            case 'pr'
                D.AutoDownloadDataset.FieldDescription = 'Precipitation rate' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/gaussian/monolevel/prate.yyyy.nc' ;
            case 'ua'
                D.AutoDownloadDataset.FieldDescription = 'zonal velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/uwnd.yyyy.nc' ;
            case 'va'
                D.AutoDownloadDataset.FieldDescription = 'meridional velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/vwnd.yyyy.nc' ;
            case 'wap'
                D.AutoDownloadDataset.FieldDescription = 'pressure vertical velocity' ;
                D.AutoDownloadDataset.Units = 'Pa/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/omega.yyyy.nc' ;
            case 'hur'
                D.AutoDownloadDataset.FieldDescription = 'Relative humidity' ;
                D.AutoDownloadDataset.Units = '%' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/rhum.yyyy.nc' ;
            case 'hus'
                D.AutoDownloadDataset.FieldDescription = 'Specific humidity' ;
                D.AutoDownloadDataset.Units = 'Unitless' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/shum.yyyy.nc' ;
            case 'ta'
                D.AutoDownloadDataset.FieldDescription = 'Temperature' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/20thC_ReanV2/Dailies/pressure/air.yyyy.nc' ;
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        % Boer Beta
        if strcmp(D.Field.Name,'zg')
            load('Topography.mat') ; 
            topolat = [-89.5:89.5];     % CREATE LAT ARRAY FOR TOPO MATRIX
            topolon = [0.5:359.5];      % CREATE LON ARRAY FOR TOPO MATRIX
            topo = topo' ;
            [X1, X2] = ndgrid(topolon,topolat);
            F = griddedInterpolant(X1,X2,topo);
            lat = -single(linspace(-90,90,91))' ;
            lon = single(linspace(0,358,180))' ;
            trepN = 24 ;
            latf = lat*-sign(lat(1)) ;
            [lonG, latG] = ndgrid(lon,latf);
            topoG = F(lonG,latG);
            topo90N = (topo(:,end)*1.5 - topo(:,end-1)*.5) ;
            topo90S = (topo(:,1  )*1.5 - topo(:,2)*.5) ;
            topoGN = interp1(topolon',topo90N,lon) ;
            topoGS = interp1(topolon',topo90S,lon) ;
            topoG(:,1) = topoGS ;
            topoG(:,end) = topoGN ;
            topo0 = (topo(1,:) + topo(end,:))/2 ;
            topoG0 = interp1(topolat,topo0,latf) ;
            topoG(1,:) = topoG0 ;
            topoG(1,  1) = (topo90S(1) + topo90S(end))/2 ;
            topoG(1,end) = (topo90N(1) + topo90N(end))/2 ;
            topoG = flipdim(topoG,2) ;
            topoRep = repmat(reshape(topoG,size(topoG,1),size(topoG,2),1),[1 1 trepN] ) ;
        end
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        for y = Year1:Year2
            URL = regexprep(D.Field.url, 'yyyy', int2str(y)) ;
            set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow ;
            FileName = [int2str(y),D.Field.Name,'20C.nc'] ;
            for da = 1:10
                try
                    urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                    break ;
                catch
                    if da < 10
                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                        pause(300) ;
                    else
                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Download Terminated.']) ; drawnow ;
                        return ;
                    end
                end
            end
            Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
            GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
            try
                load(fullfile(GOATAddress,'Dims.mat'))
            catch
                lon = Data.lon ;
                lat = Data.lat ;
                switch D.Field.LevelType
                    case '3D'
                        P = Data.P ;
                    case 'Surface'
                        P = 1000 ;
                    case 'TOA'
                        P = 0    ;
                end
                Units = D.Field.Units ;
                FieldDescription = D.Field.FieldDescription ;
                VerticalOrdinate = D.Field.VerticalOrdinate ;
                try
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                catch
                    mkdir(GOATAddress) ;
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                end
            end
            
            % Check to see whether data size fits 1 year daily
            notcorruptedIdx = 0 ;
            set(D.h_message,'string',['Installing ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow;
            if GOAT_leapyear(y) ;
                if strcmp(D.Field.LevelType,'3D')
                    if size(Data.var1,4) == 366
                        notcorruptedIdx = 1 ;
                    end
                else
                    if size(Data.var1,3) == 366
                        notcorruptedIdx = 1 ;
                    end
                end
            else
                if strcmp(D.Field.LevelType,'3D')
                    if size(Data.var1,4) == 365
                        notcorruptedIdx = 1 ;
                    end
                else
                    if size(Data.var1,3) == 365
                        notcorruptedIdx = 1 ;
                    end
                end
            end
            
            if ~notcorruptedIdx
                if GOAT_leapyear(y)
                    set(D.h_message,'string',['corruped file ',FileName,' --> time index does not match for leap-year']) ;  drawnow ;
                else
                    set(D.h_message,'string',['corruped file ',FileName,' --> time index does not match for non leap-year']) ; drawnow ;
                end
            end
            % Distribute files into Daily Frames
            if notcorruptedIdx
                J = 1 ;
                for monthIdx = 1:12
                    switch monthIdx
                        case {1,3,5,7,8,10,12}
                            monthdays = 31 ;
                        case 2
                            if GOAT_leapyear(y)
                                monthdays = 29 ;
                            else
                                monthdays = 28 ;
                            end
                        case {4,6,9,11}
                            monthdays = 30 ;
                    end
                    for i = 1:monthdays
                        if strcmp(D.Field.LevelType,'3D')
                            data = Data.var1(:,:,:,J) ;
                        else
                            data = Data.var1(:,:,J) ;
                        end
                        FileAddress = fullfile(GOATAddress,int2str(y),[int2str(monthIdx*10000+i*100),'.mat']);
                        try
                            save(FileAddress,'data') ;
                        catch
                            [pathstr, name, ext] = fileparts(FileAddress) ;
                            mkdir(pathstr) ;
                            save(FileAddress,'data') ;
                        end
                        if strcmp(D.Field.Name,'zg')
                            bata = data*0 ;
                            bata(data>=topoRep) = 1 ;
                            FileAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(monthIdx*10000 + i*100),'.mat']) ;
                            [pathstr, name, ext] = fileparts(FileAddress) ;
                            try
                                save(FileAddress,'bata') ;
                            catch
                                mkdir(pathstr) ;
                                save(FileAddress,'bata') ;
                            end
                        end
                        J = J+1 ;
                    end
                    delete(fullfile(DownloadFolder,FileName)) ;
                    % Monthly Means
                    GOATAddressMonthly = fullfile(D.DataSet.Address,'Monthly',D.Field.Name) ;
                    set(D.h_message,'string',['Producing Monthly Averages ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow ;
                    M = 0 ;
                    for monthIdx = 1:12
                        switch monthIdx
                            case {1,3,5,7,8,10,12}
                                monthdays = 31 ;
                            case 2
                                if GOAT_leapyear(y)
                                    monthdays = 29 ;
                                else
                                    monthdays = 28 ;
                                end
                            case {4,6,9,11}
                                monthdays = 30 ;
                        end
                        if strcmp(D.Field.LevelType,'3D')
                            data = mean(Data.var1(:,:,:,M+1:M+monthdays),4) ;
                        else
                            data = mean(Data.var1(:,:,M+1:M+monthdays),3) ;
                        end
                        FileAddress = fullfile(GOATAddressMonthly,int2str(y),[int2str(monthIdx*10000),'.mat']);
                        try
                            save(FileAddress,'data') ;
                        catch
                            [pathstr, name, ext] = fileparts(FileAddress) ;
                            mkdir(pathstr) ;
                            save(FileAddress,'data') ;
                        end
                        if strcmp(D.Field.Name,'zg')
                            bata = data*0 ;
                            bata(data>=topoRep) = 1 ;
                            FileAddress = fullfile(D.DataSet.Address,'Monthly','BoerBeta',int2str(y),[int2str(monthIdx*10000),'.mat']) ;
                            [pathstr, name, ext] = fileparts(FileAddress) ;
                            try
                                save(FileAddress,'bata') ;
                            catch
                                mkdir(pathstr) ;
                                save(FileAddress,'bata') ;
                            end
                        end
                        M = M + monthdays ;
                    end
                end
            end
        end
        VerticalOrdinate = 'Pressure (mb)' ;
        save(fullfile(GOATAddressMonthly,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
        if strcmp(D.Field.Name,'zg')
            FieldDescription = 'Boer Beta Function'    ;
            save(fullfile(D.DataSet.Address,'Monthly','BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
            save(fullfile(D.DataSet.Address,'Daily'  ,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end

return