% GOAT_AutoDownload_GODAS
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_GODAS(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'GODAS Monthly' ;
        D.AutoDownloadDataset.FolderName = 'GODAS' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['GODAS is a real-time ocean analysis and a reanalysis.', ...
            '\nIt is used for monitoring, retrospective analysis as well as for providing oceanic initial conditions for the CFS.', ...
            '\nBoth temperature and synthetic salinity profiles are assimilated in a 3DVAR scheme.', ...
            '\nBeginning with March 2007, altimetry sea level is assimilated.', ...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 1 ; %  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'so','totaldwnhflx','thetao','uo','uflx','vo','vflx','w','sflx','sshg','MixLayerdbss','IsoLayerdbss'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Depth (m)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1980 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Monthly' ;
        switch Field
            case 'so'
                D.AutoDownloadDataset.FieldDescription = 'Salinity' ;
                D.AutoDownloadDataset.Units = 'Unitless' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/salt.xxxx.nc' ;
            case 'totaldwnhflx'
                D.AutoDownloadDataset.FieldDescription = 'Total downward heat flux at surface' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/thflx.xxxx.nc' ;
            case 'thetao'
                D.AutoDownloadDataset.FieldDescription = 'potential temperature' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/pottmp.xxxx.nc' ;
            case 'uo'
                D.AutoDownloadDataset.FieldDescription = 'zonal component of current' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/ucur.xxxx.nc' ;
            case 'uflx'
                D.AutoDownloadDataset.FieldDescription = 'zonal component of momentum flux' ;
                D.AutoDownloadDataset.Units = 'N/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/uflx.xxxx.nc' ;
            case 'vo'
                D.AutoDownloadDataset.FieldDescription = 'meridional component of surface' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/vcur.xxxx.nc' ;
            case 'vflx'
                D.AutoDownloadDataset.FieldDescription = 'meridional component of momentum flux' ;
                D.AutoDownloadDataset.Units = 'N/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/vflx.xxxx.nc' ;
            case 'w'
                D.AutoDownloadDataset.FieldDescription = 'Geometric vertical velocity (dz/dt)' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/dzdt.xxxx.nc' ;
            case 'sflx'
                D.AutoDownloadDataset.FieldDescription = 'Salt flux' ;
                D.AutoDownloadDataset.Units = 'g/cm^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/sltfl.xxxx.nc' ;
            case 'sshg'
                D.AutoDownloadDataset.FieldDescription = 'Sea Surface Height relative to Geoid' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/sshg.xxxx.nc' ;
            case 'MixLayerdbss'
                D.AutoDownloadDataset.FieldDescription = 'Mixing Layer geometric depth below Sea Surface' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/dbss_obml.xxxx.nc' ;
            case 'IsoLayerdbss'
                D.AutoDownloadDataset.FieldDescription = 'Isothermal Later geometric depth below Sea Surface' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/godas/dbss_obil.xxxx.nc' ;
                
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        for y = Year1:Year2
            URL = regexprep(D.Field.url, 'xxxx', int2str(y)) ;
            set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow ;
            FileName = [int2str(y),D.Field.Name,'GODAS.nc'] ;
            for da = 1:10
                try
                    urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                    break ;
                catch
                    if da < 10
                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                        pause(300) ;
                    else
                        set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Download Terminated.']) ; drawnow ;
                        return ;
                    end
                end
            end
            Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
            GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
            try
                load(fullfile(GOATAddress,'Dims.mat'))
            catch
                lon = Data.lon ;
                lat = Data.lat ;
                switch D.Field.LevelType
                    case '3D'
                        P = Data.P ;
                    case 'Surface'
                        P = 0 ;
                end
                Units = D.Field.Units ;
                FieldDescription = D.Field.FieldDescription ;
                VerticalOrdinate = D.Field.VerticalOrdinate ;
                try
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                catch
                    mkdir(GOATAddress) ;
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                end
            end
            
            for m = 1:length(Data.Time)
                if strcmp(D.Field.LevelType,'3D')
                    if length(lon) == size(Data.var1,1)
                        data = Data.var1(:,:,:,m) ;
                    elseif length(lon) == size(Data.var2,1)
                        data = Data.var2(:,:,:,m) ;
                    elseif length(lon) == size(Data.var3,1)
                        data = Data.var3(:,:,:,m) ;
                    elseif length(lon) == size(Data.var4,1)
                        data = Data.var4(:,:,:,m) ;
                    elseif length(lon) == size(Data.var5,1)
                        data = Data.var5(:,:,:,m) ;
                    end
                else
                    if length(lon) == size(Data.var1,1)
                        data = Data.var1(:,:,m) ;
                    elseif length(lon) == size(Data.var2,1)
                        data = Data.var2(:,:,m) ;
                    elseif length(lon) == size(Data.var3,1)
                        data = Data.var3(:,:,m) ;
                    elseif length(lon) == size(Data.var4,1)
                        data = Data.var4(:,:,m) ;
                    elseif length(lon) == size(Data.var5,1)
                        data = Data.var5(:,:,m) ;
                    end
                end
                FileAddress = fullfile(GOATAddress,int2str(y),[int2str(m*10000),'.mat']);
                try
                    save(FileAddress,'data') ;
                catch
                    [pathstr, name, ext] = fileparts(FileAddress) ;
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
            end
            delete(fullfile(DownloadFolder,FileName)) ;
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
        
end

return