% GOAT_AutoDownload_NCEPI
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_NCEPI(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'NCEP-NCAR Reanalysis I Daily' ;
        D.AutoDownloadDataset.FolderName = 'NCEPI' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['NCEP-NCAR (R1) is the original reanalysis effort.', ...
            '\nIt uses a frozen global state-of-the-art global data assimilation system (as of 11 January 1995).', ...
            '\nThe original database was enhanced (additional, quality checked datasets) by NCAR''s Data Support Section.', ...
            '\nOriginally planned to span 1957-96 ("40-Year Reanalysis Project"), it was extended back to 1948 and continues to this day.', ...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 1 ; % 1/0 Flag. allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'prw','hus','ps','ta', ...
            'hfls','rlns','rsns','hfss','rsdt','rlnt','rsut','ts','ua','va','wap','zg','pr','uas','vas','hur','AbsVor','StrFunc','RelDiv','RelVort','VelPot','ugrd','vgrd','per'} ;% 
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1948 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Daily' ;
        switch Field
            case 'zg'
                D.AutoDownloadDataset.FieldDescription = 'Geopotential divided by g' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.phi/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'pr'
                D.AutoDownloadDataset.FieldDescription = 'Precipitation rate' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.prate/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'prw'
                D.AutoDownloadDataset.FieldDescription = 'Precipitable water' ;
                D.AutoDownloadDataset.Units = 'kg/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic /.total_air_column/.PWAT/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;  
            case 'per'
                D.AutoDownloadDataset.FieldDescription = 'Potential evaporation rate' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis.dailyavgs/surface_gauss/pevpr.sfc.gauss.YYYY.nc' ;
            case 'ua'
                D.AutoDownloadDataset.FieldDescription = 'zonal velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.u/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'va'
                D.AutoDownloadDataset.FieldDescription = 'meridional velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.v/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'wap'
                D.AutoDownloadDataset.FieldDescription = 'pressure vertical velocity' ;
                D.AutoDownloadDataset.Units = 'Pa/s' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.vvel/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'ugrd'
                D.AutoDownloadDataset.FieldDescription = 'Zonal wind 1829, 2743 and 3658 m above groud' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                D.AutoDownloadDataset.VerticalOrdinate = 'Height (m)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.above_MSL/.u/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.above_MSL/.u/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'vgrd'
                D.AutoDownloadDataset.FieldDescription = 'Meridional wind 1829, 2743 and 3658 m above groud' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                D.AutoDownloadDataset.VerticalOrdinate = 'Height (m)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.above_MSL/.v/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.above_MSL/.v/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'uas'
                D.AutoDownloadDataset.FieldDescription = 'Zonal wind 10m above ground' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.Units = 'm/s' ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.above_ground/.u/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.above_ground/.u/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'vas'
                D.AutoDownloadDataset.FieldDescription = 'Meridional wind 10m above ground' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.Units = 'm/s' ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.above_ground/.v/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.above_ground/.v/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'hus'
                D.AutoDownloadDataset.FieldDescription = 'Specific humidity' ;
                D.AutoDownloadDataset.Units = 'Unitless' ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.qa/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.qa/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'hur'
                D.AutoDownloadDataset.FieldDescription = 'Relative humidity' ;
                D.AutoDownloadDataset.Units = '%' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.rhum/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.rhum/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'ps'
                D.AutoDownloadDataset.FieldDescription = 'Surface pressure' ;
                D.AutoDownloadDataset.Units = 'Pa' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.FirstYear = 1999 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.pressure/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.pressure/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'ts'
                D.AutoDownloadDataset.FieldDescription = 'Surface temperature' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.temp/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.temp/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'ta'
                D.AutoDownloadDataset.FieldDescription = 'Temperature' ;
                D.AutoDownloadDataset.Units = 'K' ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.temp/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.temp/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'hfls'
                D.AutoDownloadDataset.FieldDescription = 'Surface latent heat flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.latent/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.latent/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'rlns'
                D.AutoDownloadDataset.FieldDescription = 'Surface net longwave flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.FirstYear = 1999 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.net/.lwflx/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.net/.lwflx/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'rsns'
                D.AutoDownloadDataset.FieldDescription = 'Surface net shortwave flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.FirstYear = 1999 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.net/.solr/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.net/.solr/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'hfss'
                D.AutoDownloadDataset.FieldDescription = 'Surface sensible heat flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.sensible/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.surface/.sensible/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'rsdt'
                D.AutoDownloadDataset.FieldDescription = 'Top of atmosphere downward shortwave flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.FirstYear = 1999 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.top/.downward/.solr/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.top/.downward/.solr/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'rlnt'
                D.AutoDownloadDataset.FieldDescription = 'Top of atmosphere upwards longwave flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.FirstYear = 1999 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.top/.upward/.lwflx/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.top/.upward/.lwflx/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'rsut'
                D.AutoDownloadDataset.FieldDescription = 'Top of atmosphere upwards shortwave flux' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.FirstYear = 1999 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.top/.upward/.solr/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Diagnostic/.top/.upward/.solr/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'AbsVor'
                D.AutoDownloadDataset.FieldDescription = 'Absolute vorticity' ;
                D.AutoDownloadDataset.Units = '1/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.ABSV/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.ABSV/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'StrFunc'
                D.AutoDownloadDataset.FieldDescription = 'Stream Function' ;
                D.AutoDownloadDataset.Units = 'm^2/s' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.psi/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.psi/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'RelDiv'
                D.AutoDownloadDataset.FieldDescription = 'Relative divergence' ;
                D.AutoDownloadDataset.Units = '1/s' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.reld/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.reld/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'RelVort'
                D.AutoDownloadDataset.FieldDescription = 'Relative vorticity' ;
                D.AutoDownloadDataset.Units = '1/s' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.relv/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.relv/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
            case 'VelPot'
                D.AutoDownloadDataset.FieldDescription = 'Velocity Potential' ;
                D.AutoDownloadDataset.Units = 'm^2/s' ;
                D.AutoDownloadDataset.FirstYear = 2002 ;
                %D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.vpot/T/%281%20Jan%20xxxx%29%2831%20Dec%20xxxx%29RANGE/data.cdf' ;
                D.AutoDownloadDataset.url = 'http://iridl.ldeo.columbia.edu/expert/SOURCES/.NOAA/.NCEP-NCAR/.CDAS-1/.DAILY/.Intrinsic/.PressureLevel/.vpot/T/%28dd%20mmm%20yyyy%29VALUES/data.cdf' ;
        end
    case 'Download'
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        % Boer Beta
        if strcmp(D.Field.Name,'zg')
            load('Topography.mat') ; 
            topolat = -89.5:89.5;     % CREATE LAT ARRAY FOR TOPO MATRIX
            topolon = 0.5:359.5;      % CREATE LON ARRAY FOR TOPO MATRIX
            topo = topo' ;
            [X1, X2] = ndgrid(topolon,topolat);
            F = griddedInterpolant(X1,X2,topo);
            lat = -single(linspace(-90,90,73))' ;
            lon = single(linspace(0,357.5,144))' ;
            trepN = 17 ;
            latf = lat*-sign(lat(1)) ;
            [lonG, latG] = ndgrid(lon,latf);
            topoG = F(lonG,latG);
            topo90N = (topo(:,end)*1.5 - topo(:,end-1)*.5) ;
            topo90S = (topo(:,1  )*1.5 - topo(:,2)*.5) ;
            topoGN = interp1(topolon',topo90N,lon) ;
            topoGS = interp1(topolon',topo90S,lon) ;
            topoG(:,1) = topoGS ;
            topoG(:,end) = topoGN ;
            topo0 = (topo(1,:) + topo(end,:))/2 ;
            topoG0 = interp1(topolat,topo0,latf) ;
            topoG(1,:) = topoG0 ;
            topoG(1,  1) = (topo90S(1) + topo90S(end))/2 ;
            topoG(1,end) = (topo90N(1) + topo90N(end))/2 ;
            topoG = flipdim(topoG,2) ;
            topoRep = repmat(reshape(topoG,size(topoG,1),size(topoG,2),1),[1 1 trepN] ) ;
        end
        GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
        GOATAddressMonthly = fullfile(D.DataSet.Address,'Monthly',D.Field.Name) ;
        Months = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'} ;
        if exist(fullfile(GOATAddress,'Dims.mat'),'file') && exist(fullfile(GOATAddressMonthly,'Dims.mat'),'file') 
            DimsFlag = 0 ; % Dims File already exists
        else
            DimsFlag = 1 ; % Need to produce Dims File
        end
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        switch D.Field.Name
            case {'per'} % download yearly files directly from:
                % http://www.esrl.noaa.gov/psd/data/gridded/data.ncep.reanalysis.html
                for y = Year1:Year2
                    URL = regexprep(D.Field.url, 'YYYY', int2str(y)) ;
                    set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow ;
                    FileName = [int2str(y),D.Field.Name,'NCEP.cdf'] ;
                    for da = 1:10
                        try
                            urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                            break ;
                        catch
                            if da < 10
                                set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                                pause(300) ;
                            else
                                set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),' Failed. Download Terminated.']) ; drawnow ;
                                return ;
                            end
                        end
                    end
                    Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                    
                    if DimsFlag
                        lon = Data.lon ;
                        lat = Data.lat ;
                        switch D.Field.LevelType
                            case '3D'
                                if sum(strcmp(D.Field.VerticalOrdinate,{'Height (m)','Depth (m)'}))
                                    P = Data.Z ;
                                else
                                    try
                                    P = Data.P ;
                                    catch
                                    P = Data.Z ;
                                    end
                                end
                            case 'Surface'
                                P = 1000 ;
                            case 'TOA'
                                P = 1.e-10    ;
                        end
                        Units = D.Field.Units ;
                        FieldDescription = D.Field.FieldDescription ;
                        VerticalOrdinate = D.Field.VerticalOrdinate ;
                        try
                            save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                            save(fullfile(GOATAddressMonthly,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        catch
                            mkdir(GOATAddress) ;
                            mkdir(GOATAddressMonthly) ;
                            save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                            save(fullfile(GOATAddressMonthly,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                        end
                        if strcmp(D.Field.Name,'zg')
                         FieldDescription = 'Boer Beta Function'    ;
                         Units = 'Unitless' ;
                         try
                         save(fullfile(D.DataSet.Address,'Monthly','BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                         save(fullfile(D.DataSet.Address,'Daily'  ,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                         catch
                         mkdir(fullfile(D.DataSet.Address,'Monthly','BoerBeta')) ;
                         mkdir(fullfile(D.DataSet.Address,'Daily','BoerBeta')) ;
                         save(fullfile(D.DataSet.Address,'Monthly','BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                         save(fullfile(D.DataSet.Address,'Daily'  ,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                         end
                        end
                       DimsFlag = 0 ;  
                    end
                    % Distribute files into Daily Frames
                    J = 1 ;
                    for monthIdx = 1:12
                        switch monthIdx
                            case {1,3,5,7,8,10,12}
                                monthdays = 31 ;
                            case 2
                                if GOAT_leapyear(y)
                                    monthdays = 29 ;
                                else
                                    monthdays = 28 ;
                                end
                            case {4,6,9,11}
                                monthdays = 30 ;
                        end
                        for i = 1:monthdays
                            if strcmp(D.Field.LevelType,'3D')
                                data = Data.var1(:,:,:,J) ;
                            else
                                data = Data.var1(:,:,J) ;
                            end
                            FileAddress = fullfile(GOATAddress,int2str(y),[int2str(monthIdx*10000+i*100),'.mat']);
                            try
                                save(FileAddress,'data') ;
                            catch
                                [pathstr, name, ext] = fileparts(FileAddress) ;
                                mkdir(pathstr) ;
                                save(FileAddress,'data') ;
                            end
                            if strcmp(D.Field.Name,'zg')
                                bata = data*0 ;
                                bata(data>=topoRep) = 1 ;
                                FileAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(monthIdx*10000+i*100),'.mat']) ;
                                [pathstr, name, ext] = fileparts(FileAddress) ;
                                try
                                    save(FileAddress,'bata') ;
                                catch
                                    mkdir(pathstr) ;
                                    save(FileAddress,'bata') ;
                                end
                            end
                            J = J+1 ;
                        end
                        % Monthly Means
                        if strcmp(D.Field.LevelType,'3D')
                           data = single(nanmean(Data.var1(:,:,:,J-monthdays:J-1),4)) ;
                        else
                           data = single(nanmean(Data.var1(:,:,J-monthdays:J-1),3)) ;
                        end
                        FileAddress = fullfile(GOATAddressMonthly,int2str(y),[int2str(monthIdx*10000),'.mat']);
                        try
                           save(FileAddress,'data') ;
                        catch
                           [pathstr, name, ext] = fileparts(FileAddress) ;
                           mkdir(pathstr) ;
                           save(FileAddress,'data') ;
                        end
                        if strcmp(D.Field.Name,'zg')
                                bata = data*0 ;
                                bata(data>=topoRep) = 1 ;
                                FileAddress = fullfile(D.DataSet.Address,'Monthly','BoerBeta',int2str(y),[int2str(monthIdx*10000),'.mat']) ;
                                [pathstr, name, ext] = fileparts(FileAddress) ;
                                try
                                    save(FileAddress,'bata') ;
                                catch
                                    mkdir(pathstr) ;
                                    save(FileAddress,'bata') ;
                                end
                        end
                    end
                    delete(fullfile(DownloadFolder,FileName)) ;
                end
            otherwise  % Daily files from Ingrid IRI
                for y = Year1:Year2
                    for monthIdx = 1:12
                        switch monthIdx
                            case {1,3,5,7,8,10,12}
                                monthdays = 31 ;
                            case 2
                                if GOAT_leapyear(y)
                                    monthdays = 29 ;
                                else
                                    monthdays = 28 ;
                                end
                            case {4,6,9,11}
                                monthdays = 30 ;
                        end
                        for d = 1:monthdays
                            FileAddress = fullfile(GOATAddress,int2str(y),[int2str(monthIdx*10000+d*100),'.mat']);
                            if exist(FileAddress,'file')
                                set(D.h_message,'string',['Skipping ',D.Field.Name,': ',Months{monthIdx},'.',int2str(d),'.',int2str(y),' . . .']) ; drawnow ;
                                continue ;
                            else
                                URL = regexprep(D.Field.url, 'yyyy', int2str(y)) ;
                                URL = regexprep(URL, 'dd', int2str(d)) ;
                                URL = regexprep(URL, 'mmm', Months{monthIdx}) ;
                                set(D.h_message,'string',['Downloading ',D.Field.Name,': ',Months{monthIdx},'.',int2str(d),'.',int2str(y),' . . .']) ; drawnow ;
                                FileName = [int2str(y),D.Field.Name,'NCEPI.cdf'] ;
                                for da = 1:10
                                    try
                                        urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                                        break ;
                                    catch
                                        if da < 10
                                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',Months{monthIdx},'.',int2str(d),'.',int2str(y),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                                            pause(300) ;
                                        else
                                            set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',Months{monthIdx},'.',int2str(d),'.',int2str(y),' Failed. Download Terminated.']) ; drawnow ;
                                            return ;
                                        end
                                    end
                                end
                                Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                                if DimsFlag
                                    lon = Data.lon ;
                                    lat = Data.lat ;
                                    switch D.Field.LevelType
                                        case '3D'
                                            if sum(strcmp(D.Field.VerticalOrdinate,{'Height (m)','Depth (m)'}))
                                                P = Data.Z ;
                                            else
                                                try
                                                P = Data.P ;
                                                catch
                                                P = Data.Z ;
                                                end
                                            end
                                        case 'Surface'
                                            P = 1000 ;
                                        case 'TOA'
                                            P = 1.e-10    ;
                                    end
                                    Units = D.Field.Units ;
                                    FieldDescription = D.Field.FieldDescription ;
                                    VerticalOrdinate = D.Field.VerticalOrdinate ;
                                    try
                                        save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                    catch
                                        mkdir(GOATAddress) ;
                                        save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                    end
                                    if strcmp(D.Field.Name,'zg')
                                        Units = 'Unitless' ;
                                        FieldDescription = 'Boer Beta Function' ;
                                        try
                                            save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                        catch
                                            mkdir(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta')) ;
                                            save(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                        end
                                    end
                                    DimsFlag = 0 ;
                                end
                                % Save File
                                data = single(Data.var1) ;
                                try
                                    save(FileAddress,'data') ;
                                catch
                                    pathstr = fileparts(FileAddress) ;
                                    mkdir(pathstr) ;
                                    save(FileAddress,'data') ;
                                end
                                if strcmp(D.Field.Name,'zg')
                                    bata = data*0 ;
                                    bata(data>=topoRep) = 1 ;
                                    FileAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(monthIdx*10000+d*100),'.mat']) ;
                                    pathstr = fileparts(FileAddress) ;
                                    try
                                        save(FileAddress,'bata') ;
                                    catch
                                        mkdir(pathstr) ;
                                        save(FileAddress,'bata') ;
                                    end
                                end
                                
                            end
                            
                        end
                        % Create Monthly File if it does not exist
                        FileAddress = fullfile(GOATAddressMonthly,int2str(y),[int2str(monthIdx*10000),'.mat']);
                        if ~exist(fullfile(GOATAddressMonthly,'Dims.mat'),'file')
                            mkdir(GOATAddressMonthly) ;
                            copyfile(fullfile(GOATAddress,'Dims.mat'),fullfile(GOATAddressMonthly,'Dims.mat')) ;
                        end
                        if strcmp(D.Field.Name,'zg')
                            mkdir(fullfile(D.DataSet.Address,'Monthly','BoerBeta')) ;
                            copyfile(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta','Dims.mat'),fullfile(D.DataSet.Address,'Monthly','BoerBeta','Dims.mat')) ;
                        end
                        fullfile(D.DataSet.Address,'Monthly','BoerBeta',int2str(y),[int2str(monthIdx*10000),'.mat']) ;
                        if ~exist(FileAddress,'file')
                            try
                                load(fullfile(GOATAddress,int2str(y),[int2str(monthIdx*10000+1*100),'.mat']));
                                MonthlyData = data*0 ;
                                if strcmp(D.Field.Name,'zg')
                                    MonthlyBata = data*0 ;
                                end
                                for d = 1:monthdays
                                    load(fullfile(GOATAddress,int2str(y),[int2str(monthIdx*10000+d*100),'.mat']));
                                    MonthlyData = MonthlyData + data ;
                                    if strcmp(D.Field.Name,'zg')
                                        load(fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(monthIdx*10000+d*100),'.mat'])) ;
                                        MonthlyBata = MonthlyBata + bata ;
                                    end
                                end
                                data = single(MonthlyData/monthdays) ;
                                try
                                    save(FileAddress,'data') ;
                                catch
                                    pathstr = fileparts(FileAddress) ;
                                    mkdir(pathstr) ;
                                    save(FileAddress,'data') ;
                                end
                                if strcmp(D.Field.Name,'zg')
                                    bata = single(MonthlyBata/monthdays) ;
                                    FileAddress = fullfile(D.DataSet.Address,'Monthly','BoerBeta',int2str(y),[int2str(monthIdx*10000),'.mat']) ;
                                    pathstr = fileparts(FileAddress) ;
                                    try
                                        save(FileAddress,'bata') ;
                                    catch
                                        mkdir(pathstr) ;
                                        save(FileAddress,'bata') ;
                                    end
                                end
                                
                            catch
                                disp(['could not produce monthly mean file',FileAddress])
                            end
                        end
                    end
                end
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end

return ;