% GOAT_AutoDownload_NCEPII  Daily
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% % D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_NCEPII(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'NCEP-DOE Reanalysis II Daily' ;
        D.AutoDownloadDataset.FolderName = 'NCEPII' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['\nNCEP-DOE Reanalysis II (R2) is an improved version of the NCEP-NCAR Reanalysis I (R1).', ...
            '\nThe improvements include an updated model, better physical parameterizations and assorted error fixes.', ...
            '\nHowever, it is still a ''first generation'' product.', ...
            '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ;
        D.AutoDownloadDataset.CurrentYear = 0 ; %  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'zg','ta','hur','wap','ua','va','pr','per'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = '3D' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1979 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'Daily' ;
        switch Field
            case 'zg'
                D.AutoDownloadDataset.FieldDescription = 'Geopotential divided by g' ;
                D.AutoDownloadDataset.Units = 'm' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/pressure/hgt.yyyy.nc' ;
            case 'pr'
                D.AutoDownloadDataset.FieldDescription = 'Precipitation rate' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/gaussian_grid/prate.sfc.gauss.yyyy.nc' ;
            case 'per'
                D.AutoDownloadDataset.FieldDescription = 'Potential evaporation rate' ;
                D.AutoDownloadDataset.Units = 'W/m^2' ;
                D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/gaussian_grid/pevpr.sfc.gauss.yyyy.nc' ;    
            case 'ua'
                D.AutoDownloadDataset.FieldDescription = 'zonal velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/pressure/uwnd.yyyy.nc' ;
            case 'va'
                D.AutoDownloadDataset.FieldDescription = 'meridional velocity' ;
                D.AutoDownloadDataset.Units = 'm/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/pressure/vwnd.yyyy.nc' ;
            case 'wap'
                D.AutoDownloadDataset.FieldDescription = 'pressure vertical velocity' ;
                D.AutoDownloadDataset.Units = 'Pa/s' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/pressure/omega.yyyy.nc' ;
            case 'hur'
                D.AutoDownloadDataset.FieldDescription = 'Relative humidity' ;
                D.AutoDownloadDataset.Units = '%' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/pressure/rhum.yyyy.nc' ;
            case 'ta'
                D.AutoDownloadDataset.FieldDescription = 'Temperature' ;
                D.AutoDownloadDataset.Units = 'K' ;
                D.AutoDownloadDataset.url = 'ftp://ftp.cdc.noaa.gov/Datasets/ncep.reanalysis2.dailyavgs/pressure/air.yyyy.nc' ;
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        % Boer Beta
        if strcmp(D.Field.Name,'zg')
            load('Topography.mat') ; 
            topolat = [-89.5:89.5];     % CREATE LAT ARRAY FOR TOPO MATRIX
            topolon = [0.5:359.5];      % CREATE LON ARRAY FOR TOPO MATRIX
            topo = topo' ;
            [X1, X2] = ndgrid(topolon,topolat);
            F = griddedInterpolant(X1,X2,topo);
            lat = -single(linspace(-90,90,73))' ;
            lon = single(linspace(0,357.5,144))' ;
            trepN = 17 ;
            
            latf = lat*-sign(lat(1)) ;
            [lonG, latG] = ndgrid(lon,latf);
            topoG = F(lonG,latG);
            topo90N = (topo(:,end)*1.5 - topo(:,end-1)*.5) ;
            topo90S = (topo(:,1  )*1.5 - topo(:,2)*.5) ;
            topoGN = interp1(topolon',topo90N,lon) ;
            topoGS = interp1(topolon',topo90S,lon) ;
            topoG(:,1) = topoGS ;
            topoG(:,end) = topoGN ;
            topo0 = (topo(1,:) + topo(end,:))/2 ;
            topoG0 = interp1(topolat,topo0,latf) ;
            topoG(1,:) = topoG0 ;
            topoG(1,  1) = (topo90S(1) + topo90S(end))/2 ;
            topoG(1,end) = (topo90N(1) + topo90N(end))/2 ;
            topoG = flipdim(topoG,2) ;
            topoRep = repmat(reshape(topoG,size(topoG,1),size(topoG,2),1),[1 1 trepN] ) ;
        end
        
        GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        try
        for y = Year1:Year2
            URL = regexprep(D.Field.url, 'yyyy', int2str(y)) ;
            set(D.h_message,'string',['Downloading Daily ',D.Field.Name,': ',int2str(y),' . . .']) ; drawnow ;
            FileName = [int2str(y),'NCEPII.nc'] ;
            urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
            Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
            
            if ~exist(fullfile(GOATAddress,'Dims.mat'),'file') || ~exist(fullfile(D.DataSet.Address,'Monthly',D.Field.Name,'Dims.mat'),'file')
                lon = Data.lon ;
                lat = Data.lat ;
                switch D.Field.LevelType
                    case '3D'
                        P = Data.P ;
                    case 'Surface'
                        P = 1000 ;
                    case 'TOA'
                        P = 1.e-10 ;
                end
                Units = D.Field.Units ;
                FieldDescription = D.Field.FieldDescription ;
                VerticalOrdinate = D.Field.VerticalOrdinate ;
                try
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                catch
                    mkdir(GOATAddress) ;
                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                end
                % Monthly
                mkdir(fullfile(D.DataSet.Address,'Monthly',D.Field.Name)) ;
                save(fullfile(D.DataSet.Address,'Monthly',D.Field.Name,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                if strcmp(D.Field.Name,'zg')
                    FieldDescription = 'Boer Beta Function'    ;
                    save(fullfile(D.DataSet.Address,'Daily'  ,'BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                    save(fullfile(D.DataSet.Address,'Monthly','BoerBeta','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                elseif strcmp(D.Field.Name,'hur')
                    FieldDescription = 'Specific Humidity derived from hur' ;
                    Units = 'Unitless';
                    save(fullfile(D.DataSet.Address,'Daily'  ,'hus','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                    save(fullfile(D.DataSet.Address,'Monthly','hus','Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                end
            end
            
            q = 0 ;
            for m = 1:12
                switch m
                    case {1,3,5,7,8,10,12}
                        mdays = 31 ;
                    case {4,6,9,11}
                        mdays = 30 ;
                    case 2
                        if GOAT_leapyear(y)
                            mdays = 29 ;
                        else
                            mdays = 28 ;
                        end
                end
                for d = 1:mdays
                    q = q+1;
                    if strcmp(D.Field.LevelType,'3D')
                        data = Data.var1(:,:,:,q) ;
                    else
                        data = Data.var1(:,:,q) ;
                    end
                    FileAddress = fullfile(GOATAddress,int2str(y),[int2str(m*10000+d*100 ),'.mat']);
                    try
                        save(FileAddress,'data') ;
                    catch
                        [pathstr, name, ext] = fileparts(FileAddress) ;
                        mkdir(pathstr) ;
                        save(FileAddress,'data') ;
                    end
                    if strcmp(D.Field.Name,'zg')
                        bata = data*0 ;
                        bata(data>=topoRep) = 1 ;
                        FileAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,'BoerBeta',int2str(y),[int2str(m*10000+ d*100 ),'.mat']) ;
                        [pathstr, name, ext] = fileparts(FileAddress) ;
                        try
                            save(FileAddress,'bata') ;
                        catch
                            mkdir(pathstr) ;
                            save(FileAddress,'bata') ;
                        end
                    elseif strcmp(D.Field.Name,'hur')
                        try
                            rhum = data ;
                            load(fullfile(D.DataSet.Address,D.Field.TimeResolution,'ta',int2str(y),[int2str(m*10000+ d*100),'.mat'])) ;
                            T = data ;
                            %L = 2.51e6 - 2.38e3*(T-273.1) ;
                            L = GOAT_Calculate_L(T,D, 'era') ;
                            data = rhum/100*0.622*0.611./Pmat.*exp(L./462.*(1/273.1 - 1./T)) ;
                            FileAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,'hus',int2str(y),[int2str(m*10000+ d*100),'.mat']) ;
                            [pathstr, name, ext] = fileparts(FileAddress) ;
                            try
                                save(FileAddress,'data') ;
                            catch
                                mkdir(pathstr) ;
                                save(FileAddress,'data') ;
                            end
                        catch
                            if q == 1
                                disp('error calculating hus field')% No T Field. Nevermind
                            end
                        end
                    end
                end
                
            end
            delete(fullfile(DownloadFolder,FileName)) ;
        end
        catch
          % possible missing data in year but ok to proceed with monthly means    
        end
        % Monthly
        set(D.h_message,'string',['Producing Monthly means . . .']) ; drawnow ;
        %mkdir(fullfile(D.DataSet.Address,'Monthly',D.Field.Name)) ;
        %save(fullfile(D.DataSet.Address,'Monthly',D.Field.Name,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
        for y = Year1:Year2
            for m = 1 : 12
                switch m
                    case {1,3,5,7,8,10,12}
                        mdays = 31 ;
                    case {4,6,9,11}
                        mdays = 30 ;
                    case 2
                        if GOAT_leapyear(y)
                            mdays = 29 ;
                        else
                            mdays = 28 ;
                        end
                end
                load(fullfile(GOATAddress,num2str(y),[int2str(m*10000+100),'.mat'])) ;
                datam = data*0 ;
                for k = 1 : mdays
                    datam = datam*0 ;
                    clear data
                    load(fullfile(GOATAddress,num2str(y),[int2str(m*10000+k*100),'.mat'])) ;
                    datam = datam + data ;
                end
                data = single(datam/mdays) ;
                FileAddress = fullfile(D.DataSet.Address,'Monthly',D.Field.Name,num2str(y),[int2str(m*10000),'.mat']) ;
                [pathstr, name, ext] = fileparts(FileAddress) ;
                try
                    save(FileAddress,'data') ;
                catch
                    mkdir(pathstr) ;
                    save(FileAddress,'data') ;
                end
                if strcmp(D.Field.Name,'zg')
                    bata = data*0 ;
                    bata(data>=topoRep) = 1 ;
                    FileAddress = fullfile(D.DataSet.Address,'Monthly','BoerBeta',int2str(y),[int2str(m*10000),'.mat']) ;
                    [pathstr, name, ext] = fileparts(FileAddress) ;
                    try
                        save(FileAddress,'bata') ;
                    catch
                        mkdir(pathstr) ;
                        save(FileAddress,'bata') ;
                    end
                elseif strcmp(D.Field.Name,'hur')
                    try
                        rhum = data ;
                        load(fullfile(D.DataSet.Address,'Monthly','ta',int2str(y),[int2str(m*10000),'.mat'])) ;
                        T = data ;
                        %L = 2.51e6 - 2.38e3*(T-273.1) ;
                        L = GOAT_Calculate_L(T,D, 'era') ;
                        data = rhum/100*0.622*0.611./Pmat.*exp(L./462.*(1/273.1 - 1./T)) ;
                        FileAddress = fullfile(D.DataSet.Address,'Monthly','hus',int2str(y),[int2str(m*10000),'.mat']) ;
                        [pathstr, name, ext] = fileparts(FileAddress) ;
                        try
                            save(FileAddress,'data') ;
                        catch
                            mkdir(pathstr) ;
                            save(FileAddress,'data') ;
                        end
                    catch
                        if q == 1
                            disp('error calculating monthly hus field')% No T Field. Nevermind
                        end
                    end
                end
            end
        end
        
        if strcmp(D.Field.Name,'zg')
            set(D.h_message,'string',[D.Field.Name,'( + BoerBeta): Installation Complete.']) ; drawnow ;
        elseif strcmp(D.Field.Name,'hur')
            set(D.h_message,'string',[D.Field.Name,'( + hus): Installation Complete.']) ; drawnow ;
        else
            set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
end

return 