% GOAT_AutoDownload_TRMMDailyX8
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013L or later.
% Input:
% Mode = 'Meta': obtain the following global dataset properties
% D.AutoDownloadDataset.ActiveFlag: 1/0 (yes/no) whether to include the Dataset in the list of datasets appearing in the GUI Fields dropdown menu.
%                  when 0 the dataset will not appear in the GUI list.
% D.AutoDownloadDataset.Name: returns How the dataset name appears in the GUI
% D.AutoDownloadDataset.FolderName: returns the name of the default physical
%                            folder name where the dataset is saved. Also used for substructure name for this dataset
% D.AutoDownloadDataset.DescriptionTip: a brief description of the dataset
% D.AutoDownloadDataset.CurrentYear:  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
% D.AutoDownloadDataset.TimeSelect: 1/0 (yes/no) whether to allow selection of years or not. set to 1 when raw data is given in years and 0 when the data
%                      the Dataset is downloaded as one block.
% D.AutoDownloadDataset.Fields.Names: a cell string list of the fields available for download
% Mode = 'Field': returns the field specifications
% Mode = 'Download': the actual field download code.
% D = GOAT_AutoDownload main structure
% There is no limit to the number of Fields or datasets that can be added.
%
% see also GOAT_VirtualFields, GOAT_AutoDownload

function D = GOAT_AutoDownload_TRMMDailyX8(Mode,D,Field)
warning off all
% This function contains all the operations required for Auto Downloading
% geophysical data and installing it in GOAT Format
switch Mode
    case 'Meta'
        D.AutoDownloadDataset.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
        D.AutoDownloadDataset.Name = 'NASA TRMM V7 DailyX8 0.25deg' ;
        D.AutoDownloadDataset.FolderName = 'TRMMX8' ;
        D.AutoDownloadDataset.DescriptionTip = sprintf(['The Tropical Rainfall Measuring Mission (TRMM) is a joint space mission between NASA and Japan''s National Space Development Agency', ... 
                 '\ndesigned to monitor and study tropical and subtropical precipitation and the associated release of energy.', ...
                 '\nThe mission uses 5 instruments: Precipitation Radar (PR), TRMM Microwave Imager (TMI), Visible Infrared Scanner (VIRS),', ... 
                 '\nClouds & Earths Radiant Energy System (CERES) and  Lightning Imaging Sensor (LSI).', ...
                 '\nMany products on varying grids and temporal resolutions are available. The TMI is the main instrument used for precipitation.',...
                 '\n(Description Source: NCAR-UCAR Climate Data Guide)']) ; 
        D.AutoDownloadDataset.CurrentYear = 1 ; %  1/0 Flag. Allow downloading files from current year (1 = yes | 0 = no)
        D.AutoDownloadDataset.TimeSelect = 1 ;
        D.AutoDownloadDataset.Fields.Names = {'pr'} ;
    case 'Field'
        % Defaults
        D.AutoDownloadDataset.VerticalOrdinate = 'Pressure (mb)' ; % Pressure (mb) | Height (m) | Depth (m) | Sigma | Potential Temperature (K) | Level
        D.AutoDownloadDataset.LevelType = 'Surface' ; % 3D | Surface | TOA
        D.AutoDownloadDataset.FirstYear = 1998 ;
        D.AutoDownloadDataset.LastYear = 2020 ; % GOAT will use year before present if LastYear is later then the current year
        D.AutoDownloadDataset.TimeResolution = 'DailyX8' ;
        switch Field
            case 'pr'
                D.AutoDownloadDataset.FieldDescription = 'Precipitation rate' ;
                D.AutoDownloadDataset.Units = 'kg/m^2/s' ;
                %D.AutoDownloadDataset.url = 'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42_daily%2Fyyyy%2Fddd%2F3B42_daily.YYYY.mm.DD.7.bin&LABEL=3B42_daily.YYYY.mm.DD.7.nc&SHORTNAME=TRMM_3B42_daily&SERVICE=HDF_TO_NetCDF&VERSION=1.02' ;
                D.AutoDownloadDataset.url = 'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2Fyyyy%2Fddd%2F3B42.YYYYmmDD.HH.7.HDF.Z&LABEL=3B42.YYYYmmDD.HH.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02' ;
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F1998%2F001%2F3B42.19980101.03.7.HDF.Z&LABEL=3B42.19980101.03.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F1998%2F001%2F3B42.19980102.00.7.HDF.Z&LABEL=3B42.19980102.00.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02'
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F1999%2F365%2F3B42.20000101.00.7A.HDF.Z&LABEL=3B42.20000101.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02'
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2000%2F002%2F3B42.20000102.21.7A.HDF.Z&LABEL=3B42.20000102.21.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2001%2F027%2F3B42.20010128.00.7A.HDF.Z&LABEL=3B42.20010128.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2002%2F012%2F3B42.20020113.00.7A.HDF.Z&LABEL=3B42.20020113.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2003%2F025%2F3B42.20030126.00.7A.HDF.Z&LABEL=3B42.20030126.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2004%2F031%2F3B42.20040201.00.7A.HDF.Z&LABEL=3B42.20040201.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2005%2F028%2F3B42.20050129.00.7A.HDF.Z&LABEL=3B42.20050129.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2006%2F301%2F3B42.20061029.00.7A.HDF.Z&LABEL=3B42.20061029.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2007%2F046%2F3B42.20070216.00.7A.HDF.Z&LABEL=3B42.20070216.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2008%2F143%2F3B42.20080523.00.7A.HDF.Z&LABEL=3B42.20080523.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2009%2F175%2F3B42.20090625.00.7A.HDF.Z&LABEL=3B42.20090625.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2010%2F015%2F3B42.20100116.00.7A.HDF.Z&LABEL=3B42.20100116.00.7A.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2011%2F029%2F3B42.20110130.00.7.HDF.Z&LABEL=3B42.20110130.00.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2012%2F235%2F3B42.20120823.00.7.HDF.Z&LABEL=3B42.20120823.00.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2013%2F365%2F3B42.20131231.09.7.HDF.Z&LABEL=3B42.20131231.09.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
                                         %  'http://disc2.nascom.nasa.gov/daac-bin/OTF/HTTP_services.cgi?FILENAME=%2Fftp%2Fdata%2Fs4pa%2FTRMM_L3%2FTRMM_3B42%2F2014%2F014%2F3B42.20140114.12.7.HDF.Z&LABEL=3B42.20140114.12.7.nc&SHORTNAME=TRMM_3B42&SERVICE=HDF_TO_NetCDF&VERSION=1.02
        end
    case 'Download'
        warning off all
        DownloadFolder = fullfile('..','Downloads','temp','') ;
        mkdir(DownloadFolder) ;
        DimsFlag = 1 ;
        Year1 = min([D.Field.BeginYear,D.Field.EndYear]) ;
        Year2 = max([D.Field.BeginYear,D.Field.EndYear]) ;
        for y = Year1:Year2
            ddd = 0 ;
            for m = 1:12
                switch m
                    case {1,3,5,7,8,10,12}
                        mdays = 31 ;
                    case {4,6,9,11}
                        mdays = 30 ;
                    case 2
                        if GOAT_leapyear(y)
                            mdays = 29 ;
                        else
                            mdays = 28 ;
                        end
                end
                for d = 1:mdays
                    ddd = ddd+1 ;
                    for hh = 1:8
                    try
                        if ddd == 1 && hh == 1
                            URL = regexprep(D.Field.url, 'yyyy', int2str(y-1)) ;
                            if GOAT_leapyear(y-1)
                                URL = regexprep(URL, 'ddd', int2str(366)) ;
                            else
                                URL = regexprep(URL, 'ddd', int2str(365)) ;
                            end
                        else
                            URL = regexprep(D.Field.url, 'yyyy', int2str(y)) ;
                            if hh == 1
                            URL = regexprep(URL, 'ddd', num2str(ddd-1,'%03.0f')) ;
                            else
                            URL = regexprep(URL, 'ddd', num2str(ddd,'%03.0f')) ;    
                            end
                        end
                        URL = regexprep(URL, 'YYYY', int2str(y)) ;
                        URL = regexprep(URL, 'mm', num2str(m,'%02.0f')) ;
                        URL = regexprep(URL, 'DD', num2str(d,'%02.0f')) ;
                        URL = regexprep(URL, 'HH', num2str((hh-1)*3,'%02.0f')) ;
                        if y>=2000 && y <=2010
                        URL = regexprep(URL, '.7.', '.7A.') ;
                        end
                        set(D.h_message,'string',['Downloading ',D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),'.',num2str(d,'%02.0f'),'.',num2str((hh-1)*3,'%02.0f'),' . . .']) ; drawnow ;
                        FileName = ['TRMMX8data.nc'] ;
                        dlimit = 2 ; 
                        for da = 1: dlimit
                            try
                                try
                                urlwrite(URL,fullfile(DownloadFolder,FileName)) ;
                                catch
                                    
                                end
                                break ;
                            catch
                                if da < dlimit
                                    set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),'.',num2str(d,'%02.0f'),'.',num2str((hh-1)*3,'%02.0f'),' Failed. Waiting 5 min and Retrying . . .']) ; drawnow ;
                                    disp([D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),'.',num2str(d,'%02.0f'),'.',num2str((hh-1)*3,'%02.0f'),' Failed. ']) ; 
                                    pause(300) ;
                                else
                                    set(D.h_message,'string',['Download attempt ',int2str(da),' of ',D.Field.Name,': ',int2str(y),'.',num2str(m,'%02.0f'),'.',num2str(d,'%02.0f'),'.',num2str((hh-1)*3,'%02.0f'),' Failed. Download Terminated.']) ; drawnow ;
                                    %return ;
                                end
                            end
                        end
                        if da == dlimit
                            continue
                        end
                        Data = GOAT_nc2mat(fullfile(DownloadFolder,FileName)) ;
                        GOATAddress = fullfile(D.DataSet.Address,D.Field.TimeResolution,D.Field.Name) ;
                        if DimsFlag
                            try
                                load(fullfile(GOATAddress,'Dims.mat'))
                                DimsFlag = 0 ;
                            catch
                                lon = Data.lon ;
                                lat = Data.lat ;
                                P = 1000 ;
                                Units = D.Field.Units ;
                                VerticalOrdinate = D.Field.VerticalOrdinate ;
                                FieldDescription = D.Field.FieldDescription ;
                                try
                                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                catch
                                    mkdir(GOATAddress) ;
                                    save(fullfile(GOATAddress,'Dims.mat'),'lon','lat','P','Units','FieldDescription','VerticalOrdinate') ;
                                end
                            end
                        end
                        data = Data.var1(:,:) ; 
                        data(data<-1000) = nan ; % non valid data are assigned the integer value -31999
                        data(data<0) = 0 ; 
                        data = data/(3600) ; % Convert from mm/hour to kg/m^2s
                        FileAddress = fullfile(GOATAddress,int2str(y),[int2str(m*10000 + d*100 + (hh-1)*3),'.mat']);
                        try
                            save(FileAddress,'data') ;
                        catch
                            [pathstr, name, ext] = fileparts(FileAddress) ;
                            mkdir(pathstr) ;
                            save(FileAddress,'data') ;
                        end
                        
                        delete(fullfile(DownloadFolder,FileName)) ;
                    catch
                        % Missing dates
                    end
                    end
                end
            end
        end
        set(D.h_message,'string',[D.Field.Name,': Installation Complete.']) ; drawnow ;
        
end

return