function GOAT_Produce_Daily_Means_From_DailyX4
% Organize and distribute data using the GOAT2012 standard
% GOAT2012 standard: 'Monthly --> 'Field' --> 'year' --> month*10000 + Day*100 + Hour(00:23)
% Example: Nov.28.75, 12:00 sits at: '1975' --> '112812.mat'
% Example: November monthly mean sits at: '1975' --> '110000.mat'
try
    DailyX4dir = uigetdir(cd,'DailyX4 Data') ;
    [DailyX4Path, DailyX4Name, ext] = fileparts(DailyX4dir) ;
    Dailydir = fullfile(DailyX4Path, 'Daily','') ;
catch
    return ;
end
tic ;
% Locate Existing Fields
Fieldsdir = dir(DailyX4dir) ;
Fields = cell(1,1) ;
NF = 0 ;
for j = 1:length(Fieldsdir)
    if Fieldsdir(j,1).isdir && ...
            ~strcmp(Fieldsdir(j,1).name,'.') && ...
            ~strcmp(Fieldsdir(j,1).name,'..')
        NF = NF + 1 ;
        Fields(NF,1) = {Fieldsdir(j,1).name} ;
    end
end

for f = 1 : length(Fields)
    yearsdir = dir(fullfile(DailyX4dir,cell2mat(Fields(f)),'')) ;
    NF = 0 ;
    years = [] ;
    for j = 1:length(yearsdir)
        if yearsdir(j,1).isdir && ...
                ~strcmp(yearsdir(j,1).name,'.') && ...
                ~strcmp(yearsdir(j,1).name,'..')
            NF = NF + 1 ;
            years(NF) = round(str2double(yearsdir(j,1).name)) ;
        end
    end
    for y = 1: length(years)
        disp(fullfile(Dailydir,cell2mat(Fields(f)),num2str(years(y)),'')) ;
        for m = 1 : 12
            switch m
                case {1,3,5,7,8,10,12}
                    mdays = 31 ;
                case {4,6,9,11}
                    mdays = 30 ;
                case 2
                    if GOAT_leapyear(years(y))
                        mdays = 29 ;
                    else
                        mdays = 28 ;
                    end
            end
            try
                if ~strcmp(cell2mat(Fields(f)),'BoerBeta')
                    load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+100),'.mat'])) ;
                    datam = data*0 ;
                    for k = 1 : mdays
                        datam = datam*0 ;
                        clear data
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 0 ),'.mat'])) ;
                        datam = datam + data ;
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 6 ),'.mat'])) ;
                        datam = datam + data ;
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 12),'.mat'])) ;
                        datam = datam + data ;
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 18),'.mat'])) ;
                        datam = datam + data ;
                        data = single(datam/4) ;
                        FileAddress = fullfile(Dailydir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100),'.mat']) ;
                        [pathstr, name, ext] = fileparts(FileAddress) ;
                        try
                            save(FileAddress,'data') ;
                        catch
                            mkdir(pathstr) ;
                            save(FileAddress,'data') ;
                        end
                    end
                else
                    load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+100),'.mat'])) ;
                    datam = bata*0 ;
                    for k = 1 : mdays
                        datam = datam*0 ;
                        clear bata
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 0 ),'.mat'])) ;
                        datam = datam + bata ;
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 6 ),'.mat'])) ;
                        datam = datam + bata ;
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 12),'.mat'])) ;
                        datam = datam + bata ;
                        load(fullfile(DailyX4dir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100 + 18),'.mat'])) ;
                        datam = datam + bata ;
                        bata = single(datam/4) ;
                        FileAddress = fullfile(Dailydir,cell2mat(Fields(f)),num2str(years(y)),[int2str(m*10000+k*100),'.mat']) ;
                        [pathstr, name, ext] = fileparts(FileAddress) ;
                        try
                            save(FileAddress,'bata') ;
                        catch
                            mkdir(pathstr) ;
                            save(FileAddress,'bata') ;
                        end
                    end
                end
            catch
                % Nevermind, Missing days
            end
            
        end
    end
    % Transfer Spatial Files
    try
        load(fullfile(DailyX4dir,cell2mat(Fields(f)),'Dims.mat')) ;
        try
        save(fullfile(Dailydir,cell2mat(Fields(f)),'Dims.mat'),'P','lat','lon','Units','FieldDescription','VerticalOrdinate') ;
        catch
        VerticalOrdinate = 'Pressure (mb)' ;    
        save(fullfile(Dailydir,cell2mat(Fields(f)),'Dims.mat'),'P','lat','lon','Units','FieldDescription','VerticalOrdinate') ;
        end
    catch
        % Nevermind
    end
    clear datam ;
end
t = toc ; 
h = floor(toc/3600) ; 
m = floor((t - h*3600)/60) ; 
s = t - h*3600 - m*60 ; 
if h > 0
TimeString = [int2str(h),' hours, ',int2str(m),' minutes and ',int2str(s),' seconds.'] ;
elseif m > 0
TimeString = [int2str(m),' minutes and ',int2str(s),' seconds.'] ;
elseif s > 0 
TimeString = [int2str(s),' seconds.'] ;
end
disp(['Conversion Completed in ',TimeString]) ;