% GOAT_nc2mat open netcdf/cdf/hdf/nc files and import to structure
% Copyright (C) Ori Adam Dec.2011
% recognized dimensions are imported to D.lat,D.lon, D.Z or D.P vectors.
% unrecognized dimensions are imported to D.dim1, D,dim2... vectors
% data vectors are placed in D.var1, D.var2, ...
%
% see also  GOAT_CMIP5_CMIP52GOAT, ERA2GOAT4XDily, ERA2GOAT_QA, GOAT_FileFolderTrees

function D = GOAT_nc2mat(filename)
verbosity = 0 ;
SizeCutoff = 1.e8 ;
Np = 5 ; 
% suited to extract
ncid = netcdf.open(filename,'NC_NOWRITE') ; % Read-Only File

% Get Contents
[numdims, numvars, natts] = netcdf.inq(ncid);


qd = 0 ;
for i = 1:numdims
    
    [varname,xtype,dimids,natts] = netcdf.inqVar(ncid,i-1);
    % Get Variable Attributes
    tmpstruct = struct();
    for jj = 1:natts
        try
            fieldname = netcdf.inqAttName(ncid, i-1, jj-1);
            tmpstruct.(fieldname) = netcdf.getAtt(ncid, i-1, fieldname );
        catch
            % nevermind, natts too big
        end
    end
    if sum(strcmpi(varname,{'Y','latitude','lat','rlat','j'})) % && any(size(netcdf.getVar(ncid,i-1)) == 1)
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,i-1) ;
        if length(inqdimids) == 1
            D.lat = netcdf.getVar(ncid,i-1,'single');
            D.lat = factordata(D.lat,tmpstruct) ;
        end
    elseif sum(strcmpi(varname,{'Z'}))
        D.Z = netcdf.getVar(ncid,i-1,'single') ;
        D.Z = factordata(D.Z,tmpstruct) ;
    elseif  sum(strcmpi(varname,{'P','levelist','levels','lev','level','plev','depth'}))           %strcmp(varname,'P') || strcmp(varname,'levelist')
        D.P = netcdf.getVar(ncid,i-1,'single') ;
        D.P = factordata(D.P,tmpstruct) ;
    elseif sum(strcmpi(varname,{'X','longitude','long','lon','rlon','i'}))  %&& any(size(netcdf.getVar(ncid,i-1)) == 1)
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,i-1) ;
        if length(inqdimids) == 1
            D.lon = netcdf.getVar(ncid,i-1,'single') ;
            D.lon = factordata(D.lon,tmpstruct) ;
        end
    elseif sum(strcmpi(varname,{'T','time','t'})) % && any(size(netcdf.getVar(ncid,i-1)) == 1)
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,i-1) ;
        if length(inqdimids) == 1
            D.Time = netcdf.getVar(ncid,i-1,'single') ;
            D.Time = factordata(D.Time,tmpstruct) ;
        end
    else
        qd = qd+1 ;
        D.(['dim',int2str(qd)]) = netcdf.getVar(ncid,i-1,'single') ;
        D.(['dim',int2str(qd)]) = factordata(D.(['dim',int2str(qd)]),tmpstruct) ;
        D.(['dim',int2str(qd),'name']) = varname ;
    end
end

I = 0 ;
for i = 1:numvars-numdims
    [varname,xtype,dimids,natts] = netcdf.inqVar(ncid,numdims+i-1);
    % Get Variable Attributes
    tmpstruct = struct();
    for jj = 1:natts
        try
            fieldname = netcdf.inqAttName(ncid,numdims+i-1, jj-1);
            tmpstruct.(fieldname) = netcdf.getAtt(ncid,numdims+ i-1, fieldname );
        catch
            % Neverming natts too big
        end
    end
    if sum(strcmpi(varname,{'Y','latitude','lat','rlat','j'})) % && any(size(netcdf.getVar(ncid, numdims+i-1)) == 1)
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,i-1) ;
        if length(inqdimids) == 1
            D.lat = netcdf.getVar(ncid, numdims+i-1,'single');
            D.lat = factordata(D.lat,tmpstruct) ;
        end
    elseif sum(strcmpi(varname,{'Z'}))
        D.Z = netcdf.getVar(ncid,numdims+i-1,'single') ;
        D.Z = factordata(D.Z,tmpstruct) ;
    elseif  sum(strcmpi(varname,{'P','levelist','levels','lev','level','plev','depth'}))
        D.P = netcdf.getVar(ncid,numdims+i-1,'single') ;
        D.P = factordata(D.P,tmpstruct) ;
    elseif sum(strcmpi(varname,{'X','longitude','long','lon','rlon','i'})) % && any(size(netcdf.getVar(ncid,numdims+i-1)) == 1)
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,i-1) ;
        if length(inqdimids) == 1
            D.lon = netcdf.getVar(ncid,numdims+i-1,'single') ;
            D.lon = factordata(D.lon,tmpstruct) ;
        end
    elseif sum(strcmpi(varname,{'T','time','t'})) % && any(size(netcdf.getVar(ncid,numdims+i-1)) == 1)
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,i-1) ;
        if length(inqdimids) == 1
            D.Time = netcdf.getVar(ncid,numdims+i-1,'single') ;
            D.Time = factordata(D.Time,tmpstruct) ;
        end
    else
        I = I + 1 ;
        % inquire about variable size and break up the calculation if it is too big
        [inqname,inqtype,inqdimids] = netcdf.inqVar(ncid,numdims+i-1) ;
        varsize = ones(1,length(inqdimids)) ;
        Size = 1 ;
        for vs = 1:length(inqdimids)
            [dimname, dimlen] = netcdf.inqDim(ncid,inqdimids(vs)) ;
            varsize(1,vs) = dimlen ;
            Size = Size*dimlen ;
        end
        if Size > SizeCutoff  % break files that are too large
            if verbosity
                disp(['warning: large nc variable [',int2str(varsize),'] . . .'])
            end
            switch length(inqdimids)
                case 2
                    if min(varsize') == 1 % same as 1D
                        D.(['var',int2str(I)]) = netcdf.getVar(ncid,numdims+i-1,'single') ;
                        D.(['var',int2str(I)]) = factordata(D.(['var',int2str(I)]),tmpstruct) ;
                    else % 
                        if verbosity
                            disp 'allocating space . . .'
                        end
                        D.(['var',int2str(I)]) = zeros(varsize) ;
                        if verbosity
                            disp 'retrieving variable . . .'
                        end
                        if varsize(1) > Np
                            Nstep = round(varsize(1)/Np) ; 
                            if varsize(1) - Np*Nstep > 0  
                            Nparts = ones(Np+1,1)*Nstep ;
                            Nparts(Np+1,1) = varsize(1) - Np*Nstep   ;
                            else
                            Nparts = ones(Np+1,1)*Np ;    
                            end
                        else
                            Nparts = ones(varsize(1),1) ; 
                        end
                        for l = 1:length(Nparts)
                            if verbosity == 2
                                disp(['part ',int2str(l),' of ',int2str(length(Nparts))])
                            end
                            var1 = netcdf.getVar(ncid,numdims+i-1,[(l-1)*Np                    0],[Nparts(l)               varsize(2)]) ;
                            D.(['var',int2str(I)])((l-1)*Np+1:(l-1)*Np+Nparts(l),:) = single(factordata(var1,tmpstruct)) ;
                        end
                    end
                case 3
                    if verbosity
                        disp 'allocating space . . .'
                    end
                    D.(['var',int2str(I)]) = zeros(varsize) ;
                    if verbosity
                        disp 'retrieving variable . . .'
                    end
                    if varsize(1) > Np
                            Nstep = round(varsize(1)/Np) ; 
                            if varsize(1) - Np*Nstep > 0  
                            Nparts = ones(Np+1,1)*Nstep ;
                            Nparts(Np+1,1) = varsize(1) - Np*Nstep   ;
                            else
                            Nparts = ones(Np+1,1)*Np ;    
                            end
                        else
                            Nparts = ones(varsize(1),1) ; 
                    end
                    for l = 1:length(Nparts)
                            if verbosity == 2
                                disp(['part ',int2str(l),' of ',int2str(length(Nparts))])
                            end
                        var1 = netcdf.getVar(ncid,numdims+i-1,[(l-1)*Np                    0 0],[Nparts(l)               varsize(2)  varsize(3)]) ;
                        D.(['var',int2str(I)])((l-1)*Np+1:(l-1)*Np+Nparts(l),:,:) = single(factordata(var1,tmpstruct)) ;    
                    end
                case 4
                    if verbosity
                        disp 'allocating space . . .'
                    end
                    D.(['var',int2str(I)]) = zeros(varsize) ;
                    if verbosity 
                        disp 'retrieving variable . . .'
                    end
                    if varsize(1) > Np
                            Nstep = round(varsize(1)/Np) ; 
                            if varsize(1) - Np*Nstep > 0  
                            Nparts = ones(Np+1,1)*Nstep ;
                            Nparts(Np+1,1) = varsize(1) - Np*Nstep   ;
                            else
                            Nparts = ones(Np+1,1)*Np ;    
                            end
                        else
                            Nparts = ones(varsize(1),1) ; 
                    end
                    for l = 1:length(Nparts)
                            if verbosity == 2
                                disp(['part ',int2str(l),' of ',int2str(length(Nparts))])
                            end
                        var1 = netcdf.getVar(ncid,numdims+i-1,[(l-1)*Np                    0 0 0],[Nparts(l)               varsize(2)  varsize(3)  varsize(4)]) ;
                        D.(['var',int2str(I)])((l-1)*Np+1:(l-1)*Np+Nparts(l),:,:,:) = single(factordata(var1,tmpstruct)) ;    
                    end
                case 5
                    if verbosity
                        disp 'allocating space . . .'
                    end
                    D.(['var',int2str(I)]) = zeros(varsize) ;
                    if verbosity
                        disp 'retrieving variable . . .'
                    end
                    if varsize(1) > Np
                            Nstep = round(varsize(1)/Np) ; 
                            if varsize(1) - Np*Nstep > 0  
                            Nparts = ones(Np+1,1)*Nstep ;
                            Nparts(Np+1,1) = varsize(1) - Np*Nstep   ;
                            else
                            Nparts = ones(Np+1,1)*Np ;    
                            end
                        else
                            Nparts = ones(varsize(1),1) ; 
                    end
                    for l = 1:length(Nparts)
                            if verbosity == 2
                                disp(['part ',int2str(l),' of ',int2str(length(Nparts))])
                            end
                        var1 = netcdf.getVar(ncid,numdims+i-1,[(l-1)*Np                    0 0 0 0],[Nparts(l)               varsize(2)  varsize(3)  varsize(4)  varsize(5)]) ;
                        D.(['var',int2str(I)])((l-1)*Np+1:(l-1)*Np+Nparts(l),:,:,:,:) = single(factordata(var1,tmpstruct)) ;    
                    end
                otherwise
                    if verbosity
                        disp 'more than 5 dimentison to variable. converting in one batch . . .'
                    end
                    % if 1D or large number of dimensions, take a chance and get the entire variable
                    D.(['var',int2str(I)]) = netcdf.getVar(ncid,numdims+i-1,'single') ;
                    D.(['var',int2str(I)]) = factordata(D.(['var',int2str(I)]),tmpstruct) ;
            end
        else
            D.(['var',int2str(I)]) = netcdf.getVar(ncid,numdims+i-1,'single') ;
            D.(['var',int2str(I)]) = factordata(D.(['var',int2str(I)]),tmpstruct) ;
        end
        D.(['var',int2str(I),'name']) = varname ;
    end
end
if verbosity
    disp(['closing nc file id = ',num2str(ncid)])
end
netcdf.close(ncid) ;
return ;
% Add Nan, Factor and Offset
function data = factordata(data,tmpstruct)
% Replace Missing Values
if (isfield(tmpstruct, 'missing_value') )
    data( data == tmpstruct.missing_value ) = NaN;
end
% Offset
if( isfield(tmpstruct, 'add_offset') )
    Offset = tmpstruct.add_offset;
else
    Offset = 0 ;
end
% Scale data
if( isfield(tmpstruct, 'scale_factor') )
    data = data .* tmpstruct.scale_factor + Offset ;
else
    if Offset ~=0
        data = data + Offset ;
    end
end
return
