% code for creating mass residual, mres, field for ERA-Interim
G = GOAT_Environment ; % Initiate GOAT Environment
Calendar = 'Daily' ; 
DataFolder = '/Volumes/data3/GOAT/Data/NCEP-I' ;         
G = GOAT_Initialize_Dataset(G,DataFolder,Calendar) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.
G = GOAT_Initialize_Field(G,'ua')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
P = double(G.lev*100) ; 
lon = G.lon ; 
lat = G.lat ;
lev0 = G.lev ;
lev = 1 ;
G = GOAT_Initialize_Field(G,'uas')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
G.lev = lev0 ;
VerticalOrdinate = 'Pressure (mb)' ; 
Units = 'Pa/s' ;



FieldDescription = 'vertically integrated mass residual' ; 
if ~exist(fullfile(DataFolder,Calendar,'mres','Dims.mat'),'file')
    mkdir(fullfile(DataFolder,Calendar,'mres')) ;
    save(fullfile(DataFolder,Calendar,'mres','Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ;
end
if ~exist(fullfile(DataFolder,Calendar,'mres_ep','Dims.mat'),'file')
    FieldDescription = 'vertically integrated mass residual due to evaporation minus precipitation' ;
    mkdir(fullfile(DataFolder,Calendar,'mres_ep')) ;
    save(fullfile(DataFolder,Calendar,'mres_ep','Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ;
end
if ~exist(fullfile(DataFolder,Calendar,'mres_div','Dims.mat'),'file')
    FieldDescription = 'vertically integrated mass residual due to divergence' ;
    mkdir(fullfile(DataFolder,Calendar,'mres_div')) ;
    save(fullfile(DataFolder,Calendar,'mres_div','Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ;
end
if ~exist(fullfile(DataFolder,Calendar,'mres_pten','Dims.mat'),'file')
    FieldDescription = 'vertically integrated mass residual due to pressure tendency' ;
    mkdir(fullfile(DataFolder,Calendar,'mres_pten')) ;
    save(fullfile(DataFolder,Calendar,'mres_pten','Dims.mat'),'lon','lat','lev','FieldDescription','Units','VerticalOrdinate') ;
end

lev = lev0 ; 

G = GOAT_Initialize_Field(G,'ps')        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
lon_ps = G.lon ; 
lat_ps = G.lat ;
G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
year1  = 1999 ; % yyyy
month1 = 1    ; % 1:12
day1   = 1    ; % 1:31 
hour1  = 0    ; % 0:3:21
year2  = 2014 ; 
month2 = 12    ; 
day2   = 31    ; 
hour2  = 0    ; 
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ; 
if G.Limits.Default.Day2same && G.Limits.Default.Sequential
    G.Limits.S2 = G.Limits.S1 ;
else
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ; 
end 
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
G.Limits.Default.sameP = 1      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1
Level1  = 0 ; % GOAT will use the nearest level to this value
Level2  = 1000 ; % GOAT will use the nearest level to this value
Lon1    = -360   ; % use values from -360 to 360. GOAT will use the nearest values to the ones specified
Lon2    = 360   ; % use values from -360 to 360. 
Lat1    = 90   ; % use values from -90 to 90. GOAT will use the nearest values to the ones specified.
Lat2    = -90   ; % Lat2 does not have to be larger than Lat1.
G.Limits.Default.Lev1 = dsearchn(double(G.lev),Level1) ; 
G.Limits.Default.Lev2 = dsearchn(double(G.lev),Level2) ; 
G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ; 
G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ; 
G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ; 
G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ; 
G.Filter.Index = ones(length(G.Addresses.Default),1) ;
G = GOAT_Compile_data_Series(G) ;
G = GOAT_Compile_Climatology(G) ; 
ps_Clim = G.DataClim ; 

% cell-2: Temporal limits
% Temporal Checkboxes/Flags
G = GOAT_Initialize_Field(G,'prw')        ;
G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
G.Limits.Default.Sequential = 1 ; % 1/0 Sequential or Seasonal averaging Flag
G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
year1  = 1979 ; % yyyy
month1 = 1    ; % 1:12
day1   = 1    ; % 1:31 
hour1  = 0    ; % 0:3:21
year2  = 2014 ; 
month2 = 12    ; 
day2   = 31    ; 
hour2  = 18    ; 
% G.Limits.S1 and G.Limits.S1 are the temporal indices used by GOAT to locate files in the GOAT archiving standard
G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ; 
if G.Limits.Default.Day2same && G.Limits.Default.Sequential
    G.Limits.S2 = G.Limits.S1 ;
else
G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ; 
end 
%
G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
G.Limits.Default.sameP = 0      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
G.Limits.Default.VerMeanInt = 1 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1
Level1  = 1000 ; % GOAT will use the nearest level to this value
Level2  = 0 ; % GOAT will use the nearest level to this value
Lon1    = 0   ; % use values from -360 to 360. GOAT will use the nearest values to the ones specified
Lon2    = 358.5   ; % use values from -360 to 360. 
Lat1    = 90   ; % use values from -90 to 90. GOAT will use the nearest values to the ones specified.
Lat2    = -90   ; % Lat2 does not have to be larger than Lat1.
G.Limits.Default.Lev1 = dsearchn(double(G.lev),Level1) ; 
G.Limits.Default.Lev2 = dsearchn(double(G.lev),Level2) ; 
G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ; 
G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ; 
G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ; 
G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ; 
% Filters, Weights and Indices: 
% No Filters, Weights or Indices Selected 
G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
% retrieve and compile data 
windowSmooth = 3 ; 
NF = length(G.DataStringVec) ; 
Nmod = ceil(NF/100) ;
GOAT_Display_message(G,'','init') ;
Z = zeros(length(lon),length(lat),1) ; 
mres = Z ; 
Ua   = Z ; 
Va   = Z ; 
DIV = Z ;
psten = Z ; 
divs  = Z ; 
divt  = Z ; 
PE    = Z ; 
dhus = zeros(length(lon),length(lat),9) ; 
switch Calendar
    case 'DailyX4'
        Delta_t = 2*6*3600 ; 
    case 'Daily'
        Delta_t = 2*86400 ; 
    case 'Monthly'
        Delta_t = 2*GOAT_DaysInMonth(m,y,'Daily')*86400 ; 
end
%%
for i = 1:NF
    
if mod(i,Nmod) == 0
                GOAT_Display_message(G,[' . . . Processing ERA-Interim mres  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
end
    datafile = cell2mat(G.Addresses.Default(G.DataStringVec(i))) ; 
    [y,m,d,h] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ; 
    
if (i == 1 && y == 1979) || (i == NF  && y == 2014)
    previousdatafile = datafile ;
    nextdatafile     = datafile ;
else
    previousdatafile = cell2mat(G.Addresses.Default(G.DataStringVec(i-1))) ; 
    nextdatafile     = cell2mat(G.Addresses.Default(G.DataStringVec(i+1))) ; 
end
[ua  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ua',datafile,i) ; % Load the data File
[va  ,G] = GOAT_LoadData(G,DataFolder,Calendar,'va',datafile,i) ; % Load the data File
[hus ,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',datafile,i) ; % Load the data File
[prw1,G] = GOAT_LoadData(G,DataFolder,Calendar,'prw',previousdatafile,i) ; % Load the data File
[prw2,G] = GOAT_LoadData(G,DataFolder,Calendar,'prw',nextdatafile,i) ; % Load the data File
% [hus1,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',previousdatafile,i) ; % Load the data File
% [hus2,G] = GOAT_LoadData(G,DataFolder,Calendar,'hus',nextdatafile,i) ; % Load the data File
try
[ps2 ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ps',nextdatafile,i) ; % Load the data File
[ps1 ,G] = GOAT_LoadData(G,DataFolder,Calendar,'ps',previousdatafile,i) ; % Load the data File
catch
    Jd = GOAT_JulianDay(y,m,d) ;
switch Jd
    case 1
        ps2 = squeeze(ps_Clim(2,:,:)) ;
        ps1 = squeeze(ps_Clim(365,:,:)) ;
    case {365,366}
        ps2 = squeeze(ps_Clim(1,:,:)) ;
        ps1 = squeeze(ps_Clim(364,:,:)) ;
    otherwise
        ps2 = squeeze(ps_Clim(Jd+1,:,:)) ;
        ps1 = squeeze(ps_Clim(Jd-1,:,:)) ;
end
    
end
[bata,G] = GOAT_load_bata(fullfile(DataFolder,Calendar,'BoerBeta',datafile),G,i)  ; 

% fix hus
hus = cat(3,hus,dhus) ; 
% 2D interpolate
ps1i = interp2(lon_ps,lat_ps',ps1',lon,lat')' ;  
ps2i = interp2(lon_ps,lat_ps',ps2',lon,lat')' ;  

Uq = sign(P(end)-P(1))*trapz(P/G.Consts.GravitationalConst,ua.*hus.*bata,3) ; 
Vq = sign(P(end)-P(1))*trapz(P/G.Consts.GravitationalConst,va.*hus.*bata,3) ; 
Ua = sign(P(end)-P(1))*trapz(P/G.Consts.GravitationalConst,ua.*bata,3) ; 
Va = sign(P(end)-P(1))*trapz(P/G.Consts.GravitationalConst,va.*bata,3) ; 
psten = (ps2i-ps1i)/Delta_t ; 
EP = (prw2-prw1)/Delta_t  + GOAT_ERA_divh(G,Uq,Vq)  ; 
divt  = GOAT_ERA_divh(G,Ua,Va) ; 

% total mass residual
 if ~exist(fullfile(DataFolder,Calendar,'mres',fileparts(datafile)),'dir')
    mkdir(fullfile(DataFolder,Calendar,'mres',fileparts(datafile)))
 end
data = single(psten - G.Consts.GravitationalConst*EP + divt*G.Consts.GravitationalConst) ;  % disregard surface pressure divergence (avoid topography related error)
save(fullfile(DataFolder,Calendar,'mres',datafile),'data') ; 
% moist component
if ~exist(fullfile(DataFolder,Calendar,'mres_ep',fileparts(datafile)),'dir')
   mkdir(fullfile(DataFolder,Calendar,'mres_ep',fileparts(datafile)))
end
data = single(EP) ; % disregard surface pressure divergence (avoid topography related error)
save(fullfile(DataFolder,Calendar,'mres_ep',datafile),'data') ; 
% pressure tendency
if ~exist(fullfile(DataFolder,Calendar,'mres_pten',fileparts(datafile)),'dir')
   mkdir(fullfile(DataFolder,Calendar,'mres_pten',fileparts(datafile)))
end
data = single(psten) ; % disregard surface pressure divergence (avoid topography related error)
save(fullfile(DataFolder,Calendar,'mres_pten',datafile),'data') ; 
% divergence
if ~exist(fullfile(DataFolder,Calendar,'mres_div',fileparts(datafile)),'dir')
   mkdir(fullfile(DataFolder,Calendar,'mres_div',fileparts(datafile)))
end
data = single(divt) ; % disregard surface pressure divergence (avoid topography related error)
save(fullfile(DataFolder,Calendar,'mres_div',datafile),'data') ; 

end

disp(['mres ',Calendar,' calculation complete!'])

