% GOAT_ALP_Install: Install linked data as *.mat
% copyright (c) Ori Adam. March.25.2014
%
%

function G = GOAT_ALP_Install(G)

G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
if ~isfield(G.ALP,'SaveSingle')
G.ALP.SaveSingle = 0 ; % save as original form
end
NF  = length(G.DataStringVec) ;
Nmod = ceil(NF/100) ;
% Save Dims.mat file
if ~exist(fullfile(G.ALP.ExportAddress,'Dims.mat'),'file')
    lev = G.lev ;
    lon = G.lon ;
    lat = G.lat ;
    Units = G.DataFieldUnits ;
    FieldDescription = G.FieldDescription ;
    VerticalOrdinate = G.VerticalOrdinate ;
    if ~exist(G.ALP.ExportAddress,'dir')
        mkdir(G.ALP.ExportAddress) ;
    end
    save(fullfile(G.ALP.ExportAddress,'Dims.mat'),'lev','lon','lat','Units','FieldDescription','VerticalOrdinate') ;
end
if G.ALP.BoerBetaFlag
    BoerExportAddress = fullfile(fileparts(G.ALP.ExportAddress),'BoerBeta') ;
    if ~exist(fullfile(BoerExportAddress,'Dims.mat'),'file')
    lev = G.lev ;
    lon = G.lon ;
    lat = G.lat ;
    Units = '1' ;
    FieldDescription = 'Boer Beta function' ;
    VerticalOrdinate = G.VerticalOrdinate ;
    if ~exist(BoerExportAddress,'dir')
        mkdir(BoerExportAddress) ;
    end
    save(fullfile(BoerExportAddress,'Dims.mat'),'lev','lon','lat','Units','FieldDescription','VerticalOrdinate') ;
    end
end
fileaddress = fileparts(G.Addresses.Default{1}) ;
if exist(fullfile(G.DataFolder,G.TimeField,G.DataField,fileaddress,'info.mat'),'file')
    load(fullfile(G.DataFolder,G.TimeField,G.DataField,fileaddress,'info.mat')) ;
    if ~exist(fullfile(G.ALP.ExportAddress,fileaddress),'dir')
        mkdir(fullfile(G.ALP.ExportAddress,fileaddress)) ;
    end
    save(fullfile(G.ALP.ExportAddress,fileaddress,'info.mat'),'infostr')
elseif exist(fullfile(G.DataFolder,G.TimeField,G.DataField,'info.mat'),'file')
    load(fullfile(G.DataFolder,G.TimeField,G.DataField,'info.mat')) ;
    save(fullfile(G.ALP.ExportAddress,'info.mat'),'infostr')
elseif exist(fullfile(G.DataFolder,G.TimeField,'info.mat'),'file')
    load(fullfile(G.DataFolder,G.TimeField,'info.mat')) ;
    save(fullfile(fileparts(G.ALP.ExportAddress),'info.mat'),'infostr')
end
if G.EnvironmentFlag
    GOAT_dispstat('','init') ; 
end
if strcmp(G.ALP.Calendar,G.TimeFieldVirtual)
    if ~G.EnvironmentFlag
         set(G.h_message,'string',[' . . . Installing linked data . . .  ']) ; drawnow ;
    else
         GOAT_dispstat([' . . . Installing linked data . . .  '])  ;
    end
    for i = 1:NF
        LinkFieldFlag = 1 ;
        fileaddress = cell2mat(G.Addresses.Default(G.DataStringVec(i))) ;
        [filePath,fileName] = fileparts(fileaddress) ;
        fullfileaddress = fullfile(G.ALP.ExportAddress,filePath,[fileName,'.mat']) ;
        if G.ALP.OverWriteFlag == 1 && exist(fullfileaddress,'file')
            LinkFieldFlag = 0 ;
            if mod(i,Nmod) == 0
                if ~G.EnvironmentFlag
                    set(G.h_message,'string',[' . . . Installing linked data . . .  ',num2str(i/NF*100,'%02.0f'),' %']) ; drawnow ;
                else
                    GOAT_dispstat([' . . . Installing linked data . . .  ',num2str(i/NF*100,'%02.0f'),' %'])  ;
                end
            end
        end
        if LinkFieldFlag
            if ~G.EnvironmentFlag
                    set(G.h_message,'string',[' . . . Installing ',fullfile(G.DataField,filePath,[fileName,'.mat']),' . . .  ',num2str(i/NF*100,'%02.0f'),' %']) ; drawnow ;
                else
                    GOAT_dispstat([' . . . Installing ',fullfile(G.DataField,filePath,[fileName,'.mat']),' . . .  ',num2str(i/NF*100,'%02.0f'),' %'])  ;
            end
            [data , G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,fileaddress,i) ;
            if ~exist(fileparts(fullfileaddress),'dir')
                mkdir(fileparts(fullfileaddress)) ;
            end
            if ~isempty(data)
            if G.ALP.SaveSingle
                data = single(data) ;
            end
            save(fullfileaddress,'data') ;
            else
            disp(['Failed to install data: ',fullfileaddress])    
            GOAT_dispstat('','init') ; 
            end
        end
        if G.ALP.BoerBetaFlag
            fileaddress = cell2mat(G.Addresses.Default(G.DataStringVec(i))) ;
            [filePath,fileName] = fileparts(fileaddress) ;
            fullfileaddress = fullfile(BoerExportAddress,filePath,[fileName,'.mat']) ;
            if G.ALP.OverWriteFlag == 1 && exist(fullfileaddress,'file')
                continue ;
            end
            try
            [bata , G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,'BoerBeta',fileaddress,i) ;
            if ~exist(fileparts(fullfileaddress),'dir')
                mkdir(fileparts(fullfileaddress)) ;
            end
            if ~isempty(bata)
            if G.ALP.SaveSingle
                bata = single(bata) ;
            end
            save(fullfileaddress,'bata') ;
            else
            disp(['Failed to install bata: ',fullfileaddress])        
            GOAT_dispstat('','init') ; 
            end
            catch
                if ~G.EnvironmentFlag
                set(G.h_message,'string',[' . . . Error while attempting to install Boer Beta function  ']) ; drawnow ;
                else
                GOAT_dispstat(['Error while installing Boer Beta function'])  ;
                end
            end
        end
    end
    if ~G.EnvironmentFlag
        set(G.h_message,'string',['Installation of linked data complete.']) ; drawnow ;
    else
        disp(['Installation of linked data complete.']) ;
    end
else
    %     SequentialFlag   = G.Limits.Default.Sequential ;
    %     DataStringVec    = G.DataStringVec      ;
    %     Addresses        = G.Default.Addresses  ;
    %     Attributes       = G.Default.Attributes ;
    %     G.DataStringVec  = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
    %     Limits_S1        = G.Limits.S1          ;
    %     Limits_S2        = G.Limits.S2          ;
    %     switch G.TimeFieldVirtual
    %         case {'DailyX4','Daily365X4','DailyX8','Daily365X8','Daily360X4','Daily360X8'}
    %             [data0,G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(1))),1)*0 ;  % Null Matrix
    %             % Convert to Daily
    %             [YYY,X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual)  ;
    %             G = GOAT_Produce_DefaultAddress(G,year1,year2,G.ALP.Calendar) ;
    %             AddressesNew = G.Default.Addresses ;
    %             G.Default.Addresses = Addresses ;
    %             G.Default.Attributes = Attributes ;
    %             Year0 = GOAT_serial2date(G.Addresses.Default,1) ;
    %             for i = 1:length(AddressesNew)
    %                 fileaddressnew = cell2mat(AddressesNew(i)) ;
    %                 fullfileaddress = fullfile(G.ALP.ExportAddress,[fileaddressnew(1:end-3),'mat']) ;
    %                 if G.ALP.OverWriteFlag == 0 && exist(fullfileaddress,'file')
    %                     continue ;
    %                 end
    %                 [y,m,d] = GOAT_serial2date(AddressesNew,i) ;
    %                 S1 = GOAT_getSerial(y,m,d,0      ,G.TimeFieldVirtual,Year0) ;
    %                 S2 = GOAT_getSerial(y,m,d,24-24/X,G.TimeFieldVirtual,Year0) ;
    %
    %                 % Better To use Limits.S1 and S2
    %                 try
    %                     datamean = zeros(size(data0),S2-S1+1) ;
    %                     for s = S1:S2
    %                         fileaddress = cell2mat(G.Addresses.Default(s)) ;
    %                         if length(size(data0)) == 2
    %                             datamean(:,:,s-S1+1) = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,fileaddress,i) ;
    %                         else
    %                             datamean(:,:,:,s-S1+1) = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,fileaddress,i) ;
    %                         end
    %                     end
    %                     data = squeeze(nanmean(datamean,length(size(datamean)))) ;
    %                     if ~exist(fileparts(fullfileaddress),'dir')
    %                         mkdir(fileparts(fullfileaddress));
    %                     end
    %                     save(fullfileaddress,'data') ;
    %                 catch
    %                     % no matching linked files
    %                 end
    %             end
    %         case {'Daily','Daily365','Daily360'}
    %
    %         otherwise
    disp(['Unsupported Link-Install Calendar ',G.TimeFieldVirtual])
    %    end
end
