% GOAT_Compile_data_Climatology Compile data into daily or monthly climatological mean
% Copyright (C) Ori Adam Aug.2013. Compatible with version 2013i or later
%
% see also GOAT_Compile_Anomaly, GOAT_Compile_data_Series

function G = GOAT_Compile_ClimAnomaly(G)
if strcmpi(G.TimeFieldVirtual,'List')
    GOAT_Display_message(G,'Aborting ClimAnomaly calculation: Cannot calculate climatology for List Calendar') ;
    return ;
end
if ~isfield(G,'DataClim')
    GOAT_Display_message(G,'Aborting ClimAnomaly calculation: No G.DataClim field') ;
    return ;
end
if ~isfield(G,'DataSeries')
    GOAT_Display_message(G,'Aborting ClimAnomaly calculation: No G.DataSeries field') ;
    return ;
end
NF = length(G.DataStringVec) ;
[YYY , X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
G.DataSeriesAnom = zeros(size(G.DataSeries)) ;
% calculate time stamp for each climatology date
if X
    ClimStamp = zeros(YYY*X,1) ;
    q = 0 ;
    for i = 1:YYY
        for h = 1:X
            q = q+1 ;
            ClimStamp(q)  = i*100 + (h-1)*24/X ;
        end
    end
else
    ClimStamp = 1:12 ;
end
GOAT_Display_message(G,'','init') ;
if NF < 12
Nmod = 0 ; 
else
 Nmod = ceil(NF/100) ;
end
qF = 0 ;
for i = 1:NF
    if mod(i,Nmod) == 0
    GOAT_Display_message(G,['Processing Anomaly DataSeries . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat') ;
    end
    [year,month,day,hour] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
    if X
        if X == 1
            hour = 0 ; 
        end
        if G.Filter.Index(G.DataStringVec(i))
            qF = qF + 1 ;
            TimeStamp = GOAT_JulianDay(1,month,day,G.TimeFieldVirtual)*100 + hour ;
            Idx = find(ClimStamp==TimeStamp) ;
            G.DataSeriesAnom(qF,:,:) = G.DataSeries(qF,:,:) - G.DataClim(Idx,:,:) ;
        end
    else
        if G.Filter.Index(G.DataStringVec(i))
            qF = qF + 1 ;
            G.DataSeriesAnom(qF,:,:) = G.DataSeries(qF,:,:) - G.DataClim(month,:,:) ;
        end
    end
end
GOAT_Display_message(G,'Processing Anomaly DataSeries . . . Done.','stat') ;
return

