% GOAT_Compile_data_Series creates a time series of the selected data
% Copyright (C) Ori Adam Jan.2012. Compatible with GOAT version 2013i or later
%
% see also GOAT_Compile_data, GOAT_Compile_data_Climatology

function G = GOAT_Compile_data_3D(G)

G.DataLimits.Lon1 = G.Limits.Default.Lon1 ; % min([G.Limits.Default.Lon1,G.Limits.Default.Lon2]) ;
G.DataLimits.Lon2 = G.Limits.Default.Lon2 ; % max([G.Limits.Default.Lon1,G.Limits.Default.Lon2]) ;
G.DataLimits.Lat1 = min([G.Limits.Default.Lat1,G.Limits.Default.Lat2]) ;
G.DataLimits.Lat2 = max([G.Limits.Default.Lat1,G.Limits.Default.Lat2]) ;
G.DataLimits.Lev1 = min([G.Limits.Default.Lev1,G.Limits.Default.Lev2]) ;
G.DataLimits.Lev2 = max([G.Limits.Default.Lev1,G.Limits.Default.Lev2]) ;
% Get 3D output sizes
if G.DataLimits.Lon2 >= G.DataLimits.Lon1
    SizeLon0 = (G.DataLimits.Lon2-G.DataLimits.Lon1+1) ;
else
    SizeLon0 = (G.DataLimits.Lon2+1 + length(G.lon) -G.DataLimits.Lon1) ;
end
SizeLat0 = G.DataLimits.Lat2 - G.DataLimits.Lat1 + 1 ;
SizeP0   = G.DataLimits.Lev2 - G.DataLimits.Lev1 + 1 ;

G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
NF  = length(G.DataStringVec) ;
if NF < G.Settings.Default.DisplayProgressCutoff
    Nmod = 0            ;
else
    Nmod = ceil(NF/100) ;
    GOAT_Display_message(G,'','init') ;
end
% Spatial Weights
G.Filter.weight = WeightFunc(G) ;
if Nmod
    GOAT_Display_message(G,' . . . Allocating memory . . .  ','stat') ;
end
if sum(G.Filter.Index(G.DataStringVec))*SizeLat0*SizeLon0*SizeP0 < G.Settings.MaxMatrixSize
    MeanTypeFlag = 1 ; % nanmean over preallocated matrix
    Event_Data = zeros(length(G.DataStringVec),SizeLon0*SizeLat0*SizeP0) ; 
else
    GOAT_Display_message(G,' . . . Requested data too large for Time Series. G.DataSeries will not be created  . . .  ','stat') ;
    MeanTypeFlag = 0 ; % calculate mean by sequentially summing all frames and dividing by total number
    Event_Data = zeros(SizeLon0*SizeLat0*SizeP0,1) ;
    Event_N    = zeros(SizeLon0*SizeLat0*SizeP0,1) ;
end
G.Filter.SimpleLoadFlag = 1 ; % when == 1 [data,G] = GOAT_LoadDataAndTimeFilter(G,i) loads the complete data without spatial truncation/projection. Spatial and temporal filters are applied to the 3D matrix

qF = 0 ;
for i = 1:NF
    if mod(i,G.Settings.breakSteps) && G.Flags.breakFlag
        break ;
    end
    if mod(i,Nmod) == 0
        GOAT_Display_message(G,[' . . . Processing Time Series  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
    end
    if G.Filter.Index(G.DataStringVec(i))
       qF = qF + 1 ;
       [data,G] = GOAT_LoadDataAndTimeFilter(G,i) ; % Loads Data and Applies Time Filters
       % Zonal Wave Number and truncation
       if sum (G.Filter.ZonalWaveNumberIdx) == -1
           if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                data = data(G.DataLimits.Lon1:G.DataLimits.Lon2,:,:) ;
           else
                data = data([G.DataLimits.Lon1:end,1:G.DataLimits.Lon2],:,:) ;
           end
       else
                data = ZonalWaveFilter(data,G.Filter.ZonalWaveNumberIdx) ;
                if G.DataLimits.Lon2 >= G.DataLimits.Lon1
                    data = data(G.DataLimits.Lon1:G.DataLimits.Lon2,:,:) ;
                else
                    data = data([G.DataLimits.Lon1:end,1:G.DataLimits.Lon2],:,:) ;
                end
       end
       % vertical and latitudinal truncation
       data = data(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2) ; 
       if MeanTypeFlag
       Event_Data(qF,:) = data(:)  ;           
       else
           data = data(:) ; 
           NotNan = ~isnan(data) ;
           Event_N = Event_N + NotNan ; 
           data(~NotNan) = 0 ; 
           Event_Data = Event_Data + data  ;
       end
     end
end
if MeanTypeFlag
Event_Data      =      Event_Data(1:qF,:,:) ;
else
    qF = 1 ;
end
if G.DataLimits.Lon2 >= G.DataLimits.Lon1
    G.x = G.lon(G.DataLimits.Lon1:G.DataLimits.Lon2) ;
else
    if G.lon(G.DataLimits.Lon2) > 0
        if size(G.lon,2) < size(G.lon,1)
            G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) ; G.lon(1:G.DataLimits.Lon2)] ;
        else
            G.x = [ (G.lon(G.DataLimits.Lon1:end) -360) , G.lon(1:G.DataLimits.Lon2)] ;
        end
    else
        if size(G.lon,2) < size(G.lon,1)
            G.x = [ G.lon(G.DataLimits.Lon1:end) ; (G.lon(1:G.DataLimits.Lon2) + 360)] ;
        else
            G.x = [ G.lon(G.DataLimits.Lon1:end) , (G.lon(1:G.DataLimits.Lon2) + 360)] ;
        end
    end
end
G.y = G.lat(G.DataLimits.Lat1:G.DataLimits.Lat2) ;
G.z = G.lev(G.DataLimits.Lev1:G.DataLimits.Lev2) ;

if Nmod && ~G.Flags.breakFlag
    GOAT_Display_message(G,[' . . . Processing Time Series  . . .  Done.'],'stat') ;
end
if G.Flags.breakFlag
    set(G.h_message,'string','Process Terminated.') ; drawnow ; % Update Message
    G.Flags.breakFlag = 0 ;
    return ;
end
G.Filter.SimpleLoadFlag = 0 ; % A flag for not performing spatial truncations
% reshape
if MeanTypeFlag
  G.DataSeries     = reshape(Event_Data,qF,SizeLon0,SizeLat0,SizeP0) ;
  G.Data = squeeze(nanmean(G.DataSeries,1)) ;
else
  G.DataSeries = [] ;   
  G.Data = Event_Data./Event_N ;
  G.Data(Event_N == 0) = nan ; 
  G.Data = reshape(G.Data,SizeLon0,SizeLat0,SizeP0) ;
end
G.Dates          = G.Addresses.Default(G.DataStringVec) ;
G.FilteredDates  = G.Dates(G.Filter.Index(G.DataStringVec)==1) ;