% GOAT_Compile_data_Climatology Compile data into daily or monthly climatological mean 
% Copyright (C) Ori Adam Aug.2013. Compatible with version 2013i or later
%
% see also GOAT_Compile_data, GOAT_Compile_data_Series

function G = GOAT_Compile_data_Climatology(G)
if ~G.ExportFlag 
G = GOAT_Compile_data_Series(G) ; 
end
Nf = length(G.DataStringVec) ; 
days = zeros(Nf,1) ;
YYY = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
switch G.TimeFieldVirtual 
    case 'Monthly'
        G.DataClim = zeros(12,size(G.DataSeries,2),size(G.DataSeries,3)) ; 
        Type = 0 ; 
    otherwise
        G.DataClim = zeros(YYY,size(G.DataSeries,2),size(G.DataSeries,3)) ; 
        Type = 1 ;
end
try
set(G.h_message,'string',[' . . . Processing Climatology . . .  ']) ; drawnow ;
end
qF = 0 ; 
for i = 1:Nf
[year,month,day] = GOAT_serial2date(G.Addresses.Default,G.DataStringVec(i)) ;
if Type
        if G.Filter.Index(G.DataStringVec(i))
        qF = qF + 1 ; 
        if month == 2 && day == 29
        days(qF) = 0 ; 
        else
        days(qF) = monthday2dayIdx(month,day) ;
        end
        end
else
        if G.Filter.Index(G.DataStringVec(i))
        qF = qF + 1 ; 
        days(qF) = month ;
        end
end
end
days = days(1:qF) ; 

if Type
    for i = 1:YYY
        idx = find(days==i) ; 
        if ~isempty(idx)
        G.DataClim(i,:,:) = squeeze(nanmean(G.DataSeries(idx,:,:),1)) ; 
        end
    end
else
    for i = 1:12
        idx = find(days==i) ; 
        if ~isempty(idx)
        G.DataClim(i,:,:) = squeeze(nanmean(G.DataSeries(idx,:,:),1)) ; 
        end
    end
end

if G.Limits.Default.DataClimSmooth > 1
    if ~mod(G.Limits.Default.DataClimSmooth,2)
       G.Limits.Default.DataClimSmooth = round(G.Limits.Default.DataClimSmooth) + 1 ; 
       disp 'Warning:  G.Limits.Default.DataClimSmooth must be an odd number.'
       disp(['G.Limits.Default.DataClimSmooth was changed from ',int2str(G.Limits.Default.DataClimSmooth-1),' to ',int2str(G.Limits.Default.DataClimSmooth)]) ; 
    end
    if ~Type && G.Limits.Default.DataClimSmooth > 11
        G.Limits.Default.DataClimSmooth = 11 ;
        disp 'Warning:  G.Limits.Default.DataClimSmooth cannot exceed 11 in monthly climatology moothing.'
        disp(['G.Limits.Default.DataClimSmooth was changed from ',int2str(G.Limits.Default.DataClimSmooth),' to 11']) ; 
        G.Limits.Default.DataClimSmooth = 11 ;
    end
    N = (G.Limits.Default.DataClimSmooth - 1)/2 ; 
    if Type
        BaseNum = YYY ; 
    else
        BaseNum = 12 ; 
    end
    DataClim = zeros(BaseNum + 2*N,size(G.DataSeries,2),size(G.DataSeries,3)) ; 
    DataClim(N+1:N+BaseNum,:,:) = G.DataClim ; 
    for i = 1:N
        DataClim(N+1-i,:,:) = G.DataClim(end-i+1,:,:) ; 
        DataClim(end-N+i,:,:) = G.DataClim(i,:,:) ; 
    end
    %DataClim = filter(ones(1,G.Limits.Default.DataClimSmooth)/G.Limits.Default.DataClimSmooth,1,DataClim,[],1) ;
    DataClim = cumsum(DataClim,1) ;
    G.DataClim = [DataClim(2*N+1,:,:) ; (DataClim(2*N+2:end,:,:) - DataClim(1:BaseNum-1,:,:)) ]./ (2*N+1) ;
    %G.DataClim = DataClim(N+1:N+BaseNum,:,:) ; 
end
    
    