% GOAT_Data_string_vec creates a cell array of addresses for the temporal span of data
% syntax: assumes all information is contained in the GOAT structure G
% >> StringVec = GOAT_Data_string_vec(G)
% Copyright (C) Ori Adam Feb.2012. 
%
% see also GOAT_getSerial, GOAT_serial2date


function StringVec = GOAT_Data_string_vec(G)
if G.Limits.Default.Sequential
    StringVec = G.Limits.S1:G.Limits.S2 ;
else
 Year0 = GOAT_serial2date(G.Addresses.Default,1) ;   
 [year1,month1,day1,hour1] = GOAT_serial2date(G.Addresses.Default,G.Limits.S1) ;
 [year2,month2,day2,hour2] = GOAT_serial2date(G.Addresses.Default,G.Limits.S2) ;
 StringVec = [] ;
 q = 0 ;   
 switch G.TimeFieldVirtual
    case {'Daily','DailyX1','DailyX2','DailyX4','DailyX6','DailyX8','DailyX12','DailyX24', ...
            'Daily365','Daily365X1','Daily365X2','Daily365X4','Daily365X6','Daily365X8','Daily365X12','Daily365X24', ... 
            'Daily360','Daily360X1','Daily360X2','Daily360X4','Daily360X6','Daily360X8','Daily365012','Daily360X24'}
            if G.Limits.Default.Day2same % 1 day longterm average
                for y = year1:year2
                    q = q + 1 ;
                    if ~GOAT_leapyear(y,G.TimeFieldVirtual) && month1 == 2 && day1 == 29
                        Day = 28 ;
                    else
                        Day = day1 ;
                    end
                    S = GOAT_getSerial(y,month1,Day,hour1,G.TimeFieldVirtual,Year0) ;
                    StringVec(q) = S ;
                end
            else  % seasonal averaging over many days/hours
                if month2*10000 + day2*100 + hour2 >= month1*10000 + day1*100 + hour1
                    for y = year1:year2
                        if ~GOAT_leapyear(y,G.TimeFieldVirtual) && month1 == 2 && day1 == 29
                            Day = 28 ;
                        else
                            Day = day1 ;
                        end
                        S1 = GOAT_getSerial(y,month1,Day,hour1,G.TimeFieldVirtual,Year0) ;
                        if ~GOAT_leapyear(y,G.TimeFieldVirtual) && month2 == 2 && day2 == 29
                            Day = 28 ;
                        else
                            Day = day2 ;
                        end
                        S2 = GOAT_getSerial(y,month2,Day,hour2,G.TimeFieldVirtual,Year0) ;
                        StringVec = [StringVec,S1:S2] ;
                    end
                else
                    for y = year1:year2-1
                        if ~GOAT_leapyear(y,G.TimeFieldVirtual) && month1 == 2 && day1 == 29
                            Day = 28 ;
                        else
                            Day = day1 ;
                        end
                        S1 = GOAT_getSerial(y,month1,Day,hour1,G.TimeFieldVirtual,Year0) ;
                        if ~GOAT_leapyear(y+1,G.TimeFieldVirtual) && month2 == 2 && day2 == 29
                            Day = 28 ;
                        else
                            Day = day2 ;
                        end
                        S2 = GOAT_getSerial(y+1,month2,Day,hour2,G.TimeFieldVirtual,Year0) ;
                        StringVec = [StringVec,S1:S2] ;
                    end
                end
            end
    case 'Monthly'
          if G.Limits.Default.Day2same % 1 month longterm average
                for y = year1:year2
                    q = q + 1 ;
                    S = GOAT_getSerial(y,month1,day1,hour1,G.TimeFieldVirtual,Year0) ;
                    StringVec(q) = S ;
                end
            else  % seasonal averaging over many days/hours
                if month2 >= month1 
                    for y = year1:year2
                        S1 = GOAT_getSerial(y,month1,day1,hour1,G.TimeFieldVirtual,Year0) ;
                        S2 = GOAT_getSerial(y,month2,day2,hour2,G.TimeFieldVirtual,Year0) ;
                        StringVec = [StringVec,S1:S2] ;
                    end
                else
                    for y = year1:year2-1
                        S1 = GOAT_getSerial(y,month1,day1,hour1,G.TimeFieldVirtual,Year0) ;
                        S2 = GOAT_getSerial(y+1,month2,day2,hour2,G.TimeFieldVirtual,Year0) ;
                        StringVec = [StringVec,S1:S2] ;
                    end
                end
          end
     otherwise
         if ~strncmp(G.TimeFieldVirtual,'Julian',6)
             disp(['GOAT_Data_string_vec error: Unrecognized TimeFieldVirtual - ',G.TimeFieldVirtual]) ; 
             StringVec = [] ; 
             return  
         end
         
         if G.Limits.Default.Day2same % 1 month longterm average
                for y = year1:year2
                    q = q + 1 ;
                    S = GOAT_getSerial(y,month1,day1,hour1,G.TimeFieldVirtual,Year0) ;
                    StringVec(q) = S ;
                end
            else  % seasonal averaging over many days/hours
                if day2 >= day1 
                    for y = year1:year2
                        S1 = GOAT_getSerial(y,month1,day1,hour1,G.TimeFieldVirtual,Year0) ;
                        S2 = GOAT_getSerial(y,month2,day2,hour2,G.TimeFieldVirtual,Year0) ;
                        StringVec = [StringVec,S1:S2] ;
                    end
                else
                    for y = year1:year2-1
                        S1 = GOAT_getSerial(y  ,month1,day1,hour1,G.TimeFieldVirtual,Year0) ;
                        S2 = GOAT_getSerial(y+1,month2,day2,hour2,G.TimeFieldVirtual,Year0) ;
                        StringVec = [StringVec,S1:S2] ;
                    end
                end
         end         
 end
end