% GOAT_Defaults Initiate Parameters, Flags and Constants
% Copyright (C) Ori Adam Dec.2012. compatible with version 2013i or later
%
% see also, GOAT_MyDefaults, GOAT, GOAT_AutoDownload

function G = GOAT_Defaults(G)
G.Consts.Version = '2015F23'  ; % version = YYYYMDD |  YYYY = year, M = A,B,C...L (1,2,3...12) | DD = 1,2,3...31. e.g., 2013H07 denotes version released Aug.7.2013 ; 
if verLessThan('matlab','8.4.0')
G.MATLAB.HG = 0 ; 
else
G.MATLAB.HG = 1 ; 
end
% Default Background
G.Display.BackgroundPic = fullfile('Art','GoatBackground.jpg') ;
G.Display.BackgroundAlpha = .5 ;
try
reset(RandStream.getglobalStream,sum(100*clock)) ;
catch
try
rand('twister',sum(100*clock)) ;
randn('state',sum(100*clock)) ;
catch
% Nevermind. Rand is not initialized
end
end

G.ALP.tempnc = fullfile(fileparts(fileparts(which('GOAT.m'))),'Downloads','temp','tempfile.nc') ;
if ~exist(fileparts(G.ALP.tempnc),'dir') ;
mkdir(fileparts(G.ALP.tempnc)) ;
end
inc = 1 ;
[TempPath,TempName,Ext] = fileparts(G.ALP.tempnc) ; 
while exist(G.ALP.tempnc,'file')
G.ALP.tempnc = fullfile(TempPath,[TempName,int2str(inc),Ext]) ; 
inc = inc+1 ;
end


GOAT_CleanJavaHeap ; % Clean Java Heap Memory (reduces GUI and nc file related errors)
load AddressRep ; G.Addresses = Addresses ;
G.Consts.MonthsStr = {'Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'} ;
% GUI Parameters
G.scrsz      = get(0,'ScreenSize') ; % Screensize (pixels) kept for compatibility with older GOAT versions
G.Display.scrsz     = G.scrsz      ; % Screensize (pixels)
G.GUI.Fonts.gcaFont          = 'sans-serif' ; % Matlab Default  = Helvetica
G.GUI.Fonts.uiButtonFont     = 'Bangla MN'  ; % Matlab Default  = Helvetica
G.GUI.Fonts.uiHeaderFont     = 'Bangla MN'  ; % Matlab Default  = Helvetica
G.GUI.Fonts.MessageFont      = 'Cambria'  ; % Matlab Default  = Helvetica
G.pos        = [G.scrsz(4) G.scrsz(4) G.scrsz(3) G.scrsz(4)].*[.1 .1 .7 .7] ;
G.Saving.PapersizeWide   = [17 13] ; % Paper size for pdf writing
G.Saving.PapersizeTall   = [13 17] ; % Paper size for pdf writing
G.Saving.PapersizeSquare = [15 15] ; % Paper size for pdf writing
G.Saving.PapersizeGUI    = [28 18] ; % Paper size for pdf writing of GUI image
G.Saving.dpdfResolution  = '-r80'  ; % resolution of printed pdf during export ('-dpdf' renderer)
G.GUI.pos.Main      = [G.scrsz(4) G.scrsz(4) G.scrsz(3) G.scrsz(4)].*[.1 .1 .7 .7] ;
G.GUI.pos.Export    = [G.scrsz(3)*2/5  G.scrsz(4)/10   G.scrsz(3)*1/3  G.scrsz(3)*1/6  ] ;
G.GUI.pos.Limits    = [G.scrsz(3)/5    G.scrsz(4)/10   G.scrsz(3)*3/5  G.scrsz(4)*4/5] ;
G.GUI.pos.Filter    = [G.scrsz(3)/5    G.scrsz(4)/10   G.scrsz(3)*2/5  G.scrsz(4)*4/5] ;
G.GUI.pos.Settings  = [G.scrsz(3)/5    G.scrsz(4)/10   G.scrsz(3)*3/5  G.scrsz(4)*4/5  ] ;
G.GUI.Flags.CloseAfterGetmcode = 0 ; % 0/1 keep/close GUI windows after pressing 'get m code' button in the Limits GUI
G.Display.message_Flag   = 0       ; % When this flag is ON (== 1) GOAT_Display_mesage is disabled
G.Display.colorbarHandle = []      ; % Colorbar Handle is saved for Hold purposes
G.Display.colorbarLimits = []      ; % Colorbar Limits is saved for Hold purposes
G.Display.TextSize_XL    = 18      ; % Standardized Text Font Size 'Large'
G.Display.TextSize_L     = 16      ; % Standardized Text Font Size 'Large'
G.Display.TextSize_M     = 14      ; % Standardized Text Font Size 'Medium'
G.Display.TextSize_S     = 12      ; % Standardized Text Font Size 'Small'
G.Display.Colors.red_cream        = [1   .3  .4 ] ; 
G.Display.Colors.orange_dark      = [255 140 0  ]/256 ; 
G.Display.Colors.orange           = [255 127 0  ]/256 ; 
G.Display.Colors.brown            = [139 87  66 ]/256 ;
G.Display.Colors.orchid           = [153 50  204]/256 ;
G.Display.Colors.gold             = [255 215 0  ]/256 ; 
G.Display.Colors.navyblue         = [35  35  142]/256 ; 
G.Display.Colors.snailblue        = [51  153 204]/256 ; 
G.Display.Colors.matlab_blue      = [0         0.4470    0.7410] ;
G.Display.Colors.matlab_orange    = [0.8500    0.3250    0.0980] ;
G.Display.Colors.matlab_yellow    = [0.9290    0.6940    0.1250] ;
G.Display.Colors.matlab_purple    = [0.4940    0.1840    0.5560] ;
G.Display.Colors.matlab_green     = [0.4660    0.6740    0.1880] ;
G.Display.Colors.matlab_lightblue = [0.3010    0.7450    0.9330] ;
G.Display.Colors.matlab_maroon    = [0.6350    0.0780    0.1840] ;
% Some Flags
G.Flags.breakFlag = 0 ; % when 1 stop Compile_data routines. default = 0 ;
G.ProjectionType = 1                    ; % Default Map Projection
G.Limits.Default.DataSeriesAnomFlag = 0 ; % During GOAT_Compile_data_Series, an anomaly data series is also calculated
G.Limits.Default.DataClimSmooth     = 1 ; % smooth Climatology in GOAT_Compile_data_Climatology, using a centered running average of rank G.Limits.Default.DataClimSmooth (1 = none, 3 5 7 9 11 ...)
G.ExportFlag     = 0                    ; % an index indicating whether during 'Export'
% Address String Default Limits (careful: wrong year limits will result in reading the wrong files for specified dates!!!)
G.Display.FigBackgroudColor = [1 1 1]   ; % White Figure Background color
G.GUI.ButtonColor       = [.9 .9 .91]       ; % Default Button Color
G.GUI.ActiveColor       = [.5  .9  .7 ]     ; % During Runs, the "Run" Button" is set to this color
G.GUI.DropTextColor     = [.25 .25 .25]     ; % Text Color in Popup Menus
G.GUI.HeaderColor       = [.1 .1 .1]        ; % Popup and button Header Color
G.GUI.ButtonTextColor   = [.11 .1 .1]       ; % Button Text Color
G.GUI.colors.celeste = [.5 .7 .9]       ; % celeste background
G.GUI.colors.HoldGrey = [.4 .4 .4]      ; % color of contour lines in hold mode
G.GUI.NumLines = 18                     ; % Number of lines in left action panel
G.GUI.NumEdit  = 13                     ; % Number of lines with a panel
G.GUI.Hgui = 1                          ; % nondimensional gui height
G.GUI.Hedge = 2/100                     ; % (%) distance from edge
G.GUI.Hparts = (G.GUI.Hgui-2*G.GUI.Hedge )/G.GUI.NumLines ; % ND Height of each rubric line
G.GUI.HbuttonHeight = G.GUI.Hparts*0.9  ; % ND Button height
G.GUI.HbuttonLength = 5/100             ; % ND Button length
G.GUI.RNumLines = 19                    ; % Number of lines in left action panel
G.GUI.RHparts = (G.GUI.Hgui-2*G.GUI.Hedge )/G.GUI.RNumLines ; % ND Height of each rubric line
G.GUI.RHbuttonHeight = G.GUI.RHparts*0.9        ; % ND Button height
G.GUI.HbuttonLength = 5/100             ; % ND Button length
G.GUI.TextLength    = 12/100            ; % ND text length
G.GUI.EditLength    = 6/100             ; % ND edit box length
G.GUI.TextSpacing   = .5/100            ; % ND text spacing
G.GUI.b_left        = .5/100            ; % ND margin from left
G.GUI.panelext      = .2/100            ; % ND panel width ~0 draws a line
G.GUI.linext        = .1*G.GUI.Hparts   ; % ND line extension
G.GUI.RHStextlength = 7/100             ; % RHS toolbar text length
G.GUI.RHS_b_right = 1-G.GUI.RHStextlength-.1/100 ; % RHS toolbar text margin
% Error
G.Error.ErrorNum = 1                    ; % error Number Identifier
try 
G.Error.Sound    = audioread(fullfile('Art','goat.wav')) ; % Goat Error Sound
catch % Compatibility with MATLAB versions older than 2014b
G.Error.Sound    = wavread(fullfile('Art','goat.wav')) ; % Goat Error Sound
end
G.Error.SoundFs  = 178040/7.5           ; % alternative goat error sound
G.Error.LatestDisplayWarning = ''       ; % latest error displayed in the command window. Allows preventing repeating the same error message for repeated calculations
% Topography and Coast
G = GOAT_GetTopography(G) ;
% Consts
G.Consts.EarthRadius        = 6.37122e6 ; % Earth's radius in m
G.Consts.SpHeatConstPr      = 1004.64   ; % Atmospheric Specific Heat at Constant Pressure for dry air
G.Consts.SpHeatConstPrMoist = 1952      ; % Atmospheric Specific Heat at Constant Pressure for moist air
G.Consts.LatentHeatOfVapor  = 2.5104e6  ; % Latent heat of vaporization 2.26
G.Consts.Omega              = 7.272e-5  ; % Rotation Rate
G.Consts.GravitationalConst = 9.80616   ; % Earth's Gravtational Constant
G.Consts.WaterDensity       = 1000      ; % mean water density kg/m^3
G.Consts.IdealGasRdry       = 287.04    ; % Ideal Dry Gas Constant (J/kg/K)
G.Consts.IdealGasRvapor     = 461       ; % Ideal water vapor Gas Constant (J/kg/K)
% Plotting
G.Plotting.GUI_PlotFlag = 1             ; % 1/0 : when 1 the plotting takes the LHS buttons into consideration
G.Plotting.TitleMessage = ''            ;
% Message Area
G.message_Limits = '' ; % A string that can be added to the Limits GUI
G.message        = 'Welcome !  Select Data Fields and analysis type. Press ''GO'' to process and display data.' ; % Default Message
G.GUI.messageColor   =  [.35  .27  .24 ]     ; % text color of displayed messages
% Settings Colormaps
q = 1 ;
G.Settings.ColorMap(q) = {'jet'}      ; q = q + 1 ;
G.Settings.ColorMap(q) = {'BlueWhiteRed'} ; q = q + 1 ;
G.Settings.ColorMap(q) = {'RYBceJulien'}    ; q = q + 1 ;
G.Settings.ColorMap(q) = {'OWBceJulien'}    ; q = q + 1 ;
G.Settings.ColorMap(q) = {'TWBceJulien'}    ; q = q + 1 ;
G.Settings.ColorMap(q) = {'OTBceJulien'}    ; q = q + 1 ;
G.Settings.ColorMap(q) = {'BluesceJulien'}  ; q = q + 1 ;
G.Settings.ColorMap(q) = {'ceJulien'} ; q = q + 1 ;
G.Settings.ColorMap(q) = {'hsv'}      ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Hot'}      ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Cool'}     ; q = q + 1 ;
G.Settings.ColorMap(q) = {'summer'}   ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Spring'}   ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Autumn'}   ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Winter'}   ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Gray'}     ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Bone'}     ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Copper'}   ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Pink'}     ; q = q + 1 ;
G.Settings.ColorMap(q) = {'Lines'}    ; q = q + 1 ;
G.Settings.ColorMaps.ceJulien     = cejulienColorMaps('ceJulien',23)     ;
G.Settings.ColorMaps.BlueWhiteRed = cejulienColorMaps('BlueWhiteRed',23) ;
G.Settings.ColorMaps.RYBceJulien = cejulienColorMaps('RYBceJulien',21) ;
G.Settings.ColorMaps.OWBceJulien = cejulienColorMaps('OWBceJulien',21) ;
G.Settings.ColorMaps.TWBceJulien = cejulienColorMaps('TWBceJulien',21) ;
G.Settings.ColorMaps.OTBceJulien = cejulienColorMaps('OTBceJulien',21) ;
G.Settings.ColorMaps.BluesceJulien = cejulienColorMaps('BluesceJulien',21) ;
G.Settings.ColorMap(q) = {'BrBG_div_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.BrBG_div_cbrewer = GOAT_cbrewer('div','BrBG', 21)     ;
G.Settings.ColorMap(q) = {'PiYG_div_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.PiYG_div_cbrewer = GOAT_cbrewer('div','PiYG', 21)     ;
G.Settings.ColorMap(q) = {'PRGn_div_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.PRGn_div_cbrewer = GOAT_cbrewer('div','PRGn', 21)     ;
G.Settings.ColorMap(q) = {'PuOr_div_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.PuOr_div_cbrewer = GOAT_cbrewer('div','PuOr', 21)     ;
G.Settings.ColorMap(q) = {'RdBu_div_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.RdBu_div_cbrewer = GOAT_cbrewer('div','RdBu', 21)     ;
G.Settings.ColorMap(q) = {'RdGy_div_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.RdGy_div_cbrewer = GOAT_cbrewer('div','RdGy', 21)     ;
G.Settings.ColorMap(q) = {'RdYlBu_div_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.RdYlBu_div_cbrewer = GOAT_cbrewer('div','RdYlBu', 21)     ;
G.Settings.ColorMap(q) = {'RdYlGn_div_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.RdYlGn_div_cbrewer = GOAT_cbrewer('div','RdYlGn', 21)     ;
G.Settings.ColorMap(q) = {'Blues_seq_cbrewer'}   ; q = q + 1 ;
G.Settings.ColorMaps.Blues_seq_cbrewer = GOAT_cbrewer('seq','Blues', 21)     ;
G.Settings.ColorMap(q) = {'BuGn_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.BuGn_seq_cbrewer = GOAT_cbrewer('seq','BuGn', 21)     ;
G.Settings.ColorMap(q) = {'BuPu_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.BuPu_seq_cbrewer = GOAT_cbrewer('seq','BuPu', 21)     ;
G.Settings.ColorMap(q) = {'GnBu_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.GnBu_seq_cbrewer = GOAT_cbrewer('seq','GnBu', 21)     ;
G.Settings.ColorMap(q) = {'Greens_seq_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.Greens_seq_cbrewer = GOAT_cbrewer('seq','Greens', 21)     ;
G.Settings.ColorMap(q) = {'Greys_seq_cbrewer'}   ; q = q + 1 ;
G.Settings.ColorMaps.Greys_seq_cbrewer = GOAT_cbrewer('seq','Greys', 21)     ;
G.Settings.ColorMap(q) = {'Oranges_seq_cbrewer'} ; q = q + 1 ;
G.Settings.ColorMaps.Oranges_seq_cbrewer = GOAT_cbrewer('seq','Oranges', 21)     ;
G.Settings.ColorMap(q) = {'OrRd_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.OrRd_seq_cbrewer = GOAT_cbrewer('seq','OrRd', 21)     ;
G.Settings.ColorMap(q) = {'PuBu_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.PuBu_seq_cbrewer = GOAT_cbrewer('seq','PuBu', 21)     ;
G.Settings.ColorMap(q) = {'PuBuGn_seq_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.PuBuGn_seq_cbrewer = GOAT_cbrewer('seq','PuBuGn', 21)     ;
G.Settings.ColorMap(q) = {'PuRd_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.PuRd_seq_cbrewer = GOAT_cbrewer('seq','PuRd', 21)     ;
G.Settings.ColorMap(q) = {'Purples_seq_cbrewer'} ; q = q + 1 ;
G.Settings.ColorMaps.Purples_seq_cbrewer = GOAT_cbrewer('seq','Purples', 21)     ;
G.Settings.ColorMap(q) = {'RdPu_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.RdPu_seq_cbrewer = GOAT_cbrewer('seq','RdPu', 21)     ;
G.Settings.ColorMap(q) = {'Reds_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.Reds_seq_cbrewer = GOAT_cbrewer('seq','Reds', 21)     ;
G.Settings.ColorMap(q) = {'YlGn_seq_cbrewer'}    ; q = q + 1 ;
G.Settings.ColorMaps.YlGn_seq_cbrewer = GOAT_cbrewer('seq','YlGn', 21)     ;
G.Settings.ColorMap(q) = {'YlGnBu_seq_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.YlGnBu_seq_cbrewer = GOAT_cbrewer('seq','YlGnBu', 21)     ;
G.Settings.ColorMap(q) = {'YlOrBr_seq_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.YlOrBr_seq_cbrewer = GOAT_cbrewer('seq','YlOrBr', 21)     ;
G.Settings.ColorMap(q) = {'YlOrRd_seq_cbrewer'}  ; q = q + 1 ;
G.Settings.ColorMaps.YlOrRd_seq_cbrewer = GOAT_cbrewer('seq','YlOrRd', 21)     ;
load(fullfile('Utils','colormaps.mat')) ;
%Maplist = who(matfile(fullfile('Utils','colormaps.mat'))) ;
for i = 1:length(Maplist)
try
if ~strcmpi(Maplist(i),'maplist')
eval(['G.Settings.ColorMap(q) = {''',cell2mat(Maplist(i)),'''} ; q = q + 1 ;']) ;
eval(['G.Settings.ColorMaps.',cell2mat(Maplist(i)),' = ',cell2mat(Maplist(i)),' ; ']) ;
end
catch
% Nevermind
end
end
G.Settings.ColorMapStr = '' ;
for i = 1: length(G.Settings.ColorMap)
G.Settings.ColorMapStr = [G.Settings.ColorMapStr,cell2mat(G.Settings.ColorMap(i)),' |'] ;
end
G.Settings.ColorMapStr = G.Settings.ColorMapStr(1:end-1) ;
% waitbar
G.Settings.waitbarPosition = [5,5,370,65] ;
% Defaults
G.Settings.Default.topoTypeFlag = G.topo.TypeFlag ;
G.Settings.Default.NanMeanFlag   = 0     ; % when 1, use nanmean(x) instead of mean(x)
G.Settings.Default.ContourIdx    = 1     ; %
G.Settings.Default.WaitbarCutoff = 10    ; % if the number of datafiles operations is more than this number, the progress message is displayed
G.Settings.Default.DisplayProgressCutoff = 3 ; % if the number of datafiles operations is more than this number, the progress message is displayed
G.Settings.breakSteps            = 5     ; % Number of loop iterations without checking if the break flag is up
G.Settings.MaxMatrixSize         = 1.e9  ; % Max allowed prod(size(x)) where x is a Matlab double precision array of elements (above this number, Matlab may crash or stall)
%G.Settings.Default.ColorMapIdx  = find(strcmp(G.Settings.ColorMap,'RdYlBu_11b'))  ; % RedYellowBlue
G.Settings.Default.ColorMapIdx  = find(strcmp(G.Settings.ColorMap,'RYB21'))  ; % RedYellowBlue
G.Settings.Default.LineWidthIdx = 1 ;
G.Settings.Default.LineWidth      = (G.Settings.Default.LineWidthIdx-2)/2 ;
G.Settings.Default.CoastLineWidthIdx = 1 ;   
G.Settings.Default.CoastLineWidth    = G.Settings.Default.CoastLineWidthIdx/2 -.5;
G.Settings.Default.XgridFlag      = 0   ; % A flag for displaying the X Grid Lines
G.Settings.Default.YgridFlag      = 0   ; % A flag for displaying the Y Grid Lines
G.Settings.Default.SymZLimitsFlag = 0   ; % A flag for setting limits symmetric with respect to zero (goes well with blue white red colormaps)
G.Settings.Default.FixZLimitsFlag = 0   ; % A flag for fixing limits (useful in auto time steps)
G.Settings.Default.LogLevelsFlag  = 0   ; % log(P) in lat-P,Lon-P projections
G.Settings.Default.ContourLevelStep = 0 ; % Manual Contour Step Level. 0 = Auto.
G.Settings.Default.ReverseColorMap = 1  ; % Reverse Colormap (1/0)
G.Settings.NanMeanFlag    = G.Settings.Default.NanMeanFlag   ; % use mean(x) function when 0 and nanmean(x) when 1 ;
G.Settings.ContourIdx     = G.Settings.Default.ContourIdx    ; % Contour Type Idx
G.Settings.WaitbarCutoff  = G.Settings.Default.WaitbarCutoff ; % Number of data loadings above which the waitbar appears
G.Settings.ColorMapIdx    = G.Settings.Default.ColorMapIdx   ; % Default ColorMap Index
G.Settings.LineWidthIdx   = G.Settings.Default.LineWidthIdx  ; % Default LineWidth Index in Plots
G.Settings.LineWidth      = G.Settings.Default.LineWidth     ;
G.Settings.CoastLineWidthIdx = G.Settings.Default.CoastLineWidthIdx   ; % Default LineWidth Index in Plots
G.Settings.CoastLineWidth    = G.Settings.Default.CoastLineWidth      ;
G.Settings.XgridFlag      = G.Settings.Default.XgridFlag        ; % A flag for displaying the X Grid Lines
G.Settings.YgridFlag      = G.Settings.Default.YgridFlag        ; % A flag for displaying the Y Grid Lines
G.Settings.SymZLimitsFlag = G.Settings.Default.SymZLimitsFlag   ; % A flag for setting limits symmetric with respect to zero (goes well with blue white red colormaps)
G.Settings.FixZLimitsFlag = G.Settings.Default.FixZLimitsFlag   ; % A flag for fixing limits (useful in auto time steps)
G.Settings.LogLevelsFlag  = G.Settings.Default.LogLevelsFlag    ; % log(P) in lat-P,Lon-P projections
G.Settings.SinLatFlag     = 0                                   ; % plot sin(latitude)
G.Settings.ContourLevelStep = G.Settings.Default.ContourLevelStep ; % Manual Contour Step Level. 0 = Auto.
G.Settings.ReverseColorMap = G.Settings.Default.ReverseColorMap ; % Reverse Colormap
G.Settings.ZlimitMax      = 1   ; % Initial Default Z limit max
G.Settings.ZlimitMin      = 0   ; % Initial Z limit min
G.Settings.ZlimitBuffer   = 0.1 ; % Z limit buffer in proportion to present axis.
G.Settings.HoldFlag       = 0   ; % hold plot checkbox
G.Settings.HoldDifFlag    = 0   ; % hold + Dif plot checkbox
G.Settings.functionLinks(1,1)      = {'GOAT_AutoDownload'}  ; % A list of default functions/GUIs that appear in the GOAT/Settings/Links dropdown menu
G.Settings.functionLinksTitle(1,1) = {'Auto Download Tool'} ; % The title fo the functions/GUIs that appear in the GOAT/Settings/Links dropdown menu
G.VerticalOrdinate        = 'Pressure (mb)' ; % Vertical Ordinate Default
G.Settings.LevelsIdx      = 1               ; % 1 = Pressure levels (mb) ; 2 = Depth (m) ; 3 = Height (m) ; 4 = Potential temperature (K)
G.Settings.ErrorSound     = 'goat'          ; % type or error sounds: 'goat','beep'
G.Settings.ExportGFlag    = 0               ; % A flag for saving the entire GOAT structure, G, instead of the data
G.Settings.KeyPressFlag   = 1               ; % Enable Key Press Functions

% Default Data Folders
G.DataFoldersAddress  = {fullfile('..','Data','')} ; % the use of fullfile allows making address strings that are compatible for all operating systems
% Personal Settings
try
G = GOAT_MyDefaults(G) ; % Personalized Settings, Locatated at the MyFunctions Folder
catch
disp 'GOAT_Mydefaults warning: an error occured while executing GOAT_Mydefaults.m' 
end
if isfield(G,'TextSize_XL')
G.Display.TextSize_XL = G.TextSize_XL ; % Compatibility with older GOAT versions
end
if isfield(G,'TextSize_L')
G.Display.TextSize_L  = G.TextSize_L ; % Compatibility with older GOAT versions
end
if isfield(G,'TextSize_M')
G.Display.TextSize_M  = G.TextSize_M ; % Compatibility with older GOAT versions
end
if isfield(G,'TextSize_S')
G.Display.TextSize_S  = G.TextSize_S ; % Compatibility with older GOAT versions
end
% Auto Time Step
G.TimeStep  = 1                ; % Day
G.TimeStepN = 1                ; % Number of automatic steps
G.Settings.TimeStepPause = .01 ; % Minimal TimeStep Pause(sec). Can Be edited in Settings
G.continueFlag = 0             ; % continue during main while loop
G.StepGUIFlag  = 0             ; % During Step mode this means GUIs are skipped
% Filter Defaults
G.Filter.WeightFactor       = 1 ; % a factor that multiplies any given field
G.Filter.SimpleLoadFlag     = 0 ; % When 1, loaded files are not spatially truncated when loaded
G.Filter.WeightIdx          = 1 ; % 1 = 'None'
G.Filter.ZonalWaveNumberLowIdx  = 1 ; % 0 = no Filter
G.Filter.ZonalWaveNumberHighIdx = 1 ; % 0 = no Filter
G.Filter.ZonalWaveNumberIdx     = - 1 ; % No Filter
G.Filter.ZEFlag             = 0 ; % 1 = Zonal Anomaly
G.Filter.RAIdx              = 1 ; % 0 = no Filter
G.Filter.REFlag             = 0 ; % 0 = no Filter
G.Filter.LowPassIdx         = 0 ; % 0 = no Filter
G.Filter.HighPassIdx        = 0 ; % 0 = no Filter
% Data Folders
G.DataFolders = cell(10,1) ; % Cell array of availabel data folders
G.DataFolderAddress = cell(1,1) ; % Cell array of availabel data folders
% All folders not named {'Export','WORK','ToolBoxes'} will be regarded as data folders
NF = 0 ;
for d = 1: length(G.DataFoldersAddress)
Goatdir = dir(cell2mat(G.DataFoldersAddress(d))) ;
for j = 1:length(Goatdir)
if  Goatdir(j,1).isdir && ...
~strncmp(Goatdir(j,1).name,'.',1) % && ... ~strcmpi(Goatdir(j,1).name,'Indices')
NF = NF + 1 ;
G.DataFolders(NF,1) = {Goatdir(j,1).name} ;
G.DataFolderAddress(NF,1) = G.DataFoldersAddress(d) ;
end
end
end
try
for d = 1: length(G.ModelFoldersAddress)
NF = NF + 1 ;
[FolderPath,FolderName] = fileparts(G.ModelFoldersAddress{d}) ; 
G.DataFolders(NF,1) = {FolderName} ;
G.DataFolderAddress(NF,1) = {FolderPath} ;
end
end
NF = NF + 1 ; G.DataFolders(NF,1) = {'Browse'} ;
if NF == 1 && ~G.EnvironmentFlag
disp 'Warning: could not find Data folder.' ;
disp 'Check G.DataFoldersAddress in GOAT_MyDefaults.m, located in the MyScripts folder' ;
disp 'Commenting G.DataFoldersAddress will set the default Data folder to ../Data' ;
end
% Create data folders string for popup menu
G.DataFolders = G.DataFolders(1:NF,1) ;
G.DataFoldersString = '' ;
for j = 1:NF
G.DataFoldersString = [G.DataFoldersString,cell2mat(G.DataFolders(j)),' |'] ;
end
G.DataFoldersString = G.DataFoldersString(1:end-1) ;
G.TimeFieldIdx = 1   ;
G.DataFolder = fullfile(cell2mat(G.DataFolderAddress(1)),cell2mat(G.DataFolders(1)),'') ;
G = GOAT_custom_field_settings(G) ; % Custom Display Settings
if ~G.EnvironmentFlag
if NF == 1
G.TempResStr       = 'None' ; 
G.DataFieldsString = 'None' ; 
G.TimeStepStr      = 'None' ;
G.x = [0 360] ; G.y = [-90 90] ; 
else
G = GOAT_Initialize_Dataset(G) ; % reviews the data folder
end
% Scripts
G.Scripts.ScriptsIdx = 1 ;
G.Scripts.Scripts = cell(100,1) ; % Cell array of available Scripts (aka Toolboxes)
G.Scripts.ScriptFileNames = cell(100,1) ; % Cell array of available Script filenames
NF = 0 ;
G.Scripts.ScriptsDir = dir(fullfile('..','Scripts','')) ;
for j = 1:length(G.Scripts.ScriptsDir)
[Filepath,FileName,FileExt] = fileparts(G.Scripts.ScriptsDir(j,1).name) ;
if  strcmp(FileExt,'.m') && ~strncmp(FileName,'.',1)
NF = NF + 1 ;
if strncmpi(FileName,'GOAT_Script_',12)
G.Scripts.Scripts(NF,1) = {FileName(13:end)} ;
else
G.Scripts.Scripts(NF,1) = {FileName} ;
end
G.Scripts.ScriptFileNames(NF,1) = {FileName} ;
end
end
% Create ToolBoxes/Scripts string for popup menu
G.Scripts.Scripts = G.Scripts.Scripts(1:NF,1) ;
G.Scripts.ScriptsString = 'None | ' ;
for j = 1:NF
G.Scripts.ScriptsString = [G.Scripts.ScriptsString,cell2mat(G.Scripts.Scripts(j)),' |'] ;
end
G.Scripts.ScriptsString = G.Scripts.ScriptsString(1:end-1) ;
end
% Auto Pilot (QA auto pilot routine)
try
if strcmpi(G.AutoPilot.Mode,'on')
G.AutoPilot.Flag = 1 ;
G.AutoPilot.ActionNumber = 1 ;
[fid,fidf] = uigetfile ;
eval(fid(1:end-2)) ;
G.AutoPilot.Actions = Actions ;
G.AutoPilot.NumberOfActions = size(Actions,2) ;
else
G.AutoPilot.Flag = 0 ;
end
catch
G.AutoPilot.Flag = 0 ;
end

