% GOAT_GetDims Get spatial info from the Dims file of a field.
% syntax: assumes all information is included in the GOAT structure G
% >>   G = GOAT_GetDims(G) ; 
% Copyright (C) Ori Adam Feb.2012. 
%
% see also GOAT_GetFieldSpan, GOAT_NewFieldLimits

function G = GOAT_GetDims(G)
try
    if ~G.DataFieldsType(find(strcmp(G.DataField,G.DataFields))) % In case a real field overrides a virtual field
        DataField = G.DataField ;
    else
        switch G.DataField
            case G.VirtualFields.List
                G = GOAT_VirtualFields(G,'ReferenceField',G.DataField) ;
                DataField = G.VirtualFields.ReferenceField ;
            case G.VirtualFields.MyList
                Idx = find(strcmp(G.DataField,G.VirtualFields.MyList)) ;
                eval(['G = ',G.VirtualFields.MyFuncs{Idx},'(''ReferenceField'',G) ; ']) ;
                DataField = G.VirtualFields.ReferenceField ;
            otherwise
                DataField = G.DataField ;
        end
    end
    try
        try
            NoOld = 0 ; 
            G.levOld =  G.lev ;
            G.latOld =  G.lat ;
            G.lonOld =  G.lon ;
            G.latlabelOld =  G.latlabel ;
            G.lonlabelOld =  G.lonlabel ;
            G.levlabelOld =  G.levlabel   ;
        catch % No Old Dims Values
            NoOld = 1 ; 
        end
        load(fullfile(G.DataFolder,G.TimeField,DataField,'Dims.mat')) ;
        if exist('lev','var')
            G.lev = lev ;
            G.P   = lev ; % Compatibility with older GOAT versions
        elseif exist('P','var')
            G.lev = P ;
            G.P   = P ; % Compatibility with older GOAT versions
        else % No vertical (3rd) dimension in Dims
            G.lev = 1 ;
            G.P   = 1 ; % Compatibility with older GOAT versions
        end
        if exist('lat','var')
            G.lat = lat ;
        else
            G.lat = 0 ;
        end
        if exist('lon','var')
            G.lon = lon ;
        else
            G.lon = 0 ;
        end
        if exist('latlabel','var')
           G.latlabel = latlabel ;
        else
           G.latlabel = 'Latitude' ;
        end
        if exist('lonlabel','var')
           G.lonlabel = lonlabel ;
        else
           G.lonlabel = 'Longitude' ;
        end
        if exist('levlabel','var')
           G.levlabel = levlabel ;
           G.Plabel   = levlabel ; % Compatibility with older GOAT versions
        elseif exist('Plabel','var')
           G.levlabel = Plabel ;
           G.Plabel   = Plabel ; % Compatibility with older GOAT versions 
        else
           G.levlabel = 'Level' ;
           G.Plabel   = 'Level' ; % Compatibility with older GOAT versions  
        end
        if exist('Units','var')
        G.DataFieldUnits = Units ;
        else
        G.DataFieldUnits = 'Unknown Units' ;
        end
        if exist('FieldDescription','var')
            G.FieldDescription = FieldDescription ;
        else
            G.FieldDescription = 'Unknown Description' ;
        end
        if NoOld
            G.levOld =  G.lev ;
            G.latOld =  G.lat ;
            G.lonOld =  G.lon ;
            G.latlabelOld =  G.latlabel ;
            G.lonlabelOld =  G.lonlabel ;
            G.levlabelOld =  G.levlabel   ;
        end
    catch
        disp 'Error loading Dims.mat at GOAT_GetDims.m. Dimensions may not be properly updated.'
    end
    if any(size(G.lon)==1)
        [G.LON,G.LAT] = meshgrid(G.lon,G.lat) ;
        [G.LEVLAT,G.LEV] = meshgrid(G.lat,G.lev) ;
        G.PLAT = G.LEVLAT ; G.PP = G.LEV  ; % Compatibility with older GOAT versions
    else
        disp 'Warning: lat and lon must be 1 dimensional vectors'
        G.LON = G.lon ;  G.LAT = G.lat ;
    end
    
    try
    if exist(fullfile(G.DataFolder,G.TimeField,DataField,'Topography.mat'),'file')
        G = GOAT_GetTopography(G,fullfile(G.DataFolder,G.TimeField,DataField,'Topography.mat')) ;
    else
        G = GOAT_GetTopography(G) ;
    end
    catch
        % ignore topography. Field and topography dimensions are inconsistent 
        disp (['GOAT_GetDims Warning: error while attempting to set topography for field: ',DataField]);
    end
catch
    disp (['GOAT_GetDims Error while attempting to load: ',fullfile(G.DataFolder,G.TimeField,DataField,'Dims.mat')]);
end
if sum(strcmpi(G.VirtualFields.List,G.DataField))
    G = GOAT_VirtualFields(G,'Dims',G.DataField) ;
    G.DataFieldUnits   = G.VirtualFields.DataFieldUnits   ;
    G.FieldDescription = G.VirtualFields.FieldDescription ;
elseif sum(strcmpi(G.VirtualFields.MyList,G.DataField))
    eval(['G = ',G.VirtualFields.MyFuncs{Idx},'(''Dims'',G) ; ']) ;
    G.DataFieldUnits   = G.VirtualFields.DataFieldUnits   ;
    G.FieldDescription = G.VirtualFields.FieldDescription ;
end

try
    switch VerticalOrdinate
        case 'Pressure (mb)'
            G.Settings.LevelsIdx = 1 ;
            G.VerticalOrdinate = VerticalOrdinate  ;
        case 'Depth (m)'
            G.Settings.LevelsIdx = 2 ;
            G.VerticalOrdinate = VerticalOrdinate  ;
        case 'Height (m)'
            G.Settings.LevelsIdx = 3 ;
            G.VerticalOrdinate = VerticalOrdinate  ;
        case 'Potential Temperature (K)'
            G.Settings.LevelsIdx = 4 ;
            G.VerticalOrdinate = VerticalOrdinate  ;
        case 'Sigma'
            G.Settings.LevelsIdx = 5 ;
            G.VerticalOrdinate = VerticalOrdinate  ;
        case 'Geometric Level'
            G.Settings.LevelsIdx = 6 ;
            G.VerticalOrdinate = VerticalOrdinate  ;    
        case 'Level'
            G.Settings.LevelsIdx = 7 ;
            G.VerticalOrdinate = VerticalOrdinate  ;      
    end
catch % Default
    G.Settings.LevelsIdx = 1 ;
    G.VerticalOrdinate = 'Pressure (mb)' ;
end


% Pressure levels
G.Limits.Levels = '' ;
for i = 1:length(G.lev)
    if G.lev(i) >= 1
        G.Limits.Levels = [G.Limits.Levels,int2str(G.lev(i)),' |'] ;
    else
        G.Limits.Levels = [G.Limits.Levels,num2str(G.lev(i),2),' |'] ;
    end
end
G.Limits.Levels = G.Limits.Levels(1:end-1);
G.Limits.Plevels = G.Limits.Levels ; % Compatibility with older GOAT versions
% Latitudes
G.Limits.Latitudes = '' ;
for i = 1:length(G.lat)
    G.Limits.Latitudes = [G.Limits.Latitudes,num2str(G.lat(i),3),' |'] ;
end
G.Limits.Latitudes = G.Limits.Latitudes(1:end-1);
% Longitudes
G.Limits.Longitudes = '' ;
for i = 1:length(G.lon)
    G.Limits.Longitudes = [G.Limits.Longitudes,num2str(G.lon(i),4),' |'] ;
end
G.Limits.Longitudes = G.Limits.Longitudes(1:end-1);
if (length(G.lon)*length(G.lat)*length(G.lev) < 1.e8) % Do not make repmat G.Vectors for data matrices that are too big
    G.Vectors.latmat  = repmat((G.lat)',[length(G.lon) 1 length(G.lev)]) ;
    G.Vectors.cosmat  = repmat((cosd(G.lat)')   ,[length(G.lon) 1 length(G.lev)]) ;
    G.Vectors.cos2mat = repmat((cosd(G.lat)').^2,[length(G.lon) 1 length(G.lev)]) ;
    G.Vectors.sinmat  = repmat((sind(G.lat)')   ,[length(G.lon) 1 length(G.lev)]) ;
    G.Vectors.sin2mat = repmat((sind(G.lat)').^2,[length(G.lon) 1 length(G.lev)]) ;
    G.Vectors.levmat  = repmat(reshape(single(G.lev),1,1,length(G.lev)),[length(G.lon) length(G.lat) 1])    ;
    G.Vectors.Pmat    = G.Vectors.levmat ; % Compatibility with older GOAT versions
    G.Vectors.fmat    = G.Vectors.sinmat.*G.Consts.Omega*2 ;
%     % half levels for vertical integration (original scheme by Tapio Schneider, 2014)
%     if length(G.lev) == 1
%         G.Vectors.lev_half = 1 ;
%     else
%         half_levels = zeros(length(G.lev)+1, 1);
%         if G.lev(end) > G.lev(1)
%             half_levels(end) = G.lev(end) ;  % lowest half level is ground
%             for j = length(G.lev) : -1 : 1
%                 half_levels(j) = 2*G.lev(j) - half_levels(j+1);
%             end
%             G.Vectors.lev_half     = half_levels(1:end-1);
%             G.Vectors.lev_half_dif = diff(half_levels) ;
%             G.Vectors.lev_half_mat = repmat(reshape(single(G.Vectors.lev_half),1,1,length(G.lev)),[length(G.lon) length(G.lat) 1])    ;
%             G.Vectors.lev_half_diff_mat = repmat(reshape(single(G.Vectors.lev_half_dif),1,1,length(G.lev)),[length(G.lon) length(G.lat) 1])    ;
%         else
%             half_levels(1) = G.lev(1) ;  % lowest half level is ground
%             for j = 1:length(G.lev)
%                 half_levels(j+1) = 2*G.lev(j) - half_levels(j);
%             end
%             G.Vectors.lev_half     = half_levels(2:end);
%             G.Vectors.lev_half_dif = diff(half_levels) ;
%             G.Vectors.lev_half_mat = repmat(reshape(single(G.Vectors.lev_half),1,1,length(G.lev)),[length(G.lon) length(G.lat) 1])    ;
%             G.Vectors.lev_half_diff_mat = repmat(reshape(single(G.Vectors.lev_half_dif),1,1,length(G.lev)),[length(G.lon) length(G.lat) 1])    ;
%         end
%     end
else
    disp 'Warning: dim(x*y*z) > 1.e8. G.Vectors was not created.' ;
end