% GOAT_GetTopography: load topography and land cover
% syntax:
% >>   G = GOAT_GetTopography(G,TopographyFileAddress) ;
% default Topography.mat file sits at GOAT/Code.
% GOAT will use the Topography file lowest down in the hierarchy of folders:
% if      exist(Dataset/Calendar/Field/YearOrLinear/Topography.mat)
% elseif  exist(Dataset/Calendar/Field/Topography.mat)  % Alongside Dims.mat
% else    load GOAT/Code/Topography.mat
% In Year or List folders, GOAT does not consider the files info.mat and Topography.mat as data

function G = GOAT_GetTopography(G,Address)
% global
%G.topo.ShadeColor  = [.5 .5 .5]          ; % Land Surfaces Shading Color
G.topo.alpha       = .4                  ; % visibility of land areas
G.topo.Lakealpha   = .4                  ; % visibility of land areas
G.topo.CoastView   = 'LandArea'          ; % 'LandArea' | 'CoastLine'
G.topo.TopotLineWidthIdx = 2             ; % '0.5 | 1 | 1.5 | 2 | 2.5 | 3 | 3.5 | 4 | 4.5 | 5 | 5.5 | 6 | 6.5 | 7 | 7.5 | 8 | 8.5 | 9 | 9.5 | 10 '
Default_Topography_Flag = 1 ;
G.topo.LakesIdx       = []               ; % Default CoastNans Indices of lakes
G.topo.AntLatAppendix = []               ; % Default. Required appendix to close surface for Southern Pole
G.topo.AntLonAppendix = []               ; % Default. Required appendix to close surface for Southern Pole
G.topo.TypeFlag       = 1                ; % Default. 1) None 2) Atmo 3) Ocean 4) All
G.topo.interval       = 1000             ; % Default. topography contour intervals
try
    load(Address) ;
    Default_Topography_Flag = 0 ;
catch
    try
        load  Topography
    catch
        load(fullfile(G.GOAT_Directory,'Topography.mat')) ;
    end
end
if  exist('CoastView','var')
    G.topo.CoastView   = CoastView          ; % 'LandArea' | 'CoastLine' | 'Mask'
end
if  exist('ocean_mask','var')
    %     G.topo.ocean_mask   = fliplr(ocean_mask)          ; % 'LandArea' | 'CoastLine' | 'Mask'
    %     G.topo.maskRI = imref2d(size(ocean_mask'));
    %     G.topo.maskRI.XWorldLimits = [min(topolon) max(topolon)] ;
    %     G.topo.maskRI.XWorldLimits = [min(topolat) max(topolat)] ;
    if strcmpi(G.topo.CoastView,'Mask')
        [long,latg,LakesIdx] = GOAT_FMS_ocean_mask2vec(ocean_mask,topolon,topolat) ;
    end
end
G.topo.landcoverlat = latg ;
G.topo.landcoverlon = long ;
G.topo.CoastNans = find(isnan(G.topo.landcoverlon)) ; % locate breaking points between surfaces
if isempty(G.topo.CoastNans)
    G.topo.CoastNans = length(G.topo.landcoverlon) + 1 ;
end
if Default_Topography_Flag
    G.topo.LakesIdx  = [13:22,94:106]        ; % CoastNans Indices of lakes
else
    try
        G.topo.LakesIdx  = LakesIdx          ; % CoastNans Indices of lakes
    end
end
if Default_Topography_Flag
    G.topo.AntLatAppendix = [-90 ; -90 ; -83.83] ; % Required appendix to close surface for Southern Pole
    G.topo.AntLonAppendix = [180 ; -180; -180  ] ; % Required appendix to close surface for Southern Pole
else
    try
        G.topo.AntLatAppendix = AntLatAppendix ; % Required appendix to close surface for Southern Pole
        G.topo.AntLonAppendix = AntLonAppendix ; % Required appendix to close surface for Southern Pole
    end
end

if ~exist('topolat','var')
    topolat      = [-89.5 : 89.5 ]                ; % CREATE LAT ARRAY FOR TOPO MATRIX
end
if ~exist('topolon','var')
    topolon      = [0.5   : 359.5]                ; % CREATE LON ARRAY FOR TOPO MATRIX
end
G.topo.lonG  = topolon ; %lonG                           ;
G.topo.latG  = topolat ; %latG                           ;
G.topo.topoG = topo ; %topoG                          ;
G.topo.topoA = topo ;  G.topo.topoA(G.topo.topoA<0) = 0 ; %topoG   ;
G.topo.topoO = topo ;  G.topo.topoO(G.topo.topoO>0) = 0 ; %topoG
if exist('topoInterval','var')
    if ~isempty(topoInterval)
        G.topo.interval = topoInterval ;
    end
end
if exist('topoType','var')
    if ~isempty(topoType)
        G.topo.TypeFlag = topoType    ; % 1) None 2) Atmo 3) Ocean 4) All
    end
end
switch G.topo.TypeFlag
    case {1,2}
        if exist('topoIntervalMin','var')
            if ~isempty(topoIntervalMin)
                G.topo.intervalmin = topoIntervalMin  ;
            else
                G.topo.intervalmin = 0     ; % Default
            end
        else
            G.topo.intervalmin = 0     ; % Default
        end
        if exist('topoIntervalMax','var')
            if ~isempty(topoIntervalMax)
                G.topo.intervalmax = topoIntervalMax  ;
            else
                G.topo.intervalmax = 8000  ;  % Default
            end
        else
            G.topo.intervalmax = 8000  ;  % Default
        end
    case 3
        if exist('topoIntervalMin','var')
            if ~isempty(topoIntervalMin)
                G.topo.intervalmin = topoIntervalMin ;
            else
                G.topo.intervalmin = -8000 ;
            end
        else
            G.topo.intervalmin = -8000 ;
        end
        if exist('topoIntervalMax','var')
            if ~isempty(topoIntervalMax)
                G.topo.intervalmax =  topoIntervalMax    ;
            else
                G.topo.intervalmax =  0    ;
            end
        else
            G.topo.intervalmax =  0    ;
        end
    case 4
        if exist('topoIntervalMin','var')
            if ~isempty(topoIntervalMin)
                G.topo.intervalmin = topoIntervalMin ;
            else
                G.topo.intervalmin = -8000 ;
            end
        else
            G.topo.intervalmin = -8000 ;
        end
        if exist('topoIntervalMax','var')
            if ~isempty(topoIntervalMax)
                G.topo.intervalmax = topoIntervalMax  ;
            else
                G.topo.intervalmax = 8000  ;
            end
        else
            G.topo.intervalmax = 8000  ;
        end
end
G.topo.topovec = G.topo.intervalmin:G.topo.interval:G.topo.intervalmax ;
try
    zeroidx = dsearchn(G.topo.topovec',0) ;
    G.topo.topovec = G.topo.topovec(G.topo.topovec~=G.topo.topovec(zeroidx))  ;
catch
    G.topo.topovec = 0 ;
end

