% GOAT_INFO_LIMITS : show and describe current LIMITS parameters
% syntax: assumes all information is contained in the GOAT structure G
% >>   GOAT_INFO_LIMITS(G)
% copyright (C) Ori Adam, May.2014
%
% see also, GOAT_INFO_PLOT

function GOAT_INFO_LIMITS(G)
%%
try
    separatorline = '----------------------------------------------------------------------------------------------------' ;
    headerline    = '====================================================================================================' ;
    %endline       = '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~' ;
    %endline       = '/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\' ;
    endline       = '(((((((((((((((((((((((((((((((((((((((((((((((((())))))))))))))))))))))))))))))))))))))))))))))))))' ;
    B             = '                                                                                                    ' ;
    L             = '|| ' ;
    Lnote         = '|--->   ' ;
    R             = '                                                                                                  ||' ;
    FlagsHeader   = '|| FLAGS:                                                                                         ||' ;
    ListHeader    = '|| List                                                                                           ||' ;
    Date1         = '|| Date 1        ' ;
    Date2         = '|| Date 2        ' ;
    Y             = '|| Year                                                                                           ||' ;
    YM            = '|| Year                   | Month                                                                 ||' ;
    YMD           = '|| Year                   | Month                  | Day                                          ||' ;
    YMDH          = '|| Year                   | Month                  | Day                    | Hour                ||' ;
    YD            = '|| Year                                            | Day                                          ||' ;
    YDH           = '|| Year                                            | Day                    | Hour                ||' ;
    Spatial1      = '|| Spatial 1                                                                                      ||' ;
    Spatial2      = '|| Spatial 2                                                                                      ||' ;
    levellonlat   = '|| Level                         | Longitude                     | Latitude                       ||' ;
    Date2Header1  = '|| [x] Same as Date 1 Flag (Date2 == Date1 [G.Limits.Default.Day2same = 1])                       ||' ;
    Date2Header0  = '|| [ ] Same as Date 1 Flag (Date2 ~= Date1 [G.Limits.Default.Day2same = 0])                       ||' ;
    Col1Idx       = regexp(YMDH,'M','once') ;
    Col2Idx       = regexp(YMDH,'D','once') ;
    Col3Idx       = regexp(YMDH,'H','once') ;
    Sol1Idx       = regexp(levellonlat,'Longitude','once') ;
    Sol2Idx       = regexp(levellonlat,'Latitude','once') ;
    
    % Projection Info
    G = GOAT_setProjectionStr(G) ;
    b =regexp(G.GUI.ProjectionStr,'[|]') ;
    b = [0 , b , length(G.GUI.ProjectionStr) + 1] ;
    bs = cell(length(b)-1,1) ;
    for i = 1:length(b)-1
        bs(i) = {G.GUI.ProjectionStr(b(i)+1:b(i+1)-1)} ;
    end
    % Calendar Info
    [YYY,X] = GOAT_ParseTimeResolution(G.TimeFieldVirtual) ;
    if strcmpi(G.TimeFieldVirtual,'Monthly')
        TimeHeader = YM ; TimeType = 'YM' ;
    else
        if strncmpi(G.TimeFieldVirtual,'Daily',5)
            if X>1
                TimeHeader = YMDH ; TimeType = 'YMDH' ;
            else
                TimeHeader = YMD  ; TimeType = 'YMD'  ;
            end
        else % Julian
            if X>1
                TimeHeader = YDH  ; TimeType = 'YDH'  ;
            else
                TimeHeader = YD   ; TimeType = 'YD'   ;
            end
        end
    end
    
    % Begin Display
    disp '  ' ;
    cprintf('text' ,'(((((((((((((((((((((((((((((((((((   ');
    cprintf('*text','GOAT INFO LIMITS STATUS ');
    cprintf('text' ,'   )))))))))))))))))))))))))))))))))))\n');
    disp(headerline) ;
    % Calendar
    LS = [L,'Calendar = ''',G.TimeField,'''  [G.TimeField = ''',G.TimeField,''']'] ;
    disp([LS,R(length(LS)+1:end)]) ;
    if ~strcmp(G.TimeField,G.TimeFieldVirtual)
        LS = [L,'Virtual Calendar = ''',G.TimeFieldVirtual,'''  [G.TimeFieldVirtual = ''',G.TimeFieldVirtual,''']'] ;
        disp([LS,R(length(LS)+1:end)]) ;
    end
    disp(headerline) ;
    % Projection
    LS = [L,'Projection = ',bs{G.ProjectionType},'  [G.ProjectionType = ',int2str(G.ProjectionType),']'] ;
    disp([LS,R(length(LS)+1:end)]) ;
    disp(headerline) ;
    
    if ~strcmp(G.TimeField,'List')
        % Date 1
        if isfield(G.Limits,'S1')
            LS = [Date1,'[G.Limits.S1 = ',int2str(G.Limits.S1),']',] ;
            disp([LS,R(length(LS)+1:end)]) ;
        else
            LS = [Date1,'[G.Limits.S1 = Unknown]',] ;
            cprintf('Errors' ,[LS,R(length(LS)+1:end),'\n']) ;
            cprintf('Blue',[Lnote,'First temporal index ''G.Limits.S1'' has not been set\n']) ;
            cprintf('Blue',[Lnote,'Try ''G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ;''\n']) ;
        end
        disp(separatorline) ;
        disp(TimeHeader) ;
        % Year1
        if isfield(G.Limits,'Year0')
            LS = [L,'[G.Limits.Year0 = ',int2str(G.Limits.Year0),']'] ;
            disp([LS,R(length(LS)+1:end)]) ;
        else
            cprintf('Blue',[Lnote,'First year of field temporal span, ''G.Limits.Year0'' has not been set\n']) ;
        end
        if isfield(G.Limits,'S1')
            
            try
                [year,month,day,hour] = GOAT_serial2date(G.Addresses.Default,G.Limits.S1) ;
                YS = ['year1 = ',int2str(year)] ;
                if regexp(TimeType,'M','once')
                    MS = ['month1 = ',int2str(month)] ;
                else
                    MS = ' ' ;
                end
                if regexp(TimeType,'D','once')
                    DS = ['day1 = ',int2str(day)] ;
                else
                    DS = ' ' ;
                end
                if regexp(TimeType,'H','once')
                    HS = ['hour1 = ',int2str(hour)] ;
                else
                    HS = ' ' ;
                end
                LS = [L,YS,B(1:Col1Idx-1-length([L,YS])),MS,B(1:Col2Idx-Col1Idx-length(MS)),DS,B(1:Col3Idx-Col2Idx-length(DS)),HS,R(Col3Idx+length(HS):end)] ;
                disp(LS) ;
            catch
                LS = [Lnote,'The address cell array ''G.Addresses,Default'' does not exist. Time series cannot be calculated.'] ;
                cprintf('Errors' ,[LS,R(length(LS)+1:end),'\n']) ;
            end
        end
        % Date 2
        disp(separatorline) ;
        if isfield(G.Limits,'S2')
            LS = [Date2,'[G.Limits.S2 = ',int2str(G.Limits.S2),']',] ;
            disp([LS,R(length(LS)+1:end)]) ;
        else
            LS = [Date2,'[G.Limits.S2 = Unknown]',] ;
            cprintf('Errors' ,[LS,R(length(LS)+1:end),'\n']) ;
            cprintf('Blue',[Lnote,'Last temporal index ''G.Limits.S2'' has not been set\n']) ;
            cprintf('Blue',[Lnote,'Try ''G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ;''\n']) ;
        end
        disp(separatorline) ;
        %disp(TimeHeader) ;
        % Year2
        if isfield(G.Limits,'S2')
            [year,month,day,hour] = GOAT_serial2date(G.Addresses.Default,G.Limits.S2) ;
            YS = ['year2 = ',int2str(year)] ;
            if regexp(TimeType,'M','once')
                MS = ['month2 = ',int2str(month)] ;
            else
                MS = ' ' ;
            end
            if regexp(TimeType,'D','once')
                DS = ['day2 = ',int2str(day)] ;
            else
                DS = ' ' ;
            end
            if regexp(TimeType,'H','once')
                HS = ['hour2 = ',int2str(hour)] ;
            else
                HS = ' ' ;
            end
            LS = [L,YS,B(1:Col1Idx-1-length([L,YS])),MS,B(1:Col2Idx-Col1Idx-length(MS)),DS,B(1:Col3Idx-Col2Idx-length(DS)),HS,R(Col3Idx+length(HS):end)] ;
            disp(LS) ;
        end
        
        if  ~any(G.ProjectionType == [5,6,7])
            disp(separatorline) ;
            if G.Limits.Default.Day2same
                disp(Date2Header1) ;
            else
                disp(Date2Header0) ;
            end
        end
        disp(headerline) ;
        disp(FlagsHeader) ;
        if G.Limits.Default.Sequential
            LS = [L,'[x] Sequential [G.Limits.Default.Sequential = 1]'] ;
        else
            LS = [L,'[x] Climatology [G.Limits.Default.Sequential = 0]'] ;
        end
        disp([LS,R(length(LS)+1:end)]) ;
        if any(G.ProjectionType == [5,6,7])
            if G.Limits.Default.Anom
                LS = [L,'[x] Anomaly [G.Limits.Default.Anom = 1]'] ;
            else
                LS = [L,'[ ] Anomaly [G.Limits.Default.Anom = 0]'] ;
            end
            disp([LS,R(length(LS)+1:end)]) ;
            if G.Limits.Default.ClimatologyContinuous
                LS = [L,'[x] Clim. Continuous [G.Limits.Default.ClimatologyContinuous = 1]'] ;
            else
                LS = [L,'[ ] Clim. Continuous [G.Limits.Default.ClimatologyContinuous = 0]'] ;
            end
            disp([LS,R(length(LS)+1:end)]) ;
        end
        
    else % List Calendar
        disp(ListHeader) ;
        disp(separatorline) ;
        LS = [L,'List Folder = ',G.Addresses.Linears{G.Addresses.LinearIdx}] ;
        disp([LS,R(length(LS)+1:end)]) ;
        LS = [L,'[G.Addresses.LinearIdx = ',int2str(G.Addresses.LinearIdx),']'] ;
        disp([LS,R(length(LS)+1:end)]) ;
        LS = [L,'Item 1 = ',G.Addresses.Default{G.Limits.S1},'  [G.Limits.S1 = ',int2str(G.Limits.S1),']' ] ;
        disp([LS,R(length(LS)+1:end)]) ;
        LS = [L,'Item 2 = ',G.Addresses.Default{G.Limits.S2},'  [G.Limits.S2 = ',int2str(G.Limits.S2),']' ] ;
        disp([LS,R(length(LS)+1:end)]) ;
    end
    disp(headerline)
    if G.ProjectionType == 10
        LS = [L,'Full original spatial span of linked fields is installed.'] ;
        disp([LS,R(length(LS)+1:end)]) ;
    else
        disp(Spatial1)
        disp(separatorline)
        disp(levellonlat) ;
        PS    = ['level1 = ',num2str(G.lev(G.Limits.Default.Lev1),5)] ;
        PSI   = ['[G.Limits.Default.Lev1 = ',int2str(G.Limits.Default.Lev1),']'] ;
        LonS  = ['Lon1 = ',num2str(G.lon(G.Limits.Default.Lon1),5)] ;
        LonSI = ['[G.Limits.Default.Lon1 = ',int2str(G.Limits.Default.Lon1),']'] ;
        LatS  = ['Lat1 = ',num2str(G.lat(G.Limits.Default.Lat1),5)] ;
        LatSI = ['[G.Limits.Default.Lat1 = ',int2str(G.Limits.Default.Lat1),']'] ;
        LS = [L,PS,B(1:Sol1Idx-1-length([L,PS])),LonS,B(1:Sol2Idx-Sol1Idx-length(LonS)),LatS,B(1:length(B)-1-Sol2Idx-length(LatS))] ;
        disp([LS,R(length(LS)+1:end)]) ;
        LS = [L,PSI,B(1:Sol1Idx-1-length([L,PSI])),LonSI,B(1:Sol2Idx-Sol1Idx-length(LonSI)),LatSI,B(1:length(B)-1-Sol2Idx-length(LatSI))] ;
        disp([LS,R(length(LS)+1:end)]) ;
        disp(separatorline)
        disp(Spatial2)
        disp(separatorline)
        PS    = ['level2 = ',num2str(G.lev(G.Limits.Default.Lev2),5)] ;
        PSI   = ['[G.Limits.Default.Lev2 = ',int2str(G.Limits.Default.Lev2),']'] ;
        LonS  = ['Lon2 = ',num2str(G.lon(G.Limits.Default.Lon2),5)] ;
        LonSI = ['[G.Limits.Default.Lon2 = ',int2str(G.Limits.Default.Lon2),']'] ;
        LatS  = ['Lat2 = ',num2str(G.lat(G.Limits.Default.Lat2),5)] ;
        LatSI = ['[G.Limits.Default.Lat2 = ',int2str(G.Limits.Default.Lat2),']'] ;
        LS = [L,PS,B(1:Sol1Idx-1-length([L,PS])),LonS,B(1:Sol2Idx-Sol1Idx-length(LonS)),LatS,B(1:length(B)-1-Sol2Idx-length(LatS))] ;
        disp([LS,R(length(LS)+1:end)]) ;
        LS = [L,PSI,B(1:Sol1Idx-1-length([L,PSI])),LonSI,B(1:Sol2Idx-Sol1Idx-length(LonSI)),LatSI,B(1:length(B)-1-Sol2Idx-length(LatSI))] ;
        disp([LS,R(length(LS)+1:end)]) ;
        disp(separatorline)
        switch G.ProjectionType
            case {1,2,8,9}
                if G.Limits.Default.sameP
                    SamePHeader = '[x] Level 1 Only (Level1 == Level2 [G.Limits.Default.sameP = 1])' ;
                else
                    SamePHeader = '[ ] Level 1 Only (Level1 ~= Level2 [G.Limits.Default.sameP = 0])' ;
                end
                disp([L,SamePHeader,R(length([L,SamePHeader])+1:end)]) ;
            case {3,4,7}
                if G.Limits.Default.sameP
                    SamePHeader = '[x] Composite levels (according to Ver. Composite [G.Limits.Default.sameP = 1])' ;
                else
                    SamePHeader = '[ ] Composite levels (according to Ver. Composite [G.Limits.Default.sameP = 0])' ;
                end
                disp([L,SamePHeader,R(length([L,SamePHeader])+1:end)]) ;
        end
        disp(separatorline)
        switch G.VerticalOrdinate
            case {'Pressure (mb)','Depth (m)'}
                switch G.Limits.Default.VerMeanInt
                    case 1
                        VerMeanInt = 'Vertical Composite = Vertical Mean ([G.Limits.Default.VerMeanInt = 1])' ;
                    case 2
                        VerMeanInt = 'Vertical Composite = Mass weighted Vertical Integral Boer ([G.Limits.Default.VerMeanInt = 2])' ;
                    case 3
                        VerMeanInt = 'Vertical Composite = Mass Weighted Vertical Integral Nan ([G.Limits.Default.VerMeanInt = 3])' ;
                end
            otherwise
                switch G.Limits.Default.VerMeanInt
                    case 1
                        VerMeanInt = 'Vertical Composite Type = Vertical Mean ([G.Limits.Default.VerMeanInt = 1])' ;
                    case 2
                        VerMeanInt = 'Vertical Composite Type = Vertical Integral Boer ([G.Limits.Default.VerMeanInt = 2])' ;
                    case 3
                        VerMeanInt = 'Vertical Composite Type = Vertical Integral Nan ([G.Limits.Default.VerMeanInt = 3])' ;
                end
        end
        disp([L,VerMeanInt,R(length([L,VerMeanInt])+1:end)]) ;
        
    end
    disp(headerline)
    % try
    %     StringVec = GOAT_Data_string_vec(G) ;
    % catch
    %     cprintf('Errors',[Lnote,'Cannot determine addresses for retrieval [error while running ''string_vec = GOAT_Data_string_vec(G)''\n']) ;
    %     cprintf('Blue',[Lnote,'The following fields may be incorrect or need to be set:\n']) ;
    %     cprintf('Blue',[Lnote,'G.TimeFieldVirtual   (Calendar string)\n']) ;
    %     cprintf('Blue',[Lnote,'G.Limits.Default.Sequential  (0/1 for climatology/sequential time-series)\n']) ;
    %     cprintf('Blue',[Lnote,'G.Limits.Default.Day2same (0/1 for setting Date2 equal/unequal to Date1)\n']) ;
    %     cprintf('Blue',[Lnote,'G.Limits.S1   (first index of time-series with respect to the first date of Calendar year)\n']) ;
    %     cprintf('Blue',[Lnote,'G.Limits.S2   (last index of time-series with respect to the first date of Calendar year)\n']) ;
    %     cprintf('Blue',[Lnote,'G.Addresses.Default  (full span of Calendar data-frame addresses)\n']) ;
    % end
    
    disp(endline) ;
    disp '  ' ;
catch
    cprintf('Errors',[Lnote,'Could not complete LIMITS status report\n']) ;
    disp(endline) ;
    disp '  ' ;
end