% GOAT_Initialize_Dataset     Obtain fields, spans, calendars, etc.
% Copyright (C) Ori Adam Jan.2012. compatible with version 2014C or later
%
% see also, GOAT_Defaults, GOAT_GetDims, GOAT_GetFieldSpan, GOAT_NewFieldLimits, GOAT_Filters, GOAT_FieldSettings, GOAT_Initialize_Field

function G = GOAT_Initialize_Dataset(G,DataFolder,TimeField)
if nargin > 1
    G.DataFolder = DataFolder ;
end
if ~exist(G.DataFolder,'dir')
    disp(['GOAT_Initialize_Dataset Error: Specified Data Folder: ',G.DataFolder,' does not exist']) ;
    return
end

% Get Time Resolutions
NF = 0 ;
G.TimeFields     = cell(10,1) ; % Cell array of available Time Resolution Folders
G.TimeResdir = dir(G.DataFolder) ;
for j = 1:length(G.TimeResdir)
    if  G.TimeResdir(j,1).isdir && ...
            ~strncmp(G.TimeResdir(j,1).name,'.',1) && ...
            ~strcmpi(G.TimeResdir(j,1).name,'data') 
        NF = NF + 1 ;
        G.TimeFields(NF,1) = {G.TimeResdir(j,1).name} ;
    end
end
G.TimeFields = G.TimeFields(1:NF,1) ;
G.TempResStr = ' ' ;
for j = 1:NF
    G.TempResStr = [G.TempResStr,cell2mat(G.TimeFields(j)),' |'] ;
end
% Virtual Calendars (e.g., DailyX1 is a subset of Daily X2,X4, X6, etc...)
G.TimeFieldsVirtual = G.TimeFields ;
DivisibleCalendars = {'X24','X12','X8','X4','X2'} ; 
CalendarDivisions = {'X12','X8','X6','X4','X2','X1'} ; 
NR = NF ; 
for j = 1:length(DivisibleCalendars)
 for n = 1:NR
     nX = DivisibleCalendars{j} ; 
     nT = G.TimeFields{n}       ;
   if strcmp(nX,nT(end-length(nX)+1:end))
    switch nX
        case 'X24'
          D = 1 ;  
        case 'X12'
          D = 3 ;   
        case 'X8'
          D = 4 ;   
        case 'X4'
          D = 5 ;   
        case 'X2'
          D = 6 ;   
        otherwise
          D = 0 ;   
    end
    for d =  D:length(CalendarDivisions)   
          NF = NF + 1 ;
          G.TimeFields(NF,1) = {nT} ; 
          vT = [nT(1:end-length(nX)),CalendarDivisions{d}] ;
          G.TimeFieldsVirtual(NF,1) = {vT} ;
          G.TempResStr = [G.TempResStr,'<HTML><FONT color="teal">',vT,'</Font></html>',' |'] ;
    end
   end
 end
end
G.TempResStr = G.TempResStr(1:end-1) ;
% Default Time Resolution
if nargin > 2
G.TimeFieldIdx = find(strcmp(TimeField,G.TimeFieldsVirtual),1,'first') ;
% check whether TimeField exists:
if isempty(G.TimeFieldIdx)
    disp(['Warning: the Temporal Resolution ',TimeField,' is not available for the dataset ',G.DataFolder]) ; 
    % Not all Is lost. If It is one of th Daily variaitons, find closest one
    ClosestTime = FindClosestResolution(G) ; 
    if ~isempty(ClosestTime) 
     G.TimeFieldIdx = find(strcmp(ClosestTime,G.TimeFieldsVirtual)) ;   
     disp(['Note: Temporal Resolution was automatically changed to the one closest to ',TimeField,'--> ',ClosestTime]) ; 
     G.TimeField        = ClosestTime ; % physical timefield 
     G.TimeFieldVirtual = ClosestTime ; % a subset of the physical timefield. e.g. DailyX2 or DailyX1 can be set as a virtual timefield of DailyX4      
    else
      return
    end
end
end
G.TimeField        = cell2mat(G.TimeFields(G.TimeFieldIdx)) ;
G.TimeFieldVirtual = cell2mat(G.TimeFieldsVirtual(G.TimeFieldIdx)) ;

% Set step options for Auto Time Step  
switch G.TimeFieldVirtual
    case 'Monthly'
        G.TimeStepStr = '1 Month | 3 Months | 6 Months | 1 Year ' ;
    case {'List','list'}
        G.TimeStepStr = '1 timestep ' ;
        G.Addresses.Default = cell(1,1) ;
    otherwise
        Xidx = regexpi(G.TimeFieldVirtual,'X') ;
        if isempty(Xidx)
            X = 1 ;
        else
            X =   str2num(G.TimeFieldVirtual(Xidx+1:end)) ;
        end
        switch X
            case 1
                G.TimeStepStr = '1 Day | 3 Day | Week | 2 Week | 30 Day | 90 Day | 1 Year ' ;
            case 2
                G.TimeStepStr = '12 Hour | 1 Day | 3 Day | Week | 2 Week | 30 Day | 90 Day | 1 Year ' ;
                
            case 4
                G.TimeStepStr = '6 Hour | 12 Hour | 1 Day | 3 Day | Week | 2 Week | 30 Day | 90 Day | 1 Year ' ;
                
            case 8
                G.TimeStepStr = '3 Hour | 6 Hour | 12 Hour | 1 Day | 3 Day | Week | 2 Week | 30 Day | 90 Day ' ;
                
            case 12
                G.TimeStepStr = '2 Hour | 4 Hour | 6 Hour | 8 Hour | 12 Hour| Day | 3 Day | Week | 2 Week  ' ;
                
            case 24
                G.TimeStepStr = '1 Hour | 2 Hour | 3 Hour | 4 Hour | 6 Hour | 8 Hour | 12 Hour | Day | 3 Day  ' ;
            otherwise
                G.TimeStepStr = '1 timestep ' ;
        end
end
try
    G.TimeStep = 1 ;
    set(G.h_Timestep,'string',G.TimeStepStr,'value',G.TimeStep) ;
catch
    % Nevermind
end

% Create Fields
G.DataFields = cell(1,1) ;
G.DataFieldsType = zeros(1,1) ; % 0 = Real | 1 = Virtual
%Fieldsdir = dir([G.DataFolder,G.Dash,G.TimeField]) ;
Fieldsdir = dir(fullfile(G.DataFolder,G.TimeField,'')) ;
NF = 0 ;
for j = 1:length(Fieldsdir)
    if  Fieldsdir(j,1).isdir && ...
            ~strncmp(Fieldsdir(j,1).name,'.',1) && ...
            ~strcmp(Fieldsdir(j,1).name,'Indices') && ...
            ~strcmp(Fieldsdir(j,1).name,'BoerBeta') && ...
            ~strcmpi(Fieldsdir(j,1).name,'data')
        NF = NF + 1 ;
        G.DataFields(NF,1) = {Fieldsdir(j,1).name} ;
        G.DataFieldsType(NF,1) = 0 ; 
    end
end

if NF > 0  %BK1
    % Inherent GOAT Virtual Fields
    G.VirtualFields.List = '' ;
%     G = GOAT_VirtualFields(G,'VirtualFieldsList') ;
%     for i = 1:length(G.VirtualFields.List)
%         if ~sum(strcmp(G.DataFields,cell2mat(G.VirtualFields.List(i))))
%             G = GOAT_VirtualFields(G,'RequiredFieldsList',cell2mat(G.VirtualFields.List(i))) ;
%             rf = 0 ;
%             for r = 1: length(G.VirtualFields.RequiredFieldsList)
%                 if sum(strcmp(cell2mat(G.VirtualFields.RequiredFieldsList(r)),G.DataFields))
%                     rf = rf + 1 ;
%                 end
%             end
%             if rf == length(G.VirtualFields.RequiredFieldsList)
%                 NF = NF+1 ;
%                 G.DataFields(NF,1) = G.VirtualFields.List(i) ;
%                 G.DataFieldsType(NF,1) = 1 ; % Flag indicating it is a Virtual Field
%             end
%         else
%             disp(['warning: the built-in virtual field ',cell2mat(G.VirtualFields.List(i)),' is disabeld because a physical field with the same name exists'])
%         end
%     end
    % Personal User GOAT Virtual Fields
    try
        NFV = 0 ;
        VFfuncs = cell(10,1) ; % Cell array of available Time Resolution Folders
        VFdir = dir(fullfile(fileparts(which('GOAT_MyDefaults.m')),'MyVirtualFields')) ;
        %VFdir = dir(fullfile('..','MyScripts','MyVirtualFields')) ;
        for j = 1:length(VFdir)
            if  ~strncmp(VFdir(j,1).name,'.',1) && strcmp(VFdir(j,1).name(end),'m')
                NFV = NFV + 1 ;
                VFfuncs(NFV,1) = {VFdir(j,1).name(1:end-2)} ;
            end
        end
        VFfuncs = VFfuncs(1:NFV,1) ;
        G.VirtualFields.MyList = {''} ;
        MVF = 0 ;
        for i = 1:NFV
            try
            eval(['V = ',VFfuncs{i},'(''Active'') ; ']) ;
            if V.VirtualFields.ActiveFlag % && ~G.EnvironmentFlag
                eval(['V = ',VFfuncs{i},'(''VirtualFieldName'') ; ']) ;
                %disp(V.VirtualFields.MyName)
                if ~ismember(V.VirtualFields.MyName,G.DataFields)
                MVF = MVF + 1 ;
                G.VirtualFields.MyList(MVF) = {V.VirtualFields.MyName} ;
                G.VirtualFields.MyFuncs(MVF) = {cell2mat(VFfuncs(i))} ;
                else
                 disp(['GOAT_Initialize_Dataset Warning: the virtual field ''',V.VirtualFields.MyName,''' is deactivated because a physical field with the same name already exists'])   
                end
            end
            catch
                disp(['GOAT_Initialize_Dataset Warning: error evaluating virtual function ',VFfuncs{i}])
            end
        end
        if MVF
            for i = 1:MVF
                if ~sum(strcmp(G.DataFields,cell2mat(G.VirtualFields.MyList(i))))
                    eval(['V = ',G.VirtualFields.MyFuncs{i},'(''RequiredFieldsList'') ; ']) ;
                    rf = 0 ;
                    for r = 1: length(V.VirtualFields.RequiredFieldsList)
                        if sum(strcmp(cell2mat(V.VirtualFields.RequiredFieldsList(r)),G.DataFields))
                            rf = rf + 1 ;
                        end
                    end
                    if rf == length(V.VirtualFields.RequiredFieldsList)
                        NF = NF+1 ;
                        G.DataFields(NF,1) = G.VirtualFields.MyList(i) ;
                        G.DataFieldsType(NF,1) = 1 ; % Flag indicating it is a Virtual Field 
                    end
                else
                    disp(['warning: the virtual field ',cell2mat(G.VirtualFields.MyList(i)),' is deactivated because a physical field with the same name already exists'])
                end
            end
        else
            G.VirtualFields.MyList = '' ;
        end
    catch
        disp 'GOAT_Initialize_Dataset: Error while loading Virtual Fields' ;
        G.VirtualFields.MyList = {''} ;
        % Nevermind, No user virtual fields
    end
    G.DataFields = G.DataFields(1:NF,1) ;
    G.DataFieldsType = G.DataFieldsType(1:NF,1) ;
    G.DataFieldsString = ' ' ;
    for j = 1:NF
        if G.DataFieldsType(j,1)
        G.DataFieldsString = [G.DataFieldsString,'<HTML><FONT color="teal">',cell2mat(G.DataFields(j)),'</Font></html>',' |'] ;
        else
        G.DataFieldsString = [G.DataFieldsString,cell2mat(G.DataFields(j)),' |'] ;
        end
    end
    G.DataFieldsString = G.DataFieldsString(1:end-1) ;
else
    disp 'No Field Folders In main GOAT Directory !!!' ;
    G.DataFieldsString = 'N/A' ;
end
% Reset defaults of linked fields (not relevant in List Calendar)
G.ALP.Attributes = [] ;
G.ALP.Addresses  = [] ;
G.ALP.Smin       = [] ; 
G.ALP.Smax       = [] ; 
G.ALP.Years      = [] ;
G.ALP.Defs       = [] ; 
% LinkedFieldsFlag = 0 ; 
% for i = 1:length(G.DataFields)
%     if exist(fullfile(G.DataFolder,G.TimeField,G.DataFields{i},'link_field.mat'),'file')
%         LinkedFieldsFlag = 1 ; 
%         G.DataField = cell2mat(G.DataFields(i))      ; % 
%         G = GOAT_GetFieldSpan(G)            ; % get available data years and days and set G.Addresses vector
%     end
% end
% if exist(fullfile(G.DataFolder,G.TimeField,'BoerBeta','link_field.mat'),'file')
%    LinkedFieldsFlag = 1 ; 
%    G.DataField = 'BoerBeta'      ; % 
%    G = GOAT_GetFieldSpan(G)            ; % get available data years and days and set G.Addresses vector
% end
% if LinkedFieldsFlag
%   if ~G.EnvironmentFlag
%     set(G.h_message,'string',['All fields good to GO.']) ; drawnow ; % Update Message
%   else
%     disp(['All fields good to GO.']) ; drawnow ; % Update Message  
%   end
% end
G.DataField = cell2mat(G.DataFields(1))      ; % Default Fiels is the first to appear
% Filter specs
G.Filter.Indices = cell(20,1) ;
try
    Indicesdir = dir(fullfile(G.DataFolder,G.TimeField,'Indices','')) ;
catch
    makdir(fullfile(G.DataFolder,G.TimeField,'Indices','')) ;
    Indicesdir = dir(fullfile(G.DataFolder,G.TimeField,'Indices','')) ;
end
NF = 0 ;
for j = 1:length(Indicesdir)
    [Filepath,FileName,FileExt] = fileparts(Indicesdir(j,1).name) ;
    if  ~Indicesdir(j,1).isdir && ...
            ~strncmp(Indicesdir(j,1).name,'.',1) && ...
            strcmp(FileExt,'.mat')
        NF = NF + 1 ;
        G.Filter.Indices(NF,1) = {FileName} ;
    end
end
if NF > 0
    G.Filter.Indices = G.Filter.Indices(1:NF,1) ;
    G.Filter.IndicesString = 'None | ' ;
    for j = 1:NF
        G.Filter.IndicesString = [G.Filter.IndicesString,cell2mat(G.Filter.Indices(j)),' |'] ;
    end
    G.Filter.IndicesString = G.Filter.IndicesString(1:end-1) ;
else
    G.Filter.Indices = [] ;
    G.Filter.IndicesString = 'None' ;
end
G.Filter.IndicesString = [G.Filter.IndicesString,' | Browse '] ;
G.Filter.IndicesIdx = 1 ; % 1 = 'None'
% Clear Filters
G.Filter.WeightIdx          = 1     ; % 1 = 'None'
G.Filter.ZonalWaveNumberIdx = -1    ; % 0 = no Filter
G.Filter.ZonalWaveNumberLowIdx  = 1 ; % Low zonal wave number truncation (1= none)
G.Filter.ZonalWaveNumberHighIdx = 1 ; % High zonal wave number truncation (1 = none)
G.Filter.RAIdx              = 1     ; % 0 = no Filter
G.Filter.REFlag             = 0     ; % No Running Eddy
G.Filter.LowPassIdx         = 0     ; % 0 = no lowpass Filter value
G.Filter.HighPassIdx        = 0     ; % 0 = no highpass Filter value
G.Filter.IndicesIdx         = 1     ; % 1 = 'None'
G.Filter.ZEFlag             = 0     ; % Zonal Anomaly Flag
G.Filter.WeightFactor       = 1     ; % Weight factor value
G = GOAT_GetDims(G)                 ; % get level, longitude, latitude and hours dimensions
G = GOAT_GetFieldSpan(G)            ; % get available data years and days and set G.Addresses vector
G = GOAT_FieldSettings(G)           ; % Customized Field Display Settings
G.Filter.Index = ones(length(G.Addresses.Default),1) ;
% Set Default spatial and temporal indices
G.Limits.Default.time1      = 1                   ; % Default Linear Time Index 1
G.Limits.Default.time2      = length(G.Addresses.Default) ; % Default Linear Time Index 2
try
G.Limits.Default.year1      = G.Limits.FirstYear  ; % Default year1 for new field
G.Limits.Default.year2      = G.Limits.LastYear   ; % Default year2 for new field
G.Limits.Default.day1       = G.Limits.FirstDay   ; % Default day1 for new field
G.Limits.Default.day2       = G.Limits.LastDay    ; % Default day2 for new field
G.Limits.Default.month1     = G.Limits.FirstMonth ; % Default Month1 for new field
G.Limits.Default.month2     = G.Limits.LastMonth  ; % Default Month2 for new field
G.Limits.Default.H1         = G.Limits.FirstHour  ; % Hour 1 idx
G.Limits.Default.H2         = G.Limits.LastHour   ; % Hour 2 idx
catch % Dummies for List Calendar
    G.Limits.Default.year1  = 1 ; % Default year1 for new field
    G.Limits.Default.year2  = 1 ; % Default year2 for new field
    G.Limits.Default.day1   = 1 ; % Default day1 for new field
    G.Limits.Default.day2   = 1 ; % Default day2 for new field
    G.Limits.Default.month1 = 1 ; % Default Month1 for new field
    G.Limits.Default.month2 = 1 ; % Default Month2 for new field
    G.Limits.Default.H1     = 1 ; % Hour 1 idx
    G.Limits.Default.H2     = 1 ; % Hour 2 idx% nevermind. Irregular Calendar
end
G.Limits.Default.Day2same   = 1                   ; % Set Date 2 same as Date 1 Flag
G.Limits.Default.Sequential = 1                   ; % Sequential averaging or Seasonal averaging Flag
G.Limits.Default.ClimatologyContinuous = 0        ; % Present Seasonal Data Sequentially
G.Limits.Default.Anom       = 0                   ; % Anomaly Flag. Relevant only for Hovmoller plots
G.Limits.Default.Lev1       = 1                   ; % Default Pressure level 1
G.Limits.Default.Lev2       = length(G.lev)         ; % Default Pressure level 2
G.Limits.Default.Lat1       = 1                   ; % Default Latitude 1
G.Limits.Default.Lat2       = length(G.lat)       ; % Default Latitude 2
G.Limits.Default.Lon1       = 1                   ; % Default Longitude 2
G.Limits.Default.Lon2       = length(G.lon)       ; % Default Longitude 1
G.Limits.Default.sameP      = 1                   ; % Same Pressure Level Flag
G.Limits.Default.VerMeanInt = 1                   ; % Vertical Mean (==1) or Integral (==2)
try
    set(G.h_Field,'TooltipString',['[ ',G.DataFieldUnits,' ]   ',G.FieldDescription]) ;
catch
    try
        set(G.h_Field,'TooltipString','Field units or description not found.') ;
    catch
        % Nevermind
    end
end
[Path,Dataset] = fileparts(G.DataFolder) ; 
% Customized Settings
G = GOAT_MySets(G,Dataset,'Dataset') ; 

function ClosestTime = FindClosestResolution(G)
ClosestTime = [] ; 
switch G.TimeFieldVirtual
        case 'Daily'
            AlternateResolution = {'Daily365','Daily360'} ; 
        case 'Daily365'
            AlternateResolution = {'Daily','Daily360'} ;     
        case 'Daily360'
            AlternateResolution = {'Daily','Daily365'} ;
        case 'DailyX2'
            AlternateResolution = {'Daily365X2','Daily360X2'} ; 
        case 'Daily365X2'
            AlternateResolution = {'DailyX2','Daily360X2'} ;     
        case 'Daily360X2'
            AlternateResolution = {'DailyX2','Daily365X2'} ;
        case 'DailyX4'
            AlternateResolution = {'Daily365X4','Daily360X4'} ; 
        case 'Daily365X4'
            AlternateResolution = {'DailyX4','Daily360X4'} ;     
        case 'Daily360X4'
            AlternateResolution = {'DailyX4','Daily365X4'} ;    
        case 'DailyX8'
            AlternateResolution = {'Daily365X8','Daily360X8'} ; 
        case 'Daily365X8'
            AlternateResolution = {'DailyX8','Daily360X8'} ;     
        case 'Daily360X8'
            AlternateResolution = {'DailyX8','Daily365X8'} ;
        case 'Monthly'
            AlternateResolution = {'Monthly360'} ; 
    otherwise
        AlternateResolution = [] ; 
end

for i = 1:length(AlternateResolution)
Time = AlternateResolution{i} ; 
  if sum(strcmp(Time,G.TimeFields))
     ClosestTime = Time ; 
     return ;
  end
end
