% GOAT_Initialize_Field  Initialize field in the GOAT environment
% Copyright (C) Ori Adam, Dec.2012. 
% Modified March.2014. Compatible with version 2014C and later versions.
%
% see also, GOAT_GetDims, GOAT_GetFieldSpan, GOAT_NewFieldLimits, GOAT_FieldSettings

function G = GOAT_Initialize_Field(G,FieldName)
switch nargin
    case 1 
        % assume G.DataField, G.TimeField, G.TimeFieldVirtual have already been set
    case 2
        G.DataField        = FieldName ; % set field's name. Assume G.TimeField and G.TimeFieldVirtual have already been set
%     case 3
%         G.DataField        = FieldName ; % set field's name. Assume G.TimeField and G.TimeFieldVirtual have already been set
%         G.TimeField        = TimeField ; % physical timefield 
%         G.TimeFieldVirtual = TimeField ; % a subset of the physical timefield. e.g. DailyX2 or DailyX1 can be set as a virtual timefield of DailyX4
%     case 4
%         G.DataField        = FieldName ; % set field's name. Assume G.TimeField and G.TimeFieldVirtual have already been set
%         G.TimeField        = TimeField ; % physical timefield 
%         G.TimeFieldVirtual = TimeFieldVirtual ; % a subset of the physical timefield. e.g. DailyX2 or DailyX1 can be set as a virtual timefield of DailyX4    
end

% check whether Field exists:
try
    if ~strcmp(G.DataFields,G.DataField)
        disp(['Warning: the field ',G.DataField,' is not available for the dataset ',fullfile(G.DataFolder,G.TimeField)]) ;
    end
catch
    cprintf('err','Error in GOAT_Initialize_Field:\n')
    if isfield(G,'DataFolder')
        if ~exist(G.DataFolder,'dir')
            cprintf('text','Dataset folder does not exits: ')
            cprintf('key',[G.DataFolder,'\n'])
        else
            cprintf('text','Potentially invalid dataset: ')
            cprintf('key',[G.DataFolder,'\n'])
            cprintf('text',' the fields ''')
            cprintf('key','G.DataFields')
            cprintf('text',''' or ''')
            cprintf('key','G.DataField')
            cprintf('text',''' do not exits. Please check dataset. \n') ;
        end
    else
        cprintf('text',' the fields ''')
        cprintf('key','G.DataFields')
        cprintf('text',''' or ''')
        cprintf('key','G.DataField')
        cprintf('text',''' do not exits. Please check dataset. \n') ;
    end
    return
end
%Assuming G.DataField, G.DataFolder and G.TimeField have already been set.
G = GOAT_GetDims(G)        ; % get spatial dimensions'                      
G = GOAT_GetFieldSpan(G)   ; % get available data years, days, hours etc. and set address vectors       
G = GOAT_NewFieldLimits(G) ; % Make sure new field dimentions matches existing limits and adjust if necessary'
G = GOAT_FieldSettings(G)  ; % Customized field display settings'           
return ;

