% GOAT_LINK_Install install linked data as *.mat files
% copyright (C) Ori Adam March19.2014
%
%

function G = GOAT_LINK_Install(G)

if ~G.EnvironmentFlag
    set(G.h_Analyze,'Enable','off')  ; drawnow ; % Disable GO button
    G = Install_LINK_GUI(G) ;
    if G.ALP.CancelFlag
        return
    end
    G = GOAT_Limits(G) ; % set temporal span
    if ~G.continueFlag
        G = GOAT_ALP_Install(G) ; % Install from server or nc file
        G.continueFlag = 0 ;
    end
    set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button
else % assuming defualt values have been set
    G = GOAT_ALP_Install(G) ; % Install from server or nc file
end

% Crerate LINK Install GUI
function G = Install_LINK_GUI(G)
G.ALP.ExportAddress = fullfile(G.DataFolder,G.TimeFieldVirtual,G.DataField) ;
G.ALP.Calendar = G.TimeFieldVirtual ;
G.ALP.OverWriteFlag  = 0 ;
G.ALP.BoerBetaFlag   = 0 ;
G.ALP.CancelFlag     = 0 ;
G.ALP.SaveSingle     = 1 ;
pos        = G.GUI.pos.Export ;
LINK_Fig = figure('name','Install linked data as *.mat','position',pos,'color',G.Display.FigBackgroudColor) ;
set(LINK_Fig,'MenuBar','none')
NumLines = 9 ;
bl = 0.02 ; % Left Edge
bt = 0.05; % Top Edge
LineHeight = (1-2*bt)/NumLines ;
Column     = (1-2*bl)/1 ;
%Top Message
messageStr = {'Install linked data as *.mat files'} ;
h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-LineHeight*.75,1,LineHeight*.75], 'string',messageStr);
uicontrol(h_message) ;  set(h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_S,'FontName',G.GUI.Fonts.MessageFont) ;
% title
Line = 2 ;
h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','Output Adress');
uicontrol(h_NameT) ;  set(h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Installed field folder','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 2 ;
h_Name = uicontrol('style','edit','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*3/5-bl,LineHeight],...
    'string',G.ALP.ExportAddress,  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(h_Name) ;  set(h_Name,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left') ;
% Browse
h_Browse = uicontrol('Units','normalized','pos',[bl + Column*4/5,1-bt-Line*LineHeight,Column/5 ,LineHeight],...
    'string','Browse',  'callback','set(gcbf,''userdata'',112); uiresume;');
uicontrol(h_Browse) ;  set(h_Browse,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% Time
TimeFields = {G.TimeFieldVirtual} ;
switch G.TimeFieldVirtual
    case {'Daily','Daily365'}
        if sum(strcmpi(G.TimeFields,'Monthly')) == 0
            TimeFields = {G.TimeFieldVirtual,'Monthly'} ;
        end
    case {'DailyX4','DailyX8'}
        if sum(strcmpi(G.TimeFields,'Daily')) == 0
            TimeFields = {G.TimeFieldVirtual,'Daily'} ;
        end
    case {'Daily365X4','Daily365X8'}
        if sum(strcmpi(G.TimeFields,'Daily365')) == 0
            TimeFields = {G.TimeFieldVirtual,'Daily365'} ;
        end
end
TimeFieldsStr = [] ;
for i = 1:length(TimeFields)
    TimeFieldsStr = [TimeFieldsStr,TimeFields{i},' |'] ;
end
TimeFieldsStr = TimeFieldsStr(1:end-1) ;

Line = 4 ;
h_TimeT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/5,LineHeight], 'string','Calendar');
uicontrol(h_TimeT) ;  set(h_TimeT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 4 ;
h_Time = uicontrol('style','popup','Units','normalized','pos',[bl*2+Column/5,1-bt-Line*LineHeight,Column*3/5-bl,LineHeight],...
    'string',TimeFieldsStr,  'callback','set(gcbf,''userdata'',12); uiresume;');
uicontrol(h_Time) ;  set(h_Time,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left') ;
set(h_Time,'enable','off') ;
if strcmp('Pressure (mb)',G.VerticalOrdinate)
    % BoerBeta
    Line = 5 ;
    h_BoerBeta = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*.9,LineHeight], 'string','Install BoerBeta Function',...
        'value',G.ALP.BoerBetaFlag,'callback','set(gcbf,''userdata'',15); uiresume;');
    uicontrol(h_BoerBeta) ;  set(h_BoerBeta,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left', ...
        'TooltipString','BoerBeta function = 0/1 below/above surface. Used in vertical integrals to cancel the contributions of artificial below-surface data.','FontName',G.GUI.Fonts.uiHeaderFont) ;
end
% Over-write
Line = 6 ;
h_OverWrite = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*.9,LineHeight], 'string','Over-Write Existing *.mat files',...
    'value',G.ALP.OverWriteFlag,'callback','set(gcbf,''userdata'',16); uiresume;');
uicontrol(h_OverWrite) ;  set(h_OverWrite,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left', ...
    'TooltipString','When checked, existing *.mat files are over-written','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 7 ;
h_Single = uicontrol('style','checkbox','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*.9,LineHeight], 'string','Save data as ''single'' precision',...
    'value',G.ALP.SaveSingle,'callback','set(gcbf,''userdata'',17); uiresume;');
uicontrol(h_Single) ;  set(h_Single,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left', ...
    'TooltipString','When checked, data is saved as single precision','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Cancel
Line = NumLines  ;
h_Cancel = uicontrol('Units','normalized','pos',[bl*2 + Column*2/3,1-bt-Line*LineHeight,Column/3 - bl,LineHeight],...
    'string','Cancel',  'callback','set(gcbf,''userdata'',99); uiresume;');
uicontrol(h_Cancel) ;  set(h_Cancel,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% OK
Line = NumLines  ;
h_OK = uicontrol('Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*2/3,LineHeight],...
    'string','OK',  'callback','set(gcbf,''userdata'',100); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',16,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;


GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',LINK_Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 11 % Folder name of export
                G.ALP.ExportAddress          = get(h_Name,'string')            ;
            case 112 % Browse
                try
                    ExportAddress = uigetdir ;
                    try
                        if ExportAddress == 0
                            continue ;
                        end
                    end
                    G.ALP.ExportAddress = ExportAddress ;
                    set(h_Name,'string',G.ALP.ExportAddress)
                catch
                    % Nevermind
                end
            case 12 % Temporal Resolution
                CurrentTimeResolution = G.ALP.Calendar ;
                G.ALP.Calendar =  TimeFields{get(h_Time,'value')}            ;
                Idx = regexp(G.ALP.ExportAddress,G.ALP.Calendar,'first') ;
                if isempty(Idx)
                    G.ALP.ExportAddress  = regexprep(G.ALP.ExportAddress,CurrentTimeResolution,G.ALP.Calendar) ;
                    set(h_Name,'string',G.ALP.ExportAddress)
                end
            case 15 % install BoerBeta Function
                G.ALP.BoerBetaFlag = get(h_BoerBeta,'value') ;
            case 16 % Overwrite existing *.mat files
                G.ALP.OverWriteFlag = get(h_OverWrite,'value') ;
            case 17 % Save as Single
                G.ALP.SaveSingle = get(h_Single,'value') ;    
            case 99 % Clear
                G.ALP.CancelFlag = 1 ;
                GUIcon = 0 ;
            case 100 % OK
                GUIcon = 0 ;
        end
    catch
        %nevermind
    end
end

try
    close(LINK_Fig) ;
catch
    % Nevermind
end
if ~G.ALP.CancelFlag
    set(G.h_message,'string',['Preparing to install linked data . . .        ']) ; % Update Message
    drawnow ;
end
return ;

