% LOAD load data and perform temporal filtering according to selected settings
% written by Ori Adam 2012. compatible with version 2013h
%
% see also VirtualFields, MyVirtualFields, GOAT_LoadData, GOAT_LoadDataAndProject 

function [data,G] = GOAT_LoadDataAndTimeFilter(G,i)
if sum([G.Filter.LowPassIdx,G.Filter.HighPassIdx]) > 0 % Low / High Pass Filter
    Nwindow = max([G.Filter.LowPassIdx,G.Filter.HighPassIdx])*2 ;
    % truncated
    if ~G.Filter.SimpleLoadFlag
        [data1,G] = GOAT_LoadDataAndProject(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i) - Nwindow + 1)),i)  ;
        dataf = zeros(size(data1,1),size(data1,2),Nwindow*2) ;
        for j = 2:Nwindow*2
            [dataf(:,:,j),G] = GOAT_LoadDataAndProject(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i) - Nwindow + j)),i)  ;
        end
        datafft = fft(dataf,[],3);  % data after fft in time
        datatoifft = datafft     ;  % data for ifft
        Fvec = linspace(0,1,Nwindow+1)*.5 ;
        FIdx = Fvec*0 ;
        if G.Filter.HighPassIdx > 0 && G.Filter.LowPassIdx > 0
            FIdx(Fvec >= 1/G.Filter.HighPassIdx & Fvec <= 1/G.Filter.LowPassIdx) = 1 ;
        elseif G.Filter.HighPassIdx > 0 && G.Filter.LowPassIdx == 0
            FIdx(Fvec >= 1/G.Filter.HighPassIdx) = 1 ;
        else
            FIdx(Fvec <= 1/G.Filter.LowPassIdx) = 1 ;
        end
        FIdx = [FIdx,fliplr(FIdx(2:end-1))] ;
        datatoifft(:,:, FIdx~= 1) = 0 ;
        dataf = real(ifft(datatoifft,[],3)) ;
        data = dataf(:,:,Nwindow).*G.Filter.weight ;
        clear datatoifft datafft data1
    else % Untruncated --> very inefficient !
        dataf = zeros(length(G.lon),length(G.lat),length(G.P),Nwindow*2) ;
        for j = 1:Nwindow*2
            [dataf(:,:,:,j),G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i) - Nwindow + j)),i)  ;
        end
        datafft = fft(dataf,[],4);  % data after fft in time
        datatoifft = datafft     ;  % data for ifft
        Fvec = linspace(0,1,Nwindow+1)*.5 ;
        FIdx = Fvec*0 ;
        if G.Filter.HighPassIdx > 0 && G.Filter.LowPassIdx > 0
            FIdx(Fvec >= 1/G.Filter.HighPassIdx & Fvec <= 1/G.Filter.LowPassIdx) = 1 ;
        elseif G.Filter.HighPassIdx > 0 && G.Filter.LowPassIdx == 0
            FIdx(Fvec >= 1/G.Filter.HighPassIdx) = 1 ;
        else
            FIdx(Fvec <= 1/G.Filter.LowPassIdx) = 1 ;
        end
        FIdx = [FIdx,fliplr(FIdx(2:end-1))] ;
        datatoifft(:,:,:, FIdx~= 1) = 0 ;
        dataf = real(ifft(datatoifft,[],4)) ;
        data = dataf(:,:,:,Nwindow) ;
        clear datatoifft datafft
    end
end
if G.Filter.RAIdx > 1 %Running Average
    if G.Filter.SimpleLoadFlag
        [data,G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i)-ceil((G.Filter.RAIdx-1)/2))),i) ;
    else
        [data,G] = GOAT_LoadDataAndProject(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i)-ceil((G.Filter.RAIdx-1)/2))),i) ;
    end
    for j = 1:G.Filter.RAIdx-1
        if G.Filter.SimpleLoadFlag      
            data = data + GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i)-ceil((G.Filter.RAIdx-1)/2)+j)),i) ;
        else
            data = data + GOAT_LoadDataAndProject(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i)-ceil((G.Filter.RAIdx-1)/2)+j)),i) ;
        end
    end
    data = data/j.*G.Filter.weight ;
    if G.Filter.REFlag
      data = GOAT_LoadDataAndProject(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i))),i).*G.Filter.weight - data ;
    end
end
% No Time Filter
if G.Filter.RAIdx <= 1 && sum([G.Filter.LowPassIdx,G.Filter.HighPassIdx]) == 0  
    if G.Filter.SimpleLoadFlag % No Spatial Truncation
        [data,G] = GOAT_LoadData(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i))),i) ;
        data = data.*G.Filter.weight ; 
    else
        [data,G] = GOAT_LoadDataAndProject(G,G.DataFolder,G.TimeField,G.DataField,cell2mat(G.Addresses.Default(G.DataStringVec(i))),i) ;
        data = data.*G.Filter.weight ; 
    end
end