% GOAT_SignificancePlot generate a figure where significant values are shaded
% written by Ori Adam 2012.
%
% see also GOAT_plotUpdate

function G = GOAT_SignificancePlot(G,Data,SigData,alpha)
    
Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
try
        Lon1 = min(G.x) ; 
        Lon2 = max(G.x) ; 
catch
        Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
        Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
end
    P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
    P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
    X = G.X ; 
    Y = G.Y ; 

    Projection = G.ProjectionType ; 
try
    ContourIdx = G.Settings.ContourIdx ;
catch
    ContourIdx = 1 ;
end
try
    LineWidth      = G.Settings.LineWidth      ;
    CoastLineWidth = G.Settings.CoastLineWidth ;
catch
    LineWidth      = 1 ;
    CoastLineWidth = 1 ;
end

SigData(SigData>alpha) = nan ; 

switch Projection
    case {1,2} % Long-Lat
         [Cs,s] = contourf(X,Y,SigData',1) ; hold on ; 
         colormap([.5 .5 .5 ; .5 .5 .5]) ; 
         set(s,'linestyle','none')
         [C,h] =  contour(X,Y,Data','k') ;
         clabel(C,h,'fontsize',13) ;
            if LineWidth > 0
                try  set(h,'linewidth',LineWidth) ; end
            else
                try  set(h,'lineWidth',2) ; end
            end
            xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
            ylabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
            if G.ProjectionType == 1
                plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
            end
        xlim([Lon1 Lon2])   ;
        ylim([Lat1 Lat2])   ;
    case 3 % Lat-Pressure
        try
            if G.Limits.Default.sameP || P2 == P1
                if LineWidth == 0 ; LineWidth = 1 ; end
                DataSig = Data ; 
                DataSig(SigData>alpha) = nan ; 
                Data(SigData<=alpha) = nan ;   
                plot(X,DataSig,'k',X,Data,'--k','linewidth',LineWidth) ;
                xlabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',13,'interpreter','latex') ;
                xlim([Lat1 Lat2])   ;
            else
                [Cs,s] = contourf(X,Y,SigData',1) ; hold on ; 
                colormap([.5 .5 .5 ; .5 .5 .5]) ; 
                set(s,'linestyle','none')
                [C,h] =  contour(X,Y,Data','k') ;
                clabel(C,h,'fontsize',13) ;
                if LineWidth > 0
                try  set(h,'linewidth',LineWidth) ; end
                else
                try  set(h,'lineWidth',2) ; end
                end
                axis ij
                xlabel('$$\textrm{Latitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
                ylim([P1 P2])
                xlim([Lat1 Lat2])   ;
            end
        catch
            % Nevermind, no Data
        end
    case 4 % Lon-Pressure
        try
            if G.Limits.Default.sameP
                if LineWidth == 0 ; LineWidth = 1 ; end
                DataSig = Data ; 
                DataSig(SigData>alpha) = nan ; 
                Data(SigData<=alpha) = nan ;   
                plot(X,DataSig,'k',X,Data,'--k','linewidth',LineWidth) ;
                xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',13,'interpreter','latex') ;
                xlim([Lon1 Lon2])   ;
            elseif Lon1 == Lon2
                if LineWidth == 0 ; LineWidth = 1 ; end
                DataSig = Data ; 
                DataSig(SigData>alpha) = nan ; 
                Data(SigData<=alpha) = nan ;   
                plot(Y,DataSig,'k',Y,Data,'--k','linewidth',LineWidth) ;
                xlabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',13,'interpreter','latex') ;
                xlim([P1 P2])   ;
            else
                [Cs,s] = contourf(X,Y,SigData',1) ; hold on ; 
                colormap([.5 .5 .5 ; .5 .5 .5]) ; 
                set(s,'linestyle','none')
                [C,h] =  contour(X,Y,Data','k') ;
                clabel(C,h,'fontsize',13) ;
                if LineWidth > 0
                try  set(h,'linewidth',LineWidth) ; end
                else
                try  set(h,'lineWidth',2) ; end
                end
                axis ij
                xlabel('$$\textrm{Longitude}$$','Fontsize',13,'interpreter','latex') ;
                ylabel('$$\textrm{Level}$$','Fontsize',13,'interpreter','latex') ;
                ylim([P1 P2])
                xlim([Lon1 Lon2])   ;
            end
        catch
            % Nevermind, no Data
        end
        
end
% Axis Configurations
try
    if G.Settings.XgridFlag
        set(gca,'XGrid','on') ;
    end
    if G.Settings.YgridFlag
        set(gca,'YGrid','on') ;
    end
end
try
    if G.Settings.ContourLevelStep
        if  G.Settings.ContourLevelStep > (max(max(Data)) - min(min(Data)))/100 ;
            set(h,'levelstep',G.Settings.ContourLevelStep) ;
        end
    end
catch
    % Nevermind
end
try
    if G.Settings.FixZLimitsFlag || G.Settings.SymZLimitsFlag
        if G.Settings.SymZLimitsFlag
            MinZ = min(min(Data));
            MaxZ = max(max(Data));
            AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
            G.Settings.ZlimitMin = - AbsMax ;
            G.Settings.ZlimitMax =   AbsMax ;
        end
        caxis([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
        if ~G.Settings.SymZLimitsFlag
            colorbar ;
        else
            MinZ = min(min(Data));
            MaxZ = max(max(Data));
            colorbar('YLim',[MinZ MaxZ],'fontname',G.GUI.Fonts.gcaFont) ;
        end
    end
end
set(gca,'fontsize',12,'fontname',G.GUI.Fonts.gcaFont) ;
hold off ;