% GOAT_getSerial Get the serial number of particular date (Monthly, DailyX1-24,Julian1-600)
% Copyright (C) Ori Adam Nov.2012. 
% syntax:
% Serial = GOAT_getSerial(year,month,day,hour,Calendar,FirstYearOfCalendar)
%
% see also GOAT_serial2date

function S = GOAT_getSerial(year,month,day,hour,Calendar,Year0)
% parse Calendar
if strncmpi(Calendar,'Mon',3)
    Type = 'Monthly' ;
elseif strncmpi(Calendar,'Daily360',8)
    Type = 'Daily360' ;
    if day>30
        day = 30 ; 
    end
elseif strncmpi(Calendar,'Daily365',8)
    Type = 'Daily365' ;
    day = min([day,GOAT_DaysInMonth(month,year,Calendar)]) ; % 
elseif strncmpi(Calendar,'Daily',5)
    Type = 'Daily' ;
    day = min([day,GOAT_DaysInMonth(month,year)]) ; 
elseif strncmp(Calendar,'Julian',6)
    Type = 'Julian' ;
else
    disp(['GOAT_getSerial error: unrecognized TimeResolution - ',Calendar]) ;
    return
end

switch Type
    case 'Monthly'
        S = (year - Year0)*12 + month ;
        return ;
    case 'Daily360'
        Xidx = regexpi(Calendar,'X') ;
        if isempty(Xidx)
            X = 1 ; % 1 Times Daily
        else
            X =   str2double(Calendar(Xidx+1:end)) ;  % X Times Daily
        end
        if X == 1
            hour = 0 ; 
        end
        Days = (month-1)*30 + day ; 
        S = ((year - Year0)*360)*X + (Days-1)*X + round(hour*X/24) + 1 ;
        return 
    case 'Daily365' % No leap years
        Xidx = regexpi(Calendar,'X') ;
        if isempty(Xidx)
            X = 1 ; % 1 Times Daily
        else
            X =   str2double(Calendar(Xidx+1:end)) ;       % X Times Daily
        end
        if X == 1
            hour = 0 ; 
        end
        switch month
            case 1
                Days = day ;
            case 2
                Days = 31 + day ;
            case 3
                Days = 59 + day ;
            case 4
                Days = 90 + day ;
            case 5
                Days = 120 + day ;
            case 6
                Days = 151 + day ;
            case 7
                Days = 181 + day ;
            case 8
                Days = 212 + day ;
            case 9
                Days = 243 + day ;
            case 10
                Days = 273 + day ;
            case 11
                Days = 304 + day ;
            case 12
                Days = 334 + day ;
        end
        S = ((year - Year0)*365)*X + (Days-1)*X + round(hour*X/24) + 1 ;
        return ;
    case 'Daily'
        Xidx = regexpi(Calendar,'X') ;
        if isempty(Xidx)
            X = 1 ; % TimesDaily
        else
            X =   str2double(Calendar(Xidx+1:end)) ;       % TimesDaily
        end
        if X == 1
            hour = 0 ; 
        end
        %Nly = sum(GOAT_leapyear(Year0-1:year-1,Calendar)) ; 
        %ly = GOAT_leapyear(year,Calendar) ;
        if year>Year0
        Nly = sum(GOAT_leapyear(Year0:year-1)) ; % No need to input Calendar since we know this is a calendar with leap years 
        else
        Nly = 0 ; 
        end
        ly = GOAT_leapyear(year)                 ;
        switch month
            case 1
                Days = day ;
            case 2
                Days = 31 + day ;
            case 3
                if ly
                    Days = 60 + day ;
                else
                    Days = 59 + day ;
                end
            case 4
                if ly
                    Days = 91 + day ;
                else
                    Days = 90 + day ;
                end
            case 5
                if ly
                    Days = 121 + day ;
                else
                    Days = 120 + day ;
                end
            case 6
                if ly
                    Days = 152 + day ;
                else
                    Days = 151 + day ;
                end
            case 7
                if ly
                    Days = 182 + day ;
                else
                    Days = 181 + day ;
                end
            case 8
                if ly
                    Days = 213 + day ;
                else
                    Days = 212 + day ;
                end
            case 9
                if ly
                    Days = 244 + day ;
                else
                    Days = 243 + day ;
                end
            case 10
                if ly
                    Days = 274 + day ;
                else
                    Days = 273 + day ;
                end
            case 11
                if ly
                    Days = 305 + day ;
                else
                    Days = 304 + day ;
                end
            case 12
                if ly
                    Days = 335 + day ;
                else
                    Days = 334 + day ;
                end
        end
        S = ((year - Year0)*365 + Nly)*X + (Days-1)*X + round(hour*X/24) + 1 ;
        return ;
    case 'Julian'
        if length(Calendar) == 6
            YYY = 365 ;
            X = 1  ;
        else
            Xidx = regexpi(Calendar,'X') ;
            if isempty(Xidx)
                YYY = str2double(Calendar(7:end)) ;
                X = 1 ;
            else
                YYY = str2double(Calendar(7:Xidx-1)) ;
                X =   str2double(Calendar(Xidx+1:end)) ;
            end
        end
        if X == 1
            hour = 0 ; 
        end
        S = ((year - Year0)*YYY )*X + (day-1)*X + round(hour*X/24) + 1 ;
end