% GOAT_load_bata central function for loading BoerBeta function into GOAT.
% bata = saved or linked BoerBeta Function. bata can be loaded from *.mat, OPenDAP or *.nc files
% copyright (C) Ori Adam, March.28.2014
%
%
function [bata,G] = GOAT_load_bata(fileaddress,G,Aidx)
if nargin == 1 || exist(fileaddress,'file')
    load(fileaddress) ; % assume is is a *.mat file by default
    return
else
    if ~isfield(G.ALP.Attributes,'BoerBeta')
        DataField0 = G.DataField ;
        G.DataField = 'BoerBeta' ;
        G = GOAT_GetFieldSpan(G) ;
        G.DataField = DataField0 ;
        G = GOAT_GetFieldSpan(G) ;
    end
    Idx = G.DataStringVec(Aidx) ;
%    Ext = G.ALP.Addresses.BoerBeta{Idx,end-2:end} ;
    %Ext = fileaddress(end-2:end) ;
    Ext = G.ALP.Attributes.BoerBeta{end,1} ;
    % Ext = mat : load from mat file
    % Ext = bds : load from opendap source, use surface pressure
    % Ext = bns : load from nc file, use surface pressure
    % Ext = dds : load from downloaded nc file, use surface pressure
    % Ext = bdn : load from opendap source, use Nans in 3D matrix
    % Ext = bnn : load from nc file, use Nans in 3D matrix
    % Ext = dnn : load from downloaded nc file, use Nans in 3D matrix
    % Do not print message as BoerBeat normally is calculated in addition to other fields
    %if ismember(Ext,{'bds','bdn','dnn'}) && ~G.EnvironmentFlag && ~G.ProjectionType == 10
    %    set(G.h_message,'string','Retrieving data from opendap server . . .') ; drawnow ; % Update Message
    %end
    
    switch Ext
        case 'mat' % *.mat files
            load(fileaddress) ; % assume is is a *.mat file
            return
        case {'bds','bns','bdn','bnn','dds','dnn'}
            % Attributes(i,1) = filename ;
            % Attributes(i,2) = varname  ;
            % Attributes(i,3) = start    ;
            % Attributes(i,4) = count    ;
            % Attributes(i,5) = factor   ;
            % Attributes(i,6) = permute   ;
            % Attributes(i,7) = reshape (add dimensions)   ;
            % Attributes(i,8) = NanValue (set to nan if data == NanValue)
            [~ , SizeA_2] = size(G.ALP.Attributes.BoerBeta) ;
            if size(G.ALP.Attributes.BoerBeta{Idx,1},1) == 0
                bata = [] ;
                Warning = ['GOAT_load_bata error: no attributes for - ',fileaddress] ;
                disp(Warning)  ;
                G.Error.LatestDisplayWarning = Warning ;
                return ;
            elseif size(G.ALP.Attributes.BoerBeta{Idx,1},1) == 1
                try
                if ismember(Ext,{'dnn','dds'})
                    data = ncread(urlwrite(G.ALP.Attributes.BoerBeta{Idx,1},G.ALP.tempnc),G.ALP.Attributes.BoerBeta{Idx,2},G.ALP.Attributes.BoerBeta{Idx,3},G.ALP.Attributes.BoerBeta{Idx,4}) ;
                else
                    data = ncread(G.ALP.Attributes.BoerBeta{Idx,1},G.ALP.Attributes.BoerBeta{Idx,2},G.ALP.Attributes.BoerBeta{Idx,3},G.ALP.Attributes.BoerBeta{Idx,4}) ;
                end
                catch
                disp(['GOAT_load_bata Error: error reading  ',G.ALP.Attributes.BoerBeta{Idx,1}])
                end
                if ~isempty(G.ALP.Attributes.BoerBeta{Idx,5}) % factor and offset [factor,offset]
                    if length(G.ALP.Attributes.BoerBeta{Idx,5}) > 1
                        data = data*G.ALP.Attributes.BoerBeta{Idx,5}(1) + G.ALP.Attributes.BoerBeta{Idx,5}(2) ;
                    else
                        data = data*G.ALP.Attributes.BoerBeta{Idx,5}(1) ;
                    end
                end
                if SizeA_2 > 5
                    if ~isempty(G.ALP.Attributes.BoerBeta{Idx,6})
                        data = permute(data,G.ALP.Attributes.BoerBeta{Idx,6}) ;
                    end
                end
                if SizeA_2 > 6
                    if ~isempty(G.ALP.Attributes.BoerBeta{1,7})
                        R = G.ALP.Attributes.BoerBeta{1,7} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7
                    if ~isempty(G.ALP.Attributes.BoerBeta{1,8})
                        NanLimits = G.ALP.Attributes.BoerBeta{1,8} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
            else
                Attributes = G.ALP.Attributes.BoerBeta(Idx,:) ;
                if ismember(Ext,{'dnn','dds'})
                    data = ncread(urlwrite(Attributes{1,1}{1},G.ALP.tempnc),Attributes{1,2}{1},Attributes{1,3}{1},Attributes{1,4}{1}) ;
                else
                    data = ncread(Attributes{1,1}{1},Attributes{1,2}{1},Attributes{1,3}{1},Attributes{1,4}{1}) ;
                end
                if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                    if length(Attributes{1,5}{1}) > 1
                        data = data*Attributes{1,5}{1}(1) + Attributes{1,5}{1}(2) ;
                    else
                        data = data*Attributes{1,5}{1}(1) ;
                    end
                end
                if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                    if ~isempty(Attributes{1,6}{1})
                        data = permute(data,Attributes{1,6}{1}) ;
                    end
                end
                if SizeA_2 > 6 &&  ~isempty(Attributes{1,7})
                    if ~isempty(Attributes{1,7}{1})
                        R = Attributes{1,7}{1} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7 && ~isempty(Attributes{1,8})
                    if ~isempty(Attributes{1,8}{1})
                        NanLimits = Attributes{1,8}{1} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
                for j = 2:size(G.ALP.Attributes.BoerBeta{Idx,1},1)
                    if ismember(Ext,{'dnn','dds'})
                        data2 = ncread(urlwrite(Attributes{1,1}{j},G.ALP.tempnc),Attributes{1,2}{j},Attributes{1,3}{j},Attributes{1,4}{j}) ;
                    else
                        data2 = ncread(Attributes{1,1}{j},Attributes{1,2}{j},Attributes{1,3}{j},Attributes{1,4}{j}) ;
                    end
                    if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                        if length(Attributes{1,5}{j}) > 1
                            data2 = data2*Attributes{1,5}{j}(1) + Attributes{1,5}{j}(2) ;
                        else
                            data2 = data2*Attributes{1,5}{j}(1) ;
                        end
                    end
                    if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                        if ~isempty(Attributes{1,6}{j})
                            data2 = permute(data2,Attributes{j,6}{j}) ;
                        end
                    end
                    if SizeA_2 > 6  && ~isempty(Attributes{1,7})
                        if ~isempty(Attributes{1,7}{j})
                            R = Attributes{1,7}{j} ;
                            for r = 1:length(R)
                                switch R(r)
                                    case 1
                                        data2 = reshape(data2,[1,size(data2)]) ;
                                    case 2
                                        data2 = reshape(data2,[size(data2,1),1,size(data2,2)]) ;
                                end
                            end
                        end
                    end
                    if SizeA_2 > 7  && ~isempty(Attributes{1,8})
                        if ~isempty(Attributes{1,8}{j})
                            NanLimits = Attributes{1,8}{j} ;
                            if length(NanLimits) == 1
                                data2(data2 < NanLimits) = nan ;
                            else
                                data2(data2 < NanLimits(1) | data2 > NanLimits(2)) = nan ;
                            end
                        end
                    end
                    data = data + data2 ;
                end
            end
            
            if ismember(Ext,{'bds','bns','dds'})
                % data is surface pressure.
                dataP = repmat(data,[1 1 length(G.P)]) ;
                bata = ones(size(G.Vectors.levmat)) ;
                bata(G.Vectors.levmat*100 > dataP) = 0 ;
            else
                % data is a loaded 3D matrix with nans.
                bata = ones(size(data)) ;
                bata(isnan(data)) = 0 ;
            end
            return ;
        case {'ers','ern'}
        
            [~ , SizeA_2] = size(G.ALP.Attributes.BoerBeta) ;
            
            if size(G.ALP.Attributes.BoerBeta{Idx,1},1) == 0
                data = [] ;
                Warning = ['GOAT_load_data error: no attributes for - ',fileaddress] ;
                disp(Warning)  ;
                G.Error.LatestDisplayWarning = Warning ;
                return ;
            end
            if size(G.ALP.Attributes.BoerBeta{Idx,1},1) == 1
                try  delete(G.ALP.tempnc) ; end % verify older data is erased and will not be read as newly donwloaded data
                if SizeA_2>8 && ~isempty(G.ALP.Attributes.BoerBeta{Idx,9})
                perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.BoerBeta.MARSAttributes;{'target',G.ALP.tempnc ;'date',G.ALP.Attributes.BoerBeta{Idx,1} } ; G.ALP.Attributes.BoerBeta{Idx,9} ])) ;
                else
                perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.BoerBeta.MARSAttributes;{'target',G.ALP.tempnc ;'date',G.ALP.Attributes.BoerBeta{Idx,1} }])) ;    
                end
                data = ncread(G.ALP.tempnc,G.ALP.Attributes.BoerBeta{Idx,2},G.ALP.Attributes.BoerBeta{Idx,3},G.ALP.Attributes.BoerBeta{Idx,4}) ;
                if ~isempty(G.ALP.Attributes.BoerBeta{Idx,5}) % factor and offset [factor,offset]
                    if length(G.ALP.Attributes.BoerBeta{Idx,5}) > 1
                        data = data*G.ALP.Attributes.BoerBeta{Idx,5}(1) + G.ALP.Attributes.BoerBeta{Idx,5}(2) ;
                    else
                        data = data*G.ALP.Attributes.BoerBeta{Idx,5}(1) ;
                    end
                end
                if SizeA_2 > 5
                    if ~isempty(G.ALP.Attributes.BoerBeta{Idx,6})
                        data = permute(data,G.ALP.Attributes.BoerBeta{Idx,6}) ;
                    end
                end
                if SizeA_2 > 6
                    if ~isempty(G.ALP.Attributes.BoerBeta{1,7})
                        R = G.ALP.Attributes.BoerBeta{1,7} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7
                    if ~isempty(G.ALP.Attributes.BoerBeta{1,8})
                        NanLimits = G.ALP.Attributes.BoerBeta{1,8} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
            else
                Attributes = G.ALP.Attributes.BoerBeta(Idx,:) ;
                try  delete(G.ALP.tempnc) ; end % verify older data is erased and will not be read as newly donwloaded data
                if SizeA_2>8 && ~isempty(Attributes{1,9})
                perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.BoerBeta.MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{1} } ; Attributes{1,9}{1} ])) ;
                else
                perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.BoerBeta.MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{1} }])) ;    
                end
                data = ncread(G.ALP.tempnc,Attributes{1,2}{1},Attributes{1,3}{1},Attributes{1,4}{1}) ;
                if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                    if length(Attributes{1,5}{1}) > 1
                        data = data*Attributes{1,5}{1}(1) + Attributes{1,5}{1}(2) ;
                    else
                        data = data*Attributes{1,5}{1}(1) ;
                    end
                end
                if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                    if ~isempty(Attributes{1,6}{1})
                        data = permute(data,Attributes{1,6}{1}) ;
                    end
                end
                if SizeA_2 > 6  && ~isempty(Attributes{1,7})
                    if ~isempty(Attributes{1,7}{1})
                        R = Attributes{1,7}{1} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7 && ~isempty(Attributes{1,8})
                    if ~isempty(Attributes{1,8}{1})
                        NanLimits = G.ALP.Attributes.BoerBeta{1,8}{1} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
                for j = 2:size(G.ALP.Attributes.BoerBeta{Idx,1},1)
                    if Aidx == 1
                        if     ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                            set(G.h_message,'string',['Retrieving nc file from MARS server (frame ',int2str(j),' of ',int2str(size(G.ALP.Attributes.BoerBeta{Idx,1},1)),') . . .']) ; drawnow ; % Update Message
                        end
                    end
                    try  delete(G.ALP.tempnc) ; end % verify older data is erased and will not be read as newly donwloaded data
                    if SizeA_2>8 && ~isempty(Attributes{1,9})
                    perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.BoerBeta.MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{j} } ; Attributes{1,9}{j}])) ;
                    else
                    perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.BoerBeta.MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{j} }])) ;    
                    end
                    data2 = ncread(G.ALP.tempnc,Attributes{1,2}{j},Attributes{1,3}{j},Attributes{1,4}{j}) ;
                    if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                        if length(Attributes{1,5}{j}) > 1
                            data2 = data2*Attributes{1,5}{j}(1) + Attributes{1,5}{j}(2) ;
                        else
                            data2 = data2*Attributes{1,5}{j}(1) ;
                        end
                    end
                    if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                        if ~isempty(Attributes{1,6}{j})
                            data2 = permute(data2,Attributes{j,6}{j}) ;
                        end
                    end
                    if SizeA_2 > 6 && ~isempty(Attributes{1,7})
                        if ~isempty(Attributes{1,7}{j})
                            R = Attributes{1,7}{j} ;
                            for r = 1:length(R)
                                switch R(r)
                                    case 1
                                        data2 = reshape(data2,[1,size(data2)]) ;
                                    case 2
                                        data2 = reshape(data2,[size(data2,1),1,size(data2,2)]) ;
                                end
                            end
                        end
                    end
                    if SizeA_2 > 7 && ~isempty(Attributes{1,8})
                        if ~isempty(Attributes{1,8}{j})
                            NanLimits = Attributes{1,8}{j} ;
                            if length(NanLimits) == 1
                                data2(data2 < NanLimits) = nan ;
                            else
                                data2(data2 < NanLimits(1) | data2 > NanLimits(2)) = nan ;
                            end
                        end
                    end
                    data = data + data2 ;
                end
            end
            
            if ismember(Ext,{'ers'})
                % data is surface pressure.
                dataP = repmat(data,[1 1 length(G.lev)]) ;
                bata = ones(size(G.Vectors.levmat)) ;
                bata(G.Vectors.levmat*100 > dataP) = 0 ;
            else
                % data is a loaded 3D matrix with nans.
                bata = ones(size(data)) ;
                bata(isnan(data)) = 0 ;
            end
            return ;    
            
        otherwise
            GOAT_Display_warning(G,['GOAT_load_bata error: unrecognized bata linked file format - ',fileaddress(end-2:end)]) ; 
    end
end

return




