% GOAT_load_data central function for loading data into GOAT.
% data can be loaded from *.mat or *.nc files (in the future from opendap and possibly grib files as-well) ;
% copyright (C) Ori Adam, March.8.2014
%
%
function [data,G] = GOAT_load_data(fileaddress,G,Aidx)

if nargin == 1
    load(fileaddress) ; % assume is is a *.mat file by default
    return
else
    % Attributes(i,1) = filename ;
    % Attributes(i,2) = varname  ;
    % Attributes(i,3) = start    ;
    % Attributes(i,4) = count    ;
    % Attributes(i,5) = factor   ;
    % Attributes(i,6) = permute   ;
    % Attributes(i,7) = reshape (add dimensions)   ;
    % Attributes(i,8) = NanLimits (set to nan if data < NanLimits(1) | data > NanLimits(2)). If length(NanLimits)==1, data < NanLimits = nan.
    % Attributes(i,9) = Dummy ; 
    % Attributes(i,10) = frame string (Applicable only for List Calendar) 
    Idx = G.DataStringVec(Aidx) ;
    Ext = fileaddress(end-2:end) ;
    switch Ext
        case 'mat' % *.mat files
            load(fileaddress) ; % assume is is a *.mat file
            return
        case {'ncf','dap','.nc','dnc'}  % nc linked files
            
            if ~isfield(G.ALP.Attributes,G.DataField)
                G = GOAT_GetFieldSpan(G) ;
            end
            if Aidx == 1
                if  strcmp(Ext,'dap') && ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                    set(G.h_message,'string','Retrieving data from OPeNDAP server . . .') ; drawnow ; % Update Message
                elseif strcmp(Ext,'dnc') && ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                    set(G.h_message,'string','Retrieving nc file from server . . .') ; drawnow ; % Update Message
                end
            end
            [~ , SizeA_2] = size(G.ALP.Attributes.(G.DataField)) ;
            if size(G.ALP.Attributes.(G.DataField){Idx,1},1) == 0
                data = [] ;
                Warning = ['GOAT_load_data error: no attributes for - ',fileaddress] ;
                disp(Warning)  ;
                G.Error.LatestDisplayWarning = Warning ;
                return ;
            elseif size(G.ALP.Attributes.(G.DataField){Idx,1},1) == 1
                if strcmp(Ext,'dnc')
                    data = ncread(urlwrite(G.ALP.Attributes.(G.DataField){Idx,1},G.ALP.tempnc),G.ALP.Attributes.(G.DataField){Idx,2},G.ALP.Attributes.(G.DataField){Idx,3},G.ALP.Attributes.(G.DataField){Idx,4}) ;
                else
                    try
                    data = ncread(G.ALP.Attributes.(G.DataField){Idx,1},G.ALP.Attributes.(G.DataField){Idx,2},G.ALP.Attributes.(G.DataField){Idx,3},G.ALP.Attributes.(G.DataField){Idx,4}) ;
                    catch
                      if ~exist(G.ALP.Attributes.(G.DataField){Idx,1},'file')
                          disp(['GOAT_load_data error: could not locate file ',G.ALP.Attributes.(G.DataField){Idx,1}])
                      end
                    end
                end
                if ~isempty(G.ALP.Attributes.(G.DataField){Idx,5}) % factor and offset [factor,offset]
                    if length(G.ALP.Attributes.(G.DataField){Idx,5}) > 1
                        data = data*G.ALP.Attributes.(G.DataField){Idx,5}(1) + G.ALP.Attributes.(G.DataField){Idx,5}(2) ;
                    else
                        data = data*G.ALP.Attributes.(G.DataField){Idx,5}(1) ;
                    end
                end
                if SizeA_2 > 5
                    if ~isempty(G.ALP.Attributes.(G.DataField){Idx,6})
                        data = permute(data,G.ALP.Attributes.(G.DataField){Idx,6}) ;
                    end
                end
                if SizeA_2 > 6
                    if ~isempty(G.ALP.Attributes.(G.DataField){1,7})
                        R = G.ALP.Attributes.(G.DataField){1,7} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7
                    if ~isempty(G.ALP.Attributes.(G.DataField){1,8})
                        NanLimits = G.ALP.Attributes.(G.DataField){1,8} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
            else
                Attributes = G.ALP.Attributes.(G.DataField)(Idx,:) ;
                if strcmp(Ext,'dnc')
                    data = ncread(urlwrite(Attributes{1,1}{1},G.ALP.tempnc),Attributes{1,2}{1},Attributes{1,3}{1},Attributes{1,4}{1}) ;
                else
                    data = ncread(Attributes{1,1}{1},Attributes{1,2}{1},Attributes{1,3}{1},Attributes{1,4}{1}) ;
                end
                if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                    if length(Attributes{1,5}{1}) > 1
                        data = data*Attributes{1,5}{1}(1) + Attributes{1,5}{1}(2) ;
                    else
                        data = data*Attributes{1,5}{1}(1) ;
                    end
                end
                if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                    if ~isempty(Attributes{1,6}{1})
                        data = permute(data,Attributes{1,6}{1}) ;
                    end
                end
                if SizeA_2 > 6 && ~isempty(Attributes{1,7})
                    if ~isempty(Attributes{1,7}{1})
                        R = Attributes{1,7}{1} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7  && ~isempty(Attributes{1,8})
                    if ~isempty(Attributes{1,8}{1})
                        NanLimits = G.ALP.Attributes.(G.DataField){1,8}{1} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
                for j = 2:size(G.ALP.Attributes.(G.DataField){Idx,1},1)
                    if Aidx == 1
                        if     strcmp(Ext,'dap') && ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                            set(G.h_message,'string',['Retrieving data from OPeNDAP server (frame ',int2str(j),' of ',int2str(size(G.ALP.Attributes.(G.DataField){Idx,1},1)),') . . .']) ; drawnow ; % Update Message
                        elseif strcmp(Ext,'dnc') && ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                            set(G.h_message,'string',['Retrieving nc file from server (frame ',int2str(j),' of ',int2str(size(G.ALP.Attributes.(G.DataField){Idx,1},1)),') . . .']) ; drawnow ; % Update Message
                        end
                    end
                    if strcmp(Ext,'dnc')
                        data2 = ncread(urlwrite(Attributes{1,1}{j},G.ALP.tempnc),Attributes{1,2}{j},Attributes{1,3}{j},Attributes{1,4}{j}) ;
                    else
                        data2 = ncread(Attributes{1,1}{j},Attributes{1,2}{j},Attributes{1,3}{j},Attributes{1,4}{j}) ;
                    end
                    if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                        if length(Attributes{1,5}{j}) > 1
                            data2 = data2*Attributes{1,5}{j}(1) + Attributes{1,5}{j}(2) ;
                        else
                            data2 = data2*Attributes{1,5}{j}(1) ;
                        end
                    end
                    if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                        if ~isempty(Attributes{1,6}{j})
                            data2 = permute(data2,Attributes{j,6}{j}) ;
                        end
                    end
                    if SizeA_2 > 6  && ~isempty(Attributes{1,7})
                        if ~isempty(Attributes{1,7}{j})
                            R = Attributes{1,7}{j} ;
                            for r = 1:length(R)
                                switch R(r)
                                    case 1
                                        data2 = reshape(data2,[1,size(data2)]) ;
                                    case 2
                                        data2 = reshape(data2,[size(data2,1),1,size(data2,2)]) ;
                                end
                            end
                        end
                    end
                    if SizeA_2 > 7  && ~isempty(Attributes{1,8})
                        if ~isempty(Attributes{1,8}{j})
                            NanLimits = Attributes{1,8}{j} ;
                            if length(NanLimits) == 1
                                data2(data2 < NanLimits) = nan ;
                            else
                                data2(data2 < NanLimits(1) | data2 > NanLimits(2)) = nan ;
                            end
                        end
                    end
                    data = data + data2 ;
                end
            end
            return ;
        case 'eri'  % ERA-Interim download protocol using perl scripts
            if ~isfield(G.ALP.Attributes,G.DataField)
                G = GOAT_GetFieldSpan(G) ;
            end
            if Aidx == 1
                if  ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                    set(G.h_message,'string','Retrieving nc file from MARS server . . .') ; drawnow ; % Update Message
                end
            end
            [~ , SizeA_2] = size(G.ALP.Attributes.(G.DataField)) ;
            
            if size(G.ALP.Attributes.(G.DataField){Idx,1},1) == 0
                data = [] ;
                Warning = ['GOAT_load_data error: no attributes for - ',fileaddress] ;
                disp(Warning)  ;
                G.Error.LatestDisplayWarning = Warning ;
                return ;
            end
            if size(G.ALP.Attributes.(G.DataField){Idx,1},1) == 1
                try  delete(G.ALP.tempnc) ; end % verify older data is erased and will not be read as newly donwloaded data
                if SizeA_2>8 && ~isempty(G.ALP.Attributes.(G.DataField){Idx,9})
                    try
                    perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.(G.DataField).MARSAttributes;{'target',G.ALP.tempnc ;'date',G.ALP.Attributes.(G.DataField){Idx,1} } ; G.ALP.Attributes.(G.DataField){Idx,9} ])) ;
                    catch PERL
                    if  ~G.EnvironmentFlag 
                    set(G.h_message,'string','Failure while attempting to retrieve nc file from MARS server . . .') ; drawnow ; % Update Message
%                     else
%                     cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;    
                    end 
                    cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;    
                    disp(PERL.message)  
                    data = [] ; 
                    return
                  end
                else
                  try 
                  perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.(G.DataField).MARSAttributes;{'target',G.ALP.tempnc ;'date',G.ALP.Attributes.(G.DataField){Idx,1} }])) ;
                  catch PERL
                    if  ~G.EnvironmentFlag 
                    set(G.h_message,'string','Failure while attempting to retrieve nc file from MARS server . . .') ; drawnow ; % Update Message
%                     else
%                     cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;        
                    end
                    cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;    
                    disp(PERL.message)
                    data = [] ; 
                    return
                  end
                end
                data = ncread(G.ALP.tempnc,G.ALP.Attributes.(G.DataField){Idx,2},G.ALP.Attributes.(G.DataField){Idx,3},G.ALP.Attributes.(G.DataField){Idx,4}) ;
                if ~isempty(G.ALP.Attributes.(G.DataField){Idx,5}) % factor and offset [factor,offset]
                    if length(G.ALP.Attributes.(G.DataField){Idx,5}) > 1
                        data = data*G.ALP.Attributes.(G.DataField){Idx,5}(1) + G.ALP.Attributes.(G.DataField){Idx,5}(2) ;
                    else
                        data = data*G.ALP.Attributes.(G.DataField){Idx,5}(1) ;
                    end
                end
                if SizeA_2 > 5
                    if ~isempty(G.ALP.Attributes.(G.DataField){Idx,6})
                        data = permute(data,G.ALP.Attributes.(G.DataField){Idx,6}) ;
                    end
                end
                if SizeA_2 > 6
                    if ~isempty(G.ALP.Attributes.(G.DataField){1,7})
                        R = G.ALP.Attributes.(G.DataField){1,7} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7
                    if ~isempty(G.ALP.Attributes.(G.DataField){1,8})
                        NanLimits = G.ALP.Attributes.(G.DataField){1,8} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
            else
                Attributes = G.ALP.Attributes.(G.DataField)(Idx,:) ;
                try  delete(G.ALP.tempnc) ; end % verify older data is erased and will not be read as newly donwloaded data
                if SizeA_2>8 && ~isempty(Attributes{1,9})
                    try
                    perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.(G.DataField).MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{1} } ; Attributes{1,9}{1} ])) ;
                    catch PERL
                    if  ~G.EnvironmentFlag 
                    set(G.h_message,'string','Failure while attempting to retrieve nc file from MARS server . . .') ; drawnow ; % Update Message
%                     else
%                     cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;        
                    end 
                    cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;        
                    disp(PERL.message)  
                    data = [] ; 
                    return
                  end
                else
                    try
                        perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.(G.DataField).MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{1} }])) ;
                    catch PERL
                    if  ~G.EnvironmentFlag 
                    set(G.h_message,'string','Failure while attempting to retrieve nc file from MARS server . . .') ; drawnow ; % Update Message
                    %else
                    %cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;        
                    end 
                    cprintf('err','Failure while attempting to retrieve nc file from MARS server . . .\n') ;        
                    disp(PERL.message)  
                    data = [] ; 
                    return
                  end
                end
                data = ncread(G.ALP.tempnc,Attributes{1,2}{1},Attributes{1,3}{1},Attributes{1,4}{1}) ;
                if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                    if length(Attributes{1,5}{1}) > 1
                        data = data*Attributes{1,5}{1}(1) + Attributes{1,5}{1}(2) ;
                    else
                        data = data*Attributes{1,5}{1}(1) ;
                    end
                end
                if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                    if ~isempty(Attributes{1,6}{1})
                        data = permute(data,Attributes{1,6}{1}) ;
                    end
                end
                if SizeA_2 > 6  && ~isempty(Attributes{1,7})
                    if ~isempty(Attributes{1,7}{1})
                        R = Attributes{1,7}{1} ;
                        for r = 1:length(R)
                            switch R(r)
                                case 1
                                    data = reshape(data,[1,size(data)]) ;
                                case 2
                                    data = reshape(data,[size(data,1),1,size(data,2)]) ;
                            end
                        end
                    end
                end
                if SizeA_2 > 7 && ~isempty(Attributes{1,8})
                    if ~isempty(Attributes{1,8}{1})
                        NanLimits = G.ALP.Attributes.(G.DataField){1,8}{1} ;
                        if length(NanLimits) == 1
                            data(data < NanLimits) = nan ;
                        else
                            data(data < NanLimits(1) | data > NanLimits(2)) = nan ;
                        end
                    end
                end
                for j = 2:size(G.ALP.Attributes.(G.DataField){Idx,1},1)
                    if Aidx == 1
                        if     ~G.EnvironmentFlag && ~(G.ProjectionType == 10)
                            set(G.h_message,'string',['Retrieving nc file from MARS server (frame ',int2str(j),' of ',int2str(size(G.ALP.Attributes.(G.DataField){Idx,1},1)),') . . .']) ; drawnow ; % Update Message
                        end
                    end
                    try  delete(G.ALP.tempnc) ; end % verify older data is erased and will not be read as newly donwloaded data
                    if SizeA_2>8 && ~isempty(Attributes{1,9})
                    perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.(G.DataField).MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{j} } ; Attributes{1,9}{j}])) ;
                    else
                    perl(GOAT_ERA_MARS_retrieve2perl([G.ALP.Defs.(G.DataField).MARSAttributes;{'target',G.ALP.tempnc ;'date',Attributes{1,1}{j} }])) ;    
                    end
                    data2 = ncread(G.ALP.tempnc,Attributes{1,2}{j},Attributes{1,3}{j},Attributes{1,4}{j}) ;
                    if ~isempty(Attributes{1,5}) % factor and offset [factor,offset]
                        if length(Attributes{1,5}{j}) > 1
                            data2 = data2*Attributes{1,5}{j}(1) + Attributes{1,5}{j}(2) ;
                        else
                            data2 = data2*Attributes{1,5}{j}(1) ;
                        end
                    end
                    if SizeA_2 > 5 && ~isempty(Attributes{1,6})
                        if ~isempty(Attributes{1,6}{j})
                            data2 = permute(data2,Attributes{j,6}{j}) ;
                        end
                    end
                    if SizeA_2 > 6 && ~isempty(Attributes{1,7})
                        if ~isempty(Attributes{1,7}{j})
                            R = Attributes{1,7}{j} ;
                            for r = 1:length(R)
                                switch R(r)
                                    case 1
                                        data2 = reshape(data2,[1,size(data2)]) ;
                                    case 2
                                        data2 = reshape(data2,[size(data2,1),1,size(data2,2)]) ;
                                end
                            end
                        end
                    end
                    if SizeA_2 > 7 && ~isempty(Attributes{1,8})
                        if ~isempty(Attributes{1,8}{j})
                            NanLimits = Attributes{1,8}{j} ;
                            if length(NanLimits) == 1
                                data2(data2 < NanLimits) = nan ;
                            else
                                data2(data2 < NanLimits(1) | data2 > NanLimits(2)) = nan ;
                            end
                        end
                    end
                    data = data + data2 ;
                end
            end
            return ;
            % TBD
        case 'hdf'  % hdf linked files
            % TBD
        case 'opd'  % opendap linked files
            % TBD
        case 'gb1'  % grib 1 linked files
            % TBD
        case 'gb2'  % grib 2 linked files
            % TBD
        otherwise
            GOAT_Display_warning(G,['GOAT_load_data error: unrecognized linked file format - ',fileaddress(end-2:end)])
    end
end
return




