% GOAT_plotUpdate GOAT main GUI plot function
% Copyright (C) Ori Adam Jan.2012.
% syntax:
% >> GOAT_plotUpdate(G,Defs)
% Defs = User specified definitions structure
% see also GOAT_Defaults, GOAT_SignificancePlot, GOAT_darken_edge

function [G, C, h] = GOAT_plotUpdate(G,Defs)
if nargin < 2
    Defs.contour_levels   = [] ; 
    Defs.longitude_factor = [] ; 
    Defs.subplotstr       = [] ; 
else
    if ~isfield(Defs,'contour_levels') 
        Defs.contour_levels       = [] ; 
    end
    if ~isfield(Defs,'longitude_factor') 
        Defs.longitude_factor     = [] ; 
    end
    if ~isfield(Defs,'subplotstr') 
        Defs.subplotstr           = [] ; 
    end
end
try
Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
catch
    Lat1 = min(G.y) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    Lat2 = max(G.y) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
end
if G.Settings.SinLatFlag && any(G.ProjectionType == [1,2,3,6])
    Lat1 = sind(Lat1) ;
    Lat2 = sind(Lat2) ;
end

try
    Lon1 = min(G.x) ; % min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    Lon2 = max(G.x) ; % max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
catch
    Lon1 = min([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
    Lon2 = max([G.lon(G.Limits.Default.Lon1),G.lon(G.Limits.Default.Lon2)]) ;
end
try
P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
catch
P1 = 1 ; P2 = 1 ; 
end

try
    if ~isempty(Defs.subplotstr) 
       eval(Defs.subplotstr) ;  
    elseif G.Plotting.GUI_PlotFlag
        subplot(1,10,[2 10])
    end
catch
    % Nevermind, PlotUpdate is used outside GUI
end

try
    ContourIdx = G.Settings.ContourIdx ;
catch
    ContourIdx = 1 ;
end
try
    LineWidth      = G.Settings.LineWidth      ;
    CoastLineWidth = G.Settings.CoastLineWidth ;
catch
    LineWidth      = 2 ;
    CoastLineWidth = 1 ;
end
try
    CoastLineColor = G.Settings.CoastLineColor ;
catch
    CoastLineColor = [.1 .1 .1] ;
end
if ContourIdx == 2 && LineWidth <=0
    LineWidth = 2 ;
end
try
    ColorMapStr = cell2mat(G.Settings.ColorMap(G.Settings.ColorMapIdx)) ;
    switch ColorMapStr
        case {'jet','hsv','Lines','Pink','Copper','Bone','Gray','Winter','Autumn','Spring','summer','Cool','Hot'}
            ColorMapDef = ColorMapStr ;
        otherwise
            eval(['ColorMapDef = G.Settings.ColorMaps.',ColorMapStr,' ; '])
    end
catch
    ColorMapDef = 'jet' ;
end
% Land Cover
if any(G.ProjectionType == [1,2])
    if G.Settings.SinLatFlag
        landcoverlat = sind(G.topo.landcoverlat) ;
        if ~isempty(G.topo.AntLatAppendix)
            AntLatAppendix = sind(G.topo.AntLatAppendix) ;
        else
            AntLatAppendix = G.topo.AntLatAppendix ;
        end
        try
        latG = sind(G.topo.latG) ; 
        end
    else
        landcoverlat = G.topo.landcoverlat ;
        AntLatAppendix = G.topo.AntLatAppendix ;
        try
        latG = G.topo.latG ; 
        end
    end
end
% Longitude factor
landcoverlon = G.topo.landcoverlon ;
AntLonAppendix = G.topo.AntLonAppendix ;
lonG = G.topo.lonG ;
if ~isempty(Defs.longitude_factor)
    LonLength = 360/Defs.longitude_factor ; 
else
    LonLength = 360 ; 
end
switch G.ProjectionType
    case {1,2} % Long-Lat
        try
            if ~G.Settings.SinLatFlag
                YL = G.Y ; 
            else
                YL = sind(G.Y) ; 
            end
            switch ContourIdx
                case 1
                    if ~isempty(Defs.contour_levels)
                        [C,h] =  contourf(G.X,YL,G.Data',Defs.contour_levels) ;
                    else
                        [C,h] =  contourf(G.X,YL,G.Data') ;
                    end
                case 2
                    if  ~isempty(Defs.contour_levels)
                        [C,h] =  contour(G.X,YL,G.Data',Defs.contour_levels) ;
                    else
                        [C,h] =  contour(G.X,YL,G.Data') ;
                    end
                case 3
                    h =  surf(double(G.X),double(YL),double(G.Data')) ;
                case 4
                    h =  surfc(double(G.X),double(YL),double(G.Data')) ;
                case 5
                    h =  mesh(double(G.X),double(YL),double(G.Data')) ;
                case 6
                    h =  meshc(double(G.X),double(YL),double(G.Data')) ;
                case 7
                    h =  meshz(double(G.X),double(YL),double(G.Data')) ;
                case 8
                    if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                        h =  imagesc([Lon1 Lon2],[Lat1 Lat2],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                    else
                        h =  imagesc([Lon1 Lon2],[Lat1 Lat2],double(G.Data')) ;
                    end
                    set(gca,'YDir','normal') ;
            end
%             else
%                 switch ContourIdx
%                     case 1
%                         if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
%                         [C,h] =  contourf(G.X,sind(G.Y),G.Data',Defs.contour_levels) ;
%                         else
%                         [C,h] =  contourf(G.X,sind(G.Y),G.Data') ;    
%                         end
%                     case 2
%                         if isfield(Defs,'contour_levels') && ~isempty(Defs.contour_levels)
%                         [C,h] =  contour(G.X,sind(G.Y),G.Data',Defs.contour_levels) ;
%                         else
%                         [C,h] =  contour(G.X,sind(G.Y),G.Data') ;    
%                         end
%                     case 3
%                         h =  surf(double(G.X),sind(double(G.Y)),double(G.Data')) ;
%                     case 4
%                         h =  surfc(double(G.X),sind(double(G.Y)),double(G.Data')) ;
%                     case 5
%                         h =  mesh(double(G.X),sind(double(G.Y)),double(G.Data')) ;
%                     case 6
%                         h =  meshc(double(G.X),sind(double(G.Y)),double(G.Data')) ;
%                     case 7
%                         h =  meshz(double(G.X),sind(double(G.Y)),double(G.Data')) ;
%                     case 8
%                         if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
%                             h =  imagesc([Lon1 Lon2],[Lat1 Lat2],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
%                         else
%                             h =  imagesc([Lon1 Lon2],[Lat1 Lat2],double(G.Data')) ;
%                         end
%                         set(gca,'YDir','normal') ;
%                 end
%             end
            if G.Settings.ReverseColorMap
                cmap = colormap(ColorMapDef) ;
                colormap(flipud(cmap)) ;
            else
                colormap(ColorMapDef) ;
            end
            
            G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;hold on
            G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            %clabel(C,h,'fontsize',13) ;
            if LineWidth > 0
                try  set(h,'linewidth',LineWidth) ; end
            elseif LineWidth == 0
                try  set(h,'linestyle','none') ; end
            else % < 0 means darken edge
                % GOAT_darken_edge(h,colormap) ;
            end
            %xlabel('$$\textrm{Longitude}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
            %ylabel('$$\textrm{Latitude}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
            %             try
            %             xlabel('Longitude','Fontsize',G.Display.TextSize_L,'fontname','MT Extra') ;
            %             ylabel('Latitude','Fontsize',G.Display.TextSize_L,'fontname','MT Extra') ;
            %             catch
            
            G.Display.xlabelhandle = xlabel(G.lonlabel,'Fontsize',G.Display.TextSize_XL) ;
            %xlabel(G.lonlabel,'Fontsize',18,'fontname','Lucida Grande') ;
            G.Display.ylabelhandle = ylabel(G.latlabel,'Fontsize',G.Display.TextSize_XL) ;
            if G.Settings.SinLatFlag
                yl = get(gca,'ytick') ; 
                ticklabels = cell(size(yl)) ; 
                for i = 1:length(yl)
                  if yl(i) == 0 
                      ticklabels(i) = {'0'} ;
                  elseif yl(i) > 0
                      ticklabels(i) = {[int2str(abs(round(asind(yl(i))))),'N']} ;
                  else
                      ticklabels(i) = {[int2str(abs(round(asind(yl(i))))),'S']} ;
                  end
                end
                set(gca,'yticklabels',ticklabels) ; 
                %ylabel(['sin( ',G.latlabel,' )'],'Fontsize',G.Display.TextSize_XL) ;
                %ylabel(['sin(',G.latlabel,')'],'Fontsize',18,'fontname','Lucida Grande') ;
%             else
%                 ylabel(G.latlabel,'Fontsize',G.Display.TextSize_XL) ;
%                 %ylabel(G.latlabel,'Fontsize',20,'fontname','Lucida Grande') ;
            end
            
            %end
            if G.ProjectionType == 1
%                if G.Settings.SinLatFlag
                    if strcmpi(G.topo.CoastView,'CoastLine') ;
                        plot(landcoverlon,landcoverlat,landcoverlon+LonLength,landcoverlat,'color',CoastLineColor,'Linewidth',CoastLineWidth) ;
                    else
                        hold all
                        l = 1:G.topo.CoastNans(1)-1 ;
                        if ~isempty(AntLonAppendix)
                            fh = fill([landcoverlon(l) ; AntLonAppendix],[landcoverlat(l) ; AntLatAppendix],G.topo.ShadeColor,[landcoverlon(l) ; AntLonAppendix]+LonLength,[landcoverlat(l) ; AntLatAppendix],G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                        else
                            fh = fill(landcoverlon(l)-LonLength,landcoverlat(l),G.topo.ShadeColor,landcoverlon(l),landcoverlat(l),G.topo.ShadeColor,landcoverlon(l)+LonLength,landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                        end
                        if length(G.topo.CoastNans) > 1
                            for i = 1:length(G.topo.CoastNans) -1
                                l = G.topo.CoastNans(i)+1:G.topo.CoastNans(i+1)-1 ;
                                if any(i == G.topo.LakesIdx)
                                    fh = fill(landcoverlon(l)-LonLength,landcoverlat(l),'w',landcoverlon(l),landcoverlat(l),'w',landcoverlon(l)+LonLength,landcoverlat(l),'w','edgecolor','none') ; alpha(fh,G.topo.Lakealpha) ; hold on
                                else
                                    fh = fill(landcoverlon(l)-LonLength,landcoverlat(l),G.topo.ShadeColor,landcoverlon(l),landcoverlat(l),G.topo.ShadeColor,landcoverlon(l)+LonLength,landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                                end
                            end
                            l = G.topo.CoastNans(end)+1:length(landcoverlat) ;
                            fh = fill(landcoverlon(l)-LonLength,landcoverlat(l),G.topo.ShadeColor,landcoverlon(l),landcoverlat(l),G.topo.ShadeColor,landcoverlon(l)+LonLength,landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
                        end
                    end
                    if CoastLineWidth > 0
                        plot(landcoverlon-LonLength,landcoverlat,landcoverlon,landcoverlat,landcoverlon+LonLength,landcoverlat,'color',CoastLineColor,'Linewidth',CoastLineWidth) ;
                    end
                    if G.topo.TypeFlag > 1
                        try
                            switch G.topo.TypeFlag
                                case 2
                                    if Lon1 < 0
                                        [Ctopo, htopo] = contour([lonG'-LonLength ; lonG'],latG ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                                    else
                                        [Ctopo, htopo] = contour(lonG,latG,G.topo.topoA,G.topo.topovec) ; hold on
                                    end
                                case 3
                                    if Lon1 < 0
                                        [Ctopo, htopo] = contour([lonG'-LonLength ; lonG'],latG ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                                    else
                                        [Ctopo, htopo] = contour(lonG,latG,G.topo.topoO,G.topo.topovec) ; hold on
                                    end
                                case 4
                                    if Lon1 < 0
                                        [Ctopo, htopo] = contour([lonG'-LonLength ; lonG'],latG ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                                    else
                                        [Ctopo, htopo] = contour(lonG,latG,G.topo.topoG,G.topo.topovec) ; hold on
                                    end
                            end
                            set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
                        catch
                            % Null topography
                        end
                    end
%                 else
%                     if strcmpi(G.topo.CoastView,'CoastLine') ;
%                         plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
% %                     elseif strcmpi(G.topo.CoastView,'Mask') ;
% %                         hold all ; 
% %                         hm = imshow(G.topo.ocean_mask',G.topo.maskRI) ; alpha(.6) ; 
%                     else
%                         hold all
%                         l = 1:G.topo.CoastNans(1)-1 ;
%                         if ~isempty(G.topo.AntLonAppendix)
%                             fh = fill([G.topo.landcoverlon(l) ; G.topo.AntLonAppendix],[G.topo.landcoverlat(l) ; G.topo.AntLatAppendix],G.topo.ShadeColor,[G.topo.landcoverlon(l) ; G.topo.AntLonAppendix]+360,[G.topo.landcoverlat(l) ; G.topo.AntLatAppendix],G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
%                         else
%                             fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l),G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
%                         end
%                         if length(G.topo.CoastNans) > 1
%                             for i = 1:length(G.topo.CoastNans) -1
%                                 l = G.topo.CoastNans(i)+1:G.topo.CoastNans(i+1)-1 ;
%                                 if any(i == G.topo.LakesIdx)
%                                     fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),'w',G.topo.landcoverlon(l),G.topo.landcoverlat(l),'w',G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),'w','edgecolor','none') ; alpha(fh,G.topo.Lakealpha) ; hold on
%                                 else
%                                     fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l),G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
%                                 end
%                             end
%                             l = G.topo.CoastNans(end)+1:length(G.topo.landcoverlat) ;
%                             fh = fill(G.topo.landcoverlon(l)-360,G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l),G.topo.landcoverlat(l),G.topo.ShadeColor,G.topo.landcoverlon(l)+360,G.topo.landcoverlat(l),G.topo.ShadeColor,'edgecolor','none') ; alpha(fh,G.topo.alpha) ; hold on
%                         end
%                     end
%                     if CoastLineWidth > 0
%                         plot(G.topo.landcoverlon-360,G.topo.landcoverlat,'k',G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
%                     end
%                     if G.topo.TypeFlag > 1
%                         try
%                             switch G.topo.TypeFlag
%                                 case 2
%                                     if Lon1 < 0
%                                         [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
%                                     else
%                                         [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoA,G.topo.topovec) ; hold on
%                                     end
%                                 case 3
%                                     if Lon1 < 0
%                                         [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
%                                     else
%                                         [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoO,G.topo.topovec) ; hold on
%                                     end
%                                 case 4
%                                     if Lon1 < 0
%                                         [Ctopo, htopo] = contour([G.topo.lonG'-360 ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
%                                     else
%                                         [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoG,G.topo.topovec) ; hold on
%                                     end
%                             end
%                             set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
%                         catch
%                             % Null topography
%                         end
%                     end
%                 end
            else
%                if G.Settings.SinLatFlag
                    if CoastLineWidth > 0
                        plot(landcoverlon-LonLength,landcoverlat,landcoverlon,landcoverlat,landcoverlon+LonLength,landcoverlat,'color',CoastLineColor,'Linewidth',CoastLineWidth) ;
                    end
                    if G.topo.TypeFlag > 1 && CoastLineWidth > 0
                        try
                            switch G.topo.TypeFlag
                                case 2
                                    if Lon1 < 0
                                        [Ctopo, htopo] = contour([lonG'-LonLength ; lonG'],latG ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                                    else
                                        [Ctopo, htopo] = contour(lonG,latG,G.topo.topoA,G.topo.topovec) ; hold on
                                    end
                                case 3
                                    if Lon1 < 0
                                        [Ctopo, htopo] = contour([lonG'-LonLength ;lonG'],latG ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                                    else
                                        [Ctopo, htopo] = contour(lonG,latG,G.topo.topoO,G.topo.topovec) ; hold on
                                    end
                                case 4
                                    if Lon1 < 0
                                        [Ctopo, htopo] = contour([lonG'-LonLength ; lonG'],latG ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                                    else
                                        [Ctopo, htopo] = contour(lonG,latG,G.topo.topoG,G.topo.topovec) ; hold on
                                    end
                            end
                            set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
                        catch
                            % Null topography
                        end
                    end
%                 else
%                     if CoastLineWidth > 0
%                         plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+LonLength,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ;
%                     end
%                     if G.topo.TypeFlag > 1 && CoastLineWidth > 0
%                         try
%                             switch G.topo.TypeFlag
%                                 case 2
%                                     if Lon1 < 0
%                                         [Ctopo, htopo] = contour([G.topo.lonG'-LonLength ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
%                                     else
%                                         [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoA,G.topo.topovec) ; hold on
%                                     end
%                                 case 3
%                                     if Lon1 < 0
%                                         [Ctopo, htopo] = contour([G.topo.lonG'-LonLength ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
%                                     else
%                                         [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoO,G.topo.topovec) ; hold on
%                                     end
%                                 case 4
%                                     if Lon1 < 0
%                                         [Ctopo, htopo] = contour([G.topo.lonG'-LonLength ; G.topo.lonG'],[G.topo.latG] ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
%                                     else
%                                         [Ctopo, htopo] = contour(G.topo.lonG,G.topo.latG,G.topo.topoG,G.topo.topovec) ; hold on
%                                     end
%                             end
%                             set(htopo,'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.25 .25 .25]) ;
%                         catch
%                             % Null topography
%                         end
%                     end
%                 end
            end
            xlim([Lon1 Lon2])   ;
            ylim([Lat1 Lat2])   ;
        catch
            hold off ;
            % Nevermind, no Data or Opening Screen
            %             if G.ProjectionType == 1
            %                 plot(G.topo.landcoverlon,G.topo.landcoverlat,'k',G.topo.landcoverlon+360,G.topo.landcoverlat,'k','Linewidth',CoastLineWidth) ; hold on
            %                 image([Lon1 Lon2],[90 -90],imread(G.Display.BackgroundPic)) ;
            %             else
            if G.Settings.SinLatFlag
                image([0 360],[-1 1],imread(G.Display.BackgroundPic)) ;
            else
                image([0 360],[-90 90],imread(G.Display.BackgroundPic)) ;
            end
            %            end
            %            alpha(G.Display.BackgroundAlpha);
        end
        hold off
        if Lon1~=Lon2
            xlim([Lon1 Lon2])   ;
        else
            xlim([Lon1 Lon2+.1]) ;
        end
        ylim([Lat1 Lat2])   ;
    case 3 % Lat-Pressure
        try
            if G.Limits.Default.sameP || P2 == P1
                if LineWidth <= 0 ; LineWidth = 2 ; end
                if G.Settings.SinLatFlag
                    plot(sind(G.X),G.Data,'k','linewidth',LineWidth) ;
                    xlabel(['$$\textrm{sin( ',G.latlabel,' )}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                else
                    plot(G.X,G.Data,'k','linewidth',LineWidth) ;
                    xlabel(['$$\textrm{',G.latlabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                end
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                xlim([Lat1 Lat2])   ;
            elseif Lat1 == Lat2
                if LineWidth <= 0 ; LineWidth = 2 ; end
                plot(G.Y,G.Data,'k','linewidth',LineWidth) ;
                xlabel(['$$\textrm{',G.Plabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                xlim([P1 P2])   ;
            else
                if G.Settings.SinLatFlag
                    switch ContourIdx
                        case 1
                            if G.Settings.LogLevelsFlag
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contourf(sind(G.X),log(single(G.Y)),G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contourf(sind(G.X),log(single(G.Y)),G.Data') ;    
                                end
                            else
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contourf(sind(G.X),G.Y,G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contourf(sind(G.X),G.Y,G.Data') ;    
                                end
                            end
                        case 2
                            if G.Settings.LogLevelsFlag
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contour(sind(G.X),log(single(G.Y)),G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contour(sind(G.X),log(single(G.Y)),G.Data') ;    
                                end
                            else
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contour(sind(G.X),G.Y,G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contour(sind(G.X),G.Y,G.Data') ;    
                                end
                            end
                        case 3
                            if G.Settings.LogLevelsFlag
                                h =  surf(sind(double(G.X)),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  surf(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                            end
                        case 4
                            if G.Settings.LogLevelsFlag
                                h =  surfc(sind(double(G.X)),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  surfc(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                            end
                        case 5
                            if G.Settings.LogLevelsFlag
                                h =  mesh(sind(double(G.X)),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  mesh(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                            end
                        case 6
                            if G.Settings.LogLevelsFlag
                                h =  meshc(sind(double(G.X)),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  meshc(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                            end
                        case 7
                            if G.Settings.LogLevelsFlag
                                h =  meshz(sind(double(G.X)),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  meshz(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                            end
                        case 8
                            if G.Settings.LogLevelsFlag
                                if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                    h =  imagesc([Lat1 Lat2],[log(single(P1)) log(single(P2))],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                                else
                                    h =  imagesc([Lat1 Lat2],[log(single(P1)) log(single(P2))],double(G.Data')) ;
                                end
                            else
                                if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                    h =  imagesc([Lat1 Lat2],[P1 P2],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                                else
                                    h =  imagesc([Lat1 Lat2],[P1 P2],double(G.Data')) ;
                                end
                            end
                    end
                else
                    switch ContourIdx
                        case 1
                            if G.Settings.LogLevelsFlag
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contourf(G.X,log(single(G.Y)),G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contourf(G.X,log(single(G.Y)),G.Data') ;    
                                end
                            else
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contourf(G.X,G.Y,G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contourf(G.X,G.Y,G.Data') ;    
                                end
                            end
                        case 2
                            if G.Settings.LogLevelsFlag
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contour(G.X,log(single(G.Y)),G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contour(G.X,log(single(G.Y)),G.Data') ;    
                                end
                             else
                                if ~isempty(Defs.contour_levels)
                                [C,h] =  contour(G.X,G.Y,G.Data',Defs.contour_levels) ;
                                else
                                [C,h] =  contour(G.X,G.Y,G.Data') ;    
                                end
                            end
                        case 3
                            if G.Settings.LogLevelsFlag
                                h =  surf(double(G.X),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  surf(double(G.X),double(G.Y),double(G.Data')) ;
                            end
                        case 4
                            if G.Settings.LogLevelsFlag
                                h =  surfc(double(G.X),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  surfc(double(G.X),double(G.Y),double(G.Data')) ;
                            end
                        case 5
                            if G.Settings.LogLevelsFlag
                                h =  mesh(double(G.X),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  mesh(double(G.X),double(G.Y),double(G.Data')) ;
                            end
                        case 6
                            if G.Settings.LogLevelsFlag
                                h =  meshc(double(G.X),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  meshc(double(G.X),double(G.Y),double(G.Data')) ;
                            end
                        case 7
                            if G.Settings.LogLevelsFlag
                                h =  meshz(double(G.X),log(double(G.Y)),double(G.Data')) ;
                            else
                                h =  meshz(double(G.X),double(G.Y),double(G.Data')) ;
                            end
                        case 8
                            if G.Settings.LogLevelsFlag
                                if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                    h =  imagesc([Lat1 Lat2],[log(single(P1)) log(single(P2))],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                                else
                                    h =  imagesc([Lat1 Lat2],[log(single(P1)) log(single(P2))],double(G.Data')) ;
                                end
                            else
                                if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                    h =  imagesc([Lat1 Lat2],[P1 P2],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                                else
                                    h =  imagesc([Lat1 Lat2],[P1 P2],double(G.Data')) ;
                                end
                            end
                    end
                end
                if G.Settings.ReverseColorMap
                    cmap = colormap(ColorMapDef) ;
                    colormap(flipud(cmap)) ;
                else
                    colormap(ColorMapDef) ;
                end
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                elseif LineWidth == 0
                    try  set(h,'linestyle','none') ; end
                else % < 0 means darken edge
                    %GOAT_darken_edge(h,colormap) ;
                end
                try
                    if any(G.Settings.LevelsIdx == [1,2,5,7])
                        axis ij
                    end
                catch
                    axis ij
                end
                if G.Settings.SinLatFlag
                    G.Display.xlabelhandle = xlabel(['$$\textrm{sin( ',G.latlabel,' )}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                else
                    G.Display.xlabelhandle = xlabel(['$$\textrm{',G.latlabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                end
                G.Display.ylabelhandle = ylabel(['$$\textrm{',G.Plabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                if G.Settings.LogLevelsFlag
                    ylim([log(single(P1)) log(single(P2))])
                else
                    ylim([P1 P2]) ;
                end
                xlim([Lat1 Lat2])   ;
            end
        catch
            % Nevermind, no Data
        end
    case 4 % Lon-Pressure
        try
            if G.Limits.Default.sameP || P2 == P1
                if LineWidth <= 0 ; LineWidth = 2 ; end
                plot(G.X,G.Data,'k','linewidth',LineWidth) ;
                G.Display.xlabelhandle = xlabel(['$$\textrm{',G.lonlabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.ylabelhandle = ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                xlim([Lon1 Lon2])   ;
            elseif Lon1 == Lon2
                if LineWidth <= 0 ; LineWidth = 2 ; end
                plot(G.Y,G.Data,'k','linewidth',LineWidth) ;
                G.Display.xlabelhandle = xlabel(['$$\textrm{',G.Plabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.ylabelhandle = ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                xlim([P1 P2])   ;
            else
                switch ContourIdx
                    case 1
                        if G.Settings.LogLevelsFlag
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contourf(G.X,log(single(G.Y)),G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contourf(G.X,log(single(G.Y)),G.Data') ;    
                            end
                        else
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contourf(G.X,G.Y,G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contourf(G.X,G.Y,G.Data') ;    
                            end
                        end
                    case 2
                        if G.Settings.LogLevelsFlag
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contour(G.X,log(single(G.Y)),G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contour(G.X,log(single(G.Y)),G.Data') ;    
                            end
                        else
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contour(G.X,G.Y,G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contour(G.X,G.Y,G.Data') ;    
                            end
                        end
                    case 3
                        if G.Settings.LogLevelsFlag
                            h =  surf(double(G.X),log(double(G.Y)),double(G.Data')) ;
                        else
                            h =  surf(double(G.X),double(G.Y),double(G.Data')) ;
                        end
                    case 4
                        if G.Settings.LogLevelsFlag
                            h =  surfc(double(G.X),log(double(G.Y)),double(G.Data')) ;
                        else
                            h =  surfc(double(G.X),double(G.Y),double(G.Data')) ;
                        end
                    case 5
                        if G.Settings.LogLevelsFlag
                            h =  mesh(double(G.X),log(double(G.Y)),double(G.Data')) ;
                        else
                            h =  mesh(double(G.X),double(G.Y),double(G.Data')) ;
                        end
                    case 6
                        if G.Settings.LogLevelsFlag
                            h =  meshc(double(G.X),log(double(G.Y)),double(G.Data')) ;
                        else
                            h =  meshc(double(G.X),double(G.Y),double(G.Data')) ;
                        end
                    case 7
                        if G.Settings.LogLevelsFlag
                            h =  meshz(double(G.X),single(double(G.Y)),double(G.Data')) ;
                        else
                            h =  meshz(double(G.X),double(G.Y),double(G.Data')) ;
                        end
                    case 8
                        if G.Settings.LogLevelsFlag
                            if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                h =  imagesc([Lon1 Lon2],[log(single(P1)) log(single(P2))],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                            else
                                h =  imagesc([Lon1 Lon2],[log(single(P1)) log(single(P2))],double(G.Data')) ;
                            end
                        else
                            if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                h =  imagesc([Lon1 Lon2],[P1 P2],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                            else
                                h =  imagesc([Lon1 Lon2],[P1 P2],double(G.Data')) ;
                            end
                        end
                end
                
                if G.Settings.ReverseColorMap
                    cmap = colormap(ColorMapDef) ;
                    colormap(flipud(cmap)) ;
                else
                    colormap(ColorMapDef) ;
                end
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                elseif LineWidth == 0
                    try  set(h,'linestyle','none') ; end
                else % < 0 means darken edge
                    %GOAT_darken_edge(h,colormap) ;
                end
                try
                    if any(G.Settings.LevelsIdx == [1,2,5,7])
                        axis ij
                    end
                catch
                    axis ij
                end
                G.Display.xlabelhandle = xlabel(['$$\textrm{',G.lonlabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.ylabelhandle = ylabel(['$$\textrm{',G.Plabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                if G.Settings.LogLevelsFlag
                    ylim([log(single(P1)) log(single(P2))])
                else
                    ylim([P1 P2])
                end
                xlim([Lon1 Lon2])   ;
            end
        catch
            % Nevermind, no Data
        end
    case 5 % Hovmoller Lon
        try
            if Lon1 == Lon2
                if LineWidth <= 0 ; LineWidth = 2 ; end
                plot(G.y,G.Data,'k','linewidth',LineWidth) ;
                %xlabel('$$\textrm{Time}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                %ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.xlabelhandle = xlabel('Time','Fontsize',G.Display.TextSize_L) ;
                G.Display.ylabelhandle = ylabel(G.DataField,'Fontsize',G.Display.TextSize_L) ;
                xlim([1 length(G.y)])   ;
                Ticks = unique(round(get(gca,'XTick'))) ;
                TickLabels = datastringvec2date(G.Dates(Ticks),hovmollerformt(G)) ;
                set(gca,'XTick',Ticks,'XTickLabel',TickLabels) ;
            else
                switch ContourIdx
                    case 1
                        if ~isempty(Defs.contour_levels)
                        [C,h] =  contourf(G.X,G.Y,G.Data',Defs.contour_levels) ;
                        else
                        [C,h] =  contourf(G.X,G.Y,G.Data') ;    
                        end
                    case 2
                        if ~isempty(Defs.contour_levels)
                        [C,h] =  contour(G.X,G.Y,G.Data',Defs.contour_levels) ;
                        else
                        [C,h] =  contour(G.X,G.Y,G.Data') ;    
                        end
                    case 3
                        h =  surf(double(G.X),double(G.Y),double(G.Data')) ;
                    case 4
                        h =  surfc(double(G.X),double(G.Y),double(G.Data')) ;
                    case 5
                        h =  mesh(double(G.X),double(G.Y),double(G.Data')) ;
                    case 6
                        h =  meshc(double(G.X),double(G.Y),double(G.Data')) ;
                    case 7
                        h =  meshz(double(G.X),double(G.Y),double(G.Data')) ;
                    case 8
                        if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                            h =  imagesc([Lon1 Lon2],[1 G.y(end)],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                        else
                            h =  imagesc([Lon1 Lon2],[1 G.y(end)],double(G.Data')) ;
                        end
                end
                
                if G.Settings.ReverseColorMap
                    cmap = colormap(ColorMapDef) ;
                    colormap(flipud(cmap)) ;
                else
                    colormap(ColorMapDef) ;
                end
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                elseif LineWidth == 0
                    try  set(h,'linestyle','none') ; end
                else % < 0 means darken edge
                    %GOAT_darken_edge(h,colormap) ;
                end
                %xlabel('$$\textrm{Longitude}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                %ylabel(['$$\textrm{',G.ylabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.xlabelhandle = xlabel(G.lonlabel,'Fontsize',G.Display.TextSize_L) ;
                G.Display.ylabelhandle = ylabel(G.ylabel,'Fontsize',G.Display.TextSize_L) ;
                ylim([1 G.y(end)])
                xlim([Lon1 Lon2])   ;
                Ticks = unique(round(get(gca,'YTick'))) ;
                TickLabels = datastringvec2date(G.Dates(Ticks),hovmollerformt(G)) ;
                set(gca,'YTick',Ticks,'YTickLabel',TickLabels) ;
            end
        catch
            % Nevermind, no Data
        end
    case 6 % Hovmoller Lat
        try
            if Lat1 == Lat2
                if LineWidth <= 0 ; LineWidth = 2 ; end
                plot(G.y,G.Data,'k','linewidth',LineWidth) ;
                %xlabel('$$\textrm{Time}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                %ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.xlabelhandle = xlabel('Time','Fontsize',G.Display.TextSize_L) ;
                G.Display.ylabelhandle = ylabel(G.DataField,'Fontsize',G.Display.TextSize_L) ;
                xlim([1 length(G.y)])   ;
                Ticks = unique(round(get(gca,'XTick'))) ;
                TickLabels = datastringvec2date(G.Dates(Ticks),hovmollerformt(G)) ;
                set(gca,'XTick',Ticks,'XTickLabel',TickLabels) ;
            else
                if G.Settings.SinLatFlag
                    switch ContourIdx
                        case 1
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contourf(sind(G.X),G.Y,G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contourf(sind(G.X),G.Y,G.Data') ;    
                            end
                        case 2
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contour(sind(G.X),G.Y,G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contour(sind(G.X),G.Y,G.Data') ;    
                            end
                        case 3
                            h =  surf(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                        case 4
                            h =  surfc(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                        case 5
                            h =  mesh(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                        case 6
                            h =  meshc(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                        case 7
                            h =  meshz(sind(double(G.X)),double(G.Y),double(G.Data')) ;
                        case 8
                            if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                h =  imagesc([Lat1 Lat2],[1 G.y(end)],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                            else
                                h =  imagesc([Lat1 Lat2],[1 G.y(end)],double(G.Data')) ;
                            end
                    end
                else
                    switch ContourIdx
                        case 1
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contourf(G.X,G.Y,G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contourf(G.X,G.Y,G.Data') ;    
                            end
                        case 2
                            if ~isempty(Defs.contour_levels)
                            [C,h] =  contour(G.X,G.Y,G.Data',Defs.contour_levels) ;
                            else
                            [C,h] =  contour(G.X,G.Y,G.Data') ;    
                            end
                        case 3
                            h =  surf(double(G.X),double(G.Y),double(G.Data')) ;
                        case 4
                            h =  surfc(double(G.X),double(G.Y),double(G.Data')) ;
                        case 5
                            h =  mesh(double(G.X),double(G.Y),double(G.Data')) ;
                        case 6
                            h =  meshc(double(G.X),double(G.Y),double(G.Data')) ;
                        case 7
                            h =  meshz(double(G.X),double(G.Y),double(G.Data')) ;
                        case 8
                            if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                                h =  imagesc([Lat1 Lat2],[1 G.y(end)],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                            else
                                h =  imagesc([Lat1 Lat2],[1 G.y(end)],double(G.Data')) ;
                            end
                    end
                end
                
                if G.Settings.ReverseColorMap
                    cmap = colormap(ColorMapDef) ;
                    colormap(flipud(cmap)) ;
                else
                    colormap(ColorMapDef) ;
                end
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                elseif LineWidth == 0
                    try  set(h,'linestyle','none') ; end
                else % < 0 means darken edge
                    %GOAT_darken_edge(h,colormap) ;
                end
                %xlabel('$$\textrm{Latitude}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                %ylabel(['$$\textrm{',G.ylabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                if G.Settings.SinLatFlag
                    G.Display.xlabelhandle = xlabel(['sin( ',G.latlabel,' )'],'Fontsize',G.Display.TextSize_L) ;
                else
                    G.Display.xlabelhandle = xlabel(G.latlabel,'Fontsize',G.Display.TextSize_L) ;
                end
                G.Display.ylabelhandle = ylabel(G.ylabel,'Fontsize',G.Display.TextSize_L) ;
                ylim([1 G.y(end)])
                xlim([Lat1 Lat2])   ;
                Ticks = unique(round(get(gca,'YTick'))) ;
                TickLabels = datastringvec2date(G.Dates(Ticks),hovmollerformt(G)) ;
                set(gca,'YTick',Ticks,'YTickLabel',TickLabels) ;
            end
        catch
            % Nevermind, no Data
        end
    case 7 % Hovmoller P
        try
            if P2 == P1 || G.Limits.Default.sameP ;
                if LineWidth <= 0 ; LineWidth = 2 ; end
                plot(G.y,G.Data,'k','linewidth',LineWidth) ;
                %xlabel('$$\textrm{Time}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                %ylabel(['$$\textrm{',G.DataField,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.xlabelhandle = xlabel('Time','Fontsize',G.Display.TextSize_L) ;
                G.Display.ylabelhandle = ylabel(G.DataField,'Fontsize',G.Display.TextSize_L) ;
                xlim([1 length(G.y)])   ;
                Ticks = unique(round(get(gca,'XTick'))) ;
                TickLabels = datastringvec2date(G.Dates(Ticks),hovmollerformt(G)) ;
                set(gca,'XTick',Ticks,'XTickLabel',TickLabels) ;
            else
                switch ContourIdx
                    case 1
                        if ~isempty(Defs.contour_levels)
                        [C,h] =  contourf(G.X,G.Y,G.Data',Defs.contour_levels) ;
                        else
                        [C,h] =  contourf(G.X,G.Y,G.Data') ;    
                        end
                    case 2
                        if ~isempty(Defs.contour_levels)
                        [C,h] =  contour(G.X,G.Y,G.Data',Defs.contour_levels) ;
                        else
                        [C,h] =  contour(G.X,G.Y,G.Data') ;    
                        end
                    case 3
                        h =  surf(double(G.X),double(G.Y),double(G.Data')) ;
                    case 4
                        h =  surfc(double(G.X),double(G.Y),double(G.Data')) ;
                    case 5
                        h =  mesh(double(G.X),double(G.Y),double(G.Data')) ;
                    case 6
                        h =  meshc(double(G.X),double(G.Y),double(G.Data')) ;
                    case 7
                        h =  meshz(double(G.X),double(G.Y),double(G.Data')) ;
                    case 8
                        if G.Settings.FixZLimitsFlag && ContourIdx ~= 8
                            h =  imagesc([P1 P2],[1 G.y(end)],double(G.Data'),[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                        else
                            h =  imagesc([P1 P2],[1 G.y(end)],double(G.Data')) ;
                        end
                end
                
                if G.Settings.ReverseColorMap
                    cmap = colormap(ColorMapDef) ;
                    colormap(flipud(cmap)) ;
                else
                    colormap(ColorMapDef) ;
                end
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
                
                if LineWidth > 0
                    try  set(h,'linewidth',LineWidth) ; end
                elseif LineWidth == 0
                    try  set(h,'linestyle','none') ; end
                else % < 0 means darken edge
                    %GOAT_darken_edge(h,colormap) ;
                end
                %xlabel('$$\textrm{Level}$$','Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                %ylabel(['$$\textrm{',G.ylabel,'}$$'],'Fontsize',G.Display.TextSize_L,'interpreter','latex') ;
                G.Display.xlabelhandle = xlabel(G.Plabel,'Fontsize',G.Display.TextSize_L) ;
                G.Display.ylabelhandle = ylabel(G.ylabel,'Fontsize',G.Display.TextSize_L) ;
                ylim([1 G.y(end)])
                xlim([P1 P2])   ;
                Ticks = unique(round(get(gca,'YTick'))) ;
                TickLabels = datastringvec2date(G.Dates(Ticks),hovmollerformt(G)) ;
                set(gca,'YTick',Ticks,'YTickLabel',TickLabels) ;
            end
        catch
            % Nevermind, no Data
        end
    case {8,9} % Polar Display
        if G.Plotting.GUI_PlotFlag
            subplot (1,10,[2 10],'replace')
        end
        if G.ProjectionType == 8
            xm = axesm('stereo','Origin',double([ 90 0]),'MapLatLimit',double([max([0 ,min([Lat1,Lat2])])  90])) ;
        else
            xm = axesm('stereo','Origin',double([-90 0]),'MapLatLimit',double([min([0, max([Lat1,Lat2])]) -90])) ;
        end
        if G.Settings.ReverseColorMap
            cmap = colormap(ColorMapDef) ;
            colormap(flipud(cmap)) ;
        else
            colormap(ColorMapDef) ;
        end
        Yz = [G.Y,G.Y(:,1)] ;
        Dataz = [G.Data ; G.Data(1,:)] ;
        if G.x(1) == 0
            Xz = [G.X,(G.X(:,1)+360-0.0001)] ;
        else
            Xz = [G.X,G.X(:,1)-0.0001] ;
        end
        set(xm,'box','off','xcolor',[.5 .5 .5],'ycolor',[.5 .5 .5],'tickdir','out')
        %setm(xm,'frame','on','grid','on','glinestyle','-','gcolor',[.5 .5 .5],'parallellabel','on','meridianlabel','on','mlabelparallel',5,'mlabellocation',[-180 -150 -120 -90 -60 -30 0 30 60 90 120 150]) ; hold on
        setm(xm,'frame','on') ; hold on
        %clm = geoshow(G.topo.landcoverlat,G.topo.landcoverlon,'DisplayType','polygon','linewidth',1,'facecolor',[0 0 0]) ; alpha(.8) ; hold on
        [C , h] = contourm(double(Yz),double(Xz),double(Dataz'),'fill','on') ; hold on % alpha(.8) ;
        G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;hold on
        G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
        if CoastLineWidth
            try
                geoshow(G.topo.landcoverlat,G.topo.landcoverlon,'color','k','linewidth',CoastLineWidth)
                switch G.topo.TypeFlag
                    case 2
                        if Lon1 < 0
                            [Ctopo, htopo] = contourm([G.topo.latG],[G.topo.lonG'-360 ; G.topo.lonG'] ,[G.topo.topoA , G.topo.topoA],G.topo.topovec) ; hold on
                        else
                            [Ctopo, htopo] = contourm(G.topo.latG,G.topo.lonG,G.topo.topoA,G.topo.topovec) ; hold on
                        end
                    case 3
                        if Lon1 < 0
                            [Ctopo, htopo] = contourm([G.topo.latG],[G.topo.lonG'-360 ; G.topo.lonG'] ,[G.topo.topoO , G.topo.topoO],G.topo.topovec) ; hold on
                        else
                            [Ctopo, htopo] = contourm(G.topo.latG,G.topo.lonG,G.topo.topoO,G.topo.topovec) ; hold on
                        end
                    case 4
                        if Lon1 < 0
                            [Ctopo, htopo] = contourm([G.topo.latG],[G.topo.lonG'-360 ; G.topo.lonG'] ,[G.topo.topoG , G.topo.topoG],G.topo.topovec) ; hold on
                        else
                            [Ctopo, htopo] = contourm(G.topo.latG,G.topo.lonG,G.topo.topoG,G.topo.topovec) ; hold on
                        end
                end
                if G.topo.TypeFlag > 1
                    tc = get(htopo,'children') ;
                    for i = 1:length(tc)
                        set(tc(i),'linewidth',G.topo.TopotLineWidthIdx/2,'color',[.2 .2 .2]) ;
                    end
                end
            catch
                % Nevermind. Possibly null topography
            end
            %axesm ('globe', 'Frame', 'on', 'Grid', 'on','MapLatLimit',[0 0]);
            %geoshow(double(G.Y),double(G.X),G.Data','Displaytype','contour') ;
            
            if LineWidth > 0
                try  set(h,'linewidth',LineWidth) ; end
            elseif LineWidth == 0
                try  set(h,'linestyle','none') ; end
            else % < 0 means darken edge
                % GOAT_darken_edge(h,colormap) ;
            end
            axis off ;
        end
end
% Axis Configurations
try
    if any(G.ProjectionType == [8,9]) && sum([G.Settings.XgridFlag,G.Settings.YgridFlag])
        setm(xm,'grid','on','glinestyle','-','gcolor',[.5 .5 .5],'parallellabel','on','meridianlabel','on','mlabelparallel',5,'mlabellocation',[-180 -150 -120 -90 -60 -30 0 30 60 90 120 150]) ;
    elseif   any(G.ProjectionType == [8,9])
        setm(xm,'grid','off') ;
    elseif G.Settings.XgridFlag
        try
            ticklevels = get(gca,'xtick') ;
            ylimits = get(gca,'ylim') ;
            hold on
            for i = 1:length(ticklevels)
                plot([ticklevels(i) ticklevels(i)],ylimits,'-','linewidth',1,'color',[.6 .6 .6])
            end
        catch
            set(gca,'XGrid','on') ;
        end
    end
    if G.Settings.YgridFlag
        try
            ticklevels = get(gca,'ytick') ;
            xlimits = get(gca,'xlim') ;
            hold on
            for i = 1:length(ticklevels)
                plot(xlimits,[ticklevels(i) ticklevels(i)],'-','linewidth',1,'color',[.6 .6 .6])
            end
        catch
            set(gca,'YGrid','on') ;
        end
    end
end

try
    if G.Settings.ContourLevelStep && isempty(Defs.contour_levels)
        if  G.Settings.ContourLevelStep > (max(max(G.Data)) - min(min(G.Data)))/100 ;
            set(h,'levelstep',G.Settings.ContourLevelStep) ;
        end
        Defs.contour_levels = get(h,'LevelList') ;
    elseif isempty(Defs.contour_levels)
        Defs.contour_levels = get(h,'LevelList') ;
    % else  Defs.contour_levels is not empty and overrides other definitions
    end
catch
    % Nevermind
end
try
    if G.Settings.FixZLimitsFlag
        if      (G.ProjectionType == 3 && (G.Limits.Default.sameP || P2 == P1)) ...
                ||(G.ProjectionType == 3 && Lat2 == Lat1) ...
                ||(G.ProjectionType == 4 && (G.Limits.Default.sameP || P2 == P1)) ...
                ||(G.ProjectionType == 4 && Lon2 == Lon1) ...
                ||(G.ProjectionType == 5 && Lon2 == Lon1) ...
                ||(G.ProjectionType == 6 && Lat2 == Lat1) ...
                ||(G.ProjectionType == 7 && P2 == P1) ...
                ylim([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
        elseif ContourIdx ~= 8
            caxis([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
            if ~G.Settings.SymZLimitsFlag
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                %G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off','YLim',[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            else
                try
                    MinZ = nanmin(G.Data(:));
                catch % in case no toolbox
                    MinZ = min(G.Data(:));
                end
                if isnan(MinZ)
                    MinZ = min(min(G.Data(~isnan(G.Data) & ~isinf(G.Data))));
                end
                try
                    MaxZ = nanmax(G.Data(:));
                catch
                    MaxZ = max(G.Data(:));
                end
                if isnan(MaxZ)
                    MaxZ = max(max(G.Data(~isnan(G.Data) & ~isinf(G.Data))));
                end
                G.Display.colorbarHandle = colorbar('YLim',[MinZ MaxZ],'fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            end
        end
    elseif G.Settings.SymZLimitsFlag
        try
            MinZ = nanmin(G.Data(:));
        catch
            MinZ = min(G.Data(:));
        end
        if isnan(MinZ)
            MinZ = min(min(G.Data(~isnan(G.Data) & ~isinf(G.Data))));
        end
        try
            MaxZ = nanmax(G.Data(:));
        catch
            MaxZ = max(G.Data(:));
        end
        if isnan(MaxZ)
            MaxZ = max(max(G.Data(~isnan(G.Data) & ~isinf(G.Data))));
        end
        AbsMax = max([abs(MinZ) , abs(MaxZ)]) ;
        G.Settings.ZlimitMin = - AbsMax ;
        G.Settings.ZlimitMax =   AbsMax ;
        if (G.ProjectionType == 3 && (G.Limits.Default.sameP || P2 == P1)) ...
                ||(G.ProjectionType == 3 && Lat2 == Lat1) ...
                ||(G.ProjectionType == 4 && (G.Limits.Default.sameP || P2 == P1)) ...
                ||(G.ProjectionType == 4 && Lon2 == Lon1) ...
                ||(G.ProjectionType == 5 && Lon2 == Lon1) ...
                ||(G.ProjectionType == 6 && Lat2 == Lat1) ...
                ||(G.ProjectionType == 7 && P2 == P1) ...
                ylim([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
        elseif ContourIdx ~= 8
            caxis([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
            if ~G.Settings.SymZLimitsFlag
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            else
                G.Display.colorbarHandle = colorbar('YLim',[MinZ MaxZ],'fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            end
        end
    elseif G.topo.TypeFlag > 1  && any(G.ProjectionType == [1,2,8,9])
        if G.ProjectionType == 8
            Data = G.Data(G.Y>=0) ;
        elseif G.ProjectionType == 9
            Data = G.Data(G.Y<=0) ;
        else
            Data = G.Data ;
        end
        try
            MinZ = nanmin(Data(:));
        catch
            MinZ = min(Data(:));
        end
        if isnan(MinZ)
            MinZ = min(Data(~isnan(Data(:)) & ~isinf(Data(:))));
        end
        try
            MaxZ = nanmax(Data(:));
        catch
            MaxZ = max(Data(:));
        end
        if isnan(MaxZ)
            MaxZ = max(Data(~isnan(Data(:)) & ~isinf(Data(:))));
        end
        G.Settings.ZlimitMin = MinZ ;
        G.Settings.ZlimitMax = MaxZ ;
        if (G.ProjectionType == 3 && (G.Limits.Default.sameP || P2 == P1)) ...
                ||(G.ProjectionType == 3 && Lat2 == Lat1) ...
                ||(G.ProjectionType == 4 && (G.Limits.Default.sameP || P2 == P1)) ...
                ||(G.ProjectionType == 4 && Lon2 == Lon1) ...
                ||(G.ProjectionType == 5 && Lon2 == Lon1) ...
                ||(G.ProjectionType == 6 && Lat2 == Lat1) ...
                ||(G.ProjectionType == 7 && P2 == P1) ...
                ylim([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
        elseif ContourIdx ~= 8
            caxis([G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
            if ~G.Settings.SymZLimitsFlag
                G.Display.colorbarHandle = colorbar('fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            else
                G.Display.colorbarHandle = colorbar('YLim',[MinZ MaxZ],'fontname',G.GUI.Fonts.gcaFont,'box','off') ;
                G.Display.colorbarLimits = get(G.Display.colorbarHandle,'YLim') ;
            end
        end
    end
end
% if G.ProjectionType > 7 && ~G.Settings.SymZLimitsFlag && ~G.Settings.FixZLimitsFlag
%     if G.ProjectionType == 8
%         Data = G.Data(G.Y>=0) ;
%     else
%         Data = G.Data(G.Y<=0) ;
%     end
%     MinZ = min(Data(:));
%     if isnan(MinZ)
%         MinZ = min(min(Data(~isnan(Data) & ~isinf(Data))));
%     end
%     MaxZ = max(Data(:));
%     if isnan(MaxZ)
%         MaxZ = max(max(G.Data(~isnan(Data) & ~isinf(Data))));
%     end
%     G.Settings.ZlimitMin = MinZ ;
%     G.Settings.ZlimitMax = MaxZ ;
%     caxis([MinZ MaxZ]) ;
%     colorbar('YLim',[MinZ MaxZ]) ;
% end

try
    if G.ProjectionType < 8
        if LineWidth < 0
            hold on
            if ~verLessThan('matlab','8.4.0') % No Children in versions >=2014b
            if G.Settings.SymZLimitsFlag
                GOAT_darken_edge(h,colormap,'clim',[G.Settings.ZlimitMin G.Settings.ZlimitMax]) ;
            else
                GOAT_darken_edge(h,colormap) ;
            end
            else
                
            end
            hold off
        end
    end
end


if ~isempty(G.Plotting.TitleMessage)
    title(G.Plotting.TitleMessage,'fontsize',G.Display.TextSize_M) ;
end
set(gca,'fontsize',G.Display.TextSize_M,'fontname',G.GUI.Fonts.gcaFont,'linewidth',2,'tickdir','out','box','off') ;
hold off ;

