% ProduceFileAddressesStrings produce a vector of address indices according to TimeFiels and Limits Settings.
% written by Ori Adam 2012. compatible with version 2013h
%
% see also, GOAT_Data_string_vec

function Addresses = ProduceFileAddressesStrings

Years = FileAddressStringsLimits ; 

% Monthly First
years = Years.MonthlyBegin:Years.MonthlyEnd ; 

MonthlyAddress = cell(1000,1) ;
q = 0 ;
for y = years(1): years(end)
    for m = 1:12
        q = q + 1 ;
        MonthlyAddress(q) = {fullfile(int2str(y),[int2str(m*10000),'.mat'])} ;   
    end
end
MonthlyAddress = MonthlyAddress(1:q) ; 

% Restrict daily data and beyond to 400 years
years = Years.DailyBegin:Years.DailyEnd ; 

% 8 Times Daily
Xd = 8 ; 
H = 24/Xd ;
DailyAddressX8 = cell(100000*Xd,1) ;
q = 0 ; 
for y = years(1): years(end)
    for m = 1:12
        switch m
            case {1,3,5,7,8,10,12}
                for d = 1:31
                    for x = 1:Xd
                    q = q + 1 ;
                    DailyAddressX8(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                    end
                end
            case {4,6,9,11}
                for d = 1:30
                    for x = 1:Xd
                    q = q + 1 ;
                    DailyAddressX8(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                    end
                end
            case 2
                if GOAT_leapyear(y)
                    for d = 1:29
                        for x = 1:Xd
                        q = q + 1 ;
                        DailyAddressX8(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                        end
                    end
                else
                    for d = 1:28
                        for x = 1:Xd
                        q = q + 1 ;
                        DailyAddressX8(q) = {fullfile(int2str(y),[int2str((m*100 + d)*100 + (x-1)*H),'.mat'])} ;
                        end
                    end
                end
        end
    end
end
DailyAddressX8 = DailyAddressX8(1:q) ;

Addresses.MonthlyAddress = MonthlyAddress ; 
Addresses.DailyAddressX8 = DailyAddressX8 ;

save('FileAddressStrings','Addresses') ; 
