% WeightFunc Apply weight to selected GOAT data
% written by Ori Adam 2012. compatible with version 2013h
% 
% see also GOAT_Filters

function weight = WeightFunc(G)

switch  G.Filter.WeightIdx
    case 1 % None
        weight = 1 ;
    case 2 % -1
        weight = G.Filter.WeightFactor ; 
    case 3 % Cos
        weight = repmat(cosd(G.lat)',[length(G.lon) 1 length(G.P)]) ;
    case 4 % -Cos
        weight = repmat(-cosd(G.lat)',[length(G.lon) 1 length(G.P)]) ;
    case 5 % 1./(a*Cos)
        weight = repmat(cosd(G.lat)'.^(-1)/G.Consts.EarthRadius,[length(G.lon) 1 length(G.P)]) ;
    case 6 % -1/(a*Cos)
        weight = repmat(-cosd(G.lat)'.^(-1)/G.Consts.EarthRadius,[length(G.lon) 1 length(G.P)]) ;
end

if length(weight) > 1
    switch G.ProjectionType
        case {1,2}
            if G.Limits.Default.sameP
                G.DataLimits.Lev1 = G.Limits.Default.Lev1 ;
                G.DataLimits.Lev2 = G.DataLimits.Lev1 ;
            end
            if G.DataLimits.Lev1 == G.DataLimits.Lev2
                weight = squeeze(weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)) ;
            else
                %weight = squeeze(mean(weight(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),3)) ; simple mean
                weight = trapz(G.DataLimits.p,weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),3)  ; % weighted mean
            end
        case 3
            if G.Limits.Default.sameP  % This Flag Means Vertical Averaging In this Projection
                if G.DataLimits.Lev1 == G.DataLimits.Lev2
                    weight = squeeze(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)) ;
                else
                    weight = squeeze(trapz(G.DataLimits.p,squeeze(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),2))' ;
                end
            else
                weight = squeeze(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)) ;
            end
        case 4
            if G.Limits.Default.sameP
                if G.DataLimits.Lev1 == G.DataLimits.Lev2
                    weight = squeeze(mean(weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1),2)) ;
                else
                    weight = squeeze(trapz(G.DataLimits.p,squeeze(mean(weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)) ;
                end
            else
                weight = squeeze(mean(weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)) ;
            end
        case 5
            if G.DataLimits.Lev1 == G.DataLimits.Lev2
                weight = mean(squeeze(weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)),2) ;
            else
                %weight = squeeze(mean(weight(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),3)) ; simple mean
                weight = trapz(G.DataLimits.p,squeeze(mean(weight(:,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),2)),2)  ; % weighted mean
            end
        case 6
            if G.DataLimits.Lev1 == G.DataLimits.Lev2
                weight = squeeze(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1)) ;
            else
                %weight = squeeze(mean(weight(G.DataLimits.Lon1:G.DataLimits.Lon2,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2),3)) ; simple mean
                weight = trapz(G.DataLimits.p,squeeze(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),2)  ; % weighted mean
            end
        case 7
            if G.DataLimits.Lev2 == G.DataLimits.Lev1
                weight = squeeze(mean(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1))) ;
            else
                weight = squeeze(mean(squeeze(weight(1,G.DataLimits.Lat1:G.DataLimits.Lat2,G.DataLimits.Lev1:G.DataLimits.Lev2)),1))' ;
            end
    end
end
return ;