% cejulienColorMaps  creates color symmetric colormaps. adapted from the original function cejulien

function cmap = cejulienColorMaps(Name,nc)
% cejulienColorMaps creates color symmetric (white = 0) color maps. 
% Based On the Function cmap = cejulien(nc);
%
% A very cute colormap with nc= 4*q+1 levels, q a small integer
%       (this is best for centered colorscales, with zero as white) 
%  
%  INPUT  :   nc : small odd integer (13 or 17 recommended)
%         :   Name: A string with the name of the Colormap
%   
%  OUTPUT :	cmap(nc,3)	- colormap RGB-value array
%     (c)    Julien Emile-Geay, LDEO, Aug 2006  
%            inspired from Gerd Krahman's ce(STUFF) series of colormaps
%  Editted and Generlized by Ori Adam Nov.2012.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin == 1
    nc = 17 ; 
elseif ~mod(nc,2)
    nc = nc + 1 ;
end

% set 5 points 
switch Name
    case 'ceJulien'
    bottom    = [0.1324    0.1011    0.8799] ;
    botmiddle = [0.3163    0.7411    1.0000] ;
    middle    = [1         1         1     ] ;
    topmiddle = [1.0000    0.7783    0.1665] ;
    top       = [0.8734    0.1333    0.1333] ;
    case 'BlueWhiteRed'
    bottom    = [0         0         0.5   ] ;
    botmiddle = [0         0.5       1     ] ;
    middle    = [1         1         1     ] ;
    topmiddle = [1         0         0     ] ;
    top       = [0.5       0         0     ] ;
    case 'Rose'
    bottom    = [0.2       0.14      0.99  ] ; % [0.1       0.2       0.99  ] ;
    botmiddle = [0.4       0.5       1     ] ; % [0.35      0.5       1     ] ;
    middle    = [1         1         1     ] ;
    topmiddle = [1         .5        0.8   ] ;
    top       = [0.99      0.14      0.28  ] ;    
    case 'RYBceJulien'
    bottom    = [0.6471    0         0.1490] ;  
    botmiddle = [0.9746    0.555     0.3216] ;  
    middle    = [1         1         0.7490] ;
    topmiddle = [0.5627    0.7647    0.8666] ;
    top       = [0.1922    0.2118    0.5843] ;    
    case 'OWBceJulien'
    bottom    = [0.75      0         0     ] ;  
    botmiddle = [1         0.5       0     ] ;  
    middle    = [1         1         1     ] ;
    topmiddle = [0         0.5       1     ] ;
    top       = [0         0         1     ] ;    
    case 'OTBceJulien'
    bottom    = [0.75      0         0     ] ;  
    botmiddle = [0.95      0.5       0     ] ;  
    middle    = [0.97      1         0.99  ] ;
    topmiddle = [0         0.5       0.99  ] ;
    top       = [0         0         1     ] ;    
    case 'TWBceJulien'
    bottom    = [.6       .8         .6    ] ;  
    botmiddle = [1         0.9       .6    ] ;  
    middle    = [1         1         1     ] ;
    topmiddle = [0         0.4       .9    ] ;
    top       = [0         0         .85   ] ;    
    case 'BluesceJulien'
    bottom    = [1.0       1.0       1.0   ] ;  
    botmiddle = [0.9098    0.9451    0.9804] ;  
    middle    = [0.6196    0.7922    0.8824] ;
    topmiddle = [0.1294    0.4431    0.7098] ;
    top       = [0.0314    0.1882    .4196 ] ;    
    otherwise
    disp 'Unrecognized ColorMap Name !!!' ; 
    disp 'Existing Colormap Names: '  ;
    disp 'ceJulien'                   ;
    disp 'BlueWhiteRed'               ; 
    disp 'Rose'                       ;
    cmap = [] ; 
    return ;
end
C = [bottom ; botmiddle ; middle ; topmiddle ; top] ; 

nk=round((nc-1)/4);
cmap=zeros(nc,3);

for ki=1:4
   xo=[ki ki+1]; % old axis
   xn=linspace(ki,ki+1,nk+1); % new axis
   bounds=(ki-1)*nk+[1:nk+1];
   cmap(bounds,1)=interp1(xo,C(ki:ki+1,1),xn);
   cmap(bounds,2)=interp1(xo,C(ki:ki+1,2),xn);
   cmap(bounds,3)=interp1(xo,C(ki:ki+1,3),xn);
end

