% date_message Generates the date message displayed at the GOAT display bar
% Copyright (C) Ori Adam March.2012. Compatible with version 2013i or later
%
function message = dateMessage(G)

P1   = min([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
P2   = max([G.lev(G.Limits.Default.Lev1),G.lev(G.Limits.Default.Lev2)]) ;
Lon1 = G.lon(G.Limits.Default.Lon1) ;
Lon2 = G.lon(G.Limits.Default.Lon2) ;
Lat1 = min([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;
Lat2 = max([G.lat(G.Limits.Default.Lat1),G.lat(G.Limits.Default.Lat2)]) ;

hours  = {'00:00','01:00','02:00','03:00','04:00','05:00','06:00','07:00','08:00','09:00','10:00','11:00','12:00','13:00','14:00','15:00','16:00','17:00','18:00','19:00','20:00','21:00','22:00','23:00'} ;

switch G.Settings.LevelsIdx
    case 1
        LevelStr = 'mb Level  ' ;
    case {2,3}
        LevelStr = 'm Level  ' ;
    case 4
        LevelStr = 'K Level  ' ;
    case 5
        LevelStr = 'Sigma Level  ' ;
    case 6
        LevelStr = 'Geometric-Level  ' ;    
    case 7
        LevelStr = 'Level  ' ;        
end

if Lon1 == Lon2
    LonStr = ['lon(',num2str(Lon1,3),')  '] ;
elseif Lon2 > Lon1
    LonStr = ['lon(',num2str(Lon1,3),' : ',num2str(Lon2,3),')  '] ;
else
    if Lon2 > 0
        LonStr = ['lon(',num2str(Lon1-360,3),' : ',num2str(Lon2,3),')  '] ;
    else
        LonStr = ['lon(',num2str(Lon1,3),' : ',num2str(Lon2+360,3),')  '] ;
    end
end
if Lat1 == Lat2
    LatStr = ['lat(',num2str(Lat1,3),')  '] ;
else
    LatStr = ['lat(',num2str(Lat1,3),' : ',num2str(Lat2,3),')  '] ;
end
if   P2 == P1
    Pstr = [num2str(P1),LevelStr] ;
elseif (any(G.ProjectionType == [1,2]) && G.Limits.Default.sameP )
    P1 = G.lev(G.Limits.Default.Lev1) ;
    Pstr = [num2str(P1),LevelStr] ;
else
    Pstr = [num2str(P2),':',num2str(P1),LevelStr] ;
end

if ~strcmpi(G.TimeField,'List')
    
    [year1,month1,day1,hour1] = GOAT_serial2date(G.Addresses.Default,G.Limits.S1) ;
    [year2,month2,day2,hour2] = GOAT_serial2date(G.Addresses.Default,G.Limits.S2) ;
    
    if G.Limits.Default.Anom && G.ProjectionType > 4
        AnomStr = ' Anomaly' ;
    else
        AnomStr = '' ;
    end
    
    switch G.TimeFieldVirtual
        case 'Daily'
            if G.Limits.S2 == G.Limits.S1
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),'.',int2str(year1) ];
            elseif G.Limits.Default.Sequential && G.ProjectionType < 5
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),'.',int2str(year1), ' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),'.',int2str(year2),' Time Mean.'];
            elseif G.Limits.Default.Sequential && G.ProjectionType > 4
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),'.',int2str(year1), ' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),'.',int2str(year2),AnomStr];
            elseif ~G.Limits.Default.Sequential && G.ProjectionType < 5 || ...
                    ~G.Limits.Default.Sequential && G.ProjectionType > 4 && ~G.Limits.Default.ClimatologyContinuous % Seasonal
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1), ' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),' for ',int2str(year1),'-',int2str(year2),' Seasonal Average.'];
            else
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1), ' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),' for ',int2str(year1),'-',int2str(year2),AnomStr,' Presented Continuously.'];
            end
        case 'Julian'
            if G.Limits.S2 == G.Limits.S1
                TimeStr = [int2str(day1),'.',int2str(year1) ];
            elseif G.Limits.Default.Sequential && G.ProjectionType < 5
                TimeStr = [int2str(day1),'.',int2str(year1), ' - ',int2str(day2),'.',int2str(year2),' Time Mean.'];
            elseif G.Limits.Default.Sequential && G.ProjectionType > 4
                TimeStr = [int2str(day1),'.',int2str(year1), ' - ',int2str(day2),'.',int2str(year2),AnomStr];
            elseif ~G.Limits.Default.Sequential && G.ProjectionType < 5 || ...
                    ~G.Limits.Default.Sequential && G.ProjectionType > 4 && ~G.Limits.Default.ClimatologyContinuous % Seasonal
                TimeStr = [int2str(day1), ' - ',int2str(day2),' for ',int2str(year1),'-',int2str(year2),' Seasonal Average.'];
            else
                TimeStr = [int2str(day1), ' - ',int2str(day2),' for ',int2str(year1),'-',int2str(year2),AnomStr,' Presented Continuously.'];
            end
        case 'Monthly'
            if G.Limits.S2 == G.Limits.S1
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(year1) ];
            elseif G.Limits.Default.Sequential && G.ProjectionType < 5
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(year1), ' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(year2),' Time Mean.'];
            elseif G.Limits.Default.Sequential && G.ProjectionType > 4
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(year1), ' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(year2),AnomStr];
            elseif ~G.Limits.Default.Sequential && G.ProjectionType < 5 || ...
                    ~G.Limits.Default.Sequential && G.ProjectionType > 4 && ~G.Limits.Default.ClimatologyContinuous % Seasonal
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),' for ',int2str(year1),'-',int2str(year2),' Seasonal Average.'];
            else
                TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),' - ', ...
                    cell2mat(G.Consts.MonthsStr(month2)),' for ',int2str(year1),'-',int2str(year2),AnomStr,' Presented Continuously.'];
            end
        otherwise
            if strncmpi(G.TimeFieldVirtual,'Daily',5)
                if G.Limits.S2 == G.Limits.S1
                    TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),'.',int2str(year1),' ',cell2mat(hours(hour1+1)) ];
                elseif G.Limits.Default.Sequential && G.ProjectionType < 5
                    TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),'.',int2str(year1),' ',cell2mat(hours(hour1+1)) ,' - ', ...
                        cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),'.',int2str(year2),' ',cell2mat(hours(hour2/3+1)),' Time Mean.'];
                elseif G.Limits.Default.Sequential && G.ProjectionType > 4
                    TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),'.',int2str(year1),' ',cell2mat(hours(hour1+1)), ' - ', ...
                        cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),'.',int2str(year2),' ',cell2mat(hours(hour2+1))];
                elseif ~G.Limits.Default.Sequential && G.ProjectionType < 5 || ...
                        ~G.Limits.Default.Sequential && G.ProjectionType > 4 && ~G.Limits.Default.ClimatologyContinuous % Seasonal
                    TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1),' ',cell2mat(hours(hour1+1)) ,' - ', ...
                        cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),' ',cell2mat(hours(hour2/3+1)),' for ',int2str(year1),'-',int2str(year2),' Seasonal Average.'];
                else
                    TimeStr = [cell2mat(G.Consts.MonthsStr(month1)),'.',int2str(day1), ' - ', ...
                        cell2mat(G.Consts.MonthsStr(month2)),'.',int2str(day2),' for ',int2str(year1),'-',int2str(year2),AnomStr,' Presented Continuously.'];
                end
            else % Julian
                if G.Limits.S2 == G.Limits.S1
                    TimeStr = [int2str(day1),'.',int2str(year1),' ',cell2mat(hours(hour1+1)) ];
                elseif G.Limits.Default.Sequential && G.ProjectionType < 5
                    TimeStr = [int2str(day1),'.',int2str(year1),' ',cell2mat(hours(hour1+1)) ,' - ',int2str(day2),'.',int2str(year2),' ',cell2mat(hours(hour2/3+1)),' Time Mean.'];
                elseif G.Limits.Default.Sequential && G.ProjectionType > 4
                    TimeStr = [int2str(day1),'.',int2str(year1),' ',cell2mat(hours(hour1+1)), ' - ',int2str(day2),'.',int2str(year2),' ',cell2mat(hours(hour2+1))];
                elseif ~G.Limits.Default.Sequential && G.ProjectionType < 5 || ...
                        ~G.Limits.Default.Sequential && G.ProjectionType > 4 && ~G.Limits.Default.ClimatologyContinuous % Seasonal
                    TimeStr = [int2str(day1),' ',cell2mat(hours(hour1+1)) ,' - ', int2str(day2),' ',cell2mat(hours(hour2/3+1)),' for ',int2str(year1),'-',int2str(year2),' Seasonal Average.'];
                else
                    TimeStr = [int2str(day1), ' - ',int2str(day2),' for ',int2str(year1),'-',int2str(year2),AnomStr,' Presented Continuously.'];
                end
            end
    end
else
    %Lidx = length(G.Addresses.Linears{G.Addresses.LinearIdx}) + 2 ;
    if G.Limits.S2 == G.Limits.S1
        TimeStr = cell2mat(G.Addresses.Default(G.Limits.S1)) ;
        [Path,Name] = fileparts(TimeStr) ; 
        %TimeStr = [' ',TimeStr(Lidx:end-4)] ;
        TimeStr = [' ',Name] ;
    else
        TimeStr1 = cell2mat(G.Addresses.Default(G.Limits.S1)) ;
        TimeStr2 = cell2mat(G.Addresses.Default(G.Limits.S2)) ;
        [Path1,Name1] = fileparts(TimeStr1) ;
        [Path2,Name2] = fileparts(TimeStr2) ; 
        %TimeStr = [' ',TimeStr1(Lidx:end-4),' to ',TimeStr2(Lidx:end-4)] ;
        TimeStr = [' ',Name1,' to ',Name2] ;
    end
end
message = [LonStr,LatStr,Pstr,TimeStr] ;
return ;