% this functiondoes
% syntax:
% >> inptu output etc..

% see also, 

function [field_3d,lat,lon,P,models,bata] = get_goat_cmip_field_3d(experiment,model,DataField,time_freq,year1,year2,month1,month2,Level1,Level2,bata_avg)                                                                             
                                                                                                                                                                              
% It is recommended though not mandatory to work from the ... GOAT/Code directory                                                                                             
% When not working from the GOAT/Code directory make sure that the following directories                                                                                      
% and all their subdirectories are added to the matlab path:                                                                                                                  
% ... GOAT/Code,   GOAT/Scripts,   and GOAT/MyScripts                                                                                                                         
% cell-1: Initiate GOAT and set: Dataset, Field and Calendar                                                                                                                  
% cell-2: Set temporal limits                                                                                                                                                 
% cell-3: Set spatial limits and Filters/weights                                                                                                                              
% cell-4: Retrieve and compile data                                                                                                                                           
% cell-5: plot                                                                                                                                                                
% cell-6: save     

if year1 == year2 && year1 == 1000
    head_dir = '/Volumes/data3/CMIP5_CLIM/';
else
    head_dir = '/Volumes/data3/CMIP5_LINK/';
end

if nargin < 11
    bata_avg = 0;
end                                                     

if strcmp(model,'all')
    folders = dir([head_dir,experiment,'/']);
    j = 1;
    for i = 1:length(folders)
        modeli = folders(i).name;
        if modeli(1) ~= '.'
            vars = dir([head_dir,experiment,'/',modeli,'/atmos/r1i1p1/Monthly/']);
            name = cell(length(vars),1);
            for n = 1:length(vars)
                name{n} = vars(n).name;
            end
            if sum(strcmp(name,DataField)) == 1
                models{j} = modeli;
                j = j + 1;
            end
        end
    end
else
    models{1} = model;
end

field_3d = cell(1,length(models));
lat = cell(1,length(models));
lon = cell(1,length(models));
P = cell(1,length(models));
bata = cell(1,length(models));

realm = '/atmos/r1i1p1/';

day1   = 1    ; % 1:31
hour1  = 0    ; % 0:3:21                                                                                                                                                      
day2   = 1    ;
hour2  = 0    ;
if nargin < 10
    Level1 = 0;
    Level2 = 1000;
end
Lon1    = -180   ; % use values from -360 to 360. GOAT will use the nearest values to the ones specified
Lon2    = 360   ; % use values from -360 to 360.
Lat1    = 90   ; % use values from -90 to 90. GOAT will use the nearest values to the ones specified.
Lat2    = -90   ; % Lat2 does not have to be larger than Lat1.
G = GOAT_Environment('enter_project_path') ; % Initiate GOAT Environment
for i = 1:length(models)
    
    %% cell-1: Initiate GOAT and set: Dataset, Field and Calendar
    
    G = GOAT_Initialize_Dataset(G,[head_dir,experiment,'/',models{i},realm],time_freq) ; % Go through contents of Data folder and Calendar. Link existing fields, temporal span, etc.
    G = GOAT_Initialize_Field(G,DataField)        ; % get spatial and temporal dimensions of field, display settings, topography, and file addresses. Link if necessary
    %G.Limits.Year0 = year1 ; % first year of the field
    %% cell-2: Temporal limits
    % Temporal Checkboxes/Flags
    G.Limits.Default.Day2same   = 0 ; % 1/0 Set Date 2 same as Date 1
    G.Limits.Default.Sequential = 0 ; % 1/0 Sequential or Seasonal averaging Flag
    G.Limits.Default.ClimatologyContinuous = 0 ; % present seasonal data sequentially in Hovmoller plots
    G.Limits.Default.Anom  = 0      ; % display anomalies in Hovmoller plots
                                                           
    G.Limits.S1 = GOAT_getSerial(year1,month1,day1,hour1,G.TimeFieldVirtual,G.Limits.Year0) ;
    if G.Limits.Default.Day2same && G.Limits.Default.Sequential
        G.Limits.S2 = G.Limits.S1 ;
    else
        G.Limits.S2 = GOAT_getSerial(year2,month2,day2,hour2,G.TimeFieldVirtual,G.Limits.Year0) ;
    end
    %% cell-3: Spatial limits
    % Checkboxes/Flags
    G.ProjectionType = 1 ; % 1) lat-lon with coast 2) lat-lon 3) lat-height 4)lon-height 5) hovmoller-lon 6) hovmoller-lat 7) hovmoller height 8/9) NH/SH polar stereographic
    G.Limits.Default.sameP = 0      ; % Projection 1,2,8,9) 1/0 set level2 = level1 | Projection 3,4,7) 1/0 composite levels
    G.Limits.Default.VerMeanInt = 2 ; % 1) Vertical Mean 2) Integral using Boer Beta 3) set Nan values to zero and vertically integrate
    G.Settings.NanMeanFlag = 0  ; % use mean(x) when 0 and nanmean(x) when 1
    
    G.Limits.Default.Lev1 = dsearchn(double(G.lev),Level1) ;
    G.Limits.Default.Lev2 = dsearchn(double(G.lev),Level2) ;
    G.Limits.Default.Lat1 = dsearchn(G.lat,Lat1)  ;
    G.Limits.Default.Lat2 = dsearchn(G.lat,Lat2)  ;
    G.Limits.Default.Lon1 = dsearchn(G.lon,Lon1)  ;
    G.Limits.Default.Lon2 = dsearchn(G.lon,Lon2)  ;
    % Filters, Weights and Indices:
    % No Filters, Weights or Indices Selected
    G.Filter.Index = ones(length(G.Addresses.Default),1) ; % default index
    %% cell-4: retrieve and compile data
                                                                                                                                                                                   
    if bata_avg
        G = GOAT_Compile_databata_3D(G) ; % obtain spatially truncated (but not projected) 3D timeseries of the BoerBeta function (when relevant)
    else
        if nargout > 5
            G = GOAT_Compile_bata_3D(G) ;
            bata{i} = G.Bata;
            G = GOAT_Compile_data_3D(G) ; % obtain spatially truncated (but not projected) 3D timeseries. Spatial and temporal filters can be applied.
        else
            G = GOAT_Compile_data_3D(G) ; % obtain spatially truncated (but not projected) 3D timeseries. Spatial and temporal filters can be applied.
        end
    end

    lon{i} = G.x ; lat{i} = G.y         ; % x and y vectors
    P{i} = G.P ;
    field_3d{i} = G.Data;
 
end
 