% GOAT_VirtualField_EHFC
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_EHFC(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  0 ; % | 1 for 'yes' and 0 for 'no'
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'EHFC' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'va','ta','wap'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'va' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = 'K/s^2' ;
        G.VirtualFields.FieldDescription = 'Eddy Heat Flux Convergence' ;
    case 'Load'
        G.DataField = 'va' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        vtag = data - repmat(nanmean(data,1),[length(G.lon) 1 1 ]) ;
        G.DataField = 'ta' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        data = data.*(1000./G.Vectors.levmat).^(2/7) ; % Potential Temperature
        Ttag = data - repmat(nanmean(data,1),[length(G.lon) 1 1 ]) ;
        EHFC = -diff(Ttag.*vtag.*G.Vectors.cosmat,1,2)/(G.lat(2)-G.lat(1))*180/pi ;
        lat2 = (G.lat(1:length(G.lat)-1)+G.lat(2:length(G.lat)))/2  ;
        data1 = 1/G.Consts.EarthRadius./G.Vectors.cosmat.*permute(interp1(lat2,permute(EHFC,[2 1 3]),G.lat,'linear','extrap'),[2 1 3]) ;
        G.DataField = 'wap' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        if size(data,3)~=length(G.lev)
            wdata = vtag*0 ;
            wdata(:,:,1:12) = data ;
        else
            wdata = data ;
        end
        wtag = wdata - repmat(nanmean(wdata,1),[length(G.lon) 1 1 1]) ;
        EHFCw = -diff(Ttag.*wtag,1,3)./diff(G.Vectors.levmat*100,1,3) ;
        P2 = (G.lev(1:end-1) + G.lev(2:end))/2 ;
        data2 = permute(interp1(double(P2),permute(EHFCw,[3 2 1]),double(G.lev),'linear','extrap'),[3 2 1]) ;
        data = data1 + data2 ;
        data(:,[1,end],:) = 0 ;
        G.DataField = 'EHFC' ; 
end
