% GOAT_VirtualField_LocalRosbyNumber
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_LocalRosbyNumber(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  0 ; % | 1 for 'yes' and 0 for 'no'
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'Ro' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'ua'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'ua' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = 'Unitless' ;
        G.VirtualFields.FieldDescription = 'Local Rossby Number' ;
    case 'Load'
        G.DataField = 'ua' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        utag = diff(data.*G.Vectors.cosmat,1,2)/(G.lat(2)-G.lat(1))*180/pi ;
        lat2 = (G.lat(1:length(G.lat)-1)+G.lat(2:length(G.lat)))/2  ;
        data = permute(interp1(lat2,permute(utag,[2 1 3]),G.lat,'linear','extrap'),[2 1 3])./(2*G.Consts.EarthRadius.*G.Consts.Omega.*G.Vectors.cosmat.*G.Vectors.sinmat) ;
end

