% GOAT_VirtualField_MeridionalStreamFunc
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_MeridionalStreamFunc(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'Psi' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'va'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'va' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = 'kg/s' ;
        G.VirtualFields.FieldDescription = 'Meridional mass Stream Function' ;
    case 'Load'
        G.DataField = 'va' ; [data,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        try
            [bata,G] = GOAT_load_bata(fullfile(DataFolder,TimeField,'BoerBeta',datafile),G,Aidx) ; 
        catch % no BoerBeta function
            bata = ones(size(data)) ;
        end
        if G.lev(1) < G.lev(end)
            data = (G.Consts.EarthRadius/G.Consts.GravitationalConst).*2.*pi.* ...
                cumtrapz(double(G.lev)*100,bata.*data.*G.Vectors.cosmat,3) ;
        else
            P = flipdim(G.lev,1) ;
            data = flipdim(bata.*data,3) ;
            data = (G.Consts.EarthRadius/G.Consts.GravitationalConst).*2.*pi.* ...
                cumtrapz(double(P)*100,data.*G.Vectors.cosmat,3) ;
            data = flipdim(data,3) ;
        end
        G.DataField = 'Psi' ;
end

