% GOAT_VirtualField_MoistStaticEnergy
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_MoistStaticEnergy(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  1 ; % 1/0   for Active/Inactive  virtual field
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'MSE' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'ta','zg','hus'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'ta' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = 'J/kg' ;
        G.VirtualFields.FieldDescription = 'Moist Static Energy' ;
    case 'Load'
        G.DataField = 'ta'  ; [T  ,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        G.DataField = 'zg'  ; [zg ,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        G.DataField = 'hus' ; [hus,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ; 
        L = GOAT_Calculate_L(T,G,'era') ;
        if isequal(size(T,3),size(hus,3)) 
         data = G.Consts.SpHeatConstPr*T + zg.*G.Consts.GravitationalConst + L.*hus ;
        else % e.g., NCEP: hus vertical dimention extends only to 300mb
         data = G.Consts.SpHeatConstPr.*T + zg.*G.Consts.GravitationalConst ;
         data(:,:,1:size(hus,3)) = data(:,:,1:size(hus,3)) + L(:,:,1:size(hus,3)).*hus ;
        end
        G.DataField = 'MSE' ; 
end