% GOAT_VirtualField_VorticityMeridional
% Copyright (C) Ori Adam Dec.2012. Revised Dec.2013. Compatible with version 2013l or later.
% Input:
% Mode = 'Active': returns 1/0 (yes/no) whether to include the field in the list of fields appearing in the GUI Fields dropdown menu.
%                  when 0 the field will not appear in the GUI Fields list but will be available via GOAT_Environment coding
% Mode = 'VirtualFieldName': returns the name of the virtual field
% Mode = 'RequiredFieldsList': returns the actual fields that need to exist for the calculation of the virtual field
% Mode = 'ReferenceField': returns a field name with the same spatial and temporal dimensions as the virtual field
% Mode = 'Dims': returns G.VirtualFields.DataFieldUnits = Virtual Field Units and G.VirtualFields.FieldDescription = Virtual Field Description 
% Mode = 'Load': the actual virtual field calculation code.
% G = GOAT main structure
% datafile = GOAT file frame address. e.g., '1979/10101.mat'
% DataFolder = GOAT data folder, e.g. 'user/GOAT/Data/NCEP-I'
% TimeField = GOAT Calendar. e.g., 'Monthly','Daily','DailyX4', etc.
% Aidx: datafile = G.DataStringVec(Aidx) ; this index can be used for calculating time derivatives.
%
% see also GOAT_VirtualFields

function [G , data] = GOAT_VirtualField_VorticityMeridional(Mode,G,datafile,DataFolder,TimeField,Aidx)

switch Mode
    case 'Active'
        G.VirtualFields.ActiveFlag =  1 ; % | 1 for 'yes' and 0 for 'no'
    case 'VirtualFieldName'
        G.VirtualFields.MyName = 'vor_x' ;
    case 'RequiredFieldsList'  % The existing fields required for the calculation of the virtual field
        G.VirtualFields.RequiredFieldsList = {'va','wap'} ;
    case 'ReferenceField' % a field with similar Dims
        G.VirtualFields.ReferenceField = 'va' ;
    case 'Dims'
        G.VirtualFields.DataFieldUnits = '1/s' ;
        G.VirtualFields.FieldDescription = 'Meridional Vorticity (pointing in the x direction)' ;
    case 'Load'
        G.DataField = 'va' ; [va,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;         
        G.DataField = 'wap'; [wap,G] = GOAT_load_data(fullfile(DataFolder,TimeField,G.DataField,datafile),G,Aidx) ;         
        % w component
        wap = wap.*G.Vectors.cosmat ;
        wtag = wap ; 
        wtag (:,2:end-1,:) = (wap(:,3:end,:) - wap(:,1:end-2,:))/(G.lat(3)-G.lat(1))*180/pi ;
        wtag = 1/G.Consts.EarthRadius./G.Vectors.cosmat.*wtag ;
        wtag(:,[1 end],:) = 0 ;
        % v component
        Pmat2 = squeeze(G.Vectors.levmat(1,:,:))*100    ;
        MeanV = squeeze(mean(va,1)) ;
        v2 = diff(MeanV,1,2)./diff(Pmat2,1,2) ;
        p2 = double(G.lev(1:end-1) + G.lev(2:end))/2 ;
        vtag = -permute(repmat(interp1(p2,v2',double(G.lev),'linear','extrap').',[1 1 size(va,1)]),[3,1,2]) ;
        data = wtag + vtag ;
        G.DataField = 'vor_x' ; 
end

