% Add or subtruct fields from the list of fields in the Fields Menu

function G = GOAT_Script_Add_Subtruct_Fields(G)
% Copyright (C) Ori Adam Jly.2012. Revised Nov.2013. Compatible with Version 2013h and onwards
if ~G.StepGUIFlag
    G = AddSubtruct_GUI_GOAT(G) ; 
    G = GOAT_Limits(G) ;
end
drawnow ;
if ~G.continueFlag
    Data0Flag = 1 ;
    for i = 1:length(G.DataFields) ;
        if G.AddSubtruct.AddDataFieldFlags(i) 
            G.DataField = cell2mat(G.DataFields(i)) ;
            G = GOAT_Compile_data_Series(G) ;
            if Data0Flag
                Data = G.AddSubtruct.AddDataFieldFlags(i)*G.Data*G.AddSubtruct.FacDataFieldFlags(i) ;
                Data0Flag = 0 ;
            else
                Data = Data + G.AddSubtruct.AddDataFieldFlags(i)*G.Data*G.AddSubtruct.FacDataFieldFlags(i) ;
            end
        end
    end
    if ~Data0Flag
    G.Data = Data ;
    end
    G.message = dateMessage(G) ;
    G.continueFlag = 0 ;
end
GOAT_plotUpdate(G)                       ; % Update Display
set(G.h_message,'string',G.message) ; % Update Message
drawnow ;

% GUI
function G = AddSubtruct_GUI_GOAT(G)
% Defaults
G.AddSubtruct.AddDataFieldFlags = zeros(length(G.DataFields),1) ; % 0 = none | 1 = Add
G.AddSubtruct.FacDataFieldFlags =  ones(length(G.DataFields),1) ; % 0 = none | 1 = Subtruct
NumAdd = min([5,length(G.DataFields)]) ; 
FieldNums = zeros(NumAdd,1) ; 
FieldFacs = ones(NumAdd,1) ; 
FieldNums(1) = 1 ; 
pos        = [G.scrsz(3)*1.5/5 G.scrsz(4)*2/5 (G.scrsz(3)*2/5) G.scrsz(4)*2/5] ;
NumLines = NumAdd + 5 ;
bl = 0.02              ; % Left Edge
bt = 0.01              ; % Top Edge
lh = (1-2*bt)/NumLines ; % Line Height
cl = (1-2*bl)/2        ; % column width (a total of 3 columns)
G.AddSubtruct_Fig = figure('name','GOAT: Add and Subtruct Fields','position',pos) ;
set(G.AddSubtruct_Fig,'Color',G.Display.FigBackgroudColor) ;
set(G.AddSubtruct_Fig,'MenuBar','none')

%Top Message
messageStr = {'Apply temporal and spatial truncations to a composite of fields.';'Fields must have the same temporal and spatial dimensions.' ;'projection and plotting settings correspond to current field.'} ;
h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-2*lh,1,lh*2], 'string',messageStr);
uicontrol(h_message) ;  set(h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_S,'FontName',G.GUI.Fonts.MessageFont) ;
% Add Title
Line = 3 ;
h_PlusT = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,1,Line), 'string','Field','fontweight','bold');
uicontrol(h_PlusT) ;  set(h_PlusT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Include Field','FontName',G.GUI.Fonts.uiHeaderFont) ;
% Factor Text
h_FactorT = uicontrol('Style', 'text','Units','normalized','pos',PosFunc(bl,bt,lh,cl,2,Line), 'string','Coefficient','fontweight','bold');
uicontrol(h_FactorT) ;  set(h_FactorT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','TooltipString','Constant Coefficient','FontName',G.GUI.Fonts.uiHeaderFont) ;
% create uicontrols
for i = 1: NumAdd
    % Fields Text
    Line = 3 + i ;
    I = int2str(i*10) ;
    h_Add(i) = uicontrol('style','popup','Units','normalized','pos',[bl,1-Line*lh,cl,lh*1], 'string',['None |',G.DataFieldsString],...
        'Value',FieldNums(i)+1,  'callback',['set(gcbf,''userdata'',',I,'); uiresume;'], ...
         'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_M,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiButtonFont) ;     
    I = int2str(i) ;
    h_Fac(i) = uicontrol('style','edit','Units','normalized','pos',[bl*2+cl,1-Line*lh,cl-bl,lh*1],...
        'string',int2str(G.AddSubtruct.FacDataFieldFlags(i)),  'callback',['set(gcbf,''userdata'',',I,'); uiresume;'], ...
        'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_L);
    uicontrol(h_Add(i)) ;
    uicontrol(h_Fac(i)) ;
    if G.AddSubtruct.AddDataFieldFlags(i)
        set(h_Add(i),'FontWeight','Bold') ; 
    end
end

Line = NumLines - .3 ;
h_OK = uicontrol('Units','normalized','pos',[bl*2,1-bt - Line*lh,1-4*bl,lh], 'string','OK',  'callback','set(gcbf,''userdata'',999); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_M,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;

GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',G.AddSubtruct_Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        I_Add = floor(select/10) ;
        if select < 10
            FieldFacs(select) = eval(get(h_Fac(select),'string'))  ;   
        elseif select ~=999
           FieldNums(I_Add) = get(h_Add(I_Add),'Value') - 1  ;   
        elseif select == 999
          GUIcon = 0 ;  
        end
    catch
        %nevermind
    end
end
FieldNums = FieldNums(FieldNums>0) ; 
FieldFacs = FieldFacs(FieldNums>0) ; 
G.AddSubtruct.AddDataFieldFlags(FieldNums) = 1 ; 
G.AddSubtruct.FacDataFieldFlags(FieldNums) =  FieldFacs ; 
try
    close(G.AddSubtruct_Fig) ;
catch
    % Nevermind
end

return ;