% Installs current Virtual Field as Real MAT
% Notes: 
% 1. Calendar folder needs to be reselected for new real field to be loaded by GOAT
% 2. It is best to convert an entire field span. Once a Real field is created, it will override the Virtual One
% 3. Ignores projection. The entire field is installed.

function G = GOAT_Script_VirtualField2MAT(G)
% copyright (C) Ori Adam, Apr.2014
% verify it is indeed a Virtual Field
if ~G.DataFieldsType(find(strcmp(G.DataField,G.DataFields)))
    G.continueFlag = 1 ; 
    set(G.h_message,'string',['Virtual Field conversion aborted: ',G.DataField,' is not a Virtual Field'])  ; drawnow ; 
    return
end
set(G.h_Analyze,'Enable','off')  ; drawnow ; % Disable GO button
G = GOAT_Limits(G) ; % set temporal span

if G.continueFlag % Cancel
    set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button
    return
end
% Install Dims assuming the field had already been initilized
P   = G.P   ; Plabel   = G.Plabel     ; 
lon = G.lon ; lonlabel = G.lonlabel   ; 
lat = G.lat ; latlabel = G.latlabel   ; 
FieldDescription = G.FieldDescription ; 
VerticalOrdinate = G.VerticalOrdinate ; 
Units            = G.DataFieldUnits   ; 
if ~exist(fullfile(G.DataFolder,G.TimeFieldVirtual,G.DataField),'dir')
    mkdir(fullfile(G.DataFolder,G.TimeFieldVirtual,G.DataField)) ; 
end
save(fullfile(G.DataFolder,G.TimeFieldVirtual,G.DataField,'Dims.mat'),'P','lon','lat','Plabel','lonlabel','latlabel','FieldDescription','VerticalOrdinate','Units') ; 

% Create string vector
G.DataStringVec = GOAT_Data_string_vec(G)                      ; % A vecotor of the Time Series File Indices
NF  = length(G.DataStringVec) ;
if NF < 3
  Nmod = 0            ;
else
  Nmod = ceil(NF/100) ;
  GOAT_Display_message(G,'','init') ; 
end

% Save each frame
for i = 1:NF
            if mod(i,Nmod) == 0
            GOAT_Display_message(G,[' . . . Installing Virtual Field  . . .  ',num2str(i/NF*100,'%02.0f'),' %'],'stat')
            end
            if G.Filter.Index(G.DataStringVec(i))
                [data,G] = GOAT_LoadData(G,G.DataFolder,G.TimeFieldVirtual,G.DataField,G.Addresses.Default{G.DataStringVec(i)},i) ; % Load the data File
                FileToSave = fullfile(G.DataFolder,G.TimeFieldVirtual,G.DataField,G.Addresses.Default{G.DataStringVec(i)}) ; 
                [FilePath,FileName,Ext] = fileparts(FileToSave) ; 
                FileToSave = fullfile(FilePath,[FileName,'.mat']) ; % In case of LINK fields
                if ~exist(FilePath,'dir')
                    mkdir(FilePath)
                end
                save(FileToSave,'data')
            end
end
GOAT_Display_message(G,[' . . . Installing Virtual Field -  ',G.DataField,' . . .  Complete.'],'stat')                
G.continueFlag = 1 ; % skip plotUpdate
set(G.h_Analyze,'Enable','on')  ; drawnow ; % re-enable GO button

