% GOAT_Script_nclink  the GOAT equivalent of 'ncview'
% Creates a browsable temporary LINK to nc files

function G = GOAT_Script_nclink(G) 

lon_dims   = {'lon','slon','ilon','longitude','X','x','long','i','rlon'}      ;
lat_dims   = {'lat','slat','ilat','latitude','Y','y','j','rlat'}              ;
level_dims = {'plev','lev','ilev','level','height','depth'} ;
time_dims  = {'time','type','loc','line','sza'}      ;
Description_Names = {'long_name','long-name','standard_name','standard-name'} ; 
Fields_To_Ignore = {'time_bnds','lat_bnds','lon_bnds'} ; 

G = LINK_GUI(G) ; 
 if G.LINK.CancelFlag
     return
 end
 switch G.LINK.VerticalOrdinateIdx
     case 1
      VerticalOrdinate = 'Pressure (mb)' ;    
     case 2
      VerticalOrdinate = 'Depth (m)' ;    
     case 3
      VerticalOrdinate = 'Height (m)' ;    
     case 4
      VerticalOrdinate = 'Potential Temperature (K)' ;    
     case 5
      VerticalOrdinate = 'Sigma' ;    
     case 6
      VerticalOrdinate = 'Geometric-Level' ;    
     case 7
      VerticalOrdinate = 'Level' ; 
 end
 
  switch G.LINK.SourceIdx
     case 1
      EXT = 'ncf' ;    
     case 2
      EXT = 'dap' ;    
 end
 %set(G.h_message,'string','Select nc file . . .')
 
[Path,TempDatasetName,ext] = fileparts(G.LINK.ncfile) ; 
TempDatasetName = regexprep(TempDatasetName,'[.]','_') ; 
a = ncinfo(G.LINK.ncfile) ; 
set(G.h_message,'string','Linking nc source . . .') ; drawnow ; 

Dimensions = {a.Dimensions(:).Name}' ; 
Variables  = {a.Variables(:).Name}' ; 
Attributes = {a.Attributes(:).Name}' ; 
Fields = Variables(~ismember(Variables,Dimensions)) ; 

LINK_Address = fullfile(fileparts(cd),'Downloads','temp','tempLINK') ; 
% create info string
try
infostr = 'Info :  Attributes' ; 
for i = 1:length(Attributes)
infostr = [infostr,'\n',Attributes{i},': '] ;
infostr = [infostr,'\n',a.Attributes(i).Value,' '] ;
end
infostr = sprintf(infostr) ; 
try
rmdir(fullfile(LINK_Address,'List'),'s') ; 
end
mkdir(fullfile(LINK_Address,'List')) ;
save(fullfile(LINK_Address,'List','info.mat'),'infostr') ; 
catch
    % Nevermind. No Info string
end
% link all fields
for i = 1:length(Fields)
  try  
   % inspect if field is linkable (1d invariant, text, etc.) and continue if not
   av = ncinfo(G.LINK.ncfile,Fields{i}) ; 
   if isempty(av.Dimensions) || ...
      isequal(av.Size,1)     || ...
      ~ismember(av.Datatype,{'double','single','int8','int16','int32','int64','uint8','uint16','uint32','uint64'}) || ...
      ismember(av.Name,Fields_To_Ignore) || ...
      0
      continue
   end
   % inspect Field dimensions
   FieldName = av.Name ; 
   DimIdx = [0,0,0,0] ; 
   for j = 1:length(av.Dimensions)
        if sum(strcmpi(av.Dimensions(j).Name,lon_dims))   ; DimIdx(1) = j ; lon_Name  = av.Dimensions(j).Name ; end
        if sum(strcmpi(av.Dimensions(j).Name,lat_dims))   ; DimIdx(2) = j ; lat_Name  = av.Dimensions(j).Name ; end
        if sum(strcmpi(av.Dimensions(j).Name,level_dims)) ; DimIdx(3) = j ; P_Name    = av.Dimensions(j).Name ; end
        if sum(strcmpi(av.Dimensions(j).Name,time_dims))  ; DimIdx(4) = j ; time_Name = av.Dimensions(j).Name ; end
   end
   if DimIdx(1) ; 
       lon = ncread(G.LINK.ncfile,lon_Name) ;  
       ad = ncinfo(G.LINK.ncfile,lon_Name) ;  
       lonlabel = 'Longitude' ; % default
       for d = 1:length(Description_Names)
         try
         lonlabel = ad.Attributes(find(strcmpi({ad.Attributes(:).Name},Description_Names{d}))).Value ;
         break ; 
         end
       end
   else
      lonlabel = 'Longitude' ;  
      lon = 0 ; 
   end
   if DimIdx(2) 
       lat = ncread(G.LINK.ncfile,lat_Name) ;  
       ad = ncinfo(G.LINK.ncfile,lat_Name) ;  
       latlabel = 'Latitude' ; % default
       for d = 1:length(Description_Names)
         try
         latlabel = ad.Attributes(find(strcmpi({ad.Attributes(:).Name},Description_Names{d}))).Value ;
         break ; 
         end
       end
   else
       latlabel = 'Latitude' ; 
       lat = 0 ; 
   end
   if DimIdx(3) 
       P   = ncread(G.LINK.ncfile,P_Name  ) ;  
       ad = ncinfo(G.LINK.ncfile,P_Name) ;  
       Plabel = 'Level' ; % default
       for d = 1:length(Description_Names)
         try
         Plabel = ad.Attributes(find(strcmpi({ad.Attributes(:).Name},Description_Names{d}))).Value ;
         break ; 
         end
       end
   else
       Plabel = 'Level' ; 
       P   = 1 ; 
   end
   % attempt to determine units
   try
       Units = av.Attributes(find(strcmpi({av.Attributes(:).Name},'units'))).Value ; 
   catch
       Units = 'Unknown Units' ; 
   end
   % attempt to determine field description
   FieldDescription = ' Description Unknown ' ;
   for d = 1:length(Description_Names)
      try
       FieldDescription = av.Attributes(find(strcmpi({av.Attributes(:).Name},Description_Names{d}))).Value ;
       break ; 
      end
   end
   mkdir(fullfile(LINK_Address,'List',FieldName)) ;
   save(fullfile(LINK_Address,'List',FieldName,'Dims.mat'),'lon','lat','P','FieldDescription','VerticalOrdinate','Units','lonlabel','latlabel','Plabel') ; 
   % move on to creating link file
   % Attributes(i,1) = filename ;
   % Attributes(i,2) = varname  ;
   % Attributes(i,3) = start    ;
   % Attributes(i,4) = count    ;
   % Attributes(i,5) = factor   ;
   % Attributes(i,6) = permute   ; < Normally, no need for CMIP5 variables
   % Attributes(i,7) = reshape (add dimensions)  ; < Normally, no need for CMIP5 variables
   % Assign Attributes to each date
   Attributes = cell(2,10) ;
   if ~DimIdx(4)
   q = 1 ;
   else
   ad = ncinfo(G.LINK.ncfile,time_Name) ;
   q = max(ad.Size) ;     
   end
   dimstrings = {'lon','lat','lev','time'} ;
   Dimstr = [] ;
       for d = 1:4
           if ismember(d,DimIdx)
           Dimstr = [Dimstr,dimstrings{find(DimIdx==d)},'x'] ;
           end
       end
       Dimstr = Dimstr(1:end-1) ;
   if q == 1
       Time = 'Invariant' ; 
       Attributes(1,1) = {G.LINK.ncfile}  ;
       Attributes(1,2) = {FieldName}  ;
       Attributes(1,5) = {[]}       ;
       Attributes(1,10) = {FieldName}       ;
       
       switch Dimstr
           case 'lon'
               start = 1 ;
               count = inf ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
           case 'lonxtime'
               start = [1 1] ;
               count = [inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
           case 'timexlon'
               start = [1 1] ;
               count = [1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;         
               Attributes(1,6) = {[2,1]} ;         
           case 'lat'
               start = 1 ;
               count = inf ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {1}   ;
           case 'latxtime'
               start = [1,1] ;
               count = [inf,1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {1}   ;    
           case 'timexlat'
               start = [1,1] ;
               count = [1,inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
           case 'lev'
               start = 1 ;
               count = inf ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {[1,1]}   ;
           case 'levxtime'
               start = [1,1] ;
               count = [inf,1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {[1,1]}   ;    
           case 'timexlev'
               start = [1,1] ;
               count = [1,inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {[1]}   ;        
           case 'lonxlat'
               start = [1 1] ;
               count = [inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
           case 'lonxlatxtime'
               start = [1 1 1] ;
               count = [inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
           case 'lonxtimexlat'
               start = [1 1 1] ;
               count = [inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
               Attributes(1,6) = {[1,3,2]} ;     
           case 'timexlonxlat'
               start = [1 1 1] ;
               count = [1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;         
               Attributes(1,6) = {[2,3,1]} ;     
           case 'latxlon'
               start = [1 1] ;
               count = [inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1]}   ;
           case 'latxlonxtime'
               start = [1 1 1] ;
               count = [inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1,3]}   ;    
           case 'latxtimexlon'
               start = [1 1 1] ;
               count = [inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,1,2]}   ;    
           case 'timexlatxlon'
               start = [1 1 1] ;
               count = [1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,2,1]}   ;        
           case 'lonxlev'
               start = [1 1] ;
               count = [inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {2}   ;
           case 'lonxlevxtime'
               start = [1 1 1] ;
               count = [inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[1,3,2]}   ;    
           case 'lonxtimexlev'
               start = [1 1 1] ;
               count = [inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
           case 'timexlonxlev'
               start = [1 1 1] ;
               count = [1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1,3]}   ;        
           case 'levxlon'
               start = [1 1] ;
               count = [inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1]}   ;
               Attributes(1,7) = {2}   ;
           case 'levxlonxtime'
               start = [1 1 1] ;
               count = [inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,3,1]}   ;    
           case 'levxtimexlon'
               start = [1 1 1] ;
               count = [inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,2,1]}   ;
           case 'timexlevxlon'
               start = [1 1 1] ;
               count = [1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,1,2]}   ;
           case 'latxlev'
               start = [1 1] ;
               count = [inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,7) = {1}   ;
           case 'latxlevxtime'
               start = [1 1 1] ;
               count = [inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,1,2]}   ;    
           case 'latxtimexlev'
               start = [1 1 1] ;
               count = [inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1,3]}   ;    
           case 'timexlatxlev'
               start = [1 1 1] ;
               count = [1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;         
           case 'levxlat'
               start = [1 1] ;
               count = [inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1]}   ;
               Attributes(1,7) = {1}   ;
           case 'levxlatxtime'
               start = [1 1 1] ;
               count = [inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,2,1]}   ;
           case 'levxtimexlat'
               start = [1 1 1] ;
               count = [inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,3,1]}   ;
           case 'timexlevxlat'
               start = [1 1 1] ;
               count = [1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[1,3,2]}   ;
           case 'lonxlatxlev'
               start = [1 1 1] ;
               count = [inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;       
           case 'lonxlatxlevxtime'
               start = [1 1 1 1] ;
               count = [inf inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
           case 'lonxlatxtimexlev'
               start = [1 1 1 1] ;
               count = [inf inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
               Attributes(1,6) = {[1,2,4,3]} ;     
           case 'lonxtimexlatxlev'
               start = [1 1 1 1] ;
               count = [inf 1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
               Attributes(1,6) = {[1,3,4,2]} ;     
           case 'timexlonxlatxlev'
               start = [1 1 1 1] ;
               count = [1 inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ;     
               Attributes(1,6) = {[2,3,4,1]} ;     
           case 'lonxlevxlat'
               start = [1 1 1] ;
               count = [inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[1,3,2]}   ;
           case 'lonxlevxlatxtime'
               start = [1 1 1 1] ;
               count = [inf inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[1,3,2,4]}   ;
           case 'lonxlevxtimexlat'
               start = [1 1 1 1] ;
               count = [inf inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[1,4,2,3]}   ;
           case 'lonxtimexlevxlat'
               start = [1 1 1 1] ;
               count = [inf 1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[1,4,3,2]}   ;
           case 'timexlonxlevxlat'
               start = [1 1 1 1] ;
               count = [1 inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,4,3,1]}   ;
           case 'latxlonxlev'
               start = [1 1 1] ;
               count = [inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1,3]}   ;
          case 'latxlonxlevxtime'
               start = [1 1 1 1] ;
               count = [inf inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1,3,4]}   ;
          case 'latxlonxtimexlev'
               start = [1 1 1 1] ;
               count = [inf inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,1,4,3]}   ;
          case 'latxtimexlonxlev'
               start = [1 1 1 1] ;
               count = [inf 1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,1,4,2]}   ;
          case 'timexlatxlonxlev'
               start = [1 1 1 1] ;
               count = [1 inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,2,4,1]}   ;
           case 'latxlevxlon'
               start = [1 1 1] ;
               count = [inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,3,1]}   ;
           case 'latxlevxlonxtime'
               start = [1 1 1 1] ;
               count = [inf inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,1,2,4]}   ;
           case 'latxlevxtimexlon'
               start = [1 1 1 1] ;
               count = [inf inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[4,1,2,3]}   ;
           case 'latxtimexlevxlon'
               start = [1 1 1 1] ;
               count = [inf 1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[4,1,3,2]}   ;
           case 'timexlatxlevxlon'
               start = [1 1 1 1] ;
               count = [1 inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[4,2,3,1]}   ;
           case 'levxlonxlat'
               start = [1 1 1] ;
               count = [inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,3,1]}   ;
           case 'levxlonxlatxtime'
               start = [1 1 1 1] ;
               count = [inf inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,3,1,4]}   ;
           case 'levxlonxtimexlat'
               start = [1 1 1 1] ;
               count = [inf inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[2,4,1,3]}   ;
           case 'levxtimexlonxlat'
               start = [1 1 1 1] ;
               count = [inf 1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,4,1,2]}   ;
           case 'timexlevxlonxlat'
               start = [1 1 1 1] ;
               count = [1 inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,4,2,1]}   ;
           case 'levxlatxlon'
               start = [1 1 1] ;
               count = [inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,2,1]}   ;
           case 'levxlatxlonxtime'
               start = [1 1 1 1] ;
               count = [inf inf inf 1] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[3,2,1,4]}   ;
           case 'levxlatxtimexlon'
               start = [1 1 1 1] ;
               count = [inf inf 1 inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[4,2,1,3]}   ;
           case 'levxtimexlatxlon'
               start = [1 1 1 1] ;
               count = [inf 1 inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[4,3,1,2]}   ;
            case 'timexlevxlatxlon'
               start = [1 1 1 1] ;
               count = [1 inf inf inf] ;
               Attributes(1,3) = {start} ; 
               Attributes(1,4) = {count} ; 
               Attributes(1,6) = {[4,3,2,1]}   ;   
       end
   else
       Time = 'TimeSeries' ; 
       Col = cell(1,2) ;
       Col(1,1) = {G.LINK.ncfile} ;
       Col(1,2) = {q} ;
       Attributes(1,1) = {Col}  ;
       Col(1,1) = {FieldName} ;
       Attributes(1,2) = {Col}       ;
       Col(1,1) = {['Col = cellstr([repmat(''Step'',[',int2str(q),' 1]),num2str((1:',int2str(q),')'')]) ; ']} ;
       Col(1,2) = {'eval'} ; 
       Attributes(1,10) = {Col}       ;
       switch Dimstr
           case 'time'
               Col(1,1) = {['Col = num2cell(1:',int2str(q),')'' ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {1} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'lonxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',2) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf , 1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'latxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',2) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf , 1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {1} ; 
               Attributes(1,7) = {Col}   ;
           case 'levxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',2) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf , 1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,1]} ; 
               Attributes(1,7) = {Col}   ;
           case 'timexlon'
               Col(1,1) = {['Col = ones(',int2str(q),',2) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1, inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2, 1]} ; 
               Attributes(1,6) = {Col} ; 
           case 'timexlat'
               Col(1,1) = {['Col = ones(',int2str(q),',2) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'timexlev'
               Col(1,1) = {['Col = ones(',int2str(q),',2) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {1} ; 
               Attributes(1,7) = {Col}   ;    
           case 'lonxlatxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'latxlonxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1]} ; 
               Attributes(1,6) = {Col} ;
           case 'lonxtimexlat'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,3,2]} ; 
               Attributes(1,6) = {Col} ;    
           case 'latxtimexlon'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,1,2]} ; 
               Attributes(1,6) = {Col} ; 
           case 'timexlonxlat'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,3,1]} ; 
               Attributes(1,6) = {Col} ;    
           case 'timexlatxlon'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,2,1]} ; 
               Attributes(1,6) = {Col} ;     
           case 'lonxlevxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {2} ; 
               Attributes(1,7) = {Col} ;
           case 'levxlonxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1]} ; 
               Attributes(1,6) = {Col} ;
               Col(1,1) = {2} ; 
               Attributes(1,7) = {Col} ;
           case 'lonxtimexlev'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'levxtimexlon'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,2,1]} ; 
               Attributes(1,6) = {Col} ; 
           case 'timexlonxlev'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1,3]} ; 
               Attributes(1,6) = {Col} ;    
           case 'timexlevxlon'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,1,2]} ; 
               Attributes(1,6) = {Col} ;         
           case 'latxlevxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {1} ; 
               Attributes(1,7) = {Col} ;
           case 'levxlatxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1,3]} ; 
               Attributes(1,6) = {Col} ;
               Col(1,1) = {1} ; 
               Attributes(1,7) = {Col} ;
           case 'latxtimexlev'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1,3]} ; 
               Attributes(1,6) = {Col} ;
           case 'levxtimexlat'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,1,2]} ; 
               Attributes(1,6) = {Col} ;
               Col(1,1) = {1} ; 
               Attributes(1,7) = {Col} ;
           case 'timexlatxlev'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'timexlevxlat'
               Col(1,1) = {['Col = ones(',int2str(q),',3) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,3,2]} ; 
               Attributes(1,6) = {Col} ;
          case 'lonxlatxlevxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,4) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
           case 'lonxlevxlatxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,4) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,3,2]} ; 
               Attributes(1,6) = {Col} ;    
           case 'latxlonxlevxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,4) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1,3]} ; 
               Attributes(1,6) = {Col} ;        
           case 'latxlevxlonxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,4) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,1,2]} ; 
               Attributes(1,6) = {Col} ;            
           case 'levxlonxlatxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,4) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,3,1]} ; 
               Attributes(1,6) = {Col} ;            
            case 'levxlatxlonxtime'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,4) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,inf,1]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,2,1]} ; 
               Attributes(1,6) = {Col} ;  
            case 'lonxlatxtimexlev'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,2,4,3]} ; 
               Attributes(1,6) = {Col} ;    
           case 'lonxlevxtimexlat'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,4,2,3]} ; 
               Attributes(1,6) = {Col} ;    
           case 'latxlonxtimexlev'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,1,4,3]} ; 
               Attributes(1,6) = {Col} ;        
           case 'latxlevxtimexlon'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[4,1,2,3]} ; 
               Attributes(1,6) = {Col} ;            
           case 'levxlonxtimexlat'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,4,1,3]} ; 
               Attributes(1,6) = {Col} ;            
            case 'levxlatxtimexlon'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,3) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,inf,1,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[4,2,1,3]} ; 
               Attributes(1,6) = {Col} ;     
            case 'lonxtimexlatxlev'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,3,4,2]} ; 
               Attributes(1,6) = {Col} ;    
           case 'lonxtimexlevxlat'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[1,4,3,2]} ; 
               Attributes(1,6) = {Col} ;    
           case 'latxtimexlonxlev'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,1,4,2]} ; 
               Attributes(1,6) = {Col} ;        
           case 'latxtimexlevxlon'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[4,1,3,2]} ; 
               Attributes(1,6) = {Col} ;            
           case 'levxtimexlonxlat'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,4,1,2]} ; 
               Attributes(1,6) = {Col} ;            
            case 'levxtimexlatxlon'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,2) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[inf,1,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[4,3,1,2]} ; 
               Attributes(1,6) = {Col} ;      
           case 'timexlonxlatxlev'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,3,4,1]} ; 
               Attributes(1,6) = {Col} ;    
           case 'timexlonxlevxlat'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[2,4,3,1]} ; 
               Attributes(1,6) = {Col} ;    
           case 'timexlatxlonxlev'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,2,4,1]} ; 
               Attributes(1,6) = {Col} ;        
           case 'timexlatxlevxlon'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[4,2,3,1]} ; 
               Attributes(1,6) = {Col} ;            
           case 'timexlevxlonxlat'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[3,4,2,1]} ; 
               Attributes(1,6) = {Col} ;            
            case 'timexlevxlatxlon'
               Col(1,1) = {['Col = ones(',int2str(q),',4) ; Col(:,1) = (1:',int2str(q),') ; Col = num2cell(Col,2) ; ']} ; 
               Col(1,2) = {'eval'} ;
               Attributes(1,3) = {Col} ; 
               Col(1,1) = {[1,inf,inf,inf]} ; 
               Col(1,2) = {q} ;
               Attributes(1,4) = {Col} ; 
               Col(1,1) = {[4,3,2,1]} ; 
               Attributes(1,6) = {Col} ;          
       end  
   end
   Attributes(end,1) = {EXT} ; % 'mat' | 'ncf' | 'dap' | 'gb1' | 'gb2'
   Attributes(end,3) = {[]} ; % Defs Struct
   Attributes(end,4) = {'List'} ;
   if q > 1
   Attributes(end,5) = {q} ;
   end
   mkdir(fullfile(LINK_Address,'List',FieldName,Time)) ; 
   save(fullfile(LINK_Address,'List',FieldName,Time,'link_field.mat'),'Attributes') ;            
  catch
      disp(['Failed to LINK Variable ',FieldName,'.'])
  end
end
try
% set Data folder to temp LINK folder
G.DataSelected = length(G.DataFolders) ;
set(G.h_DataSelection,'Value',G.DataSelected) ;
G.DataFolder = LINK_Address ;
set(G.h_DataSelection,'Tooltipstring',G.LINK.ncfile) ;
G.TimeFieldIdx = 1 ;
G = GOAT_Initialize_Dataset(G) ;
if strcmp(G.TimeField,'Monthly') ;
    G.Limits.Default.DataClimSmooth = 1 ;
end
if exist(fullfile(G.DataFolder,G.TimeField,'info.mat'),'file')
    try
        load(fullfile(G.DataFolder,G.TimeField,'info.mat')) ;
        G.Limits.LinearInfo = infostr ;
    catch
        G.Limits.LinearInfo = '' ;
    end
else
    G.Limits.LinearInfo = '' ;
end
set(G.h_info,'TooltipString',G.Limits.LinearInfo) ;
set(G.h_Field,'string',G.DataFieldsString,'value',1) ;
set(G.h_TempRes,'string',G.TempResStr,'value',G.TimeFieldIdx);
try
G = FilterColor(G) ;
end
try
    set(G.h_Field,'TooltipString',['[ ',G.DataFieldUnits,' ]   ',G.FieldDescription]) ;
end
G = GOAT_setProjectionStr(G) ;
set(G.h_Projection,'string',G.GUI.ProjectionStr) ;
% set script to None
G.Scripts.ScriptsIdx = 1 ; 
set(G.h_ToolBoxes,'value',1) ;
set(G.h_ToolBoxes,'TooltipString','select a script and press ''GO''')
% inform that LINK is complete
set(G.h_message,'string','nc file temporary LINK ready to GO') ; drawnow
catch
set(G.h_message,'string','nc LINK failed') ; drawnow    
end


function G = LINK_GUI(G)

G.LINK.ncfile = fullfile(cd,'ncfile.nc') ;
G.LINK.VerticalOrdinateIdx = 1 ; 
G.LINK.SourceIdx = 1 ; 
G.LINK.CancelFlag = 0 ;
pos        = G.GUI.pos.Export ;
G.LINK.Fig = figure('name','Link MIP netCDF files','position',pos,'color',G.Display.FigBackgroudColor) ;
set(G.LINK.Fig,'MenuBar','none')
NumLines = 8 ;
bl = 0.02 ; % Left Edge
bt = 0.05; % Top Edge
LineHeight = (1-2*bt)/NumLines ;
Column     = (1-2*bl)/1 ;
%Top Message
messageStr = 'Browse NetCDF files or OPeNDAP urls by creating a temp LINK archive' ;
G.LINK.h_message = uicontrol('Style', 'text','Units','normalized','pos',[0,1-LineHeight,1,LineHeight], 'string',messageStr);
uicontrol(G.LINK.h_message) ;  set(G.LINK.h_message,'ForegroundColor',G.GUI.messageColor,'FontSize',G.Display.TextSize_S,'FontName',G.GUI.Fonts.MessageFont) ;
% title
Line = 2 ;
h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column,LineHeight], 'string','NetCDF file or OPeNDAP url address:');
uicontrol(h_NameT) ;  set(h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 3 ;
G.LINK.h_Name = uicontrol('style','edit','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*4/5-bl,LineHeight],...
    'string',G.LINK.ncfile,  'callback','set(gcbf,''userdata'',11); uiresume;');
uicontrol(G.LINK.h_Name) ;  set(G.LINK.h_Name,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',12,'FontName',G.GUI.Fonts.uiButtonFont,'HorizontalAlignment','left','TooltipString','A folder containing MIP netCDF files') ;
% Browse
G.LINK.h_Browse = uicontrol('Units','normalized','pos',[bl + Column*4/5,1-bt-Line*LineHeight,Column/5 ,LineHeight],...
    'string','Browse',  'callback','set(gcbf,''userdata'',112); uiresume;');
uicontrol(G.LINK.h_Browse) ;  set(G.LINK.h_Browse,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
Line = 5 ;
h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/2.1-bl,LineHeight], 'string','Vertical Ordinate Type:');
uicontrol(h_NameT) ;  set(h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 6 ;
h_Levels = uicontrol('style','popup','Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column/2.1-bl,LineHeight],...
    'string','Pressure (mb) | Depth (m) | Height (m) | Potential Temperature (K) | Sigma | Geometric-Level (increasing with height) | Level (decreasing with height)','value',G.Settings.LevelsIdx,'callback','set(gcbf,''userdata'',205); uiresume;');
uicontrol(h_Levels) ;  set(h_Levels,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','Vertical ordinate type','FontName',G.GUI.Fonts.uiButtonFont) ;
Line = 5 ;
h_NameT = uicontrol('Style', 'text','Units','normalized','pos',[bl+Column/2,1-bt-Line*LineHeight,Column/2.1-bl,LineHeight], 'string','Source:');
uicontrol(h_NameT) ;  set(h_NameT,'BackgroundColor',G.Display.FigBackgroudColor,'ForegroundColor',G.GUI.HeaderColor,'FontSize',G.Display.TextSize_L,'HorizontalAlignment','left','FontName',G.GUI.Fonts.uiHeaderFont) ;
Line = 6 ;
h_Link = uicontrol('style','popup','Units','normalized','pos',[bl+Column/2.1,1-bt-Line*LineHeight,Column/2.1-bl,LineHeight],...
    'string','NetCDF file | OPeNDAP URL ','value',G.Settings.LevelsIdx,'callback','set(gcbf,''userdata'',105); uiresume;');
uicontrol(h_Link) ;  set(h_Link,'BackgroundColor',[1 1 1],'ForegroundColor',[.2 .2 .2],'FontSize',G.Display.TextSize_S,'HorizontalAlignment','left', ...
    'TooltipString','LINK source type','FontName',G.GUI.Fonts.uiButtonFont) ;
% Cancel
Line = NumLines  ;
h_Cancel = uicontrol('Units','normalized','pos',[bl*2 + Column*2/3,1-bt-Line*LineHeight,Column/3 - bl,LineHeight],...
    'string','Cancel',  'callback','set(gcbf,''userdata'',99); uiresume;');
uicontrol(h_Cancel) ;  set(h_Cancel,'BackgroundColor',G.GUI.ButtonColor,'ForegroundColor',G.GUI.DropTextColor,'FontSize',G.Display.TextSize_L,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;
% OK
Line = NumLines  ;
h_OK = uicontrol('Units','normalized','pos',[bl,1-bt-Line*LineHeight,Column*2/3,LineHeight],...
    'string','OK',  'callback','set(gcbf,''userdata'',100); uiresume;');
uicontrol(h_OK) ;  set(h_OK,'BackgroundColor',G.GUI.colors.celeste,'ForegroundColor',G.GUI.DropTextColor,'FontSize',16,'fontweight','bold','FontName',G.GUI.Fonts.uiButtonFont) ;


GUIcon = 1 ;
while GUIcon
    try
        set(0,'CurrentFigure',G.LINK.Fig) ;
        uiwait(gcf);
        select = get(gcf,'userdata');
    catch
        GUIcon = 0 ;
    end
    try
        switch select
            case 11 % Folder name of export
                G.LINK.ncfile          = get(G.LINK.h_Name,'string')            ;
            case 112 % Browse
                try
                    [fid,fidf] = uigetfile({'*.*','All Files'}) ;    
                    try
                        if fid == 0
                            continue ;
                        end
                    end
                    G.LINK.ncfile = [fidf,fid] ; 
                    set(G.LINK.h_Name,'string',G.LINK.ncfile)
                catch
                    % Nevermind
                end
            case 105
                G.LINK.SourceIdx = get(h_Link,'value') ;     
            case 205
                G.LINK.VerticalOrdinateIdx = get(h_Levels,'value') ; 
            case 99 % Clear
                G.LINK.CancelFlag = 1 ;
                GUIcon = 0 ;
            case 100 % OK
                GUIcon = 0 ;
        end
    catch
        %nevermind
    end
end

close(G.LINK.Fig) ; drawnow


% Conditions for Filter Color (green or gray)
function G = FilterColor(G)
if G.Filter.WeightIdx          ~= 1 || ...
        sum(G.Filter.ZonalWaveNumberIdx) > -1 || ...
        G.Filter.RAIdx              ~= 1 || ...
        G.Filter.LowPassIdx         ~= 0 || ...
        G.Filter.HighPassIdx        ~= 0 || ...
        G.Filter.IndicesIdx         ~= 1 || ...
        G.Filter.ZEFlag
    set(G.h_Filters,'Backgroundcolor',G.GUI.ActiveColor)
else
    set(G.h_Filters,'Backgroundcolor',G.GUI.ButtonColor)
end
